
EnableExplicit

XIncludeFile "..\_sgx_includer.pbi"

InitSprite()
InitMouse()
InitKeyboard()

Global event, quit, ww, wh

; windowed or full screen
If 01
  ww=600
  wh=400
  OpenWindowedScreen( OpenWindow(0, 1,1, ww,wh,"..", #PB_Window_BorderLess | #PB_Window_ScreenCentered), 0,0,ww,wh,0,0,0)
Else
  ww=800
  wh=600
  OpenScreen(ww,wh,32,"",#PB_Screen_NoSynchronization)
EndIf

Global design1=sgxLoadDesign("..\..\sgxDesigns\next")
If Not design1   :   MessageRequester("error","can't load design") : End   :   EndIf


; this example shows how to implement a file requester with sgx.
; the example also shows:
; - handling of a window's 'resized' event to adjust it's child elements
; - using a scrollbar to implement some listicon-like functionality including creating/destroying/positioning
;   sgx elements for visible/invisble entries of a some list.
; - a general aproach of creating a compound object containing sgx elements and custom data.
;
; in the main loop we use fileRequesterSgx_open() to create one and fileRequesterSgx_examine() to
; let the requester handle its events. the requester will handle its resizing and scrolling on its
; own, we only have to call the examine function which also will tell us the requester's state or 
; if it has been finished/destroyed. after that we can use the global map fileRequesterSgx_selectedEntries()
; to get the selected entries. since we only use one such global map, we disable the 'open' button while
; the requester is open to avoid multiple instances.
;
; the requester can be created with several options:
; - the title
; - the initial path (relative/absolute/empty)
; - a pattern (which will be handed to ExamineDirectory() as is, so you might not see any directory entries
;   if you specifiy a file pattern for example)
; - 'multiple' to allow multiple selection
; - 'selectFolders' to allow selection of directories. if disabled then the requester will traverse into
;   these directories instead if a directory entry is clicked.
;


; type of the entry list elements of a requester.
; each directoy entry will have such an element but the sgx elements in it get created only if the entry is
; currently visible and get destroyed if the element becomes invisible while scrolling.
Structure s_fileRequesterSgx_entry ;{
  sgxCheckbox.i  ; - the sgx elements of a currently visible directory entry
  sgxText.i      ; /
  
  type.i         ; file or directory
  name.s         ; filename
  fullpath.s     ; filepath + filename
  ;}
EndStructure


; type of the requester.
; it contains some sgx elements during it's lifetime like the window and buttons etc.
; besides some properties it also contains a list of all directory entries (with additional sgx elements if the entry
; is currently visible).
Structure s_fileRequesterSgx ;{
  sgxWindow.i
  sgxTextPath.i
  sgxScrollbar.i
  sgxButtonCancel.i
  sgxButtonOK.i
  
  state.i
  path.s
  pattern.s
  multiple.i
  selectFolders.i
  List entries.s_fileRequesterSgx_entry()
  entryWidth.i
  entryHeight.i
  maxShowEntries.i
  margin.i
  ;}
EndStructure

; requester states returned by fileRequesterSgx_examine()
; the requester will be 'idle' until completely finished (including the window animation)
Enumeration 1
  #fileRequesterSgx_state_idle
  #fileRequesterSgx_state_closed
  #fileRequesterSgx_state_canceled
  #fileRequesterSgx_state_ok
EndEnumeration

; macro isRelativePath()
; -------------
; origin: http://www.purebasic.fr/english/viewtopic.php?p=277703#p277703
CompilerIf #PB_Compiler_OS = #PB_OS_Windows
  Macro isRelativePath(file)
    (Mid(file, 2, 1) <> ":")
  EndMacro
CompilerElse
  Macro isRelativePath(file)
    (Left(file, 1) <> "/")
  EndMacro
CompilerEndIf
; -------------

; normalizePath()
; resolves and removes redundant "\..\" and "\.\" parts from a path
Procedure.s normalizePath(path.s)
  Protected i, n
  Protected Dim result.s(0)
  
  ; remove double backslashes
  While FindString(path, "\\")
    path = ReplaceString(path, "\\", "\")
  Wend
  
  ; remove "\.\" parts
  While FindString(path, "\.\")
    path = ReplaceString(path, "\.\", "\")
  Wend
  
  ; find any number of chars that are not a backslash followed by a backslash, 2 dots and a backslash..
  Protected regex = CreateRegularExpression(#PB_Any, "[^\\]*\\\.\.\\")
  If regex
    Repeat
      n = ExtractRegularExpression(regex, path, result.s())
      If n > 0
        ; ..and remove them
        path = ReplaceString(path, result(0), "")
      EndIf
    Until n=0
    FreeRegularExpression(regex)
  EndIf
  
  ProcedureReturn path
EndProcedure


; creates and returns a requester, or 0 if allocation fails or if the directory could not be examined
Declare fileRequesterSgx_open(title.s, path.s, pattern.s="", multiple=#False, selectFolders=#False)

; let a requester handle its events and return its current state
Declare fileRequesterSgx_examine(*requester.s_fileRequesterSgx)

; called by fileRequesterSgx_open() and fileRequesterSgx_examine() to set/change the requester's path
Declare fileRequesterSgx_setDirectory(*requester.s_fileRequesterSgx, path.s)

; called by fileRequesterSgx_examine() to resize all elements
Declare fileRequesterSgx_resize(*requester.s_fileRequesterSgx)

; called by fileRequesterSgx_examine() to create and position/move and delete sgx elements for visible/invisible entries
Declare fileRequesterSgx_arrangeEntries(*requester.s_fileRequesterSgx)


; selecting/deselecting entries will add/remove them from that map during the requester's 'idle' state.
; the map keys (filenames) can be inquired after the requester finished with 'ok'. since we use only one single global
; map, we can not use multiple reuqesters with this example and ensure this by disabling the 'open file
; requester' button while a requester is open
Global NewMap fileRequesterSgx_selectedEntries()


Procedure fileRequesterSgx_open(title.s, path.s, pattern.s="", multiple=#False, selectFolders=#False)
  Protected *requester.s_fileRequesterSgx
  
  ; just make sure the selection is reset
  ClearMap(fileRequesterSgx_selectedEntries())
  
  ; create the requester including an empty entries list
  *requester = AllocateStructure(s_fileRequesterSgx)
  
  If *requester
    *requester\pattern = pattern
    *requester\multiple = Bool(multiple)
    *requester\selectFolders = Bool(selectFolders)
    *requester\state = #fileRequesterSgx_state_idle
    
    If fileRequesterSgx_setDirectory(*requester, path)
    
      ; default size of the requester
      Protected ww = 300
      Protected wh = 200
      Protected y, n
      ; center the requester on screen
      Protected wx = sgx_dimensions\w / 2 - ww / 2
      Protected wy = sgx_dimensions\h / 2 - wh / 2
      ; will be set to real values in fileRequesterSgx_resize()
      *requester\entryWidth = 1
      *requester\entryHeight = 1
      *requester\maxShowEntries = 1
      ; space between sgx elements
      *requester\margin = 8
      ; create the general sgx elements. position and size will be handled later in fileRequesterSgx_resize()
      *requester\sgxWindow = sgxWin(-1, wx, wy, ww, wh, title)
        *requester\sgxTextPath     = sgxText     (-1, 0,0, 1,1, *requester\path, #sgxStyleThin)
        *requester\sgxScrollbar    = sgxScrollbar(-1, 0,0, 1,1, 20,20, 1) ; (updating scrollbar range later)
        *requester\sgxButtonOK     = sgxButton   (-1, 0,0, 1,1, "OK")
        *requester\sgxButtonCancel = sgxButton   (-1, 0,0, 1,1, "cancel")
      sgxCloseLevel()
      ; the 'ok' button will be disabled if no selection
      sgxSetDisabled(*requester\sgxButtonOK)
      
      ; reposition and resize the sgx elements. this will also create the sgx elements for the visible directoy entries.
      fileRequesterSgx_resize(*requester)
      
      ; update the scrollbar's range using the updated maximum visible entries
      ; if we would use only ListSize() as max then the last entry could be scrolled out of the top.
      sgxSetScrollMax(*requester\sgxScrollbar, ListSize(*requester\entries()) - *requester\maxShowEntries)
      
      ; make the window resizable with a minimum size
      sgxSetResizable(*requester\sgxWindow, #True)
      sgxSetMinSize(*requester\sgxWindow, 200, 200)
      
      ; present the window animated
      sgxPresentWin(*requester\sgxWindow)
      
    Else
      ; if ExamineDirectory() fails, we free the requester memory including the empty entries list and return 0.
      FreeStructure(*requester)
      *requester = 0
    EndIf
    
  EndIf
  
  ; return the requester object
  ProcedureReturn *requester
EndProcedure


Procedure fileRequesterSgx_setDirectory(*requester.s_fileRequesterSgx, path.s)
  
  ; examine directory
  Protected dir = ExamineDirectory(#PB_Any, path, *requester\pattern)
  If dir
    
    ; delete the sgx elements of all entries (in case there are any)
    ForEach *requester\entries()
      If *requester\entries()\sgxText
        sgxFree(*requester\entries()\sgxText)
        *requester\entries()\sgxText = 0
      EndIf
      If *requester\entries()\sgxCheckbox
        sgxFree(*requester\entries()\sgxCheckbox)
        *requester\entries()\sgxCheckbox = 0
      EndIf
    Next
    ; remove all entries
    ClearList(*requester\entries())
    
    ; reset selection
    ClearMap(fileRequesterSgx_selectedEntries())
    
    ; convert relative paths to absolute/full path
    If isRelativePath(path)
      path = GetCurrentDirectory() + path
    EndIf
    
    ; ensure closing backslash
    If Len(path) > 0  And Right(path, 1) <> "\"
      path + "\"
    EndIf
    
    ; remove redundant "\..\" and "\.\" parts from the path
    path = normalizePath(path)
    
    *requester\path = path
    Protected entryType, entryName.s
    ; temporarily store filenames separately to add them later, so we have all the directories listed first
    Protected NewList files.s_fileRequesterSgx_entry()
    While NextDirectoryEntry(dir)
      entryType = DirectoryEntryType(dir)
      entryName = DirectoryEntryName(dir)
      If entryType = #PB_DirectoryEntry_Directory
        ; we don't need the self reference of a directoy
        If entryName <> "."
          ; add directories directly to the requesters entries list
          AddElement(*requester\entries())
          *requester\entries()\type = entryType
          *requester\entries()\name = entryName
          *requester\entries()\name + "\"
        EndIf
      ElseIf entryType = #PB_DirectoryEntry_File
        ; add files to the temporary list
        AddElement(files())
        files()\type = entryType
        files()\name = entryName
      EndIf
    Wend
    ; add list of files to the requester's list after the directories
    MergeLists(files(), *requester\entries())
    ClearList(files())
    
    ; populate the 'fullpath' property (path + name) of all entries
    ForEach *requester\entries()
      *requester\entries()\fullpath = path
      If Len(*requester\entries()\fullpath) > 0  And Right(*requester\entries()\fullpath, 1) <> "\"
        *requester\entries()\fullpath + "\"
      EndIf
      *requester\entries()\fullpath + *requester\entries()\name
    Next
    
    FinishDirectory(dir)
    
    ; return succes
    ProcedureReturn #True
  EndIf
  
  ; examine failed
  ProcedureReturn #False
EndProcedure


Procedure fileRequesterSgx_examine(*requester.s_fileRequesterSgx)
  Protected mapkey.s
  Protected NewMap normalized()
  
  ; default return value if an invaild requester has been passed
  Protected state = -1
  
  If *requester
    
    ; return idle state per default if nothing else happens
    ; (the requester's own state changes with a button click and is used internally, but the state will be returned only
    ; if requester has finished its closing animation, either immediately or with a later examine call)
    state = #fileRequesterSgx_state_idle
    
    ; if the window does not exist anymore..
    If Not sgx(*requester\sgxWindow)
      Select *requester\state
        
        Case #fileRequesterSgx_state_idle
          ; if still idle, then the window has been closed by rightclicking it's close gadget and has been freed by sgx
          state = #fileRequesterSgx_state_closed
          ; reset selection
          ClearMap(fileRequesterSgx_selectedEntries())
        
        Case #fileRequesterSgx_state_canceled
          ; the 'cancel' button has closed the window
          state = #fileRequesterSgx_state_canceled
          ; reset selection
          ClearMap(fileRequesterSgx_selectedEntries())
        
        Case #fileRequesterSgx_state_ok
          ; the 'ok' button has closed the window
          state = #fileRequesterSgx_state_ok
      EndSelect
      
    Else
      ; 'ok' button clicked
      If sgx(*requester\sgxWindow) And sgx(*requester\sgxButtonOK)\leftup
        *requester\state = #fileRequesterSgx_state_ok
        If sgxCloseWin(*requester\sgxWindow)
          ; animation has finished immediately
          state = #fileRequesterSgx_state_ok
        EndIf
      EndIf
      
      ; 'cancel' button clicked
      If sgx(*requester\sgxWindow) And sgx(*requester\sgxButtonCancel)\leftup
        *requester\state = #fileRequesterSgx_state_canceled
        ; reset selection
        ClearMap(fileRequesterSgx_selectedEntries())
        If sgxCloseWin(*requester\sgxWindow)
          ; animation has finished immediately
          state = #fileRequesterSgx_state_canceled
        EndIf
      EndIf
      
      ; resized
      If sgx(*requester\sgxWindow) And sgx(*requester\sgxWindow)\resized
        fileRequesterSgx_resize(*requester)
      EndIf
      
      ; scrolled
      If sgx(*requester\sgxWindow) And sgx(*requester\sgxScrollbar)\changed
        fileRequesterSgx_arrangeEntries(*requester)
      EndIf
      
      ; check entry clicks
      If sgx(*requester\sgxWindow)
        Protected isInSelection, checked
        Protected singleChecked = -1
        Protected changedDir = #False
        
        ForEach *requester\entries()
          
          ; check only visible entries with existing sgx elements
          If *requester\entries()\sgxText And *requester\entries()\sgxCheckbox
            
            ; traverse into directory if allowed
            If (*requester\entries()\type = #PB_DirectoryEntry_Directory) And (Not *requester\selectFolders)
              If sgx(*requester\entries()\sgxText)\leftup
                If fileRequesterSgx_setDirectory(*requester, *requester\entries()\fullpath)
                  ; cancel further entry traversal
                  changedDir = #True
                  ; --------
                  Break
                  ; -------
                EndIf
              EndIf
            EndIf
            
            ; is entry currently selected?
            isInSelection = FindMapElement(fileRequesterSgx_selectedEntries(), *requester\entries()\fullpath)
            ; is checkbox checked?
            checked       = sgx(*requester\entries()\sgxCheckbox)\state
            
            ; text clicked
            ; (the checkbox's 'changed' event will handle the selection map)
            If sgx(*requester\entries()\sgxText)\leftup
              If checked
                sgxSetState(*requester\entries()\sgxCheckbox, 0)
              Else
                sgxSetState(*requester\entries()\sgxCheckbox, 1)
              EndIf
            EndIf
            
            ; checkbox changed
            If sgx(*requester\entries()\sgxCheckbox)\changed
              
              If (Not checked) And isInSelection
                ; remove from selection
                DeleteMapElement(fileRequesterSgx_selectedEntries(), *requester\entries()\fullpath)
              ElseIf checked And (Not isInSelection)
                ; add to selection
                AddMapElement(fileRequesterSgx_selectedEntries(), *requester\entries()\fullpath)
              EndIf
              
              ; single selection only
              If Not *requester\multiple
                If checked
                  singleChecked = *requester\entries()\sgxCheckbox
                EndIf
              EndIf
    
            EndIf
          EndIf
        Next
        
        If changedDir
          ; update path text
          sgxSetText(*requester\sgxTextPath, *requester\path)
          
          ; update the scrollbar's range using the maximum visible entries
          ; if we would use only ListSize() as max then the last entry could be scrolled out of the top.
          sgxSetScrollMax(*requester\sgxScrollbar, ListSize(*requester\entries()) - *requester\maxShowEntries)
          
          ; create and position/move and delete sgx elements for visible/invisible entries
          fileRequesterSgx_arrangeEntries(*requester)
        Else
          ; select our checkbox 'singleChecked' and deselect all others
          If singleChecked <> -1
            ForEach *requester\entries()
              
              ; if checkbox is existing / entry is visisble
              If *requester\entries()\sgxCheckbox
                If *requester\entries()\sgxCheckbox <> singleChecked
                  ; will trigger 'changed' for checked checkboxes
                  sgxSetState(*requester\entries()\sgxCheckbox, 0)
                EndIf
              Else
                ; remove from selection map manually, because we cannot trigger the checkbox
                DeleteMapElement(fileRequesterSgx_selectedEntries(), *requester\entries()\fullpath)
              EndIf
            Next
          EndIf
        EndIf
        
        ; enable the 'ok' button if there is a selection
        If MapSize(fileRequesterSgx_selectedEntries())
          sgxSetDisabled(*requester\sgxButtonOK, #False)
        Else
          sgxSetDisabled(*requester\sgxButtonOK, #True)
        EndIf
        
      EndIf
    EndIf
  EndIf
  
  ; if finished
  Select state
    Case #fileRequesterSgx_state_closed, #fileRequesterSgx_state_canceled, #fileRequesterSgx_state_ok
      ; all sgx elements have already been freed as childs of the window so we can free our structure memory including
      ; the entries list and its elements
      FreeStructure(*requester)
      
      If state = #fileRequesterSgx_state_ok
        
        ; normalize the selected filepaths into a temporary map..
        ForEach fileRequesterSgx_selectedEntries()
          mapkey = MapKey(fileRequesterSgx_selectedEntries())
          mapkey = normalizePath(mapkey)
          AddMapElement(normalized(), MapKey)
        Next
        ; ..and recreate the selection map from that
        ClearMap(fileRequesterSgx_selectedEntries())
        ForEach normalized()
          mapKey = MapKey(normalized())
          AddMapElement(fileRequesterSgx_selectedEntries(), mapKey)
        Next
        ClearMap(normalized())
        
      EndIf
  EndSelect
  
  ProcedureReturn state
EndProcedure


Procedure fileRequesterSgx_resize(*requester.s_fileRequesterSgx)
  If *requester
    If sgx(*requester\sgxWindow)
      ; get the current window size
      Protected ww = sgx(*requester\sgxWindow)\w
      Protected wh = sgx(*requester\sgxWindow)\h
      ; define some metrics
      Protected titlebarH = sgx(*requester\sgxWindow)\width ; we use the window's titlebar, also to respect the font height
      Protected margin = *requester\margin
      Protected edW = ww - 2 * margin
      Protected edH = titlebarH - 4
      Protected btW = 80
      Protected btH = titlebarH
      Protected scW = 20
      Protected scH = wh - titlebarH - edH - btH - 4 * margin
      Protected entryW = ww - scW - 3 * margin
      
      ; reposition and resize the general sgx elements
      
      sgxMove(*requester\sgxTextPath, margin, titlebarH + margin)
      sgxResize(*requester\sgxTextPath, edW, edH)
      
      sgxMove(*requester\sgxButtonOK,  ww - 2 * margin - 2 * btW,  wh - margin - btH)
      sgxResize(*requester\sgxButtonOK,  btW, btH)
      
      sgxMove(*requester\sgxButtonCancel,  ww - margin - btW,  wh - margin - btH)
      sgxResize(*requester\sgxButtonCancel,  btW, btH)
      
      sgxMove(*requester\sgxScrollbar,  ww - margin - scW,  titlebarH + edH + 2 * margin)
      sgxResize(*requester\sgxScrollbar,  scW, scH)
      
      ; store some metrics for arranging entries
      *requester\entryWidth = entryW
      *requester\entryHeight = titlebarH - 4
      ; store how many entries can be visible
      *requester\maxShowEntries = Int(scH / *requester\entryHeight)
      
      ; update the scrollbar's range using the updated maximum visible entries
      ; if we would use only ListSize() as max then the last entry could be scrolled out of the top.
      sgxSetScrollMax(*requester\sgxScrollbar, ListSize(*requester\entries()) - *requester\maxShowEntries)
      
      ; create and position/move and delete sgx elements for visible/invisible entries
      fileRequesterSgx_arrangeEntries(*requester)
    EndIf
  EndIf
EndProcedure


Procedure fileRequesterSgx_arrangeEntries(*requester.s_fileRequesterSgx)
  If *requester
    If sgx(*requester\sgxWindow)
      ; define some metrics
      Protected margin = *requester\margin
      Protected yStart, y
      Protected checkboxWidth = 13
      Protected checkboxX = margin
      Protected checkboxYOffset
      Protected textWidth = *requester\entryWidth - margin - checkboxWidth
      Protected textX = margin + checkboxWidth + margin
      
      ; the in-window y-position where entries should begin to show up (below the path text element)
      yStart = sgx(*requester\sgxTextPath)\yToParent + sgx(*requester\sgxTextPath)\h + margin
      ; the in-window y-position for the first entry (might get even negative while scrolling down)
      y = yStart - *requester\entryHeight * sgx(*requester\sgxScrollbar)\state
      
      ForEach *requester\entries()
        
        ; check if entry should be shown..
        If y >= yStart  And  y < yStart + *requester\entryHeight * *requester\maxShowEntries
          
          ; create the checkbox element if not exists
          If Not *requester\entries()\sgxCheckbox
            *requester\entries()\sgxCheckbox = sgxCheckbox(-1, checkboxX, y, checkboxWidth, checkboxWidth)
            sgxSetParent(*requester\entries()\sgxCheckbox, *requester\sgxWindow)
          EndIf
          ; create the text element if not exists
          If Not *requester\entries()\sgxText
            *requester\entries()\sgxText = sgxText(-1, textX, y, textWidth, *requester\entryHeight, *requester\entries()\name, #sgxStyleHover)
            sgxSetParent(*requester\entries()\sgxText, *requester\sgxWindow)
          EndIf
          
          ; position created, or move existing objects
          checkboxYOffset = (*requester\entryHeight / 2) - (sgx(*requester\entries()\sgxCheckbox)\h2 / 2)
          sgxMove(*requester\entries()\sgxCheckbox, sgx(*requester\entries()\sgxCheckbox)\xToParent, y + checkboxYOffset )
          sgxMove(*requester\entries()\sgxText, sgx(*requester\entries()\sgxText)\xToParent, y)
          
          ; set checkbox state according to selection
          If FindMapElement(fileRequesterSgx_selectedEntries(), *requester\entries()\fullpath)
            sgxSetState(*requester\entries()\sgxCheckbox, 1)
          Else
            sgxSetState(*requester\entries()\sgxCheckbox, 0)
          EndIf
          
          ; resize text element if necessary (we don't need to resize the checkbox)
          If sgx(*requester\entries()\sgxText)\w <> textWidth
            sgxResize(*requester\entries()\sgxText, textWidth, -2)
          EndIf
          
          ; just hide checkbox if selectFolders is disabled
          If Not *requester\selectFolders And *requester\entries()\type = #PB_DirectoryEntry_Directory
            sgxSetShow(*requester\entries()\sgxCheckbox, #False)
          EndIf
          
        Else
          ; delete sgx elements for invisible entries
          If *requester\entries()\sgxText
            sgxFree(*requester\entries()\sgxText)
            *requester\entries()\sgxText = 0
          EndIf
          If *requester\entries()\sgxCheckbox
            sgxFree(*requester\entries()\sgxCheckbox)
            *requester\entries()\sgxCheckbox = 0
          EndIf
        EndIf
        ; next entry position
        y + *requester\entryHeight
      Next
    EndIf
  EndIf
EndProcedure


sgxCursor(1)
sgxButton(4, 20,40, 150,25, "open file requester")
sgxText(5, 200,40, -1,-1, "")

Define *myRequester.s_fileRequesterSgx
Define text.s
Define state, statePrevious


Repeat
  ExamineMouse()
  ExamineKeyboard()
  
  ; pb window events
  If IsWindow(0)
    Repeat
      event=WindowEvent()
      Select event
        Case #PB_Event_CloseWindow
          quit = #True
      EndSelect
    Until Not event
  EndIf
  
  sgxExamine()
  
  ; if we have a requester currently
  If *myRequester
    statePrevious = state
    state = fileRequesterSgx_examine(*myRequester)
    
    ; check requester state change (happens when requester opened or finished)
    If state <> statePrevious
      If state = -1
        sgxSetText(5, "invalid requester")
      ElseIf state = #fileRequesterSgx_state_idle
        sgxSetText(5, "requester is idle")
      ElseIf state = #fileRequesterSgx_state_closed
        sgxSetText(5, "requester has been closed")
      ElseIf state = #fileRequesterSgx_state_canceled
        sgxSetText(5, "requester has been canceled")
      ElseIf state = #fileRequesterSgx_state_ok
        text = "requester has finished" + #LF$
        text = "selected entries (" + MapSize(fileRequesterSgx_selectedEntries()) + "):" + #LF$
        ForEach fileRequesterSgx_selectedEntries()
          text + MapKey(fileRequesterSgx_selectedEntries()) + #LF$
        Next
        sgxSetText(5, text)
        ; reset/free the selection if we don't need it anymore (not necessary but cleaner)
        ClearMap(fileRequesterSgx_selectedEntries())
      EndIf
      If state <> #fileRequesterSgx_state_idle
        ; a state <> 'idle' means the requester has been destroyed (or was invalid)
        *myRequester = 0
        ; reset our own local state
        state = 0
        ; enable the 'open' button again
        sgxSetDisabled(4, #False)
      EndIf
    EndIf
    
    ; the 'open' button has been bushed
  ElseIf sgx(4)\leftup
    
    ;*myRequester = fileRequesterSgx_open("choose file", "C:\Users\user\Desktop\pb\_sgx", "", #False, #False)
    ;*myRequester = fileRequesterSgx_open("choose file", "..", "", #True, #True)
    *myRequester = fileRequesterSgx_open("choose file", "", "", #True, #False)
    ;*myRequester = fileRequesterSgx_open("choose file", "C:\", "", #False)
    
    If *myRequester
      ; disable the 'open' button
      sgxSetDisabled(4)
      sgxSetText(5, "")
    Else
      sgxSetText(5, "creation of requester failed! invalid path specified?")
    EndIf
  EndIf
  
  sgxDisplay()
  
  StartDrawing(ScreenOutput())
    DrawingMode(#PB_2DDrawing_Transparent)
    DrawText(10,10, "sgxNumberOfElements: " + Str(sgxNumberOfElements))
  StopDrawing()
  
  FlipBuffers()
  ClearScreen($002222)
  Delay(5)
Until quit Or KeyboardPushed(#PB_Key_Escape)



; IDE Options = PureBasic 5.50 beta 1 (Windows - x64)
; CursorPosition = 329
; FirstLine = 117
; Folding = Ak
; Markers = 613
; HideErrorLog
; CompileSourceDirectory