
EnableExplicit

XIncludeFile "_sgx_includer.pbi"


; custom drawing callbacks used by
; - example 'sgxDoc\16customDraw2.pb'
; - as well as 'demo.pb'


Procedure myOnBeginDraw(id, sprType, w, h, disabled=#False)
  Static retval, borderColor, bgColor, x, boxmode, gradients
  
  If disabled
    ; sgx should not modify anything for 'disabled' and we will handle that in myOnEndDraw() on our own
    ProcedureReturn #sgxDrawNone
    
  ElseIf sgx(id)\type = #sgxTypeCursor
    ; we don't want to change the mouse cursor.
    ProcedureReturn #sgxDrawDesign | #sgxDrawText
    
  ElseIf sgx(id)\type = #sgxTypeEdit And sprType = #sgx_GadgetSprTextCursor
    ; in our case we don't want to change the text cursor of sgxEdit element.
    ProcedureReturn #sgxDrawDesign | #sgxDrawText
  EndIf
  
  If sprType = #sgx_GadgetSprTip
    ; the element's tooltip sprite is currently being redrawn
    DrawingMode(#PB_2DDrawing_Default | #PB_2DDrawing_AllChannels)
    RoundBox(0,0, w,h, 3,3, RGBA( 10, 10, 10,200))
    DrawingMode(#PB_2DDrawing_Outlined | #PB_2DDrawing_AlphaBlend)
    RoundBox(0,0, w,h, 3,3, RGBA(210,210,210,200))
    ; let sgx draw only the text
    ProcedureReturn #sgxDrawText
    
  ElseIf sprType = #sgx_GadgetSprResize
    ; the element's resize gadget is currently being redrawn.
    ; we don't change this and use the default resize gadget instead
    ; but we could replace it:
    ;DrawingMode(#PB_2DDrawing_Default | #PB_2DDrawing_AllChannels)
    ;LineXY(  4,h-1, w-1,  4, RGBA(250,250,250,200))
    ;LineXY(w-1,  4, w-1,h-1, RGBA(250,250,250,200))
    ;LineXY(w-1,h-1,   4,h-1, RGBA(250,250,250,200))
    ;ProcedureReturn #sgxDrawNone
  Else
    ; we handle all other cases in a general way, with some adaptation for checkboxes and combos
    
    ; alternate some colors if element is pushed
    If sprType = #sgx_GadgetSprPushed  Or  sprType = #sgx_GadgetSprHoverPushed
      bgColor     = RGBA( 70,  60,  60, 200)
      borderColor = RGBA(100,  90,  60, 200)
    Else
      bgColor     = RGBA( 70,  70,  70, 200)
      borderColor = RGBA( 90,  90,  90, 200)
    EndIf
    
    ; only used/changed for left text checkboxes
    x = 0
    
    If sgx(id)\type = #sgxTypeCheckbox
      ; using knob size without label
      w = sgx(id)\w
      h = sgx(id)\h
      ; draw knob on the right side if label is on the left
      If sgx(id)\textalign & #sgxAlignCheckboxTextLeft
        x = sgx(id)\w2 - w
      EndIf
    EndIf
    
    boxmode = #PB_2DDrawing_Default
    ; the hover effect of open combos should be 'hollow' because it is displayed above the entries so we use outlined here
    If sgx(id)\type = #sgxTypeCombo And sprType = #sgx_GadgetSprHover2
      boxmode = #PB_2DDrawing_Outlined
    EndIf
    
    ; draw border
    DrawingMode(boxmode | #PB_2DDrawing_AllChannels)
    RoundBox(x+0, 0,  w,   h,    3,3, RGBA(Red(borderColor),Green(borderColor),Blue(borderColor),Alpha(borderColor)/2))
    
    ; use a gradient for some element types
    gradients = #False
    Select sgx(id)\type
      Case #sgxTypeButton, #sgxTypeButtonToggle, #sgxTypeCheckbox, #sgxTypeCombo,
           #sgxTypeScrollbarVer, #sgxTypeScrollbarHor, #sgxTypeTrackbarVer, #sgxTypeTrackbarHor
        gradients = #True
    EndSelect
    
    ; draw background
    If  gradients
      DrawingMode(boxmode | #PB_2DDrawing_AllChannels | #PB_2DDrawing_Gradient)
      
      ; define gradient
      BackColor(         RGBA(Red(bgColor)*0.8, Green(bgColor)*0.8, Blue(bgColor)*0.8, Alpha(bgColor)))
      GradientColor(0.1, RGBA(Red(bgColor)*1.0, Green(bgColor)*1.0, Blue(bgColor)*1.0, Alpha(bgColor)))
      GradientColor(0.5, RGBA(Red(bgColor)*0.8, Green(bgColor)*0.8, Blue(bgColor)*0.8, Alpha(bgColor)))
      GradientColor(0.9, RGBA(Red(bgColor)*0.6, Green(bgColor)*0.6, Blue(bgColor)*0.6, Alpha(bgColor)))
      FrontColor(        RGBA(Red(bgColor)*0.1, Green(bgColor)*0.1, Blue(bgColor)*0.1, Alpha(bgColor)))
      
      ; simpler gradient
      ;BackColor(RGBA(Red(bgColor), Green(bgColor), Blue(bgColor), Alpha(bgColor)))
      ;FrontColor(RGBA(Red(bgColor)/3, Green(bgColor)/3, Blue(bgColor)/3, Alpha(bgColor)))
      
      ; reversed gradient for pushed buttons
      If (sgx(id)\type = #sgxTypeButton Or sgx(id)\type = #sgxTypeButtonToggle)  And  (sprType = #sgx_GadgetSprPushed  Or  sprType = #sgx_GadgetSprHoverPushed)
        LinearGradient(0,h,0,0)
      Else
        LinearGradient(0,0,0,h)
      EndIf
      ; draw gradient background
      RoundBox(x+1, 1,  w-2, h-2,  3,3)
    Else
      ; draw background
      DrawingMode(boxmode | #PB_2DDrawing_AllChannels)
      RoundBox(x+1, 1,  w-2, h-2,  3,3, bgColor)
    EndIf
    
    ; checkboxes 'checked' part
    If sgx(id)\type = #sgxTypeCheckbox And sprType = #sgx_GadgetSprChecked
      DrawingMode(#PB_2DDrawing_Default | #PB_2DDrawing_AlphaBlend)
      Circle(w/2, h/2, w/6, RGBA(255,125,0,20))
      Circle(w/2, h/2, w/4, RGBA(255,125,0,20))
    EndIf
    
    ; brighten up all the hover states with a thin white
    If sprType = #sgx_GadgetSprHover  Or  sprType = #sgx_GadgetSprHoverPushed  Or  sprType = #sgx_GadgetSprHover2
      DrawingMode(#PB_2DDrawing_Default | #PB_2DDrawing_AlphaBlend)
      If sgx(id)\type = #sgxTypeCombo And sprType = #sgx_GadgetSprHover2
        ; the hover effect of open combos should be 'hollow' because it is displayed above the entries so we use outlined here
        DrawingMode(#PB_2DDrawing_Outlined | #PB_2DDrawing_AlphaBlend)
      EndIf
      RoundBox(x+0, 0,  w, h, 3,3,  RGBA(255,255,255,10))
      DrawingMode(#PB_2DDrawing_Outlined | #PB_2DDrawing_AlphaBlend)
      RoundBox(x+0, 0,  w, h, 3,3,  RGBA(230,180,0,80))
    EndIf
    
    ; we let sgx handle the drawing of text
    ProcedureReturn #sgxDrawTextNoOffset
    
  EndIf
  
  ; our default returnvalue (sgx should draw everthing as usual after this function has finished)
  ProcedureReturn #sgxDrawDesign | #sgxDrawText
EndProcedure


Procedure myOnEndDraw(id, sprType, w, h, disabled=#False)
  ; works just like myOnBeginDraw() except you don't have to return anything.
  
  If disabled
    ; using alpha clip to keep transparent areas transparent (i.e. don't dim checkbox label background)
    DrawingMode(#PB_2DDrawing_Default | #PB_2DDrawing_AlphaClip)
    RoundBox(1, 1,  w-2, h-2, 3,3,  RGBA(50,50,50,127))
  EndIf
EndProcedure


; IDE Options = PureBasic 5.50 beta 1 (Windows - x64)
; CursorPosition = 137
; FirstLine = 117
; Folding = -
; HideErrorLog
; CompileSourceDirectory