

;** sgx_PassStandardValues
;* macro to initalize element property default for most element type during creation. used by the compose functions like <a href=#f_sgxbutton>sgxButton</a> etc.
Macro sgx_PassStandardValues()
  sgxSetParent(id, sgx_CurrentParent)
  sgx(id)\design=*sgx_CurrentDesign
  sgx(id)\alpha = sgx_defaultAlpha
  sgx(id)\xToParent=x
  sgx(id)\yToParent=y
  sgx(id)\x=sgx_GetGadgetX(id)
  sgx(id)\y=sgx_GetGadgetY(id)
  sgx(id)\w=w
  sgx(id)\h=h
  sgx(id)\origW=w
  sgx(id)\origH=h
  sgx(id)\show=1
  sgx(id)\x2 = sgx(id)\x
  sgx(id)\y2 = sgx(id)\y
  sgx(id)\w2 = sgx(id)\w
  sgx(id)\h2 = sgx(id)\h
  sgx(id)\minWidth = sgx_resizeGadgetSize
  sgx(id)\minHeight = sgx_resizeGadgetSize
EndMacro


;** sgx_infoTail
;* used to add debug information to the text of sgx elements (must be activated in the code)
Macro sgx_infoTail()
  ""
  ;" {"+Str(id)+"} "
  ;"["+Str(id)+"|"+Str(sgx(id)\parent)+"]"
  ;" #"+Str( sgx_fontID )
EndMacro




; -------------------------------------- compose





; --------- ELEMENTS


;** sgxCursor
;* creates an sgx cursor element and returns it's ID. _
;* the notes on <a href=#f_sgxnew>sgxNew</a> apply to return value and parameter 'id'.
;** .id : ID for the new sgx element.
Procedure sgxCursor(id)
  id = sgxNew(id)
  If id
    sgx(id)\parent=-1
    sgx(id)\design=*sgx_CurrentDesign
    sgx(id)\type=#sgxTypeCursor
    If sgx_BuildRefresh
      sgxRefresh(id)
    EndIf
    sgx(id)\show=1
    sgx_m\cursor=id
  EndIf
  ProcedureReturn id
EndProcedure


;** sgxButton
;* creates an sgx button element and returns it's ID. _
;* the notes on <a href=#f_sgxnew>sgxNew</a> apply to return value and parameter 'id'. _
;* if a sprite is specified then you are responsible for the sprites lifetime, <a href=#f_sgxfree>sgxFree</a> will not
;* free the sprite. also you should not free the sprite before you have freed the sgxButton, otherwise the sprite
;* simply won't be displayed, but doesn't generate an error. _
;* a togglebutton's state will change when leftclicked, or will be set to pushed if it is part of a radiogroup.
;* rightclick behaviour can be configured via variable sgxRightclickUnset. _
;* you can use <a href=#f_sgxsetstate>sgxSetState</a> with toggle buttons. _
;** .id     : ID for the new sgx element.
;** .x      : x-position relative to parent element
;** .y      : y-position relative to parent element
;** .w      : width
;** .h      : height
;** .text   : caption. use #LF$ / Chr(10) for line breaks
;** .toggle : one of these values: _
;*            #sgxStateNone (default) for a standard button _
;*            #sgxStateOff for a toggle button _
;*            #sgxStateOn for a toggle button initialized as 'pushed' _
;** .sprNr  : an optional PB #sprite to be displayed on the button. you can then also use -1 for the w/h-parameters to adjust the button to the size of the sprite.
;** .x2     : x-offset for the additional sprite
;** .y2     : y-offset for the additional sprite
Procedure sgxButton(id, x, y, w, h, text.s, toggle=#sgxStateNone , sprNr=-1, x2=5, y2=5)
  id = sgxNew(id)
  If id
    sgx_PassStandardValues()
    If toggle = #sgxStateOff  Or  toggle = #sgxStateOn
      sgx(id)\type=#sgxTypeButtonToggle
      If toggle = #sgxStateOn
        sgxSetState(id, 1)
      EndIf
    Else
      sgx(id)\type=#sgxTypeButton
    EndIf
    sgx(id)\text=text + sgx_infoTail()
    sgx(id)\textfontID=sgx_CurrentFontID
    If IsSprite(sprNr)
      sgx(id)\x2 = x2
      sgx(id)\y2 = y2
      sgx(id)\spr[#sgx_GadgetSprNormal2] = sprNr
      If sgx(id)\w = -1
        sgx(id)\w = SpriteWidth (sgx(id)\spr[#sgx_GadgetSprNormal2]) + 2 * sgx(id)\x2 + 1
      EndIf
      If sgx(id)\h = -1
        sgx(id)\h = SpriteHeight(sgx(id)\spr[#sgx_GadgetSprNormal2]) + 2 * sgx(id)\y2 + 1
      EndIf
    ElseIf w=-1 Or h=-1
      sgx_err("you have to specify a sprite to use automatic size adjustment for id " + id + ".")
    EndIf
    If sgx_BuildRefresh
      sgxRefresh(id)
    EndIf
    sgxMoveOnTop(id)
  EndIf
  ProcedureReturn id
EndProcedure


;** sgxCheckbox
;* creates an sgx checkbox element and returns it's ID. _
;* the notes on <a href=#f_sgxnew>sgxNew</a> apply to return value and parameter 'id'. _
;* _
;* the specified position and size will define the knob. the overall size including the label will be stored
;* in the fields w2 and h2. the fields x2 and y2 store the display position of the elements sprite (the visual size including
;* the label) relative to the knob position. with #sgxAlignCheckboxTextLeft for example the label will start on screen
;* at (x+x2),(y+y2) where x2 will be negative and y2 will be negative if the text is larger than the knob height. _
;* _
;* a checkbox's state will change when leftclicked, or will be set to checked if it is part of a radiogroup.
;* rightclick behaviour can be configured via variable sgxRightclickUnset. _
;* you can use <a href=#f_sgxsetstate>sgxSetState</a> with this element. _
;** .id    : ID for the new sgx element.
;** .x     : knob x-position relative to parent element
;** .y     : knob y-position relative to parent element
;** .w     : knob width
;** .h     : knob height
;** .checked  : #sgxStateOff (default) for unchecked or #sgxStateOn for checked
;** .text  : label
;** .align : can be #sgxAlignCheckboxTextRight to have the label on the right (default), or #sgxAlignCheckboxTextLeft to have it on the left.
Procedure sgxCheckbox(id, x, y, w, h, checked=#sgxStateOff, text.s="", align=#sgxAlignCheckboxTextRight)
  id = sgxNew(id)
  If id
    sgx_PassStandardValues()
    sgx(id)\type=#sgxTypeCheckbox
    sgx(id)\text=text + sgx_infoTail()
    sgx(id)\textalign=align
    sgx(id)\textfontID=sgx_CurrentFontID
    If checked = #sgxStateOn
      sgxSetState(id, 1)
    EndIf
    
    ; for checkboxes we ne need at least to update size and position according to the label
    ; so a possible parent table can rely on it. but we will skip the drawing in sgxRefresh
    ; in case sgx_BuildRefresh is set
    If sgx_BuildRefresh
      sgxRefresh(id)
    Else
      sgxRefresh(id, #sgx_refreshSkipTableCheckboxRedraw)
    EndIf
    
    sgxMoveOnTop(id)
  EndIf
  ProcedureReturn id
EndProcedure


;** sgxScrollbar
;* creates an sgx scrollbar element and returns it's ID. _
;* the notes on <a href=#f_sgxnew>sgxNew</a> apply to return value and parameter 'id'. _
;* if height is greater than width, the scrollbar will be vertical, otherwise horizontal. _
;* you can use <a href=#f_sgxsetstate>sgxSetState</a> and <a href=#f_sgxsetscrollmax>sgxSetScrollMax</a> with this element. _
;* note that for example for a scrollbar length of 10 pixel and a max state value of 100 you will only be able to receive every 10th state. _
;* likewise for a scrollbar length of 100 pixel and a max state value of 10 you will only receive state changes every 10th pixel. _
;** .id    : ID for the new sgx element.
;** .x     : x-position relative to parent element
;** .y     : y-position relative to parent element
;** .w     : width (bar)
;** .h     : height (bar)
;** .w2    : width (knob)
;** .h2    : height (knob)
;** .max   : maximum possible scroll state (minimum is 0). this must be greater than 0 and will be corrected to 1 
;*           <span></span> automatically if necessary.
;** .state : initial state (default is 0)
;** .style : possible values (you can combine flags with '|' (bitwise OR)): _
;*           0 (scrollbar default) _
;*           #sgxStyleGrid: draws a pseudo-grid. _
;*           #sgxStyleReverse : the state 0 will be at the left for horizontal and at the top for vertical scrollbars per default. by specifying the reverse flag the 0 state will be at the other end of the scrollbar. _
Procedure sgxScrollbar(id, x, y, w, h, w2, h2, max, state=0, style=0)
  id = sgxNew(id)
  If id
    sgx_PassStandardValues()
    sgx(id)\w2 = w2
    sgx(id)\h2 = h2
    If sgx(id)\h > sgx(id)\w
      sgx(id)\type=#sgxTypeScrollbarVer
      sgx(id)\width = h2
    Else
      sgx(id)\type=#sgxTypeScrollbarHor
      sgx(id)\width = w2
    EndIf
    sgx(id)\scrollpos=0
    If max < 1
      max = 1
    EndIf
    sgx(id)\max=max
    sgx(id)\style=style
    sgx(id)\textfontID=sgx_CurrentFontID
    sgxSetState(id, state)
    If sgx_BuildRefresh
      sgxRefresh(id)
    EndIf
    sgxMoveOnTop(id)
  EndIf
  ProcedureReturn id
EndProcedure


;** sgxTrackbar
;* creates an sgx trackbar element and returns it's ID. _
;* the notes on <a href=#f_sgxnew>sgxNew</a> apply to return value and parameter 'id'. _
;* if height is greater than width, the trackbar will be vertical, otherwise horizontal. _
;* you can use <a href=#f_sgxsetstate>sgxSetState</a> and <a href=#f_sgxsetscrollmax>sgxSetScrollMax</a> with this element. _
;* note that for example for a trackbar length of 10 pixel and a max state value of 100 you will only be able to receive every 10th state. _
;* likewise for a trackbar length of 100 pixel and a max state value of 10 you will only receive state changes every 10th pixel. _
;** .id    : ID for the new sgx element.
;** .x     : x-position relative to parent element
;** .y     : y-position relative to parent element
;** .w     : width (bar)
;** .h     : height (bar)
;** .w2    : width (knob)
;** .h2    : height (knob)
;** .max   : maximum possible scroll state (minimum is 0). this must be greater than 0 and will be corrected to 1  
;*           <span></span> automatically if necessary.
;** .state : initial state (default is 0)
;** .style : possible values (you can combine flags with '|' (bitwise OR)): _
;*           0 _
;*           #sgxStyleGrid: draws a pseudo-grid (trackbar default) _
;*           #sgxStyleReverse : the state 0 will be at the left for horizontal and at the bottom for vertical trackbars per default. by specifying the reverse flag the 0 state will be at the other end of the trackbar. _
Procedure sgxTrackbar(id, x, y, w, h, w2, h2, max, state=0, style=#sgxStyleGrid)
  id = sgxNew(id)
  If id
    sgx_PassStandardValues()
    sgx(id)\w2 = w2
    sgx(id)\h2 = h2
    If sgx(id)\h > sgx(id)\w
      sgx(id)\type=#sgxTypeTrackbarVer
      sgx(id)\width=h2
    Else
      sgx(id)\type=#sgxTypeTrackbarHor
      sgx(id)\width=w2
    EndIf
    sgx(id)\scrollpos=0
    If max < 1
      max = 1
    EndIf
    sgx(id)\max=max
    sgx(id)\style=style
    sgx(id)\textfontID=sgx_CurrentFontID
    sgxSetState(id, state)
    If sgx_BuildRefresh
      sgxRefresh(id)
    EndIf
    sgxMoveOnTop(id)
  EndIf
  ProcedureReturn id
EndProcedure


;** sgxEdit
;* creates an sgx edit element and returns it's ID. _
;* the notes on <a href=#f_sgxnew>sgxNew</a> apply to return value and parameter 'id'. _
;* this is for user editable text only. for static labels use <a href=#f_sgxtext>sgxText</a>.
;** .id       : ID for the new sgx element.
;** .x        : x-position relative to parent element
;** .y        : y-position relative to parent element
;** .w        : width
;** .h        : height
;** .text     : inital text. use #LF$ / Chr(10) for line breaks
;** .maxLines : you can limit the amount of lines that can be entered by the user. use -1 (default) to not limit the amount of lines.
Procedure sgxEdit(id, x, y, w, h, text.s="", maxLines=-1)
  Protected i
  id = sgxNew(id)
  If id
    sgx_PassStandardValues()
    sgx(id)\type=#sgxTypeEdit
    sgx(id)\textfontID=sgx_CurrentFontID
    sgx(id)\text=text + sgx_infoTail()
    sgx(id)\textpos=0
    sgx(id)\textalign=0
    sgx(id)\scrollpos=0
    sgx(id)\max=maxLines
    ExamineKeyboard()
    sgx(id)\text = sgx_keyboardEdit( sgx(id)\text , sgx(id)\textpos)
    sgx(id)\textpos = sgx_keyboardEdit_lastTextPos
    
    If sgx_BuildRefresh
      sgxRefresh(id)
    EndIf
    sgxMoveOnTop(id)
  EndIf
  ProcedureReturn id
EndProcedure


;** sgxWin
;* creates an sgx window element and returns it's ID. _
;* the notes on <a href=#f_sgxnew>sgxNew</a> apply to return value and parameter 'id'. _
;* this element will be the parent for new sgx elements until you call <a href=#f_sgxcloselevel>sgxCloseLevel</a> or until
;* you create another parent element. _
;* _
;* a window's state can be changed via the functions
;* <a href=#f_sgxpresentwin>sgxPresentWin</a>, <a href=#f_sgxminimizewin>sgxMinimizeWin</a>, 
;* <a href=#f_sgxrestorewin>sgxRestoreWin</a>, <a href=#f_sgxclosewin>sgxCloseWin</a> and can be inquired by checking the
;* 'state' field for #sgxWinState.. values. _
;* a closed window does not exists anymore and thus has no state. use the macro <a href=#f_sgx>sgx()</a> without structure
;* field access to test the vailidty in that case. _
;* _
;* you can use <a href=#f_sgxpresentwin>sgxPresentWin</a> directly after <a href=#f_sgxcloselevel>sgxCloseLevel</a>
;* to present the window instead of showing it immeditately. _
;* if you want the window to be created hidden, use <a href=#f_sgxsetshow>sgxSetShow</a> (see notes about child visibility)
;* to hide the window after you finished the creation with <a href=#f_sgxcloselevel>sgxCloseLevel</a>. you can then use
;* <a href=#f_sgxpresentwin>sgxPresentWin</a> later to show the window. _
;** .id    : ID for the new sgx element.
;** .x     : x-position relative to parent element
;** .y     : y-position relative to parent element
;** .w     : width
;** .h     : height
;** .text  : caption in titlebar. no text will cause the window to have no titlebar. use a space for an empty titlebar.
;** .style : 0 (default) _
;*           #sgxStyleResizable to make the window resizable via mouse.
Procedure sgxWin(id, x, y, w, h, text.s="", style=0)
  id = sgxNew(id)
  If id
    sgx_PassStandardValues()
    sgx(id)\type=#sgxTypeWin
    sgx(id)\text=text + sgx_infoTail()
    ;sgx(id)\style=style
    ;sgx(id)\textalign=align
    sgx(id)\textfontID=sgx_CurrentFontID
    If style & #sgxStyleResizable
      sgxSetResizable(id, #True)
    EndIf
    If sgx_BuildRefresh
      sgxRefresh(id)
    EndIf
    sgxMoveOnTop(id)
    sgxSetCurrentParent(id)
  EndIf
  ProcedureReturn id
EndProcedure


;** sgxSpriteGadget
;* creates an sgx sprite container and returns it's ID. _
;* the notes on <a href=#f_sgxnew>sgxNew</a> apply to return value and parameter 'id'. _
;* you are responsible for the sprites lifetime, <a href=#f_sgxfree>sgxFree</a> will not free the sprite. also you should not free the sprite before you have freed the sgxSpriteGadget, otherwise the sprite simply won't be displayed, but doesn't generate an error. _
;* you can use <a href=#f_sgxsetstate>sgxSetState</a> with this element. _
;** .id    : ID for the new sgx element.
;** .x     : x-position relative to parent element
;** .y     : y-position relative to parent element
;** .sprNr : a valid PB Sprite number, or 0. you can change the sprite via <a href=#f_sgxsetstate>sgxSetState</a>.
Procedure sgxSpriteGadget(id, x, y, sprNr)
  Protected w
  Protected h
  id = sgxNew(id)
  If id
    If IsSprite(sprNr)
      w = SpriteWidth(sprNr)  ; - w and h are needed in macro sgx_PassStandardValues()
      h = SpriteHeight(sprNr) ; /
    Else
      w = 0
      h = 0
    EndIf
    sgx_PassStandardValues()
    sgx(id)\type = #sgxTypeSpriteGadget
    If IsSprite(sprNr)
      sgx(id)\state = sprNr
    Else
      sgx(id)\state = 0
    EndIf
    sgx(id)\textfontID=sgx_CurrentFontID
    If sgx_BuildRefresh
      sgxRefresh(id)
    EndIf
    sgxMoveOnTop(id)
    ProcedureReturn id
  EndIf
  ProcedureReturn 0
EndProcedure


;** sgxArea
;* creates an sgx area container and returns it's ID. _
;* the notes on <a href=#f_sgxnew>sgxNew</a> apply to return value and parameter 'id'. _
;* use sgx area elements to get mouse events without using any visuals. _
;* areas dont't move up in z-order when clicked as other types do but other elements may move in front of an area. you can use <a href=#f_sgxmoveontop>sgxMoveOnTop</a> to move it up manually if needed. _
;** .id    : ID for the new sgx element.
;** .x     : x-position relative to parent element
;** .y     : y-position relative to parent element
;** .w    : width
;** .h    : height
Procedure sgxArea(id, x, y, w, h)
  id = sgxNew(id)
  If id
    sgx_PassStandardValues()
    sgx(id)\type = #sgxTypeArea
    If sgx_BuildRefresh
      sgxRefresh(id)
    EndIf
    sgxMoveOnTop(id)
    ProcedureReturn id
  EndIf
  ProcedureReturn 0
EndProcedure


;** sgxTip
;* creates a tooltip for an sgx element.
;** .id   : ID of a valid and existing sgx element
;** .text : inital text of the tooltip. use #LF$ / Chr(10) for line breaks
Procedure sgxTip(id, text.s)
  If sgx(id)
    If sgx(id)\tip = 0
      sgx(id)\tip = AllocateMemory(SizeOf(sgxS_Tip))
      If sgx(id)\tip = 0
        ProcedureReturn 0
      EndIf
    EndIf
    sgx(id)\tip\text = text
    ;sgx(id)\tip\timer = 0
    ;sgx(id)\tip\show = 0
    ;sgx(id)\tip\spr = -1
    If sgx_BuildRefresh
      sgxRefresh(id, #sgx_refreshOnlyTip)
    EndIf
    ProcedureReturn id
  EndIf
  ProcedureReturn 0
EndProcedure


;** sgxCombo
;* creates an sgx combo element and returns it's ID. _
;* the notes on <a href=#f_sgxnew>sgxNew</a> apply to return value and parameter 'id'. _
;* you can use <a href=#f_sgxsetstate>sgxSetState</a> with this element. _
;** .id   : ID for the new sgx element.
;** .x    : x-position relative to parent element
;** .y    : y-position relative to parent element
;** .w    : width
;** .h    : height
;** .text : the possible entries separated by #LF$ / Chr(10)
Procedure sgxCombo(id, x, y, w, h, text.s="")
  id = sgxNew(id)
  If id
    sgx_PassStandardValues()
    sgx(id)\type=#sgxTypeCombo
    sgx(id)\text=text + sgx_infoTail()
    sgx(id)\textfontID=sgx_CurrentFontID
    sgx(id)\state = 1
    If sgx_BuildRefresh
      sgxRefresh(id)
    EndIf
    sgxMoveOnTop(id)
  EndIf
  ProcedureReturn id
EndProcedure


;** sgxText
;* creates an sgx text element and returns it's ID. _
;* the notes on <a href=#f_sgxnew>sgxNew</a> apply to return value and parameter 'id'. _
;* this is for static labels only. for user editable text use <a href=#f_sgxedit>sgxEdit</a>. _
;* take in consideration that changing the text via <a href=#f_sgxsettext>sgxSetText</a> will cause an element 
;* refresh, including drawing of design images as well as drawing of text onto the element's sprites, possibly also recreation 
;* of sprites if the size changes too. this will have an impact on performance, so don't take it lightly by changing it's
;* text every frame for example. _
;** .id        : ID for the new sgx element.
;** .x         : x-position relative to parent element
;** .y         : y-position relative to parent element
;** .w         : width. -1 (default) will adjust width to textwidth
;** .h         : height. -1 (default) will adjust height to textheigt
;** .text      : inital text. use #LF$ / Chr(10) for line breaks
;** .style     : with #sgxStyleThin the element will have a frame. #sgxStyleHover will cause it to have a mouseover effect. you can combine flags with '|' (bitwise OR), or use #sgxStyleHoverThin.
;** .margin    : space between text and border
;** .textalign : one of these values: _
;*               #sgxAlignTextLeft (Default) _
;*               #sgxAlignTextRight _
;*               #sgxAlignTextCenter _
;*               #sgxAlignTextJustifySpaces (text justified per word) _
;*               #sgxAlignTextJustifyCharacters (text justified per character)
;** .wordwrap  : #False (Default), #True for automatic word wrap. works only if width (parameter w) is not set to -1.
Procedure sgxText(id, x, y, w=-1, h=-1, text.s="", style=0, margin=3, textalign=#sgxAlignTextLeft, wordwrap=#False)
  id = sgxNew(id)
  If id
    sgx_PassStandardValues()
    If w=-1
      sgx(id)\auto | #sgx_AutoW
    EndIf
    If h=-1
      sgx(id)\auto | #sgx_AutoH
    EndIf
    sgx(id)\type = #sgxTypeText
    sgx(id)\text=text + sgx_infoTail()
    sgx(id)\origText = sgx(id)\text
    sgx(id)\textfontID=sgx_CurrentFontID
    sgx(id)\textalign=textalign
    sgx(id)\style=style
    sgx(id)\margin=margin
    If wordwrap  And  Not (sgx(id)\auto & #sgx_AutoW)
      sgx(id)\auto | #sgx_AutoNL
    EndIf
    If sgx_BuildRefresh Or (sgx(id)\auto & (#sgx_AutoW|#sgx_AutoH)) ; bypass refresh deactivation to get proper w/h of element according to text
      sgxRefresh(id)
    EndIf
    sgxMoveOnTop(id)
  EndIf
  ProcedureReturn id
EndProcedure


;** sgxTable
;* creates an sgx table element and returns it's ID. _
;* the notes on <a href=#f_sgxnew>sgxNew</a> apply to return value and parameter 'id'. _
;* this element will be the parent for new sgx elements until you call <a href=#f_sgxcloselevel>sgxCloseLevel</a> or 
;* until you create another parent element. _
;* use <a href=#f_sgxcellassign>sgxCellAssign</a> to place elements in the table, <a href=#f_sgxcellid>sgxCellId</a> and
;* <a href=#f_sgxcell>sgxCell</a> to access the cell elements. _
;* each column will be as wide as the widest element in it. _
;* each line will be as high as the highest element in it. _
;* the size of the table will adapt to the sum of column widths / line heights.
;* child elements of the table might not be completely refreshed until table creation is finished
;* with <a href=#f_sgxcloselevel>sgxCloseLevel</a>. _
;* _
;* the parameters 'columns' and 'lines' specify the amount, so cell indices will range from 0 to amount-1. it does not
;* necessarily hurt to oversize these dimensions if you're not shure how big you need the table to be later. each cell
;* will need 4 bytes of memory, no matter if assigned or left empty and each colum as well as each lines needs
;* additional 4 bytes of memory. _
;* _
;* big tables with a lot of elements, possibly also subtables and especially in combination with #sgxStyleFitCellW and/or 
;* #sgxStyleFitCellH can impact performance if cell elements need to be refreshed often since this could cause refresh of
;* other cell elements and the table itself. using #sgxStyleNoBG will also make a table refresh faster. _
;* tables are desinged to be a quick way for simple layouts and in some cases it might be better
;* to position, resize and refesh elements on you own without using a table.
;** .id      : ID for the new sgx table element.
;** .x       : x-position relative to parent element
;** .y       : y-position relative to parent element
;** .columns : amount of maximum columns (see notes above)
;** .lines   : amount of maximum lines (see notes above)
;** .margin  : space between cells
;** .style   : possible values (you can combine flags with '|' (bitwise OR)): _
;*             0 _
;*             #sgxStyleGridV    : draw vertical grid lines _
;*             #sgxStyleGridH    : draw horizontal grid lines _
;*             #sgxStyleGrid (default) : draw vertical and horizontal grid lines _
;*             #sgxStyleFitCellW : width of cell elements will be overwritten according to column width except for vertical scrollbars/trackbars, checkboxes and sgxText elements with automatic width adjustment. _
;*             #sgxStyleFitCellH : height of cell elements will be overwritten according to line height except for horizontal scrollbars/trackbars, checkboxes and sgxText elements with automatic height adjustment. _
;*             #sgxStyleNoBG     : no background / only cells will be visible. (this will disable the grid flags) _
Procedure sgxTable(id, x, y, columns, lines, margin, style=#sgxStyleGrid)
  id = sgxNew(id)
  If id
    ;If margin < 1
    ;  margin = 1
    ;EndIf
    Protected w = 1
    Protected h = 1
    sgx_PassStandardValues()
    sgx(id)\margin = margin
    sgx(id)\type=#sgxTypeTable
    sgx(id)\text="";text + sgx_infoTail()
    sgx(id)\textfontID=sgx_CurrentFontID
    sgx(id)\style=style
    sgx(id)\table = AllocateMemory(SizeOf(sgxS_Table))
    sgx(id)\table\w = columns
    sgx(id)\table\h = lines
    sgx(id)\table\cells = AllocateMemory(columns*lines*SizeOf(Long))
    sgx(id)\table\gridx = AllocateMemory(columns*SizeOf(Long))
    sgx(id)\table\gridy = AllocateMemory(lines*SizeOf(Long))
    If style & #sgxStyleNoBG
      sgxSetShow(id, #False, #False)
    EndIf
    
    ; from now on only initialize child elements on creation without refresh
    ; and refresh them later during table refresh in sgxCloseLevel, since
    ; table and child dimensions can influence each other by #sgxStyleFitCellW/H
    ; and we want avoid double refreshes
    sgx_PushBuildRefresh(0)
    
    sgxMoveOnTop(id)
    sgxSetCurrentParent(id)
  EndIf
  ProcedureReturn id
EndProcedure







; IDE Options = PureBasic 5.50 beta 1 (Windows - x64)
; CursorPosition = 321
; FirstLine = 301
; Folding = ---
; UseMainFile = demo.pb
; Executable = sgxDemo.exe
; CompileSourceDirectory