



;** sgx_RemakeDesignImage
;* frees an design image (in case it is a valid one) and recreates it with the specified dimensions
;** ._img : index for the *design\img[] array
;** ._w   : new width
;** ._w   : new height
Macro sgx_RemakeDesignImage(_img, _w, _h)
  sgx_FreeImageChecked(*design\img[_img]\nr)
  *design\img[_img]\nr = CreateImage(#PB_Any,_w,_h, 32)
  *design\img[_img]\w = _w
  *design\img[_img]\h = _h
EndMacro


; Macro Save_(_img)
;   If IsImage(*design\img[_img]\nr)
;     SaveImage(*design\img[_img]\nr, RSet(Str(_img),2,"0") +"_"+ Mid( M_quote#_img#M_quote+".bmp", 11, 999))
;   ElseIf IsSprite(*design\img[_img]\nr)
;     SaveSprite(*design\img[_img]\nr, RSet(Str(_img),2,"0") +"_"+ Mid( M_quote#_img#M_quote+".bmp", 11, 999))
;   EndIf
; EndMacro


; -------------------------------------- init


;** sgx_MakePalette
;* creates a color pallete for the design derived from a basecolor. _
;* basically an array of some colors derived from a base color and to be used as text color when hovered or for thin borders of text elements etc. _
;* called by <a href=#f_sgxrefreshdesign>sgxRefreshDesign</a>
;** .*design   : the design to create the pallete for
;** .basecolor : the base color to derive the pallete from
Procedure sgx_MakePalette(*design.sgxS_Design, basecolor)
  Protected i
  
  ; search for '\c[' to see where colors are used
  
  *design\basecolor = basecolor
  *design\c[00] = sgx_fade(*design\basecolor,-500)
  *design\c[01] = sgx_fade(*design\basecolor,-700)
  *design\c[02] = sgx_fade(*design\basecolor,-300)
  *design\c[03] = sgx_fade(*design\basecolor,-200)
  *design\c[04] = sgx_fade(*design\basecolor,-400)
  *design\c[05] = sgx_fade(*design\basecolor,-150)
  *design\c[06] = sgx_fade(*design\basecolor,-050)
  
  *design\c[07] = *design\basecolor
  
  *design\c[08] = sgx_fade(*design\basecolor, 050)
  *design\c[09] = sgx_fade(*design\basecolor, 150)
  *design\c[10] = sgx_fade(*design\basecolor, 200)
  *design\c[11] = sgx_fade(*design\basecolor, 300)
  *design\c[12] = sgx_fade(*design\basecolor, 400)
  *design\c[13] = sgx_fade(*design\basecolor, 500)
  *design\c[14] = sgx_fade(*design\basecolor, 600)
  *design\c[15] = sgx_fade(*design\basecolor, 300)
  
  *design\c[16] = sgx_fade(*design\basecolor,-600) ; table grid color 1 normal
  *design\c[17] = sgx_fade(*design\basecolor,-300) ; table grid color 2 normal
  *design\c[18] = sgx_fade(*design\basecolor,-600) ; table grid color 1 hover
  *design\c[19] = sgx_fade(*design\basecolor,-350) ; table grid color 2 hover
  
  ; redo zero-fades (avoid black)
  For i=0 To 15
    If RGB(Red(*design\c[i]),Green(*design\c[i]),Blue(*design\c[i])) = $000000
      *design\c[i] = RGBA(Red(0),Green(0),Blue(1), Alpha(*design\basecolor))
    EndIf
  Next
  
  ;save pallette (debugging)
  If 00
    Protected h = 30, w = 400
    Protected pal = CreateSprite(#PB_Any,w,20*h+20, #PB_Sprite_AlphaBlending)
    Protected font = LoadFont(#PB_Any, "courier",8)
    Protected color, y
    TransparentSpriteColor(pal, RGB(0,0,0))
    StopDrawing()
    StartDrawing(SpriteOutput(pal))
      DrawingMode(#PB_2DDrawing_AllChannels)
      Box(0,0,SpriteWidth(pal),SpriteHeight(pal),RGBA(0,0,0,0))
      Box(100,0,20,SpriteHeight(pal),RGBA($00,$00,$00,$ff));black
      Box(120,0,20,SpriteHeight(pal),RGBA($77,$77,$77,$ff));gray
      Box(140,0,20,SpriteHeight(pal),RGBA($ff,$ff,$ff,$ff));white
      Box(160,0,20,SpriteHeight(pal),RGBA($ff,$00,$00,$ff));red
      Box(180,0,20,SpriteHeight(pal),RGBA($00,$ff,$00,$ff));green
      Box(200,0,20,SpriteHeight(pal),RGBA($00,$00,$ff,$ff));blue
      For i=0 To 19
        y = i*h+20
        color = *design\c[i]
        DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Transparent)
        Box(1,y+1,w-2,h-2, color)
        DrawingFont(FontID(font))
        DrawText(20,y+2, Str(i)+" : $"+Hex(color)+" / RGBA("+Red(color) + ","+Green(color)+","+Blue(color)+","+Alpha(color)+")" ,RGBA($ff,$ff,$ff,$ff))
        DrawingMode(#PB_2DDrawing_AllChannels | #PB_2DDrawing_Outlined)
        Box(3,y+3,h-7,h-7, RGBA(Red(color),Green(color),Blue(color),$ff))
      Next
      DrawingMode(#PB_2DDrawing_AllChannels | #PB_2DDrawing_Outlined)
      ;Debug SpriteWidth(pal)
      ;Debug SpriteHeight(pal)
      Box(0,0,SpriteWidth(pal)-1,SpriteHeight(pal)-1,RGBA($ff,$66,$66,$ff))
    StopDrawing()
    UsePNGImageEncoder()
    UsePNGImageEncoder()
    Define sav
    sav = SaveSprite(pal,"palette_" + *design\name + ".png", #PB_ImagePlugin_PNG)
    ;Debug sav
    FreeSprite(pal)
    ;RunProgram("palette_" + *design\name + ".png")
    ;Delay(5000)
    End
  EndIf
EndProcedure


;** sgxLoadDesign:
;* loads a design. _
;* returns the loaded design or 0 if loading fails. _
;* you don't necessarily need the return value, but you can store it for later use with <a href=#f_sgxsetdesign>sgxSetDesign</a> in which case an integer variable is sufficient. _
;* if you want to access structure fields of the design you should use a pointer of type <a href=#s_sgxs_design>sgxS_Design</a> as variable. _
;* if you call this procedure before you opened a screen, make use of <a href=#f_sgxsetdimensions>sgxSetDimensions</a>.
;** .design : the path / directory that contains the design images, or "lowbudget" (default) for an instant design
;** .basecolor : basecolor to be used for the design, or -1 (default) to use the color as found in the image data. 'lowbudget' will be fully colored. file-designs only support this for color of text and thin borders.
Procedure sgxLoadDesign(design.s="lowbudget", basecolor=-1)
  Protected ret
  Protected i
  If Not ( design="lowbudget" Or FileSize(design)=-2 )
    sgx_err("can't load design. no such directory: " + design + " (currentDirectory: "+GetCurrentDirectory()+")")
    ProcedureReturn 0
  EndIf
  
  Static sgx_is_initialized = 0
  If Not sgx_is_initialized
    sgx_is_initialized = 1
    sgx_m\cursor=-1
    If sgx_dimensions\w = 0
      sgx_dimensions\w = ScreenWidth()
    EndIf
    If sgx_dimensions\h = 0
      sgx_dimensions\h = ScreenHeight()
    EndIf
    sgx_m\dlimit = 3
    
    ; default anim styles
    sgx_animStyle_minimizing = #sgxAnimZoom | #sgxAnimFade 
    sgx_animStyle_restoring  = #sgxAnimZoom | #sgxAnimFade
    sgx_animStyle_closing    = #sgxAnimSize | #sgxAnimFade
    sgx_animStyle_presenting = #sgxAnimSize | #sgxAnimFade
    
    ; call sgxExamine() once for other initializations.
    ; (sgx_ems needs to be non-zero for window animation to work, in case you would
    ; call sgxPresentWin() before the first call of sgxExamine() for example.)
    sgxExamine()
  EndIf

  AddElement(sgxDesign())
  sgxDesign()\name = design
  sgxDesign()\basecolor = basecolor
  sgxRefreshDesign(sgxDesign())
  *sgx_CurrentDesign = @sgxDesign()
  ret = @sgxDesign()
  
  ProcedureReturn ret
EndProcedure


;** sgxRefreshDesign
;* refresh the design. i.e. after you changed the basecolor of the design. the images will be regenerated / reloaded from disk.
;** .*design : pointer to a valid design
Procedure sgxRefreshDesign(*design.sgxS_Design)
  
  Protected designPath.s = *design\name + Mid("\",1,Bool(0 Or Right(*design\name,1)<>"\")) ; (ensure trailing backslash)
  Protected isLowbudget = Bool(designPath = "lowbudget\")
  Protected imgParts
  Protected imgPartsW
  Protected imgPartsH
  Protected noColor
  Protected p
  Protected m
  Protected isBlank
  Protected isBlankPrevious
  Protected i
  Protected k
  Protected w
  Protected h
  Protected r, g, b, a, mean
  Protected c1, c2, c3
  Protected bgColor
  Protected rnd.f
  Protected Dim edgesY(9)
  Protected Dim edgesX(17)
  Protected Dim edgesX2(3)
  Protected Dim edgesX3(1)
  
  ; reset design images
  For i=#sgx_DesignBackgroundNormal To #sgx_DesignMouseCursor
    sgx_FreeImageChecked(*design\img[i]\nr)
  Next
  
  
  
  
  
  
  If isLowbudget
  
    ; loading lowbudget parts image from file during sgx development, or from datasection otherwise.
    ; (use lowbudget_toDataSection.exe to update datasection from image file)
    If 01
      imgParts = CatchImage(#PB_Any, ?sgx_lowbudget_png_start, ?sgx_lowbudget_png_end - ?sgx_lowbudget_png_start)
    Else
      imgParts = LoadImage(#PB_Any, "lowbudget.png")
    EndIf
  
  ElseIf FileSize(designPath) = -2 ; is directory?
      imgParts = LoadImage(#PB_Any, designPath + "parts" + ".png")
      *design\img[#sgx_DesignBackgroundNormal]\nr = LoadImage(#PB_Any,designPath + "BackgroundNormal" + ".png")
      *design\img[#sgx_DesignBackgroundHover ]\nr = LoadImage(#PB_Any,designPath + "BackgroundHover"  + ".png")
      *design\img[#sgx_DesignMouseCursor]\nr = LoadImage(#PB_Any, designPath + "MouseCursor.png")
  EndIf
  
  ;{ -- parts
  
  If IsImage(imgParts)
    imgPartsW = ImageWidth(imgParts)
    imgPartsH = ImageHeight(imgParts)
    
    StartDrawing(ImageOutput(imgParts))
      DrawingMode(#PB_2DDrawing_AllChannels)
      
      ; get the separating color from top left pixel
      noColor = Point(0,0)
      
      ; if no basecolor is specified explicitly, then get the base color from bottom left pixel
      If *design\basecolor = -1
        ;Debug "..picking"
        *design\basecolor = Point(0,imgPartsH-1)
      EndIf
      
      ;sgx_debugColorC("noColor", noColor)
      ;sgx_debugColorC("*design\basecolor", *design\basecolor)
      
      If isLowbudget
        ; apply basecolor to every pixel that is not black and not the separating color
        For i=0 To imgPartsW-1
          For k=0 To imgPartsH-1
            c1 = Point(i,k)
            If c1 And c1<>noColor
              r = (c1&$000000ff)
              g = (c1&$0000ff00) >> 8
              b = (c1&$00ff0000) >> 16
              a = (c1&$ff000000) >> 24
              c2 =    (*design\basecolor&$000000ff)      * r/255
              c2 | ( ((*design\basecolor&$0000ff00)>>8)  * g/255 ) << 8
              c2 | ( ((*design\basecolor&$00ff0000)>>16) * b/255 ) << 16
              c2 | ( ((*design\basecolor&$ff000000)>>24) * a/255 ) << 24
              Plot(i,k, c2)
            Else
              Plot(i,k, c1)
            EndIf
          Next
        Next
      EndIf
      
      ; use the separating color to get the coordinates of the empty parts in the image (horizontal/vertical gaps/lines)
      ; (see section below to debug these)
      
      ;{ -- Y-gaps
      isBlank = 1
      m = 0
      
      For k=0 To imgPartsH-1
        isBlankPrevious = isBlank
        isBlank = 1
        For i=0 To imgPartsW-1
          p = Point(i,k)
          If p<>noColor
            isBlank = 0
            Break
          EndIf
        Next
        If (isBlank<>isBlankPrevious)
          If isBlank
            edgesY(m) = k-1
          ElseIf isBlankPrevious
            edgesY(m) = k
          EndIf
          m+1
          If m>9
            Break
          EndIf
        EndIf
      Next
      ;}
      ;  --
      
      ;{ -- X-gaps (parts)
      isBlank = 1
      m = 0
      
      For i=0 To imgPartsW-1
        isBlankPrevious = isBlank
        isBlank = 1
        For k=0 To edgesY(5)
          p = Point(i,k)
          If p<>noColor
            isBlank = 0
            Break
          EndIf
        Next
        If (isBlank<>isBlankPrevious)
          If isBlank
            edgesX(m) = i-1
          ElseIf isBlankPrevious
            edgesX(m) = i
          EndIf
          m+1
          If m>17
            Break
          EndIf
        EndIf
      Next
      ;}
      ;  --
      
      ;{ -- X2-gaps (closegadget)
      isBlank = 1
      m = 0
      
      For i=0 To imgPartsW-1
        isBlankPrevious = isBlank
        isBlank = 1
        For k=edgesY(5)+1 To edgesY(7)
          p = Point(i,k)
          If p<>noColor
            isBlank = 0
            Break
          EndIf
        Next
        If (isBlank<>isBlankPrevious)
          If isBlank
            edgesX2(m) = i-1
          ElseIf isBlankPrevious
            edgesX2(m) = i
          EndIf
          m+1
          If m>4
            Break
          EndIf
        EndIf
      Next
      ;}
      ;  --
      
      ;{ -- X3-gaps (checkboxChecked)
      isBlank = 1
      m = 0
      
      For i=0 To imgPartsW-1
        isBlankPrevious = isBlank
        isBlank = 1
        For k=edgesY(7)+1 To edgesY(9)
          p = Point(i,k)
          If p<>noColor
            isBlank = 0
            Break
          EndIf
        Next
        If (isBlank<>isBlankPrevious)
          If isBlank
            edgesX3(m) = i-1
          ElseIf isBlankPrevious
            edgesX3(m) = i
          EndIf
          m+1
          If m>1
            Break
          EndIf
        EndIf
      Next
      ;}
      ;  --
      
      ;{ debug edges
      If 00
        Protected _i
        Debug "edgesY: "
        For _i=0 To ArraySize(edgesY())
          Debug Str(_i)+": y="+Str(edgesY(_i))
        Next
        Debug "edgesX: "
        For _i=0 To ArraySize(edgesX())
          Debug Str(_i)+": x="+Str(edgesX(_i))
        Next
        Debug "edgesX2: "
        For _i=0 To ArraySize(edgesX2())
          Debug Str(_i)+": x="+Str(edgesX2(_i))
        Next
        Debug "edgesX3: "
        For _i=0 To ArraySize(edgesX3())
          Debug Str(_i)+": x="+Str(edgesX3(_i))
        Next
      EndIf
      ;}
      
    StopDrawing()
    
    ; create some other colors from the basecolor
    sgx_MakePalette(*design, *design\basecolor )
    
    ;{ -- grab parts
    i=0  :  k=0  :  *design\img[#sgx_DesignBorderNormalCorner1]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
    i=2  :  k=0  :  *design\img[#sgx_DesignBorderNormalHor1   ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
    i=4  :  k=0  :  *design\img[#sgx_DesignBorderNormalCorner2]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
    
    i=6  :  k=0  :  *design\img[#sgx_DesignBorderHoverCorner1 ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
    i=8  :  k=0  :  *design\img[#sgx_DesignBorderHoverHor1    ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
    i=10 :  k=0  :  *design\img[#sgx_DesignBorderHoverCorner2 ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
    
    i=12 :  k=0  :  *design\img[#sgx_DesignBorderPushedCorner1]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
    i=14 :  k=0  :  *design\img[#sgx_DesignBorderPushedHor1   ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
    i=16 :  k=0  :  *design\img[#sgx_DesignBorderPushedCorner2]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
    

    i=0  :  k=2  :  *design\img[#sgx_DesignBorderNormalVer1   ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
    i=4  :  k=2  :  *design\img[#sgx_DesignBorderNormalVer2   ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
    
    i=6  :  k=2  :  *design\img[#sgx_DesignBorderHoverVer1    ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
    i=10 :  k=2  :  *design\img[#sgx_DesignBorderHoverVer2    ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
    
    i=12 :  k=2  :  *design\img[#sgx_DesignBorderPushedVer1   ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
    i=16 :  k=2  :  *design\img[#sgx_DesignBorderPushedVer2   ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
    

    i=0  :  k=4  :  *design\img[#sgx_DesignBorderNormalCorner3]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
    i=2  :  k=4  :  *design\img[#sgx_DesignBorderNormalHor2   ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
    i=4  :  k=4  :  *design\img[#sgx_DesignBorderNormalCorner4]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
    
    i=6  :  k=4  :  *design\img[#sgx_DesignBorderHoverCorner3 ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
    i=8  :  k=4  :  *design\img[#sgx_DesignBorderHoverHor2    ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
    i=10 :  k=4  :  *design\img[#sgx_DesignBorderHoverCorner4 ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
    
    i=12 :  k=4  :  *design\img[#sgx_DesignBorderPushedCorner3]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
    i=14 :  k=4  :  *design\img[#sgx_DesignBorderPushedHor2   ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
    i=16 :  k=4  :  *design\img[#sgx_DesignBorderPushedCorner4]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
    
    i=0  :  k=6  :  *design\img[#sgx_DesignCloseGadgetNormal  ]\nr = GrabImage(imgParts,#PB_Any, edgesX2(i),edgesY(k), edgesX2(i+1)-edgesX2(i)+1, edgesY(k+1)-edgesY(k)+1 )
    i=2  :  k=6  :  *design\img[#sgx_DesignCloseGadgetHover   ]\nr = GrabImage(imgParts,#PB_Any, edgesX2(i),edgesY(k), edgesX2(i+1)-edgesX2(i)+1, edgesY(k+1)-edgesY(k)+1 )
    
    i=0  :  k=8  :  *design\img[#sgx_DesignCheckboxChecked    ]\nr = GrabImage(imgParts,#PB_Any, edgesX3(i),edgesY(k), edgesX3(i+1)-edgesX3(i)+1, edgesY(k+1)-edgesY(k)+1 )
    
    ;}
    ;  --
    
    sgx_FreeImageChecked(imgParts)
    ;}
    
  Else
    sgx_err("invalid parts image (design: "+designPath +")")
  EndIf
  
  
  If isLowbudget

    ;{ -- (lowbudget) BackgroundNormal
    w = 80
    h = 50
    sgx_RemakeDesignImage(#sgx_DesignBackgroundNormal,w,h)
    StartDrawing(ImageOutput(*design\img[#sgx_DesignBackgroundNormal]\nr))
      DrawingMode(#PB_2DDrawing_AllChannels)
      bgColor = *design\c[0]
      Box(0,0,w,h,bgColor)
      If sgx_lowbudgetDistortion
        ;a = Alpha(bgColor)
        For i=0 To w-1
          For k=0 To h-1
            ; get a small variance of each component of the bgcolor and add the variances to the slightly 
            ; reduced bgcolor basically something like 90% bgcolor + random(20% bgcolor) so we get random 
            ; pixels in the range 90%-110% bgcolor brightness (and color distribution)
            If sgx_lowbudgetDistortColors
              ; random per component
              r = Random(  Red(sgx_fade(bgColor,-1000 + sgx_lowbudgetDistortion * 1000 )))
              g = Random(Green(sgx_fade(bgColor,-1000 + sgx_lowbudgetDistortion * 1000 )))
              b = Random( Blue(sgx_fade(bgColor,-1000 + sgx_lowbudgetDistortion * 1000 )))
            Else
              ; keep component distribution
              rnd = Random(1000) / 1000
              r = rnd * Red  (sgx_fade(bgColor,-1000 + sgx_lowbudgetDistortion * 1000 ))
              g = rnd * Green(sgx_fade(bgColor,-1000 + sgx_lowbudgetDistortion * 1000 ))
              b = rnd * Blue (sgx_fade(bgColor,-1000 + sgx_lowbudgetDistortion * 1000 ))
            EndIf
            Plot(i,k, sgx_fade(bgColor,-(1000 * sgx_lowbudgetDistortion/2)) + r | g<<8 | b<<16 ); | a<<24)  alpha stays untouched
          Next
        Next
      EndIf
    StopDrawing() ;}
    ;  --
    
    ;{ -- (lowbudget) BackgroundHover (copy and brighten up a bit)
    w = *design\img[#sgx_DesignBackgroundNormal]\w
    h = *design\img[#sgx_DesignBackgroundNormal]\h
    *design\img[#sgx_DesignBackgroundHover]\w = w
    *design\img[#sgx_DesignBackgroundHover]\h = h
    sgx_FreeImageChecked(*design\img[#sgx_DesignBackgroundHover]\nr)
    *design\img[#sgx_DesignBackgroundHover]\nr = CopyImage(*design\img[#sgx_DesignBackgroundNormal]\nr, #PB_Any)
    StartDrawing(ImageOutput(*design\img[#sgx_DesignBackgroundHover]\nr))
      DrawingMode(#PB_2DDrawing_AlphaBlend)
      Box(0,0, w,h, RGBA(255,255,255,5))
    StopDrawing()
    ;}
    ;  --
    
    ;{ -- (lowbudget) cursor 
    sgx_FreeImageChecked(*design\img[#sgx_DesignMouseCursor]\nr)
    *design\img[#sgx_DesignMouseCursor]\nr = CreateImage(#PB_Any, 32, 32, 32)
    StartDrawing(ImageOutput(*design\img[#sgx_DesignMouseCursor]\nr))
      DrawingMode(#PB_2DDrawing_AllChannels)
      Box(0,0,32,32, RGBA(0,0,0,0))
      DrawingMode(#PB_2DDrawing_AlphaBlend)
      c1 = RGBA(Red(*design\basecolor), Green(*design\basecolor), Blue(*design\basecolor), 0.3*Alpha(*design\basecolor))
      c2 = RGBA(Red(*design\basecolor), Green(*design\basecolor), Blue(*design\basecolor), 0.8*Alpha(*design\basecolor))
      c3 = RGBA(Red(*design\basecolor), Green(*design\basecolor), Blue(*design\basecolor), 0.1*Alpha(*design\basecolor))
      LineXY( 0, 1,  5,25, c1)
      LineXY( 6,25,  9,18, c1)
      LineXY(10,18, 16,17, c1)
      LineXY(16,16,  0, 0, c1)
      FillArea(4,6, -1, c2)
      LineXY( 1, 2,  8,17, c3)
    StopDrawing()
    
    ;}
    ;  --
    
  Else    
  
    ;{ -- BackgroundNormal
    If IsImage( *design\img[#sgx_DesignBackgroundNormal]\nr )
      *design\img[#sgx_DesignBackgroundNormal]\w = ImageWidth(*design\img[#sgx_DesignBackgroundNormal]\nr)
      *design\img[#sgx_DesignBackgroundNormal]\h = ImageHeight(*design\img[#sgx_DesignBackgroundNormal]\nr) ;}
    EndIf
    ;  --
  
    ;{ -- BackgroundHover
    If IsImage( *design\img[#sgx_DesignBackgroundHover]\nr )
      *design\img[#sgx_DesignBackgroundHover]\w = ImageWidth(*design\img[#sgx_DesignBackgroundHover]\nr)
      *design\img[#sgx_DesignBackgroundHover]\h = ImageHeight(*design\img[#sgx_DesignBackgroundHover]\nr) ;}
    EndIf
    ;  --
    
    ;{ -- cursor
    If IsImage( *design\img[#sgx_DesignMouseCursor]\nr )
      *design\img[#sgx_DesignMouseCursor]\w = ImageWidth(*design\img[#sgx_DesignMouseCursor]\nr)
      *design\img[#sgx_DesignMouseCursor]\h = ImageHeight(*design\img[#sgx_DesignMouseCursor]\nr) ;}
    EndIf
    ;  --
    
  EndIf
  
  ; get images width and height
  For i=#sgx_DesignBackgroundNormal To #sgx_DesignBorderPushedVer2
    *design\img[i]\w = ImageWidth(*design\img[i]\nr)
    *design\img[i]\h = ImageHeight(*design\img[i]\nr)
  Next
  
EndProcedure






; IDE Options = PureBasic 5.50 beta 1 (Windows - x64)
; CursorPosition = 121
; FirstLine = 103
; Folding = -+j
; UseMainFile = demo.pb
; HideErrorLog
; CompileSourceDirectory