

;** sgx_prepareSprite
;* creates an element sprite it not done yet or frees and recreates the sprite if the size is different.
;** .spr : variable that contains the pb sprite number.
;** .w   : new width
;** .h   : new height
;** .id  : ID of the sgx element that is shown in a debugger error if the sprite creation fails.
Macro sgx_prepareSprite(spr, w, h, id)
  ; values of 0 are ignored, handled as unititialized and are not freed (sprite number 0 might be a valid sprite outside of sgx)
  If Not (spr)  Or  Not IsSprite(spr)  Or  (w) <> SpriteWidth(spr)  Or  (h) <> SpriteHeight(spr)
    sgx_FreeSpriteChecked(spr)
    spr = CreateSprite(#PB_Any, w, h, #PB_Sprite_AlphaBlending)
    If Not spr
      sgx_err("sprite creation failed with size "+Str(w)+", "+Str(h)+": sgx("+id+")")
    Else
      TransparentSpriteColor(spr, RGB(0,0,0))
    EndIf
  EndIf
EndMacro



;** sgxRefreshAll
;* refresh all existing sgx elements, if you modified the design for instance.
Procedure sgxRefreshAll()
  Protected id
  For id=0 To sgx_CURRENT_MAX_DYNAMIC_GADGETS
    sgx_SkipUnused(id)
    If sgx(id)
      sgxRefresh(id)
    EndIf
  Next
EndProcedure


;** sgxRefresh
;* refresh an sgx element.
;** .id   : ID of the sgx element to be refreshed.
;** .mode : #sgx_refreshNormalWithTip           : normal refresh (including sgx-tip). _
;*          #sgx_refreshOnlyTip                 : only the sgx-tip will be refreshed. _
;*          #sgx_refreshNormalWithChildsOfTables: normal refresh, but includes childs of tables. _
;*          #sgx_refreshSkipTableCheckboxRedraw : skip redraw for checkboxes created in tables. _
;*          #sgx_refreshEditMouseclickCursor    : for edit elements: update the character textpos from the current cursor pixel position (as opposed to other way round). used when positioning cursor by mouseclick. _
;*          #sgx_refreshEditMousewheelScroll    : for edit elements: disable the correction of textalign/scrollpos if cursor is outside clientarea. used when scrolling by mousewheel.
Procedure sgxRefresh(id, mode=#sgx_refreshNormalWithTip)
  Protected w, h
  Protected i, nbChars
  Protected i2
  Protected temp.s
  Protected char.s, text.s
  Protected lastwhitespace, linestart, inserted, maxLineWidth
  Protected textcursorX_prev, textcursorY_prev
  Protected sprW, sprH, xClip, yClip, wClip, hClip
  Protected wPrev, hPrev, w2Prev, h2Prev
  Protected NewList sizes.sgx_S_metrics(), _id_, doRefresh
  
  
  If sgx(id)
    
    AddElement(sgx_pendingEvents())
    sgx_pendingEvents()\id = id
    sgx_pendingEvents()\refreshed = 1
    
    ; (debug)
    ; Static indent.s, parent
    ; indent = ""
    ; parent = sgx(id)\parent
    ; While parent <> -1
    ;   indent + "    "
    ;   parent = sgx(parent)\parent
    ; Wend
    ; indent + "sgxRefresh("+id+", "+mode+")" + sgx_getTypeString(id)
    ; Debug indent
    
    wPrev = sgx(id)\w
    hPrev = sgx(id)\h
    w2Prev = sgx(id)\w2
    h2Prev = sgx(id)\h2
    
    ;- refresh tip
    If sgx(id)\tip
      sgx_DrawTip(id)
    EndIf
    
    sgx_DrawResizeGadget(id)
    
    If mode <> #sgx_refreshOnlyTip
      
      Select sgx(id)\type
          
          ;- refresh cursor
        Case #sgxTypeCursor ;{
          sgx_DrawCursor(id, #sgx_GadgetSprNormal)
          ;}
          
          ;- refresh button/togglebutton
        Case #sgxTypeButton, #sgxTypeButtonToggle ;{
          StartDrawing(ScreenOutput())
            DrawingFont(sgx(id)\textfontID)
            sgx(id)\texth = TextHeight("X")
          StopDrawing()
          sgx_DrawButton(id, #sgx_GadgetSprNormal)
          sgx_DrawButton(id, #sgx_GadgetSprHover)
          sgx_DrawButton(id, #sgx_GadgetSprPushed)
          sgx_DrawButton(id, #sgx_GadgetSprHoverPushed)
          ;}
          
          ;- refresh checkbox
        Case #sgxTypeCheckbox ;{
          
          ; get text width and line height
          StartDrawing(ScreenOutput())
            DrawingFont(sgx(id)\textfontID)
            sgx(id)\textw = TextWidth(sgx(id)\text)
            sgx(id)\texth = TextHeight("X")
          StopDrawing()
          
          ; overall width = knob width + label width
          sgx(id)\w2 = sgx(id)\w + sgx(id)\textw+5
          ; overall height = knob height or label height
          sgx(id)\h2 = sgx_max(sgx(id)\h, sgx(id)\texth)
          
          ; knob-relative display x-position..
          If sgx(id)\textalign = #sgxAlignCheckboxTextLeft
            ; ..is shifted 'labelwidth' the left
            sgx(id)\x2 = -(sgx(id)\textw+5)
          Else
            ; ..is same as knob position
            sgx(id)\x2 = 0
          EndIf
          
          ; knob-relative display y-position will be 0 (if h=h2) or negative (if text is larger then knob height)
          sgx(id)\y2 = sgx(id)\h/2 - sgx(id)\h2/2
          
          ; skip redraw for now for checkboxes created in tables (for checkboxes created during table creation
          ; we only got here to update the metrics above. the table will trigger the elements redraw later)
          If Not mode=#sgx_refreshSkipTableCheckboxRedraw
            sgx_DrawCheckbox(id, #sgx_GadgetSprNormal)
            sgx_DrawCheckbox(id, #sgx_GadgetSprHover)
            sgx_DrawCheckbox(id, #sgx_GadgetSprChecked)
          EndIf
          ;}
          
          ;- refresh scrollbar/trackbar
        Case #sgxTypeScrollbarVer, #sgxTypeScrollbarHor, #sgxTypeTrackbarVer, #sgxTypeTrackbarHor ;{
          If sgx(id)\h > sgx(id)\w ; ### VER |
            If sgx(id)\type & #sgxTypeScrollbar
              sgx(id)\type=#sgxTypeScrollbarVer
            ElseIf sgx(id)\type & #sgxTypeTrackbar
              sgx(id)\type=#sgxTypeTrackbarVer
            EndIf
            sgx(id)\width=sgx(id)\h2
            If sgx(id)\h < sgx(id)\width+1
              sgx(id)\h = sgx(id)\width+1
            EndIf
            sgx(id)\x2 = (sgx(id)\w-sgx(id)\w2)/2
            ;sgx(id)\y2 = (sgx(id)\h-sgx(id)\h2)/2
          Else                     ; ### HOR --
            If sgx(id)\type & #sgxTypeScrollbar
              sgx(id)\type=#sgxTypeScrollbarHor
            ElseIf sgx(id)\type & #sgxTypeTrackbar
              sgx(id)\type=#sgxTypeTrackbarHor
            EndIf
            sgx(id)\width = sgx(id)\w2
            If sgx(id)\w < sgx(id)\width+1
              sgx(id)\w = sgx(id)\width+1
            EndIf
            ;sgx(id)\x2 = (sgx(id)\w-sgx(id)\w2)/2
            sgx(id)\y2 = (sgx(id)\h-sgx(id)\h2)/2
          EndIf
          ;sgx(id)\scrollpos=0;sgx(id)\width/2
          ;sgx_SetScrollPos(id,sgx(id)\scrollpos)
          ;sgx(id)\state=0
          sgx_DrawScrollbar(id, #sgx_GadgetSprNormal)  ; -- bar
          sgx_DrawScrollbar(id, #sgx_GadgetSprHover)   ; /
          sgx_DrawScrollbar(id, #sgx_GadgetSprNormal2) ; -- knob
          sgx_DrawScrollbar(id, #sgx_GadgetSprHover2)  ; /
          ;}
          
          ;- refresh edit
        Case #sgxTypeEdit ;{
          ; get 'client area'
          sgx(id)\x2 = sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w+2
          sgx(id)\y2 = sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h+2
          sgx(id)\w2 = sgx(id)\w - sgx(id)\x2 - sgx(id)\design\img[#sgx_DesignBorderNormalVer2]\w-2
          sgx(id)\h2 = sgx(id)\h - sgx(id)\y2 - sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h-2
          If sgx(id)\w2 < 1
            sgx(id)\w2 = 1
          EndIf
          If sgx(id)\h2 < 1
            sgx(id)\h2 = 1
          EndIf
          Static moveTextPos, scrollToCursor
          
          moveTextPos = #False   ; while normal typing / cursor keys
          scrollToCursor = #True ; while normal typing / cursor keys
          
          If mode = #sgx_refreshEditMouseclickCursor
            moveTextPos = #True ; while cursor positioning by mouseclick
          ElseIf mode = #sgx_refreshEditMousewheelScroll
            scrollToCursor = #False ; disable scrollToCursor while scrolling by mousewheel
          EndIf
          ; update metrics
          sgx_DrawEdit(id, 0, #True, moveTextPos, scrollToCursor)
          
          ; redraw (while focused only redraw the hover sprite)
          sgx_DrawEdit(id, #sgx_GadgetSprHover)
          If sgx_editing <> id
            sgx_DrawEdit(id, #sgx_GadgetSprNormal)
            sgx_DrawEdit(id, #sgx_GadgetSprTextCursor)
          EndIf
          
          ; clip textcursor sprite if necessary
          
          ; reset clipping to get proper sprite size
          ClipSprite(sgx(id)\spr[#sgx_GadgetSprTextCursor], #PB_Default, #PB_Default, #PB_Default, #PB_Default)
          sprW = SpriteWidth (sgx(id)\spr[#sgx_GadgetSprTextCursor])
          sprH = SpriteHeight(sgx(id)\spr[#sgx_GadgetSprTextCursor])
          
          xClip = 0
          yClip = 0
          wClip = sprW
          hClip = sprH
          If sgx(id)\textcursor\x < 0
            xClip = -sgx(id)\textcursor\x
          EndIf
          If sgx(id)\textcursor\x > (sgx(id)\w2 - sprW)
            wClip = sprW - (sgx(id)\textcursor\x - (sgx(id)\w2 - sprw))
          EndIf
          If sgx(id)\textcursor\y < 0
            yClip = -sgx(id)\textcursor\y
          EndIf
          If sgx(id)\textcursor\y > (sgx(id)\h2 - sprH)
            hClip = sprH - (sgx(id)\textcursor\y - (sgx(id)\h2 - sprH))
          EndIf
          ClipSprite(sgx(id)\spr[#sgx_GadgetSprTextCursor], xClip, yClip, wClip, hClip)
          
          ; store offset for proper sprite positioning
          sgx_textCursorClippingOffset\x = xClip
          sgx_textCursorClippingOffset\y = yClip
          ;}
          
          ;- refresh window
        Case #sgxTypeWin ;{
          StartDrawing(ScreenOutput())
            DrawingFont(sgx(id)\textfontID)
            sgx(id)\textw = TextWidth(sgx(id)\text)
            sgx(id)\texth = TextHeight(sgx(id)\text)
          StopDrawing()
          
          ; get titlebar height
          If sgx(id)\texth
            sgx(id)\width = sgx(id)\texth + 2 + sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h + sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h
          Else
            sgx(id)\width = 0
          EndIf
          
          ; close gadget (keep that order; second relies on first!)
          sgx_DrawWin(id, #sgx_GadgetSprNormal2)
          sgx_DrawWin(id, #sgx_GadgetSprHover2)
          
          ; get close button size and position
          If sgx(id)\width
            sgx(id)\sys0\x1 = sgx(id)\w - (sgx(id)\width-SpriteHeight(sgx(id)\spr[#sgx_GadgetSprNormal2]))/2 - SpriteWidth(sgx(id)\spr[#sgx_GadgetSprNormal2])
            sgx(id)\sys0\y1 = (sgx(id)\width-SpriteHeight(sgx(id)\spr[#sgx_GadgetSprNormal2]))/2
          Else
            sgx(id)\sys0\x1 = sgx(id)\w - SpriteWidth(sgx(id)\spr[#sgx_GadgetSprNormal2]) - sgx(id)\design\img[#sgx_DesignBorderHoverVer2]\w - 1
            sgx(id)\sys0\y1 = sgx(id)\design\img[#sgx_DesignBorderHoverHor2]\h + 1
          EndIf
          sgx(id)\sys0\x2 = sgx(id)\sys0\x1 + SpriteWidth(sgx(id)\spr[#sgx_GadgetSprNormal2])
          sgx(id)\sys0\y2 = sgx(id)\sys0\y1 + SpriteHeight(sgx(id)\spr[#sgx_GadgetSprNormal2])
          
          ; these rely on sys0 to be initialized
          sgx_DrawWin(id, #sgx_GadgetSprNormal)
          sgx_DrawWin(id, #sgx_GadgetSprHover)
          sgx_DrawWin(id, #sgx_GadgetSprPushed)
          ;}
          
          ;- refresh table
        Case #sgxTypeTable ;{
          StartDrawing(ScreenOutput())
            DrawingFont(sgx(id)\textfontID)
            sgx(id)\textw = TextWidth(sgx(id)\text)
            sgx(id)\texth = TextHeight(sgx(id)\text)
          StopDrawing()
          ; in case the table has text, it gets a garbage titlebar (not realy usefull at the moment but inherited from window code)  
          If sgx(id)\texth
            sgx(id)\width = sgx(id)\texth + 2 + sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h + sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h
          Else
            sgx(id)\width = 0
          EndIf
          
          ; store all elements metrics to check size change after the following call of sgx_ArrangeTable()
          ; this is somewhat experimental and a bit oversized but with the avoidance of redundant table
          ; child refreshes there is some perfomance gain.
          ; could be reduced to only store table's childs recursivley and only if style fitW/H is set.
          For sgxEach(_id_)
            AddElement(sizes())
            sizes()\id = _id_
            sizes()\w = sgx(_id_)\w
            sizes()\h = sgx(_id_)\h
            sizes()\w2 = sgx(_id_)\w2
            sizes()\h2 = sgx(_id_)\h2
          Next
          
          ; collecting and adjusting metrics of table and childs without refreshing
          sgx_ArrangeTable(id)
          
          ; table child refresh (flat, but sub tables will refresh their childs too, causing recursion)
          If mode=#sgx_refreshNormalWithChildsOfTables And sgx(id)\treeElem\node
            sgx_llIterator(Protected *it, sgx(id)\treeElem\node)
            If sgx_BuildRefresh
                ; get previous element size from before sgx_ArrangeTable()..
                ForEach sizes()
                  If (sizes()\id = *it\id)
                    ; and refresh only if size changed
                    doRefresh = #False
                    If (sgx(*it\id)\type <> #sgxTypeCheckbox)  And  (sgx(*it\id)\w  <> sizes()\w  Or sgx(*it\id)\h  <> sizes()\h )
                      ; (non-checkboxes: check w/h change)
                      doRefresh = #True
                    ElseIf (sgx(*it\id)\type  = #sgxTypeCheckbox)  And  (sgx(*it\id)\w2 <> sizes()\w2 Or sgx(*it\id)\h2 <> sizes()\h2)
                      ; (checkboxes: check w2/h2 overall size change)
                      doRefresh = #True
                    EndIf
                    If doRefresh
                      sgxRefresh( *it\id, #sgx_refreshNormalWithChildsOfTables)
                    EndIf
                  EndIf
                Next
              Else
                sgxRefresh( *it\id, #sgx_refreshNormalWithChildsOfTables)
              EndIf
            sgx_llNext(*it)
          EndIf
          
          ; remove temporary metrics
          ClearList(sizes())
          
          ; tables use the window visuals
          sgx_DrawWin(id, #sgx_GadgetSprNormal)
          sgx_DrawWin(id, #sgx_GadgetSprHover)
          ;}
          
          ;- refresh combo
        Case #sgxTypeCombo ;{
          StartDrawing(ScreenOutput())
            DrawingFont(sgx(id)\textfontID)
            sgx(id)\texth = TextHeight("X")
          StopDrawing()
          sgx_DrawCombo(id, #sgx_GadgetSprNormal)
          sgx_DrawCombo(id, #sgx_GadgetSprHover)
          sgx_DrawCombo(id, #sgx_GadgetSprPushed) ; the opened combo
          sgx_DrawCombo(id, #sgx_GadgetSprHover2) ; open combo hover effect
          ;}
          
          ;- refresh text
        Case #sgxTypeText ;{
          StartDrawing(ScreenOutput())
          DrawingFont(sgx(id)\textfontID)
            ; get longest line widht and line heigth
            sgx(id)\textw = sgx_textWidthLF(sgx(id)\text)
            sgx(id)\texth = TextHeight("X")
            
            If sgx(id)\auto & #sgx_AutoW
              ; (margin will be added when drawn)
              sgx(id)\w = sgx(id)\textw +sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w +sgx(id)\design\img[#sgx_DesignBorderNormalVer2]\w
            EndIf
            
            If sgx(id)\w > 0
              
              ; automatic word wrap (add newlines where necessary)
              If sgx(id)\auto & #sgx_AutoNL
                nbChars = Len(sgx(id)\text)
                maxLineWidth = sgx(id)\w - 2*sgx(id)\margin
                lastwhitespace = 0
                For i=1 To nbChars
                  char = Mid(sgx(id)\text,i,1)
                  text + char
                  Select Asc(char)
                    Case 32,9 ; (space,tab)
                      lastwhitespace = i+inserted
                    Case #LF
                      lastwhitespace = i+inserted
                      ; next char will be begin of line
                      linestart = i+1
                  EndSelect
                  If TextWidth(Mid(text,linestart))  >  maxLineWidth  And  lastwhitespace<>linestart-1
                    ; dont break before first line/char
                    If lastwhitespace = 0
                      ; we're still in the first word of text, so we just go on and will break with next whitespace occurance
                      lastwhitespace+1
                      linestart = lastwhitespace+1
                    Else
                      ; insert newline (get text from beginning to last space, add newline, add the rest of the text)
                      text = Mid(text,1,lastwhitespace) + #LF$ + Mid(text,lastwhitespace+1)
                      ; count our additional added newline
                      inserted+1
                      lastwhitespace+1
                      linestart = lastwhitespace+1
                    EndIf
                  EndIf
                Next
                sgx(id)\text = text
              EndIf
              ; update longest line width
              sgx(id)\textw = sgx_textWidthLF(sgx(id)\text)
              
              If sgx(id)\auto & #sgx_AutoW
                sgx(id)\w = sgx(id)\margin + sgx(id)\textw + sgx(id)\margin
              EndIf
              If sgx(id)\auto & #sgx_AutoH
                ; lines * line height
                sgx(id)\h = sgx(id)\margin + (CountString(sgx(id)\text,#LF$)+1)*sgx(id)\texth + sgx(id)\margin
              EndIf
              
            EndIf
          StopDrawing()
          sgx_DrawText(id, #sgx_GadgetSprNormal)
          If sgx(id)\style & #sgxStyleHover
            sgx_DrawText(id, #sgx_GadgetSprHover)
          EndIf
          ;}
          
          ;- refresh area
        Case #sgxTypeArea ;{
          ; nothing to do
          ;}
          
      EndSelect
      
      
      ; disabled elements
      If sgx(id)\disabled
        ; we only modify those sprites that are used while disabled (i.e. not 'hover' etc.)
        ; user sprites are not modified but dimmed by alpha for disabled elements when displayed.
        sgx_DrawDisabled(id, #sgx_GadgetSprNormal)
        
        If sgx(id)\type & (#sgxTypeScrollbar|#sgxTypeTrackbar)
          sgx_DrawDisabled(id, #sgx_GadgetSprNormal2) ; knob
          
        ElseIf sgx(id)\type = #sgxTypeCheckbox
          sgx_DrawDisabled(id, #sgx_GadgetSprChecked) ; checked
          
        ElseIf sgx(id)\type = #sgxTypeButtonToggle
          sgx_DrawDisabled(id, #sgx_GadgetSprPushed)  ; pushed
          
        ElseIf sgx(id)\type = #sgxTypeWin
          sgx_DrawDisabled(id, #sgx_GadgetSprNormal2)  ; close gadget
        EndIf
      EndIf
      
    EndIf
    
    ; check if size changed (for checkbox also check overall size change including label)
    If (sgx(id)\w <> wPrev Or sgx(id)\h <> hPrev)  Or  (sgx(id)\type = #sgxTypeCheckbox And (sgx(id)\w2 <> w2Prev Or sgx(id)\h2 <> h2Prev))
      ; prepare event triggering
      AddElement(sgx_pendingEvents())
      sgx_pendingEvents()\id = id
      sgx_pendingEvents()\resized = 1
    EndIf
    
  EndIf
EndProcedure




; -------------------------------------- drawing auxiliary




;** sgx_Fill
;* draws a background to the current drawing output using the design's background image (will be tiled to fill specified area).
;** .*design.sgxS_Design : the design you want to use for the drawing.
;** .x                   : x-position to begin drawing
;** .y                   : y-position to begin drawing
;** .w                   : width of background to be drawn
;** .h                   : height of background to be drawn
;** .appearance          : a value like #sgx_AppearanceNormal, #sgx_AppearanceHover or #sgx_AppearancePushed etc.
Procedure sgx_Fill(*design.sgxS_Design, x, y, w, h, appearance)
  Static i, k
  Static img2, img3, img4
  Static width2, height3
  Static imgType
  
  Select appearance
  Case #sgx_AppearanceNormal
    imgType = #sgx_DesignBackgroundNormal
  Case #sgx_AppearanceHover
    imgType = #sgx_DesignBackgroundHover
  Default
    ProcedureReturn
  EndSelect
  
  ; heeere (fill)
  If 0
    DrawingMode(#PB_2DDrawing_AllChannels)
    ;DrawingMode(#PB_2DDrawing_AlphaBlend)
    DrawImage(ImageID(*design\img[imgType]\nr), x,y, w,h)
    
    ;DrawingMode(#PB_2DDrawing_Default | #PB_2DDrawing_AlphaBlend)
;     DrawingMode(#PB_2DDrawing_Default | #PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Gradient)
    
    ;DrawingMode(#PB_2DDrawing_Gradient)      
;     BackColor($aa666666)
;     GradientColor(0.95, $aa666666)
;     FrontColor($aa222222)
;     BoxedGradient(0,0,w,h)
;     Box(x,y,w,h)
    
;     Box(x,y,w,h,$aa666666)
    
    ProcedureReturn
  EndIf
  
  width2 = w % *design\img[imgType]\w
  height3 = h % *design\img[imgType]\h
  If width2>0
    img2 = GrabImage(*design\img[imgType]\nr, #PB_Any, 0,0, width2, *design\img[imgType]\h)
  EndIf
  
  If height3>0
    img3 = GrabImage(*design\img[imgType]\nr, #PB_Any, 0,0, *design\img[imgType]\w, height3)
  EndIf
  
  If width2>0 And height3>0
    img4 = GrabImage(*design\img[imgType]\nr, #PB_Any, 0,0, width2, height3)
  EndIf
  
  ; full size
  If IsImage(*design\img[imgType]\nr)
    k=y
    While k <= y+h-*design\img[imgType]\h
      i=x
      While i <= x+w-*design\img[imgType]\w
        DrawImage(ImageID(*design\img[imgType]\nr),i,k)
        i+*design\img[imgType]\w
      Wend
      k+*design\img[imgType]\h
    Wend
  EndIf

  ; remaining right
  If IsImage(img2) 
    k=y
    While k <= y+h-*design\img[imgType]\h
      DrawImage(ImageID(img2),i,k)
      k+*design\img[imgType]\h
    Wend
    sgx_FreeImageChecked(img2)
  EndIf

  ; remaining bottom
  If IsImage(img3) 
    i=x
    While i <= x+w-*design\img[imgType]\w
      DrawImage(ImageID(img3),i,k)
      i+*design\img[imgType]\w
    Wend
    sgx_FreeImageChecked(img3)
  EndIf

  ; remaining bottom-right
  If IsImage(img4) 
    DrawImage(ImageID(img4),i,k)
    sgx_FreeImageChecked(img4)
  EndIf
EndProcedure


;** sgx_BorderLine
;* draws a straight part (a line, as opposed to some corner part) of a border to the current drawing output using the design's border images. _
;* used by <a href=#f_sgx_borderbox>sgx_Borderbox</a>.
;** .*design.sgxS_Design : the design you want to use for the drawing.
;** .x                   : x-position to begin drawing
;** .y                   : y-position to begin drawing
;** .V                   : length of the line to be drawn. a positive value for horizontal lines and negative value for vertical lines.
;** .appearance          : a value like #sgx_AppearanceNormal, #sgx_AppearanceHover or #sgx_AppearancePushed etc.
;** .side                : the side of the border to be drawn: _
;*                         #sgx_SideTopOrLeft draws the top edge border (for horizontal) or left edge border (for vertical) respectively. _
;*                         #sgx_SideBottomOrRight draws the bottom edge border (for horizontal) or right edge border (for vertical) respectively.
Procedure sgx_BorderLine(*design.sgxS_Design, x, y, V, appearance, side)
  Static i
  Static *designImg1.sgx_S_Image
  Static full
  Static img2
  Static width2
  Static height2
  
  
  ;horizontal line
  If V>0
    
    ; get the design image
    Select appearance
    Case #sgx_AppearanceNormal
      If side = #sgx_SideTopOrLeft
        *designImg1 = *design\img[#sgx_DesignBorderNormalHor1] ; use left border parts
      ElseIf side = #sgx_SideBottomOrRight
        *designImg1 = *design\img[#sgx_DesignBorderNormalHor2] ; use right border parts
      EndIf
    Case #sgx_AppearanceHover
      If side = #sgx_SideTopOrLeft
        *designImg1 = *design\img[#sgx_DesignBorderHoverHor1] ; left
      ElseIf side = #sgx_SideBottomOrRight
        *designImg1 = *design\img[#sgx_DesignBorderHoverHor2] ; right
      EndIf
    Case #sgx_AppearancePushed
      If side = #sgx_SideTopOrLeft
        *designImg1 = *design\img[#sgx_DesignBorderPushedHor1] ; left
      ElseIf side = #sgx_SideBottomOrRight
        *designImg1 = *design\img[#sgx_DesignBorderPushedHor2] ; right
      EndIf
    Default
      ProcedureReturn
    EndSelect
    
    ; get full amount of images to be drawn
    full  = V / *designImg1\w
    ; get the remaining image width to be drawn
    width2 = V % *designImg1\w
    
    ; draw full images
    For i=0 To full-1
      DrawImage(ImageID(*designImg1\nr), x+i**designImg1\w, y)
    Next
    ; draw remaining image part
    If width2>0
      img2 = GrabImage(*designImg1\nr, #PB_Any, 0,0, width2, *designImg1\h)
      DrawImage(ImageID(img2), x + i * *designImg1\w, y)
      FreeImage(img2)
    EndIf
  
  
  
  ;vertical lines
  ElseIf v<0
    
    ; get the design image
    Select appearance
    Case #sgx_AppearanceNormal
      If side = #sgx_SideTopOrLeft
        *designImg1 = *design\img[#sgx_DesignBorderNormalVer1]
      ElseIf side = #sgx_SideBottomOrRight
        *designImg1 = *design\img[#sgx_DesignBorderNormalVer2]
      EndIf
    Case #sgx_AppearanceHover
      If side = #sgx_SideTopOrLeft
        *designImg1 = *design\img[#sgx_DesignBorderHoverVer1]
      ElseIf side = #sgx_SideBottomOrRight
        *designImg1 = *design\img[#sgx_DesignBorderHoverVer2]
      EndIf
    Case #sgx_AppearancePushed
      If side = #sgx_SideTopOrLeft
        *designImg1 = *design\img[#sgx_DesignBorderPushedVer1]
      ElseIf side = #sgx_SideBottomOrRight
        *designImg1 = *design\img[#sgx_DesignBorderPushedVer2]
      EndIf
    Default
      ProcedureReturn
    EndSelect
    
    ; get positive length
    V = -V
    
    ; get full amount of images to be drawn
    full  = V / *designImg1\h
    ; get the remaining image height to be drawn
    height2 = V % *designImg1\h
    
    ; draw full images
    For i=0 To full-1
      DrawImage(ImageID(*designImg1\nr), x, y+i**designImg1\h)
    Next
    ; draw remaining image part
    If height2>0
      img2 = GrabImage(*designImg1\nr, #PB_Any, 0,0, *designImg1\w, height2)
      DrawImage(ImageID(img2), x, y+i* *designImg1\h)
      FreeImage(img2)
    EndIf

  EndIf
EndProcedure


;** sgx_Borderbox
;* draws a box to the current drawing output using the design's border images. _
;** .*design.sgxS_Design : the design you want to use for the drawing.
;** .x                   : x-position to begin drawing
;** .y                   : y-position to begin drawing
;** .w                   : width of box to be drawn
;** .h                   : height of box to be drawn
;** .appearance          : a value like #sgx_AppearanceNormal, #sgx_AppearanceHover or #sgx_AppearancePushed etc.
Procedure sgx_Borderbox(*design.sgxS_Design, x, y, w, h, appearance)
  Static *designImgCorner1.sgx_S_Image
  Static *designImgCorner2.sgx_S_Image
  Static *designImgCorner3.sgx_S_Image
  Static *designImgCorner4.sgx_S_Image
  Static i
  
  ; get design's corner images
  Select appearance
  ; ----------------
  Case #sgx_AppearanceNormal ;{
    *designImgCorner1 = *design\img[#sgx_DesignBorderNormalCorner1]
    *designImgCorner2 = *design\img[#sgx_DesignBorderNormalCorner2]
    *designImgCorner3 = *design\img[#sgx_DesignBorderNormalCorner3]
    *designImgCorner4 = *design\img[#sgx_DesignBorderNormalCorner4]
    ;}
  
  Case #sgx_AppearanceHover ;{
    *designImgCorner1 = *design\img[#sgx_DesignBorderHoverCorner1]
    *designImgCorner2 = *design\img[#sgx_DesignBorderHoverCorner2]
    *designImgCorner3 = *design\img[#sgx_DesignBorderHoverCorner3]
    *designImgCorner4 = *design\img[#sgx_DesignBorderHoverCorner4]
    ;}
  
  Case #sgx_AppearancePushed ;{
    *designImgCorner1 = *design\img[#sgx_DesignBorderPushedCorner1]
    *designImgCorner2 = *design\img[#sgx_DesignBorderPushedCorner2]
    *designImgCorner3 = *design\img[#sgx_DesignBorderPushedCorner3]
    *designImgCorner4 = *design\img[#sgx_DesignBorderPushedCorner4]
    ;}
  
  Default
    ProcedureReturn
  EndSelect
  
  ; heeere (borderbox)
  If 0
    DrawingMode(#PB_2DDrawing_Outlined | #PB_2DDrawing_AlphaBlend)
;     Box(x+0, y+0, w-0, h-0, $66336633)
    ;Box(x+1, y+1, w-2, h-2, $88336633)
    ;Box(x+2, y+2, w-4, h-4, $aa336633)
    ProcedureReturn
  EndIf
  
  ; draw borders:
  ; top 
  ; bottom
  ; left
  ; right
  sgx_BorderLine(*design, x+*designImgCorner1\w,                          y,                                              w-*designImgCorner1\w-*designImgCorner2\w,    appearance, #sgx_SideTopOrLeft)
  sgx_BorderLine(*design, x+*designImgCorner1\w,                          y+h-*design\img[#sgx_DesignBorderNormalHor2]\h, w-*designImgCorner1\w-*designImgCorner2\w,    appearance, #sgx_SideBottomOrRight)
  sgx_BorderLine(*design, x,                                              y+*designImgCorner1\h,                          -(h-*designImgCorner2\h-*designImgCorner4\h), appearance, #sgx_SideTopOrLeft)
  sgx_BorderLine(*design, x+w-*design\img[#sgx_DesignBorderNormalVer2]\w, y+*designImgCorner2\h,                          -(h-*designImgCorner2\h-*designImgCorner4\h), appearance, #sgx_SideBottomOrRight)
  
  ; draw corners:
  ; top left
  ; top right
  ; bottom left
  ; bottom right
  DrawImage(ImageID(*designImgCorner1\nr), x,                       y)
  DrawImage(ImageID(*designImgCorner2\nr), x+w-*designImgCorner2\w, y)
  DrawImage(ImageID(*designImgCorner3\nr), x,                       y+h-*designImgCorner3\h)
  DrawImage(ImageID(*designImgCorner4\nr), x+w-*designImgCorner4\w, y+h-*designImgCorner3\h)
  
EndProcedure


;** sgx_CopyImageToSprite
;* creates and returns a sprite duplicated from an image (for example the mouse sprite from a loaded mouse design image, or the window's close buttons). _
;* the optional parameters will be specified by the element drawing routines to let this function also handle the custom drawing callbacks.
;** .imgNr            : a PB image number.
;** .onBeginDraw      : callback that should be called before drawing the image content to newly created sprite.
;** .onEndDraw        : callback that should be called after drawing the image content to newly created sprite.
;** .id               : id that should be passed to the callbacks.
;** .sprType          : sprType that should be passed to the callbacks.
Procedure sgx_CopyImageToSprite(imgNr, onBeginDraw.sgx_onBeginDraw=0, onEndDraw.sgx_onEndDraw=0, id=0, sprType=0)
  Protected ret = 0
  Protected w = ImageWidth(imgNr)
  Protected h = ImageHeight(imgNr)
  Protected i
  Protected k
  Protected draw
  If IsImage(imgNr)
    ret = CreateSprite(#PB_Any, w, h, #PB_Sprite_AlphaBlending)
    If Not ret
      sgx_err("sprite creation failed with size "+w+", "+h) ; +": sgx("+id+")")
    Else
      TransparentSpriteColor(ret, RGB(0,0,0))
      StartDrawing(SpriteOutput(ret))
        draw = #sgxDrawDesign | #sgxDrawText
        If onBeginDraw
          draw = onBeginDraw(id, sprType, w, h)
        EndIf
        If draw & #sgxDrawDesign
          DrawingMode(#PB_2DDrawing_AllChannels)
          DrawImage(ImageID(imgNr),0,0)
        EndIf
        If onEndDraw
          onEndDraw(id, sprType, w, h)
        EndIf
      StopDrawing()
    EndIf
  EndIf
  ProcedureReturn ret
EndProcedure



; -------------------------------------- draw sgx components ( refresh only; not display )


;** sgx_DrawCursor
;* redraws a particular state sprite of an sgxCursor element (called by <a href=#f_sgxrefresh>sgxRefresh</a>).
;** .id      : ID of the sgx element
;** .sprType : the state / sprite type that should be redrawn (not all elements use every state): #sgx_GadgetSprNormal or #sgx_GadgetSprHover etc.
Procedure sgx_DrawCursor(id, sprType)
  sgx_FreeSpriteChecked(sgx(id)\spr[sprType])
  sgx(id)\spr[sprType] = sgx_CopyImageToSprite(sgx(id)\design\img[#sgx_DesignMouseCursor]\nr, sgx(id)\design\onBeginDraw, sgx(id)\design\onEndDraw, id, sprType)
EndProcedure


;** sgx_DrawButton
;* redraws a particular state sprite of an sgxButton element (called by <a href=#f_sgxrefresh>sgxRefresh</a>).
;** .id      : ID of the sgx element
;** .sprType : the state / sprite type that should be redrawn (not all elements use every state): #sgx_GadgetSprNormal or #sgx_GadgetSprHover etc.
Procedure sgx_DrawButton(id, sprType)
  Static color1
  Static appearance_fill
  Static appearance_border
  Static textOffset, x, y, lines
  Static draw
  
  sgx_prepareSprite(sgx(id)\spr[sprType], sgx(id)\w, sgx(id)\h, id)
  If sgx(id)\spr[sprType]
    StartDrawing(SpriteOutput(sgx(id)\spr[sprType]))
      DrawingMode(#PB_2DDrawing_AllChannels)
      Box(0,0,sgx(id)\w,sgx(id)\h, RGBA(0,0,0,0))
      draw = #sgxDrawDesign | #sgxDrawText
      If sgx(id)\design\onBeginDraw
        draw = sgx(id)\design\onBeginDraw(id, sprType, sgx(id)\w, sgx(id)\h)
        ; store text offset for 'pushed' for later display of user sprite
        If draw & #sgxDrawTextNoOffset
          sgx(id)\margin = 0
        Else
          sgx(id)\margin = 1
        EndIf
      EndIf
      
      DrawingMode(#PB_2DDrawing_AlphaBlend)
      Select sprType
      Case #sgx_GadgetSprNormal
        If sgx(id)\type=#sgxTypeButtonToggle : color1 = sgx(id)\design\c[03]
        Else                                 : color1 = sgx(id)\design\basecolor
        EndIf
        appearance_fill = #sgx_AppearanceNormal
        appearance_border = #sgx_AppearanceNormal
        textOffset = 0
      Case #sgx_GadgetSprHover
        If sgx(id)\type=#sgxTypeButtonToggle : color1 = sgx(id)\design\c[09]
        Else                                 : color1 = sgx(id)\design\c[09]
        EndIf
        appearance_fill = #sgx_AppearanceHover
        appearance_border = #sgx_AppearanceHover
        textOffset = 0
      Case #sgx_GadgetSprPushed
        If sgx(id)\type=#sgxTypeButtonToggle : color1 = sgx(id)\design\basecolor
        Else                                 : color1 = sgx(id)\design\c[03]
        EndIf
        appearance_fill = #sgx_AppearanceNormal
        appearance_border = #sgx_AppearancePushed
        textOffset = 1
        If draw &  #sgxDrawTextNoOffset
          textOffset = 0
        EndIf
      Case #sgx_GadgetSprHoverPushed
        If sgx(id)\type=#sgxTypeButtonToggle : color1 = sgx(id)\design\c[09]
        Else                                 : color1 = sgx(id)\design\basecolor
        EndIf
        appearance_fill = #sgx_AppearanceHover
        appearance_border = #sgx_AppearancePushed
        textOffset = 1
        If draw &  #sgxDrawTextNoOffset
          textOffset = 0
        EndIf
      Default
        ProcedureReturn
      EndSelect
      
      If draw & #sgxDrawDesign
        DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Outlined)
        sgx_Fill( sgx(id)\design, sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w, sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h, sgx(id)\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer2]\w, sgx(id)\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h, appearance_fill)
        
        DrawingMode(#PB_2DDrawing_AlphaBlend)
        sgx_BorderBox(sgx(id)\design, 0, 0, sgx(id)\w, sgx(id)\h, appearance_border)
      EndIf
      
      If draw & (#sgxDrawText | #sgxDrawTextNoOffset)
        DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Transparent)
        DrawingFont(sgx(id)\textfontID)
        
        ; get text start drawing position to have it centered on the button
        lines = CountString(sgx(id)\text, #LF$) + 1
        x = textOffset + (sgx(id)\w - sgx_textWidthLF(sgx(id)\text)) / 2
        y = textOffset + (sgx(id)\h - (lines * sgx(id)\texth) ) / 2
        ; draw text centered
        sgx_drawTextLF( x, y, sgx(id)\text, color1, -1, #LF$, #sgxAlignTextCenter)
      EndIf
      
      If sgx(id)\design\onEndDraw
        sgx(id)\design\onEndDraw(id, sprType, sgx(id)\w, sgx(id)\h)
      EndIf
    StopDrawing()
  EndIf
    
EndProcedure


;** sgx_DrawCheckbox
;* redraws a particular state sprite of an sgxCheckbox element (called by <a href=#f_sgxrefresh>sgxRefresh</a>).
;** .id      : ID of the sgx element
;** .sprType : the state / sprite type that should be redrawn (not all elements use every state): #sgx_GadgetSprNormal or #sgx_GadgetSprHover etc.
Procedure sgx_DrawCheckbox(id, sprType)
;   Static x1
;   Static y1
;   Static x2
;   Static y2
  Static color1
  Static imgType_fill
  Static appearance_fill
  Static appearance_border
  Static draw
  
  Select sprType
    Case #sgx_GadgetSprNormal
      color1 = sgx(id)\design\basecolor
      appearance_fill = #sgx_AppearanceNormal
      appearance_border = #sgx_AppearanceNormal
    Case #sgx_GadgetSprHover
      color1 = sgx(id)\design\c[09]
      appearance_fill = #sgx_AppearanceHover
      appearance_border = #sgx_AppearanceHover
    Case #sgx_GadgetSprChecked
      color1 = sgx(id)\design\basecolor
    Default
      ProcedureReturn
  EndSelect
  
  Select sprType
    Case #sgx_GadgetSprNormal, #sgx_GadgetSprHover
      sgx_prepareSprite(sgx(id)\spr[sprType], sgx(id)\w2, sgx(id)\h2, id)
      If sgx(id)\spr[sprType]
        StartDrawing(SpriteOutput(sgx(id)\spr[sprType]))
          DrawingMode(#PB_2DDrawing_AllChannels)
          Box(0,0,sgx(id)\w2,sgx(id)\h2, RGBA(0,0,0,0))
          draw = #sgxDrawDesign | #sgxDrawText
          If sgx(id)\design\onBeginDraw
            draw = sgx(id)\design\onBeginDraw(id, sprType, sgx(id)\w2, sgx(id)\h2)
          EndIf
          If draw & (#sgxDrawDesign | #sgxDrawText | #sgxDrawTextNoOffset)
            DrawingMode(#PB_2DDrawing_AlphaBlend)
            Select sgx(id)\textalign
              Case #sgxAlignCheckboxTextRight
                
                ; box left, text right
                If draw & #sgxDrawDesign
                  sgx_Fill( sgx(id)\design, sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w, sgx(id)\h2/2-sgx(id)\h/2+sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h, sgx(id)\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer2]\w, sgx(id)\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h, appearance_fill)
                  sgx_Borderbox(sgx(id)\design, 0,sgx(id)\h2/2-sgx(id)\h/2, sgx(id)\w,sgx(id)\h, appearance_border)
                EndIf
                If draw & (#sgxDrawText | #sgxDrawTextNoOffset)
                  DrawingFont(sgx(id)\textfontID)
                  DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Transparent)
                  DrawText( sgx(id)\w+5, sgx(id)\h2/2-sgx(id)\texth/2, sgx(id)\text, color1 )
                EndIf
                
              Case #sgxAlignCheckboxTextLeft
                
                ; text left, box right
                If draw & #sgxDrawDesign
                  sgx_Fill( sgx(id)\design, sgx(id)\textw+5 + sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w, sgx(id)\h2/2-sgx(id)\h/2+sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h, sgx(id)\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer2]\w, sgx(id)\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h, appearance_fill)
                  sgx_Borderbox(sgx(id)\design, sgx(id)\textw+5,sgx(id)\h2/2-sgx(id)\h/2, sgx(id)\w,sgx(id)\h, appearance_border)
                EndIf
                If draw & (#sgxDrawText | #sgxDrawTextNoOffset)
                  DrawingFont(sgx(id)\textfontID)
                  DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Transparent)
                  DrawText( 0, sgx(id)\h2/2-sgx(id)\texth/2, sgx(id)\text, color1 )
                EndIf
              
            EndSelect
          EndIf
          If sgx(id)\design\onEndDraw
            sgx(id)\design\onEndDraw(id, sprType, sgx(id)\w2, sgx(id)\h2)
          EndIf
        StopDrawing()
      EndIf
    Case #sgx_GadgetSprChecked
      ; this is only the 'checked' part (without the knob).
      ; we only need the knob width but we use the full height to not worry about y offset when displaying.
      sgx_prepareSprite(sgx(id)\spr[sprType], sgx(id)\w, sgx(id)\h, id)
      If sgx(id)\spr[sprType]
        StartDrawing(SpriteOutput(sgx(id)\spr[sprType]))
          DrawingMode(#PB_2DDrawing_AllChannels)
          Box(0,0,sgx(id)\w,sgx(id)\h, RGBA(0,0,0,0))
          draw = #sgxDrawDesign | #sgxDrawText
          If sgx(id)\design\onBeginDraw
            draw = sgx(id)\design\onBeginDraw(id, sprType, sgx(id)\w, sgx(id)\h)
          EndIf
          If draw & #sgxDrawDesign
            DrawingMode(#PB_2DDrawing_AlphaBlend)
            ; old lowbudged 'checked' x
            ;If sgx(id)\design\name = "lowbudget"
            ;  x1 = sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w + 1
            ;  x2 = sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w + sgx(id)\w2 - 2 - 2*sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w
            ;  y1 = sgx(id)\h/2 - sgx(id)\h2/2 + sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h + 1
            ;  y2 = sgx(id)\h/2 - sgx(id)\h2/2 + sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h + sgx(id)\h2 - 2 - 2*sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h
            ;  LineXY(x1,y1,x2,y2,color1)
            ;  LineXY(x1,y2,x2,y1,color1)
            ;Else
            DrawImage(ImageID(sgx(id)\design\img[#sgx_DesignCheckboxChecked]\nr),  sgx(id)\w/2 - ImageWidth(sgx(id)\design\img[#sgx_DesignCheckboxChecked]\nr)/2,  sgx(id)\h/2 - ImageHeight(sgx(id)\design\img[#sgx_DesignCheckboxChecked]\nr)/2  )
            ;EndIf
          EndIf
          If sgx(id)\design\onEndDraw
            sgx(id)\design\onEndDraw(id, sprType, sgx(id)\w, sgx(id)\h)
          EndIf
        StopDrawing()
      EndIf
  EndSelect
EndProcedure


;** sgx_DrawScrollbar
;* redraws a particular state sprite of an sgxScrollbar/sgxTrackbar element (called by <a href=#f_sgxrefresh>sgxRefresh</a>).
;** .id      : ID of the sgx element
;** .sprType : the state / sprite type that should be redrawn (not all elements use every state): #sgx_GadgetSprNormal or #sgx_GadgetSprHover etc.
Procedure sgx_DrawScrollbar(id, sprType)
  Static appearance_fill
  Static appearance_border
  Static x, y
  Static sw, sh
  Static draw
  
  Select sprType
  Case #sgx_GadgetSprNormal, #sgx_GadgetSprNormal2
    appearance_fill = #sgx_AppearanceNormal
    appearance_border = #sgx_AppearanceNormal
  Case #sgx_GadgetSprHover, #sgx_GadgetSprHover2
    appearance_fill = #sgx_AppearanceHover
    appearance_border = #sgx_AppearanceHover
  Default
    ProcedureReturn
  EndSelect
  
  Select sprType
    Case #sgx_GadgetSprNormal, #sgx_GadgetSprHover
      ; bar
      sgx_prepareSprite(sgx(id)\spr[sprType], sgx(id)\w, sgx(id)\h, id)
      If sgx(id)\spr[sprType]
        sw = SpriteWidth(sgx(id)\spr[sprType])
        sh = SpriteHeight(sgx(id)\spr[sprType])
        StartDrawing(SpriteOutput(sgx(id)\spr[sprType]))
          DrawingMode(#PB_2DDrawing_AllChannels)
          Box(0,0,sgx(id)\w,sgx(id)\h, RGBA(0,0,0,0))
          draw = #sgxDrawDesign | #sgxDrawText
          If sgx(id)\design\onBeginDraw
            draw = sgx(id)\design\onBeginDraw(id, sprType, sgx(id)\w, sgx(id)\h)
          EndIf
          If draw & #sgxDrawDesign
            DrawingMode(#PB_2DDrawing_AlphaBlend)
            sgx_Fill( sgx(id)\design, sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w, sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h, sgx(id)\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer2]\w, sgx(id)\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h, appearance_fill)
            If sgx(id)\style & #sgxStyleGrid
              Select sgx(id)\type
                Case #sgxTypeScrollbarVer, #sgxTypeTrackbarVer
                  For y=0 To sgx(id)\h-2 Step 6
                    For x = 0 To sgx(id)\w-2
                      If x>=0 And x<sw And y>=0 And y<sh
                        Plot(x, y  , sgx_fade(Point(x, y  ),-200)|1 )
                      EndIf
                      If x>=0 And x<sw And (y+1)>=0 And (y+1)<sh
                        Plot(x, y+1, sgx_fade(Point(x, y+1),  50)|1 )
                      EndIf
                    Next
                  Next
                Case #sgxTypeScrollbarHor, #sgxTypeTrackbarHor
                  For x=0 To sgx(id)\w-2 Step 6
                    For y = 0 To sgx(id)\h-2
                      If x>=0 And x<sw And y>=0 And y<sh
                        Plot(x,  y, sgx_fade(Point(x,  y),-200)|1 )
                      EndIf
                      If (x+1)>=0 And (x+1)<sw And y>=0 And y<sh
                        Plot(x+1,y, sgx_fade(Point(x+1,y),  50)|1 )
                      EndIf
                    Next
                  Next
              EndSelect
            EndIf
            sgx_Borderbox(sgx(id)\design, 0,0, sgx(id)\w,sgx(id)\h, appearance_border)
          EndIf
          If sgx(id)\design\onEndDraw
            sgx(id)\design\onEndDraw(id, sprType, sgx(id)\w, sgx(id)\h)
          EndIf
        StopDrawing()
      EndIf
    Case #sgx_GadgetSprNormal2, #sgx_GadgetSprHover2
      ; knob
      sgx_prepareSprite(sgx(id)\spr[sprType], sgx(id)\w2, sgx(id)\h2, id)
      If sgx(id)\spr[sprType]
        StartDrawing(SpriteOutput(sgx(id)\spr[sprType]))
          DrawingMode(#PB_2DDrawing_AllChannels)
          Box(0,0,sgx(id)\w2,sgx(id)\h2, RGBA(0,0,0,0))
          draw = #sgxDrawDesign | #sgxDrawText
          If sgx(id)\design\onBeginDraw
            draw = sgx(id)\design\onBeginDraw(id, sprType, sgx(id)\w2, sgx(id)\h2)
          EndIf
          If draw & #sgxDrawDesign
            DrawingMode(#PB_2DDrawing_AlphaBlend)
            sgx_Fill( sgx(id)\design, sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w, sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h, sgx(id)\w2-sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer2]\w, sgx(id)\h2-sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h, appearance_fill)
            sgx_Borderbox(sgx(id)\design, 0,0, sgx(id)\w2,sgx(id)\h2, appearance_border)
          EndIf
          If sgx(id)\design\onEndDraw
            sgx(id)\design\onEndDraw(id, sprType, sgx(id)\w2, sgx(id)\h2)
          EndIf
        StopDrawing()
      EndIf
  EndSelect
EndProcedure


;** sgx_DrawEdit
;* redraws a particular state sprite of an sgxEdit element (called by <a href=#f_sgxrefresh>sgxRefresh</a>).
;** .id      : ID of the sgx element
;** .sprType : the state / sprite type that should be redrawn (not all elements use every state): #sgx_GadgetSprNormal or #sgx_GadgetSprHover etc.
;** .updateMetricsOnly : #True to update only metrics (calculate text offset and cursor position) and skip the actual drawing. #False (default) to also redraw.
;** .moveTextpos       : #False (default) to use current char pos (while typing / cursor keys). #True to move the character position to the current textcursor pixel position (while cursor positioning by mouse click)
;** .scrollToCursor    : #True (default) to scroll to current character/cursor position. #False while scrolling via mouse wheel.
Procedure sgx_DrawEdit(id, sprType, updateMetricsOnly=#False, moveTextpos=#False, scrollToCursor=#True)
  Static textCursorHeight
  Static color1, color2, color3, color4
  Static appearance_fill
  Static appearance_border
  Static tempImg
  Static linesAboveCursor, charsBeforeCursor, cursorX, cursorY
  Static i, textLen, lines, countChars, lineWidth, currentLine, line.s, skipTraversal
  Static *c.Character
  Static skipLines, useLines, startPos, endPos, check
  Static draw
  
  ; reuse tmp image if possible
  If  Not tempImg  Or  Not IsImage(tempImg)  Or  sgx(id)\w2 <> ImageWidth(tempImg)  Or  sgx(id)\h2 <> ImageHeight(tempImg)
    sgx_FreeImageChecked(tempImg)
    tempImg = CreateImage(#PB_Any, sgx(id)\w2, sgx(id)\h2, 32)
    If Not tempImg
      sgx_err("image creation failed with size "+Str(sgx(id)\w2)+", "+Str(sgx(id)\h2)+": sgx("+id+")")
    EndIf
  EndIf
  
  ; calculate text offset and cursor position
  If updateMetricsOnly
    
    If tempImg
      ; we only start drawing here for using TextHeight() and TextWidth()
      StartDrawing(ImageOutput(tempImg))
        DrawingFont(sgx(id)\textfontID)
        sgx(id)\texth = TextHeight("X")
        
        ; get cursor line and char pos
        
        linesAboveCursor = 0
        charsBeforeCursor = 0
        If moveTextpos=#False
          ; calculate the cursor pixel position (textcursor\x \y) from current character position (textpos)
          ; normal (i.e. typing and cursor keys)
          
          DisableDebugger
          If sgx(id)\textpos > 0
            ; go backwards in text from textpos and count the characters in it's line and count the lines above it
            *c = @sgx(id)\text + (sgx(id)\textpos-1) * SizeOf(Character)
            For i=sgx(id)\textpos-1 To 0 Step -1
              ;c = PeekC(@sgx(id)\text + i * SizeOf(Character))
              If *c\c = #LF
                linesAboveCursor + 1
              EndIf
              If linesAboveCursor = 0
                charsBeforeCursor + 1
              EndIf
              *c - SizeOf(Character)
            Next
          EndIf
          EnableDebugger
          
        Else
          ; calculate the character position (textpos) from current cursor pixel position (textcursor\x \y) 
          ; (i.e. cursor positioning by mouseclick)
          
          DisableDebugger
          textLen = Len(sgx(id)\text)
          countChars = #False
          lineWidth = 0
          linesAboveCursor = (sgx(id)\scrollpos + sgx(id)\textcursor\y) / sgx(id)\texth
          lines = CountString(sgx(id)\text, #LF$) + 1
          skipTraversal = #False
          If linesAboveCursor > (lines-1)
            ; if clicked in an empty space in the bottom part of an edit where no
            ; text might exist then we use the last line / end of text
            linesAboveCursor = lines - 1
            line = StringField(sgx(id)\text, lines, #LF$)
            charsBeforeCursor = Len(line)
            sgx(id)\textpos = textLen
            skipTraversal = #True
          EndIf
          
          If Not skipTraversal
            currentLine = 1
            *c = @sgx(id)\text
            For i=0 To textLen-1
              ;c = PeekC(@sgx(id)\text + i * SizeOf(Character))
              
              ; count lines
              If *c\c = #LF
                If countChars
                  ; stop if eol while counting chars (i.e. if clicked beyound the end of line)
                  Break
                Else
                  currentLine + 1
                EndIf
              EndIf
              
              ; count chars and sum up char width until intended x position is reached
              If countChars
                If *c\c <> #LF
                  lineWidth + TextWidth(Chr(*c\c))
                  If lineWidth < (sgx(id)\textalign + sgx(id)\textcursor\x)
                    charsBeforeCursor + 1
                  Else
                    ; reached intended x/y position
                    Break
                  EndIf
                Else
                  Break
                EndIf
              ElseIf (currentLine > linesAboveCursor)
                ; we reached the intended line. now we count characters from beginning to indented position or eol
                countChars = #True
                
                ; if first line is intended..
                If linesAboveCursor = 0
                  If *c\c = #LF
                    ; ..and is empty line then just stop
                    Break
                  Else
                    ; ..otherwise count the first char explicitly
                    lineWidth + TextWidth(Chr(*c\c))
                    If lineWidth < (sgx(id)\textalign + sgx(id)\textcursor\x)
                      charsBeforeCursor + 1
                    Else
                      ; reached intended x/y position (beginning of text)
                      Break
                    EndIf
                  EndIf
                EndIf
              EndIf
              *c + SizeOf(Character)
            Next
            sgx(id)\textpos = i
            EnableDebugger
          EndIf
        EndIf
        
        ; cursor pixel position in complete text
        cursorX = TextWidth(Mid(sgx(id)\text, 1 + sgx(id)\textpos - charsBeforeCursor, charsBeforeCursor))
        cursorY = linesAboveCursor * sgx(id)\texth
        
        ; move view (scroll) if cursor would be outside of client area.
        ; scrollToCursor is false when scrolling by mousewheel.
        If scrollToCursor
          ; left edge contact
          If cursorX < sgx(id)\textalign
            sgx(id)\textalign - (sgx(id)\textalign - cursorX)
          EndIf
          ; right edge contact
          If cursorX > (sgx(id)\textalign + sgx(id)\w2)
            sgx(id)\textalign + (cursorX - (sgx(id)\textalign + sgx(id)\w2))
          EndIf
          ; top edge contact
          If cursorY < sgx(id)\scrollpos
            sgx(id)\scrollpos - (sgx(id)\scrollpos - cursorY)
          EndIf
          ; bottom edge contact
          If cursorY > (sgx(id)\scrollpos + sgx(id)\h2 - sgx(id)\texth)
            sgx(id)\scrollpos + (cursorY - (sgx(id)\scrollpos + sgx(id)\h2 - sgx(id)\texth))
          EndIf
        EndIf
        
        ; translate in-text pixel position to in-clientarea pixel position
        sgx(id)\textcursor\x = cursorX - sgx(id)\textalign
        sgx(id)\textcursor\y = cursorY - sgx(id)\scrollpos
        
      StopDrawing()
    EndIf
    
  Else
    ; actual drawing (not updateMetricsOnly)
    
    Select sprType
    Case #sgx_GadgetSprNormal, #sgx_GadgetSprHover
  
      If sprType = #sgx_GadgetSprNormal
        color1 = sgx(id)\design\basecolor
        appearance_fill = #sgx_AppearanceNormal
        appearance_border = #sgx_AppearanceNormal
      ElseIf sprType = #sgx_GadgetSprHover
        color1 = sgx(id)\design\c[09]
        appearance_fill = #sgx_AppearanceHover
        appearance_border = #sgx_AppearanceHover
      EndIf
      
      ; text will be drawn on temporary image first for clipping to client area
      If tempImg
        StartDrawing(ImageOutput(tempImg))
          DrawingMode(#PB_2DDrawing_AllChannels)
          Box(0,0,sgx(id)\w,sgx(id)\h, RGBA(0,0,0,0))
          DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Transparent)
          DrawingFont(sgx(id)\textfontID)
          
          ; get lines visible inside clientarea and get the start/end
          ; character position in text to draw only non-clipped text
          
          DisableDebugger
          skipLines = sgx(id)\scrollpos / sgx(id)\texth
          useLines = sgx(id)\h2 / sgx(id)\texth + 3
          textLen = Len(sgx(id)\text)
          currentLine = 1
          startPos = -1
          endPos = -1
          If skipLines = 0
            startPos = 0
          EndIf
          
          check = #True
          
          *c = @sgx(id)\text
          For i=0 To textLen-1
            ;c = PeekC(@sgx(id)\text + i * SizeOf(Character))
            
            ; check only once per line
            If check
              check = #False
              ; get start character position if not done yet and unnecessary lines have been skipped
              If startPos = -1 And currentLine > skipLines
                startPos = i
              EndIf
              If startPos >= 0 And currentLine > (skipLines + useLines)
                ; get end character position (outside the loop after 'Next' in case we never get here)
                Break
              EndIf
            EndIf
            If *c\c = #LF
              currentLine + 1
              check = #True
            EndIf
            *c + SizeOf(Character)
          Next
          endpos = i + 1
          
          ; draw only visible part of text but add y-offset for the skipped lines
          sgx_DrawTextLF( -sgx(id)\textalign, -(sgx(id)\scrollpos - (skipLines) * sgx(id)\texth), Mid(sgx(id)\text, startPos+1, endPos-startPos), color1 )
          EnableDebugger
          
        StopDrawing()
      EndIf
      
      sgx_prepareSprite(sgx(id)\spr[sprType], sgx(id)\w, sgx(id)\h, id)
      If sgx(id)\spr[sprType]
        StartDrawing(SpriteOutput(sgx(id)\spr[sprType]))
          DrawingMode(#PB_2DDrawing_AllChannels)
          Box(0,0,sgx(id)\w,sgx(id)\h, RGBA(0,0,0,0))
          draw = #sgxDrawDesign | #sgxDrawText
          If sgx(id)\design\onBeginDraw
            draw = sgx(id)\design\onBeginDraw(id, sprType, sgx(id)\w, sgx(id)\h)
          EndIf
          If draw & #sgxDrawDesign
            DrawingMode(#PB_2DDrawing_AlphaBlend)
            sgx_Fill( sgx(id)\design, sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w, sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h, sgx(id)\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer2]\w, sgx(id)\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h, appearance_fill)
          EndIf
          
          If draw & (#sgxDrawText | #sgxDrawTextNoOffset)
            ; draw our client area tempImage (the visible text) onto the sprite
            If tempImg
              DrawImage(ImageID(tempImg), sgx(id)\x2, sgx(id)\y2, sgx(id)\w2, sgx(id)\h2)
            EndIf
          EndIf
          
          If draw & #sgxDrawDesign
            sgx_Borderbox(sgx(id)\design, 0,0, sgx(id)\w,sgx(id)\h, appearance_border)
          EndIf
          
          If sgx(id)\design\onEndDraw
            sgx(id)\design\onEndDraw(id, sprType, sgx(id)\w, sgx(id)\h)
          EndIf
        StopDrawing()
      EndIf
      
    Case #sgx_GadgetSprTextCursor
      
      ; textcursors are hardcoded / not a design issue
      
      color1 = sgx(id)\design\c[09]
      color2 = sgx(id)\design\basecolor
      color3 = sgx(id)\design\c[05]
      color4 = sgx(id)\design\c[11]
      textCursorHeight = 1
      If sgx(id)\texth > 0
        textCursorHeight = sgx(id)\texth
      EndIf
      sgx_prepareSprite(sgx(id)\spr[sprType], 3, textCursorHeight, id)
      If sgx(id)\spr[sprType]
        StartDrawing(SpriteOutput(sgx(id)\spr[sprType]))
          DrawingMode(#PB_2DDrawing_AllChannels)
          Box(0,0,3,textCursorHeight, RGBA(0,0,0,0))
          draw = #sgxDrawDesign | #sgxDrawText
          If sgx(id)\design\onBeginDraw
            draw = sgx(id)\design\onBeginDraw(id, sprType, 3, textCursorHeight)
          EndIf
          If draw & #sgxDrawDesign
            DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Outlined)
            Select sgx_textcursorType
              Case #sgxTextcursorTypeIBeam
                LineXY(0,0,0,0, color2)
                LineXY(1,0,1,0, color3)
                LineXY(2,0,2,0, color2)
                LineXY(0,textCursorHeight-1,0,textCursorHeight-1, color2)
                LineXY(1,textCursorHeight-1,1,textCursorHeight-1, color3)
                LineXY(2,textCursorHeight-1,2,textCursorHeight-1, color2)
                LineXY(1,1,1,textCursorHeight-2, color1)
              Case #sgxTextcursorTypeTeeth
                LineXY(0,0,2,0, color2)
                LineXY(0,textCursorHeight-1,2,textCursorHeight-1, color2)
                LineXY(1,0,1,2, color1)
                LineXY(1,textCursorHeight-3,1,textCursorHeight-1, color1)
                LineXY(0,1,0,1, color3)
                LineXY(2,1,2,1, color3)
                LineXY(0,textCursorHeight-2,0,textCursorHeight-2, color3)
                LineXY(2,textCursorHeight-2,2,textCursorHeight-2, color3)
              Case #sgxTextcursorTypeLine
                LineXY(1,0,1,textCursorHeight-1, color1)
              Case #sgxTextcursorTypeLine2
                LineXY(0,0,0,textCursorHeight-1, color1)
                LineXY(1,0,1,textCursorHeight-1, color1)
              Case #sgxTextcursorTypeLine3
                LineXY(0,0,0,textCursorHeight-1, RGBA(Red(color1),Green(color1),Blue(color1),Alpha(color1)*0.4))
                LineXY(1,0,1,textCursorHeight-1, RGBA(Red(color4),Green(color4),Blue(color4),Alpha(color4)*0.5))
                LineXY(2,0,2,textCursorHeight-1, RGBA(Red(color1),Green(color1),Blue(color1),Alpha(color1)*0.4))
              Case #sgxTextcursorTypeCaret
                LineXY(0,textCursorHeight-1,0,textCursorHeight-1, color1)
                LineXY(1,textCursorHeight-2,1,textCursorHeight-2, color1)
                LineXY(2,textCursorHeight-1,2,textCursorHeight-1, color1)
                LineXY(1,textCursorHeight-1,1,textCursorHeight-1, color3)
            EndSelect
          EndIf
          If sgx(id)\design\onEndDraw
            sgx(id)\design\onEndDraw(id, sprType, 3, textCursorHeight)
          EndIf
        StopDrawing()
      EndIf
    EndSelect
  EndIf
EndProcedure


;** sgx_DrawWin
;* redraws a particular state sprite of an sgxWin/sgxTable element (called by <a href=#f_sgxrefresh>sgxRefresh</a>).
;** .id      : ID of the sgx element
;** .sprType : the state / sprite type that should be redrawn (not all elements use every state): #sgx_GadgetSprNormal or #sgx_GadgetSprHover etc.
Procedure sgx_DrawWin(id, sprType)
  Static color1
  Static color2
  Static color3
  Static color4
  Static color5
  Static color6
  Static appearance_fill
  Static appearance_border
  Static margin = 4   ; (for minimized)
  Static i, k, x, y
  Static *c.Character, textLen, width, maxWidth, title.s
  Static text.s
  Static draw
  
  ;{ maximized
  If sgx(id)\state <> #sgxWinStateMinimized
    Select sprType
    Case #sgx_GadgetSprNormal
      color1 = sgx(id)\design\basecolor
      color2 = sgx(id)\design\c[01]
      color5 = sgx(id)\design\c[16]
      color6 = sgx(id)\design\c[17]
      appearance_fill = #sgx_AppearanceNormal
      appearance_border = #sgx_AppearanceNormal
    Case #sgx_GadgetSprHover
      color1 = sgx(id)\design\c[09]
      color2 = sgx(id)\design\c[01]
      color5 = sgx(id)\design\c[18]
      color6 = sgx(id)\design\c[19]
      appearance_fill = #sgx_AppearanceNormal
      If sgx(id)\type = #sgxTypeTable
        appearance_fill = #sgx_AppearanceHover
      EndIf
      appearance_border = #sgx_AppearanceHover
    Case #sgx_GadgetSprPushed
      color1 = sgx(id)\design\c[09]
      color2 = sgx(id)\design\c[01]
      color5 = sgx(id)\design\c[18]
      color6 = sgx(id)\design\c[19]
      appearance_fill = #sgx_AppearanceHover
      appearance_border = #sgx_AppearanceHover
    EndSelect
    
    Select sprType
    Case #sgx_GadgetSprNormal,  #sgx_GadgetSprHover,  #sgx_GadgetSprPushed
      
      If sgx(id)\type = #sgxTypeTable And sgx(id)\style & #sgxStyleNoBG
        ; no background sprites needed
      Else
        
        sgx_prepareSprite(sgx(id)\spr[sprType], sgx(id)\w, sgx(id)\h, id)
        If sgx(id)\spr[sprType]
          StartDrawing(SpriteOutput(sgx(id)\spr[sprType]))
            DrawingMode(#PB_2DDrawing_AllChannels)
            Box(0,0,sgx(id)\w,sgx(id)\h, RGBA(0,0,0,0))
            draw = #sgxDrawDesign | #sgxDrawText
            If sgx(id)\design\onBeginDraw
              draw = sgx(id)\design\onBeginDraw(id, sprType, sgx(id)\w, sgx(id)\h)
            EndIf
            If draw & #sgxDrawDesign
              DrawingMode(#PB_2DDrawing_AlphaBlend)
              
              sgx_Fill( sgx(id)\design, sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w, sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h, sgx(id)\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer2]\w, sgx(id)\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h, appearance_fill)
              
              ;  ------------------------------
              ;{ draw grid (tables only)
              
              If sgx(id)\type = #sgxTypeTable  And  Not sgx(id)\style & #sgxStyleNoBG
                
                ; ver lines |
                
                If sgx(id)\style & #sgxStyleGridV
                  For i=0 To sgx(id)\table\w-2
                    x = PeekL( sgx(id)\table\gridx + i*SizeOf(Long) )
                    ; don't draw useless grid lines for empty table columns
                    ;If x
                    ;  LineXY(x  ,0, x  ,sgx(id)\h-1, color5)
                    ;  LineXY(x+1,0, x+1,sgx(id)\h-1, color6)
                    ;EndIf
                    ; don't draw useless grid lines for empty table columns
                    If x And x < sgx(id)\w
                      For y=0 To sgx(id)\h-1
                        ;If ..check bounds
                        Plot(x  , y, sgx_fade(Point(x  , y),-200)|1 )
                        Plot(x+1, y, sgx_fade(Point(x+1, y),  50)|1 )
                      Next
                    EndIf
                  Next
                EndIf
                
                ; hor lines --
                
                If sgx(id)\style & #sgxStyleGridH
                  For k=0 To sgx(id)\table\h-2
                    y = PeekL( sgx(id)\table\gridy + k*SizeOf(Long) )
                    ;; don't draw useless grid lines for empty table lines/rows
                    ;If y
                    ;  LineXY(0,y  , sgx(id)\w-1,y  , color5)
                    ;  LineXY(0,y+1, sgx(id)\w-1,y+1, color6)
                    ;EndIf
                    ; don't draw useless grid lines for empty table lines/rows
                    If y And y < sgx(id)\h
                      For x=0 To sgx(id)\w-1
                        ;If ..check bounds
                        Plot(x, y  , sgx_fade(Point(x, y  ),-200)|1 )
                        Plot(x, y+1, sgx_fade(Point(x, y+1),  50)|1 )
                      Next
                    EndIf
                  Next
                EndIf
              EndIf
              ;}
              ; ------------------------------
              
              ; separator below the titlebar
              If sgx(id)\width
                ; heeere (win line)
                sgx_BorderLine(sgx(id)\design, 0,sgx(id)\width-sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h, sgx(id)\w, appearance_border, #sgx_SideBottomOrRight)
              EndIf
              
              ; window border
              sgx_Borderbox(sgx(id)\design, 0,0, sgx(id)\w, sgx(id)\h, appearance_border)
            EndIf
            
            If draw & (#sgxDrawText | #sgxDrawTextNoOffset)
              ; draw title if window has a titlebar
              If sgx(id)\width
                DrawingFont(sgx(id)\textfontID)
                DrawingMode(#PB_2DDrawing_AlphaBlend |#PB_2DDrawing_Transparent)
                ; this means: maxWidth = xBeginOfCloseGadget - xBeginningOfText
                maxWidth = (sgx(id)\sys0\x1 - (sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w+1) - TextWidth("..") - 1)
                width = 0
                textLen = Len(sgx(id)\text)
                *c = @sgx(id)\text
                ; detect if title is to long
                For i=0 To textLen-1
                  width + TextWidth(Chr(*c\c))
                  If width > maxWidth
                    Break
                  EndIf
                  *c + SizeOf(Character)
                Next
                ; shorten title if too long
                title = Mid(sgx(id)\text,1,i)
                If i < textLen
                  title + ".."
                EndIf
                sgx_DrawTextLF( sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w+1,sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h+1, title, color1 )
              EndIf
            EndIf
            
            If sgx(id)\design\onEndDraw
              sgx(id)\design\onEndDraw(id, sprType, sgx(id)\w, sgx(id)\h)
            EndIf
          StopDrawing()
        EndIf
      EndIf
      
      ;close gadget
      Case #sgx_GadgetSprNormal2
        sgx_FreeSpriteChecked(sgx(id)\spr[sprType])
        sgx(id)\spr[sprType] = sgx_CopyImageToSprite(sgx(id)\design\img[#sgx_DesignCloseGadgetNormal]\nr, sgx(id)\design\onBeginDraw, sgx(id)\design\onEndDraw, id, sprType)
      Case #sgx_GadgetSprHover2
        sgx_FreeSpriteChecked(sgx(id)\spr[sprType])
        sgx(id)\spr[sprType] = sgx_CopyImageToSprite(sgx(id)\design\img[#sgx_DesignCloseGadgetHover]\nr, sgx(id)\design\onBeginDraw, sgx(id)\design\onEndDraw, id, sprType)
  
    EndSelect
  ;}
  
  
  ;{ minimized (draw as taskbar buttons)
  Else
    Select sprType
    Case #sgx_GadgetSprNormal
      color1 = sgx(id)\design\basecolor
      color2 = sgx(id)\design\c[05]
      color3 = sgx(id)\design\c[03]
      color4 = sgx(id)\design\c[02]
      appearance_fill = #sgx_AppearanceNormal
      appearance_border = #sgx_AppearanceNormal
    Case #sgx_GadgetSprHover
      color1 = sgx(id)\design\c[09]
      color2 = sgx(id)\design\c[01]
      color3 = sgx(id)\design\c[01]
      color4 = sgx(id)\design\c[03]
      appearance_fill = #sgx_AppearanceHover
      appearance_border = #sgx_AppearanceHover
    EndSelect
    
    sgx_prepareSprite(sgx(id)\spr[sprType], sgx(id)\w, sgx(id)\h, id)
    If sgx(id)\spr[sprType]
      StartDrawing(SpriteOutput(sgx(id)\spr[sprType]))
        DrawingMode(#PB_2DDrawing_AllChannels)
        Box(0,0,sgx(id)\w,sgx(id)\h, RGBA(0,0,0,0))
        draw = #sgxDrawDesign | #sgxDrawText
        If sgx(id)\design\onBeginDraw
          draw = sgx(id)\design\onBeginDraw(id, sprType, sgx(id)\w, sgx(id)\h)
        EndIf
        If draw & #sgxDrawDesign
          sgx_Fill( sgx(id)\design, sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w, sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h, sgx(id)\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer2]\w, sgx(id)\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h, appearance_fill)
          sgx_Borderbox(sgx(id)\design, 0,0, sgx(id)\w, sgx(id)\h, appearance_border)
        EndIf
        If draw & (#sgxDrawText | #sgxDrawTextNoOffset)
          DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Outlined | #PB_2DDrawing_Transparent)
          ; use a uniform font for all taskbar buttons
          DrawingFont(FontID(sgxStdFont))
          ; shorten title if too long
          If TextWidth(sgx(id)\text) < sgx(id)\w-margin
            text = sgx(id)\text
          Else
            i=2
            While TextWidth( Left(sgx(id)\text,i)+".." ) < sgx(id)\w-margin
              i+1
            Wend
            text = Left(sgx(id)\text,i-1)+".."
          EndIf
          sgx_DrawTextLF( 3,3, text, color1 )
        EndIf
        If sgx(id)\design\onEndDraw
          sgx(id)\design\onEndDraw(id, sprType, sgx(id)\w, sgx(id)\h)
        EndIf
      StopDrawing()
    EndIf
  EndIf ;}

EndProcedure


;** sgx_DrawTip
;* redraws the tooltip sprite of an sgx element (called by <a href=#f_sgxrefresh>sgxRefresh</a>).
;** .id      : ID of the sgx element
Procedure sgx_DrawTip(id)
  Protected i
  Protected texth
  Protected w
  Protected h
  Protected temp
  Protected max
  Protected margin = 3
  Protected draw

  StartDrawing(ScreenOutput())
    DrawingFont(sgx(id)\textfontID)
    texth = TextHeight(sgx(id)\tip\text)
    max = CountString(sgx(id)\tip\text,#LF$)+1
    For i=1 To max
      temp = TextWidth(StringField(sgx(id)\tip\text,i,#LF$))
      If temp > w
        w = temp
      EndIf
      h + texth
    Next
  StopDrawing()
  w + 2*margin
  h + 2*margin
  ; the tip has its own sprite, wich is not part of the sgx(id)\spr[sprType] array
  sgx_prepareSprite(sgx(id)\tip\spr, w, h, id)
  If sgx(id)\tip\spr
    StartDrawing(SpriteOutput(sgx(id)\tip\spr))
      DrawingMode(#PB_2DDrawing_AllChannels)
      Box(0,0,w,h, RGBA(0,0,0,0))
      draw = #sgxDrawDesign | #sgxDrawText
      If sgx(id)\design\onBeginDraw
        draw = sgx(id)\design\onBeginDraw(id, #sgx_GadgetSprTip, w, h)
      EndIf
      If draw & #sgxDrawDesign
        DrawingMode(#PB_2DDrawing_AlphaBlend)
        ;Box(0,0, w,h, sgx(id)\design\c[01])
        sgx_Fill( sgx(id)\design, 2,2, w-4,h-4, #sgx_AppearanceNormal)
        DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Outlined)
        Box(0,0, w,h, sgx(id)\design\c[03])
        Box(1,1, w-2,h-2, sgx(id)\design\c[02])
      EndIf
      If draw & (#sgxDrawText | #sgxDrawTextNoOffset)
        DrawingFont(sgx(id)\textfontID)
        DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Transparent)
        sgx_DrawTextLF(margin,margin,sgx(id)\tip\text,sgx(id)\design\basecolor)
      EndIf
      If sgx(id)\design\onEndDraw
        sgx(id)\design\onEndDraw(id, #sgx_GadgetSprTip, w, h)
      EndIf
    StopDrawing()
  EndIf
  sgx(id)\tip\w = w
  sgx(id)\tip\h = h
EndProcedure


;** sgx_DrawResizeGadget
;* creates the resize gadget sprite of an sgxWin element if it does not exist yet. (called by <a href=#f_sgxrefresh>sgxRefresh</a>).
;** .id      : ID of the sgx element
Procedure sgx_DrawResizeGadget(id)
  Static color1, color2, i, w, borderBottom, borderRight, border
  Static draw
  color1 = sgx(id)\design\basecolor
  color1 = RGBA($aa,$aa,$aa,$aa)
  color1 = $fffffffa
  color2 = $ff00000a
  color1 = sgx(id)\design\basecolor
  color2 = sgx(id)\design\c[01]
  w = sgx_resizeGadgetSize
  sgx_prepareSprite(sgx(id)\spr[#sgx_GadgetSprResize], w, w, id)
  If sgx(id)\spr[#sgx_GadgetSprResize]
    TransparentSpriteColor(sgx(id)\spr[#sgx_GadgetSprResize], RGBA(0,0,0,0))
    StartDrawing(SpriteOutput(sgx(id)\spr[#sgx_GadgetSprResize]))
      DrawingMode(#PB_2DDrawing_AllChannels)
      Box(0,0, w,w, RGBA(0,0,0,0))
      draw = #sgxDrawDesign | #sgxDrawText
      If sgx(id)\design\onBeginDraw
        draw = sgx(id)\design\onBeginDraw(id, #sgx_GadgetSprResize, w, w)
      EndIf
      If draw & #sgxDrawDesign
        DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Outlined | #PB_2DDrawing_Transparent)
        borderBottom = sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h
        borderRight  = sgx(id)\design\img[#sgx_DesignBorderNormalVer2]\w
        border = (borderBottom + borderRight) / 2
        For i=border To w-1 Step 4
          LineXY(i+0, w-border,  w-border, i+0, color1)
          LineXY(i+1, w-border,  w-border, i+1, color1)
          LineXY(i+2, w-border,  w-border, i+2, color2)
          LineXY(i+3, w-border,  w-border, i+3, color2)
        Next
        ; triangle
        ;color1 = $ffffffff
        ;LineXY(0,w, w,w, color1)
        ;LineXY(w,0, w,w, color1)
        ;LineXY(0,w, w,0, color1)
      EndIf
      If sgx(id)\design\onEndDraw
        sgx(id)\design\onEndDraw(id, #sgx_GadgetSprResize, w, w)
      EndIf
    StopDrawing()
  EndIf
EndProcedure


;** sgx_DrawCombo
;* redraws a particular state sprite of an sgxCombo element (called by <a href=#f_sgxrefresh>sgxRefresh</a>).
;** .id      : ID of the sgx element
;** .sprType : the state / sprite type that should be redrawn (not all elements use every state): #sgx_GadgetSprNormal or #sgx_GadgetSprHover etc.
Procedure sgx_DrawCombo(id, sprType)
  Static color1
  Static color2
  Static appearance_fill
  Static appearance_border
  Static arrowOffset
  Static x, y, i, k
  Static count
  Static h
  Static sw,sh,_x,_y,_c
  Static draw
  
  ; line height
  If sgx(id)\max=1
    h = sgx(id)\h2
  Else
    h = sgx(id)\h
  EndIf
  
  Select sprType
  Case #sgx_GadgetSprNormal
    color1 = sgx(id)\design\basecolor
    color2 = sgx(id)\design\c[01]
    appearance_fill = #sgx_AppearanceNormal
    appearance_border = #sgx_AppearanceNormal
    arrowOffset = 0
    count=1
  Case #sgx_GadgetSprHover
    color1 = sgx(id)\design\c[09]
    color2 = sgx(id)\design\c[01]
    appearance_fill = #sgx_AppearanceHover
    appearance_border = #sgx_AppearanceHover
    arrowOffset = 0
    count=1
  Case #sgx_GadgetSprPushed ; (the opened combo)
    color1 = sgx(id)\design\c[03]
    color2 = $0
    appearance_fill = #sgx_AppearanceNormal
    appearance_border = #sgx_AppearancePushed
    arrowOffset = 1
    count = CountString(sgx(id)\text,#LF$) + 2  ; (there is one more line/entry than number of linebreaks, plus 1 for displaying the currently selected entry)
  Case #sgx_GadgetSprHover2
    ; (the hover effect when combo is open)
    sgx_prepareSprite(sgx(id)\spr[sprType], sgx(id)\w, h+1, id)
    If sgx(id)\spr[sprType]
      StartDrawing(SpriteOutput(sgx(id)\spr[sprType]))
        DrawingMode(#PB_2DDrawing_AllChannels)
        Box(0,0,sgx(id)\w, h+1, RGBA(0,0,0,0))
        draw = #sgxDrawDesign | #sgxDrawText
        If sgx(id)\design\onBeginDraw
          draw = sgx(id)\design\onBeginDraw(id, sprType, sgx(id)\w, h+1)
        EndIf
        If draw & #sgxDrawDesign
          DrawingMode(#PB_2DDrawing_AlphaBlend)
          sgx_Borderbox(sgx(id)\design, 0,0, sgx(id)\w-1, h+1, #sgx_AppearanceHover)
          Box(0,0,sgx(id)\w-1, h+1, RGBA(0,0,0,0))
        EndIf
        If sgx(id)\design\onEndDraw
          sgx(id)\design\onEndDraw(id, sprType, sgx(id)\w, h+1)
        EndIf
      StopDrawing()
    EndIf
    ProcedureReturn
    
  Default
    ProcedureReturn
  EndSelect
  
  ; (normal, hover and pushed/open)
  sgx_prepareSprite(sgx(id)\spr[sprType], sgx(id)\w, h*count, id)
  If sgx(id)\spr[sprType]
    sw = SpriteWidth(sgx(id)\spr[sprType])
    sh = SpriteHeight(sgx(id)\spr[sprType])
    StartDrawing(SpriteOutput(sgx(id)\spr[sprType]))
      DrawingMode(#PB_2DDrawing_AllChannels)
      Box(0,0,sgx(id)\w, h*count, RGBA(0,0,0,0))
      draw = #sgxDrawDesign | #sgxDrawText
      If sgx(id)\design\onBeginDraw
        draw = sgx(id)\design\onBeginDraw(id, sprType, sgx(id)\w, h*count)
      EndIf
      If draw & #sgxDrawDesign
        DrawingMode(#PB_2DDrawing_AlphaBlend)
        sgx_Fill( sgx(id)\design, sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w, sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h, sgx(id)\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer2]\w, h*count-sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h, appearance_fill)
      EndIf
      If draw & (#sgxDrawText | #sgxDrawTextNoOffset)
        DrawingFont(sgx(id)\textfontID)
        DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Transparent)
      EndIf
      
      ; draw the selected entry and all possible entries
      For i=0 To count-1
        If i=0
          If draw & (#sgxDrawText | #sgxDrawTextNoOffset)
            ; draw the selected entry
            DrawText(  sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w+1,  h*i+(h-sgx(id)\texth)/2,  StringField(sgx(id)\text,sgx(id)\state,#LF$),  color1 )
          EndIf
        Else
          If draw & #sgxDrawDesign
            ; draw horizontal separator lines
            For x = 0 To sgx(id)\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w-1
              _y = h*i
              If x>=0 And x<sw And _y>=0 And _y<sh
                Plot(x, _y, sgx_fade(Point(x, _y),-200)|1 )
              EndIf
              _y = h*i+1
              If x>=0 And x<sw And _y>=0 And _y<sh
                Plot(x, _y, sgx_fade(Point(x, _y),  50)|1 )
              EndIf
            Next
          EndIf
          If draw & (#sgxDrawText | #sgxDrawTextNoOffset)
            ; draw possible entries (i = 1..count-1)
            DrawText(  sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w+1,  h*i+(h-sgx(id)\texth)/2,  StringField(sgx(id)\text,i,#LF$),  color1 )
          EndIf
        EndIf
      Next
      If draw & #sgxDrawDesign
        sgx_Borderbox(sgx(id)\design, 0,0, sgx(id)\w, h*count, appearance_border)
        
        ; the 'open' arrow
        x = sgx(id)\w - h/2 - 8
        y = h/2 - 3
        For i=0 To 5
          For k=0 To 12
            If i And i>=k
              _x = arrowOffset+x+i
              _y = arrowOffset+y+k
              If _x>=0 And _x<sw And _y>=0 And _y<sh
                Plot(_x ,_y, sgx_fade(Point(_x ,_y),-250+arrowOffset*350)|1 )
              EndIf
              _x = arrowOffset+x+11-i
              _y = arrowOffset+y+k
              If _x>=0 And _x<sw And _y>=0 And _y<sh
                Plot(_x ,_y, sgx_fade(Point(_x ,_y),-250+arrowOffset*350)|1 )
              EndIf
            EndIf
          Next
        Next
      EndIf
      
      If sgx(id)\design\onEndDraw
        sgx(id)\design\onEndDraw(id, sprType, sgx(id)\w, h*count)
      EndIf
    StopDrawing()
  EndIf
EndProcedure


;** sgx_DrawText
;* redraws a particular state sprite of an sgxText element (called by <a href=#f_sgxrefresh>sgxRefresh</a>).
;** .id      : ID of the sgx element
;** .sprType : the state / sprite type that should be redrawn (not all elements use every state): #sgx_GadgetSprNormal or #sgx_GadgetSprHover etc.
Procedure sgx_DrawText(id, sprType)
  Static color1, color2, color3, color4
  Static appearance_fill, appearance_border
  Static w, h, margin
  Static draw

  Select sprType
  Case #sgx_GadgetSprNormal
    color1 = sgx(id)\design\basecolor
    color2 = sgx(id)\design\c[01]
    color3 = sgx(id)\design\c[03]
    color4 = sgx(id)\design\c[02]
    appearance_fill = #sgx_AppearanceNormal
    appearance_border = #sgx_AppearanceNormal
  Case #sgx_GadgetSprHover
    color1 = sgx(id)\design\c[09]
    color2 = sgx(id)\design\c[01]
    color3 = sgx(id)\design\c[05]
    color4 = sgx(id)\design\c[03]
    appearance_fill = #sgx_AppearanceHover
    appearance_border = #sgx_AppearanceHover
  Default
    ProcedureReturn
  EndSelect
  
  w = sgx(id)\w
  h = sgx(id)\h
  margin = sgx(id)\margin
  
  sgx_prepareSprite(sgx(id)\spr[sprType], w, h, id)
  If sgx(id)\spr[sprType]
    StartDrawing(SpriteOutput(sgx(id)\spr[sprType]))
      DrawingMode(#PB_2DDrawing_AllChannels)
      Box(0,0,w,h, RGBA(0,0,0,0))
      draw = #sgxDrawDesign | #sgxDrawText
      If sgx(id)\design\onBeginDraw
        draw = sgx(id)\design\onBeginDraw(id, sprType, w, h)
      EndIf
      If draw & #sgxDrawDesign
      DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Outlined)
        If sgx(id)\style & #sgxStyleThin 
          sgx_Fill( sgx(id)\design, 2,2, w-4,h-4, appearance_fill)
          ; heeere (text border)
          DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Outlined)
          Box(0,0, w,h, color3)
          Box(1,1, w-2,h-2, color4)
        EndIf
      EndIf
      If draw & (#sgxDrawText | #sgxDrawTextNoOffset)
        DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Transparent)
        DrawingFont(sgx(id)\textfontID)
        Select sgx(id)\textalign
          Case #sgxAlignTextLeft
            sgx_DrawTextLF(                 margin, margin, sgx(id)\text, color1 )
          Case #sgxAlignTextRight
            sgx_DrawTextLF( w-margin-sgx(id)\textw, margin, sgx(id)\text, color1, -1, #LF$, #sgxAlignTextRight )
          Case #sgxAlignTextCenter
            sgx_DrawTextLF( (w-sgx(id)\textw)/2,    margin, sgx(id)\text, color1, -1, #LF$, #sgxAlignTextCenter )
          Case #sgxAlignTextJustifySpaces
            sgx_DrawTextLF(                 margin, margin, sgx(id)\text, color1, -1, #LF$, #sgxAlignTextJustifySpaces )
          Case #sgxAlignTextJustifyCharacters
            sgx_DrawTextLF(                 margin, margin, sgx(id)\text, color1, -1, #LF$, #sgxAlignTextJustifyCharacters )
        EndSelect
      EndIf
      
      If sgx(id)\design\onEndDraw
        sgx(id)\design\onEndDraw(id, sprType, w, h)
      EndIf
    StopDrawing()
  EndIf
  
  ;sgx_Fill( sgx(id)\design, 0,0, sgx(id)\w, sgx(id)\h, appearance_fill)
  ;DrawingFont(sgx(id)\textfontID)
  ;DrawingMode(#PB_2DDrawing_Transparent|#PB_2DDrawing_Outlined)
  ;Box(0,0, sgx(id)\w, sgx(id)\h, $555555)
  ;DrawText(1,1, sgx(id)\text, color1)
EndProcedure


;** sgx_DrawDisabled
;* modifies a particular state sprite of an sgx element to visually represent a 'disabled' state (called by <a href=#f_sgxrefresh>sgxRefresh</a>).
;** .id      : ID of the sgx element
;** .sprType : the state / sprite type that should be modified (not all states are modified to 'disabled'): #sgx_GadgetSprNormal or #sgx_GadgetSprPushed etc.
Procedure sgx_DrawDisabled(id, sprType)
  Static w, h, x, y
  Static c, r,g,b,a, mean
  Static draw
  
  If sgx(id)\disabled  And  sgx(id)\spr[sprType]  And  IsSprite(sgx(id)\spr[sprType])
    w = SpriteWidth(sgx(id)\spr[sprType])
    h = SpriteHeight(sgx(id)\spr[sprType])
    ; reduce saturation and brightness of 'normal' sprite
    StartDrawing(SpriteOutput(sgx(id)\spr[sprType]))
      draw = #sgxDrawDesign | #sgxDrawText
      If sgx(id)\design\onBeginDraw
        draw = sgx(id)\design\onBeginDraw(id, sprType, w, h, #True)
      EndIf
      If draw & #sgxDrawDesign
        ; using alpha clip to keep transparent areas transparent (i.e. don't dim checkbox label background)
        DrawingMode(#PB_2DDrawing_Default | #PB_2DDrawing_AlphaClip)
        Box(0,0,w,h,sgx_disabledColor1)
        DrawingMode(#PB_2DDrawing_Default | #PB_2DDrawing_AlphaClip)
        Box(0,0,w,h,sgx_disabledColor2)
        
        ; (older method, slower)
        ; For x=0 To w-1
        ;   For y=0 To h-1
        ;     c = Point(x,y)
        ;     r = Red(c)
        ;     g = Green(c)
        ;     b = Blue(c)
        ;     a = Alpha(c)
        ;     ; reduce brightness
        ;     r * 0.8
        ;     g * 0.8
        ;     b * 0.8
        ;     ; get components mean brightness
        ;     mean = (r+g+b)/3
        ;     ; use mean between reduced component and components mean
        ;     r = (r + mean) / 2
        ;     g = (g + mean) / 2
        ;     b = (b + mean) / 2
        ;     Plot(x,y,RGBA(r,g,b,a))
        ;   Next
        ; Next
      EndIf
      If sgx(id)\design\onEndDraw
        sgx(id)\design\onEndDraw(id, sprType, w, h, #True)
      EndIf
    StopDrawing()
  EndIf
EndProcedure


















; IDE Options = PureBasic 5.50 beta 1 (Windows - x64)
; CursorPosition = 54
; FirstLine = 433
; Folding = 0-A+--
; Markers = 497,731,1584,2002
; UseMainFile = demo.pb
; CompileSourceDirectory