
; Macro MouseWheel()
;   Bool(KeyboardPushed(#PB_Key_LeftControl)) * -sgx_m\dy
; EndMacro

; Procedure sgx_windowCallback(hWnd, uMsg, WParam, LParam)
;   Static wd.w
;   If uMsg = #WM_MOUSEWHEEL
;     wd.w = - ((WParam>>16) & $ffff)
;     wd.w / 120
;     sgx_mouseWheelDelta - wd
;   EndIf 
;   ProcedureReturn #PB_ProcessPureBasicEvents 
; EndProcedure 

;SetWindowCallback(@sgx_windowCallback())



;** sgxExamine
;* process actions / events and update the field data of all sgx element. _
;* it returns the ID of the sgx element that is hovered by the mouse, or -1 if the mouse isn't hovering an element. _
;* you can pass mouse coordinates if needed (for instance if you have self-calculated accelerated mouse movement). otherwise sgx will simply inquire the PB mouse library commands.
;** .mousex : (optional)
;** .mousey : (optional)
Procedure sgxExamine( mousex=-1, mousey=-1)
  Static id, id2, parent, temp
  Static topID
  Static group
  Static emsPrev
  Static i, x, y, w, h, dy, lines
  Static textposPrev
  Static *it.sgx_llS_elem ; use Protected when using recursion!
  Static ignore
  Static otherFocused
  Static mousexPrev
  Static mouseyPrev
  Static *animation.sgxS_Animation
  Static changeLater.sgxS_ListWrapper
  Static destroyed
  Static editWasFocused
  Static sizeOk, dw, dh
  
  emsPrev=sgx_ems
  If *sgx_emsContainer
    sgx_ems = *sgx_emsContainer\i
  Else
    sgx_ems = ElapsedMilliseconds()
  EndIf
  
  ; ############################################################################
  ; ############################################################################
  
  ;{ -- GET MOUSE DATA
  
  mousexPrev = sgx_m\x
  mouseyPrev = sgx_m\y
  If mousex=-1
    sgx_m\x  = MouseX()
    sgx_m\dx = MouseDeltaX()
  Else
    sgx_m\x  = mousex
    sgx_m\dx = mousex - mousexPrev
  EndIf
  If mousey=-1
    sgx_m\y  = MouseY()
    sgx_m\dy = MouseDeltaY()
  Else
    sgx_m\y  = mousey
    sgx_m\dy = mousey - mouseyPrev
  EndIf
  
  ; (mouselib vs winapi)
  If 01
    ; use pb mouse lib for buttons and wheel
    ; problems:
    ; - mouse wheel may not work in windowed screen (http://www.purebasic.fr/english/viewtopic.php?f=4&t=65533)
    ; - mouse button may reset without actual button release (http://www.purebasic.fr/english/viewtopic.php?f=4&t=65799)
    
    If MouseWheel() >= 0
      sgx_m\wheelSign = 1
      sgx_m\wheel     = MouseWheel()
    Else
      sgx_m\wheelSign = -1
      sgx_m\wheel     = -MouseWheel()
    EndIf
    sgx_m\leftdown_prev = sgx_m\leftdown
    sgx_m\rightdown_prev = sgx_m\rightdown
    
    sgx_m\leftdown = MouseButton(1)
    sgx_m\rightdown = MouseButton(2)
    
    sgx_m\leftclick = 0
    If sgx_m\leftdown And Not sgx_m\leftdown_prev
      sgx_m\leftclick = 1
    EndIf
    sgx_m\leftup = 0
    If sgx_m\leftdown_prev And Not sgx_m\leftdown
      sgx_m\leftup = 1
    EndIf
    sgx_m\rightclick = 0
    If sgx_m\rightdown And Not sgx_m\rightdown_prev
      sgx_m\rightclick = 1
    EndIf
    sgx_m\rightup = 0
    If sgx_m\rightdown_prev And Not sgx_m\rightdown
      sgx_m\rightup = 1
    EndIf
  Else
    ; window callback (see above) not working in fullscreen?
    
    ; If sgx_mouseWheelDelta >= 0
    ;   ; scrolling 'up' (or not scrolling)
    ;   sgx_m\wheelSign = 1
    ;   sgx_m\wheel     = sgx_mouseWheelDelta
    ; Else
    ;   ; scrolling 'down'
    ;   sgx_m\wheelSign = -1
    ;   sgx_m\wheel     = -sgx_mouseWheelDelta
    ; EndIf
    ; sgx_mouseWheelDelta = 0
    
    ; If GetAsyncKeyState_(#VK_LBUTTON)&(1<<15)
    ;   sgx_m\leftdown = 1
    ; Else
    ;   sgx_m\leftdown = 0
    ; EndIf
    ; If GetAsyncKeyState_(#VK_RBUTTON)&(1<<15)
    ;   sgx_m\rightdown = 1
    ; Else
    ;   sgx_m\rightdown = 0
    ; EndIf
    
  EndIf
  ;}
  ;  --
  
  ; ############################################################################
  ; ############################################################################
  
  ; global topID
  sgxTopID = sgxGetElementAt(sgx_m\x, sgx_m\y)
  ; static local topID
  ; keep static topID until mouse release, for example during scrollbar/trackbar knob dragging even
  ; if mouse is outside the element
  If Not sgx_m\leftdown
    topID = sgxTopID
  EndIf
  
  ; reset window focus if clicked 'nowhere'
  If sgxActiveWin <> -1  And  topID = -1  And  (sgx_m\leftup Or sgx_m\rightup)
    sgxActiveWin = -1
  EndIf
  
  ; we check if an edit is currently focused before we loop through the elements because if the edit looses
  ; focus, elements processed after the edit would already receive events but we want to behave them as if
  ; the edit still had focus for this examine/frame.
  editWasFocused = 0
  If sgx_editing<>-1
    editWasFocused = sgx_editing
  EndIf
  
  ; ############################################################################
  ; ############################################################################
  
  For id=0 To sgx_CURRENT_MAX_DYNAMIC_GADGETS    ;   <<<<<<<      MAIN LOOP
    
    ; ############################################################################
    ; ############################################################################
    
    ;{ -- KICKOUTS / event reset / triggering the pending events
    
    ; skip gap from highest static ID to first dynamic one
    sgx_SkipUnused(id)
    
    ; skip invalid
    If Not sgx(id)
      Continue
    EndIf
    
    ; reset single fired events
    sgx(id)\leftclick = 0
    sgx(id)\leftup = 0
    sgx(id)\rightclick = 0
    sgx(id)\rightup = 0
    
    sgx(id)\lefthold = 0
    
    sgx(id)\changed = 0
    sgx(id)\resized = 0
    sgx(id)\refreshed = 0
    
    ; mouse button release -> reset element mousedown and focus
    If sgx_m\leftup
      sgx(id)\leftdown = 0
      sgx(id)\lefthold_timer = 0
      If id = sgx_focusCatchedBy
        sgx(id)\leftup = 1
        sgx_focusCatchedBy = -1
      EndIf
      If id = sgx_moving
        sgx(id)\leftup = 1
        sgx_moving = -1
      EndIf
    EndIf
    If sgx_m\rightup
      sgx(id)\rightdown = 0
      sgx(id)\lefthold_timer = 0
      If id = sgx_focusCatchedBy
        sgx(id)\rightup = 1
        sgx_focusCatchedBy = -1
      EndIf
    EndIf
    
    ; we trigger the pending events that sgxSetState() or sgxResize() etc. might have caused
    If ListSize(sgx_pendingEvents())
      ForEach sgx_pendingEvents()
        If sgx_pendingEvents()\id = id
          If sgx_pendingEvents()\changed
            sgx(id)\changed = sgx_pendingEvents()\changed
          EndIf
          If sgx_pendingEvents()\resized
            sgx(id)\resized = sgx_pendingEvents()\resized
          EndIf
          If sgx_pendingEvents()\refreshed
            sgx(id)\refreshed = sgx_pendingEvents()\refreshed
          EndIf
          DeleteElement(sgx_pendingEvents())
        EndIf
      Next
    EndIf
    
    ; ###### CONTINUE / IGNORE THE ELEMENT IN SOME CASES
    
    ; if an edit or tackbar for example have catched focus, the current element
    ; does not need hover/tooltips ect and can be handled briefly
    otherFocused = #False
    If (sgx_focusCatchedBy<>-1 And id<>sgx_focusCatchedBy)  Or  (sgx_editing<>-1 And id<>sgx_editing)
      otherFocused = #True
    EndIf
    
    ignore = #False
    ; in most situations we can handle the element in a brief way
    ; i.e. when no focus and element is not hovered and not being resized (exeption: animated windows must be processed anyway)
    If sgx_focusCatchedBy=-1 And id<>topID And id<>sgx_resizing And id<>sgx_editing And (Not (sgx(id)\type=#sgxTypeWin And sgx(id)\state<>#sgxWinStateNormal))  ; And (Not sgxIsParentage(id,topID)
      ignore = #True
    EndIf
    
    If sgx(id)\disabled
      ignore = #True
      If id = topID; Or id = sgxTopID
        ; make all parents up to the first window hover too.
        sgx_setParentWindowHovered(id)
      EndIf
    EndIf
    
    If Not sgx(id)\show
      Continue
    ElseIf (otherFocused Or ignore)
      ; avoid hover change (flickering) while manually resizing elements (mouse may temporarily move outside the element)
      If sgx_resizing = -1
        sgx(id)\hover=0
      EndIf
      If sgx(id)\tip
        sgx(id)\tip\show = 0
        sgx(id)\tip\timer = 0
      EndIf
      Continue
    EndIf
    ;}
    ;  --
    
    ; ############################################################################
    ; ############################################################################
    
    ; ######## HOVER (type independent)
    
    If sgx_isInElement(id, sgx_m\x, sgx_m\y)
      
      ;{ -- set element hovered
      If id = topID
        ; avoid hover change (flickering) while manually resizing elements (mouse may temporarily move outside the element)
        If sgx_resizing = -1
          sgx(id)\hover=1
        EndIf
      EndIf
      
      ; make all parents up to the first window hover too.
      ; avoid hover change (flickering) while manually resizing elements (mouse may temporarily move outside the element)
      If sgx_resizing = -1
        sgx_setParentWindowHovered(id)
      EndIf
      ;}
      ;  --
      
      
      ;{ -- resize (manually via mouse)
      ; initialize resizing if not done yet and if resize gadget clicked
      If sgx(id)\resizable  And  sgx_resizing = -1  And  sgx_m\leftclick  And  sgx_MousehoverResize(id)  And  (Not editWasFocused Or id=sgx_editing)
        ; resize windows only in normal state, combos only when closed
        If  (sgx(id)\type <> #sgxTypeWin Or sgx(id)\state=#sgxWinStateNormal)  And  (sgx(id)\type <> #sgxTypeCombo Or sgx(id)\max<>1)
          sgx_resizing = id
          
          ; get mouse cursor distance from bottom right element corner, [-2,-1] for example.
          sgx_vec\x = sgx_m\x - (sgx(id)\x + sgx(id)\w)
          sgx_vec\y = sgx_m\y - (sgx(id)\y + sgx(id)\h)
          
          ; exit edit focused on resizing
          If sgx(id)\type = #sgxTypeEdit And sgx_editing=id
            sgx_editing = -1
          EndIf
        EndIf
      EndIf ;}
      ;  --
      
      
      ;{ -- mouse events (hover)
      
      If id <> sgx_resizing
        
        ; catch focus with first mousedown
        
        ; LEFT
        If sgx_m\leftclick  And  ((Not editWasFocused) Or (id=editWasFocused))
          sgx(id)\leftclick=1
          sgx_focusCatchedBy = id
          sgx(id)\leftdown=1 ; this will stay set until mouse release
          sgx(id)\lefthold_timer = sgx_ems + sgxHoldDelay ; init timer for first hold event
        EndIf
        
        ; RIGHT
        If sgx_m\rightclick  And  ((Not editWasFocused) Or (id=editWasFocused))
          sgx(id)\rightclick=1
          sgx_focusCatchedBy = id
          sgx(id)\rightdown=1 ; this will stay set until mouse release
          sgx(id)\lefthold_timer = sgx_ems + sgxHoldDelay ; init timer for first hold event
        EndIf
        
      EndIf
      ;}
      ;  --
      
      
      ;{ -- make direct parent window active if child is clicked
      If sgx(id)\leftclick Or sgx(id)\rightclick
        ; skip areas, they don't move in front (i'm not sure if a parent window should become active nonetheless for areas?)
        If sgx(id)\type <> #sgxTypeArea
          sgxMoveOnTop(id)
        EndIf
      EndIf
      ;}
      ;  --
      
      
      ;{ -- TIPS (show / hide)
      If sgx(id)\tip
        ; hide tip if mouse moves fast enough
        If (sgx_m\dx > sgx_m\dlimit)  Or (sgx_m\dy > sgx_m\dlimit) Or (sgx_m\dx < -sgx_m\dlimit)  Or (sgx_m\dy < -sgx_m\dlimit)
          sgx(id)\tip\timer=0
          If sgx(id)\tip\show = 1
            sgx(id)\tip\show = -1  ; (mark as 'has been shown')
          EndIf
        ElseIf sgx(id)\tip\timer=0  And  sgx(id)\tip\show = 0   ; (show=-1 if has already been shown)
          ; init timer
          sgx(id)\tip\timer = sgx_ems + sgxTipDelay
        ElseIf (sgx_ems > sgx(id)\tip\timer)  And  sgx(id)\tip\show=0
          sgx(id)\tip\show = 1
        EndIf
      EndIf ;}
      ;  --
      
      
      ; ############################################################################
      ; ############################################################################
      
      ; ###### HOVER (type dependent)
      
      Select sgx(id)\type
        
        
        ;  -- BUTTONTOGGLE (hover)
        ;{ -- CHECKBOX     (hover)
        Case #sgxTypeButtonToggle, #sgxTypeCheckbox
          If (Not editWasFocused) And id <> sgx_resizing
            If sgx(id)\leftclick
              If sgxGetGroup(id) <> -1
                ; radiogroup element (we don't untoggle those by leftclick)
                sgxSetState(id, 1, #True, @changeLater)
              Else
                ; no radiogroup element (toggle state)
                sgxSetState(id, sgx(id)\state!1, #True, @changeLater)
              EndIf
            ElseIf sgx(id)\rightclick And (sgxRightclickUnset=#sgxRightclickUnsetAll Or (sgxRightclickUnset=#sgxRightclickUnsetNoGroup And sgxGetGroup(id)=-1))
              ; untoggle according to righclick settings
              sgxSetState(id, 0, #False, @changeLater)
            EndIf
          EndIf ;}
        ;  --
        
        
        ;{ -- WIN (hover)
        Case #sgxTypeWin
          
          If (Not editWasFocused)
            If sgx(id)\leftclick And id=topID
  
              ; minimize window
              If sgx(id)\state = #sgxWinStateNormal  And  sgx(id)\width  And  sgx_MousehoverClosebutton(id)
                If sgxActiveWin <> id
                  sgxMoveOnTop(id)
                EndIf
                sgxMinimizeWin(id)
                
              ; restore window
              ElseIf sgx(id)\state = #sgxWinStateMinimized
                sgxRestoreWin(id)
              EndIf
  
            ElseIf sgx(id)\rightclick
              
              ; close window
              If sgx(id)\state = #sgxWinStateNormal  And  sgx(id)\width  And  sgx_MousehoverClosebutton(id)  And  sgxWinCloseEnabled
                If sgxActiveWin <> id
                  sgxMoveOnTop(id)
                EndIf
                destroyed = sgxCloseWin(id) ; returns 1 if the element has been destroyed by a finished closing animation
                If destroyed
                  Continue
                EndIf
              EndIf
              
            ElseIf sgx(id)\state=#sgxWinStateNormal And sgx(id)\leftdown
              
              ; activate window (make foreground)
              If sgxActiveWin <> id
                sgxMoveOnTop(id)
              EndIf
              
              ; move
              If sgx_moving = -1  And  sgx_m\y < (sgx(id)\y+sgx(id)\width) ; if not already dragging and mouse is in titlebar
                sgx_moving = id
                ; store the in-titlebar mouse position for proper positioning while moving
                sgx_vec\x = sgx_m\x - sgx(id)\x
                sgx_vec\y = sgx_m\y - sgx(id)\y
              EndIf
              
            EndIf
            
          EndIf ;}
        ;  --
        
        
        ;{ -- TABLE (hover) <<<<<<<<<<<<<<<<<<<<
        Case #sgxTypeTable
          
          ;If (Not editWasFocused)
            ; [nothing to handle yet]
          ;EndIf
;}
        ;  --
        
        
        ;{ -- EDIT (hover)
        Case #sgxTypeEdit
          If (Not editWasFocused)
            If sgx(id)\leftclick And id <> sgx_resizing
              ; focus
              sgx_editing = id
              ; make tip appear immediately on edit focus (..?)
              If sgx(id)\tip
                sgx(id)\tip\timer = sgx_ems
              EndIf
            EndIf
          ElseIf sgx_editing=id And sgx(id)\leftclick
            ; position textcursor by mouseclick
            x = sgx_m\x - (sgx(id)\x + sgx(id)\x2)
            y = sgx_m\y - (sgx(id)\y + sgx(id)\y2)
            ; clicked in client area?
            If x >= 0  And  x < sgx(id)\w2  And  y >= 0  And  y < sgx(id)\h2
              ; store mouse click position in client area
              sgx(id)\textcursor\x = x
              sgx(id)\textcursor\y = y
              textposPrev = sgx(id)\textpos
              ; reposition textpos from cursor position (as opposed to other way round normally)
              sgxRefresh(id, #sgx_refreshEditMouseclickCursor)
              ; trigger a 'changed' event if 'textpos' changed
              If sgx(id)\textpos <> textposPrev
                AddElement(sgx_pendingEvents())
                sgx_pendingEvents()\id = id
                sgx_pendingEvents()\changed = 1
              EndIf
              ; avoid confusion in sgx_keyboardEdit() and cursor key handling
              sgx_keyboardEdit_lastTextPos = sgx(id)\textpos
            EndIf
          EndIf ;}
        ;  --
        
        
        ;{ -- COMBO (hover)
        Case #sgxTypeCombo
          If (Not editWasFocused)  And id <> sgx_resizing
            If sgx(id)\leftclick
              sgx(id)\max!1
              If sgx(id)\max=1
                ; open
                sgx(id)\h2 = sgx(id)\h
                sgx(id)\h = sgx(id)\h*(2+CountString(sgx(id)\text,#LF$))
                sgx_llMoveEnd(sgx(id)\treeElem)
              Else
                ; close (and select)
                temp = (sgx_m\y - sgx(id)\y) / sgx(id)\h2
                sgx(id)\h = sgx(id)\h2
                If temp
                  If sgx(id)\state<>temp
                    sgx(id)\changed = 1
                    sgx(id)\state=temp
                    sgxRefresh(id)
                  EndIf
                EndIf
              EndIf
            ElseIf sgx(id)\rightclick
              ; close (cancel)
              If sgx(id)\max=1
                sgx(id)\h = sgx(id)\h2
                sgx(id)\max=0
              EndIf
            EndIf
          EndIf ;}
        ;  --
  
        
      EndSelect
      
      
      ;{ -- MOUSEWHEEL (hover) (scrollbars/trackbars and edit)
      If sgx_m\wheel And (Not editWasFocused Or sgx_editing=id) And id <> sgx_resizing
        
        ; scrollbars/trackbars handled only
        sgx_SetScrollPos(id, sgx(id)\scrollpos - sgx_m\wheelSign * 10 * sgx_m\wheel)
        
        If sgx(id)\type = #sgxTypeEdit
          If sgx_m\wheel
            dy = (sgx_m\wheel * sgx(id)\texth) ; (texth is lineheight)
            
            ; scrolling up (reduce scrollpos)
            If sgx_m\wheelSign > 0
              If (sgx(id)\scrollpos - dy) < 0
                dy - -(sgx(id)\scrollpos - dy)
              EndIf
              sgx(id)\scrollpos - dy
              sgx(id)\textcursor\y - dy
              
            ; scrolling down (increase scrollpos)
            ElseIf sgx_m\wheelSign < 0
              lines = CountString(sgx(id)\text, #LF$) + 1
              If (lines * sgx(id)\texth) < sgx(id)\h2
                ; less lines than clientarea height
                sgx(id)\scrollpos = 0
                sgx(id)\textcursor\y = 0
                dy = 0
              ElseIf (sgx(id)\scrollpos + dy) > (lines * sgx(id)\texth - sgx(id)\h2)
                dy - ( (sgx(id)\scrollpos + dy) - (lines * sgx(id)\texth - sgx(id)\h2) )
              EndIf
              sgx(id)\scrollpos + dy
              sgx(id)\textcursor\y + dy
            EndIf
            ; #sgx_refreshEditMousewheelScroll disables 'scrollToCursor'
            sgxRefresh(id, #sgx_refreshEditMousewheelScroll)
          EndIf
        EndIf
      EndIf ;}
      ;  --
    
    ; ############################################################################
    ; ############################################################################
    
    ; ######## NO HOVER
    Else ;{
      
      ; unset hover
      ; (we won't get here if other element has catched focus)
      ; avoid hover change (flickering) while manually resizing elements (mouse may temporarily move outside the element)
      If sgx_resizing = -1
        sgx(id)\hover=0
      EndIf
      If sgx(id)\tip
        sgx(id)\tip\show = 0
        sgx(id)\tip\timer = 0
      EndIf
      ;}
    EndIf

    
    ; ############################################################################
    ; ############################################################################
    
    ; ######## HOVER INDEPENDENT
    
    ; there are some 'Continue's up there
    ; so you might not get here!
    
    Select sgx(id)\type
      
      ;{ -- SCROLLBARS, TRACKBARS (click or drag)
      Case #sgxTypeScrollbarVer, #sgxTypeTrackbarVer ; |
        If  (Not editWasFocused) And id <> sgx_resizing
          If sgx_focusCatchedBy=id
            sgx_SetScrollPos(id, sgx_m\y - sgx(id)\y)
          EndIf
        EndIf
      Case #sgxTypeScrollbarHor, #sgxTypeTrackbarHor ; --
        If (Not editWasFocused) And id <> sgx_resizing
          If sgx_focusCatchedBy=id
            sgx_SetScrollPos(id, sgx_m\x - sgx(id)\x)
          EndIf
        EndIf ;}
      ;  --
      
      
      ;{ -- EDIT (focused)
      Case #sgxTypeEdit
        If sgx_editing=id
          If KeyboardReleased(#PB_Key_Escape)  Or  ( (sgx_m\leftdown Or sgx_m\rightdown) And Not (sgx_isInElement(id, sgx_m\x, sgx_m\y) And id=topID) )
            ; exit edit
            sgx_editing = -1
            ; disable 'scrollToCursor' with #sgx_refreshEditMousewheelScroll
            sgxRefresh(id, #sgx_refreshEditMousewheelScroll) ; (this call might be redundant)
          Else
            ; typing/focused
            ; (we need to do this continuously while edit is focused to have proper keystroke delay reset)
            ;If KeyboardPushed(#PB_Key_All) Or KeyboardReleased(#PB_Key_All)
              sgx(id)\text = sgx_keyboardEdit( sgx(id)\text , sgx(id)\textpos, sgx(id)\max)
            ;EndIf
            sgx(id)\textpos = sgx_keyboardEdit_lastTextPos
            If sgx_keyboardEdit_TextModified
              sgx(id)\changed = 1
              sgxRefresh(id)
            EndIf
          EndIf
        EndIf ;}
      ;  --
      
      
      ;{ -- WIN (animation and moving)
      Case #sgxTypeWin
        
        ; ------------------------------------------------------
        ; window animations
        
        *animation = 0
        
        If sgx(id)\state <> #sgxWinStateNormal
          *animation = FindMapElement(sgx_animations(), Str(id))
        EndIf
        
        If *animation
          
          ; adjust the element properties according to the current animation progress
          destroyed = sgx_animate(id, *animation) ; returns 1 if the element has been destroyed by a finished closing animation
          If destroyed
            Continue
          EndIf
        EndIf
        
        ; ------------------------------------------------------
        ; moving window
        
        If sgx_moving = id
          sgxMove( id, sgx_m\x - sgx_vec\x, sgx_m\y - sgx_vec\y, #False )
          If Not sgx_m\leftdown
            sgx_moving = -1
          EndIf
        EndIf ;}
      ;  --
          
    EndSelect
    
    
    ;{ -- resizing manually via mouse
    If sgx_resizing = id
      If Not sgx_m\leftdown
        sgx_resizing = -1
      ElseIf sgx_m\dx Or sgx_m\dy
        ; new size = mouse-in-element + remaining-to-bottom-right-corner
        w = sgx_m\x - sgx(id)\x + (-sgx_vec\x)
        h = sgx_m\y - sgx(id)\y + (-sgx_vec\y)
        
        If w < sgx(id)\minWidth
          w = sgx(id)\minWidth
        EndIf
        If h < sgx(id)\minHeight
          h = sgx(id)\minHeight
        EndIf
        If w <> sgx(id)\w  Or  h <> sgx(id)\h
          sgxResize(id, w, h)
        EndIf
        
      EndIf
    EndIf
    ;}
    ;  --
    
    
    ;{ -- trigger next hold event
    If sgx(id)\lefthold_timer And sgx(id)\lefthold_timer < sgx_ems
      ; fire a single hold event
      sgx(id)\lefthold = 1
      ; init timer for next hold event
      sgx(id)\lefthold_timer = sgx_ems + sgxHoldInterval
    EndIf
    ;}
    ;  --
    
    
  Next ; finished [For id=0 To ...]                                                                  <<<<<<< MAIN LOOP END
  
  ; ############################################################################
  ; ############################################################################
  
  
  ; elements in radiogroups might reset their 'changed' flag at the beginning of the loop though
  ; it was possibly set by another group element that has been processed in a previous turn
  ; so we stored the change and set it now after all element have been processed.
  ForEach changeLater\list_()
    sgx( changeLater\list_() )\changed = 1
  Next
  ClearList( changeLater\list_() )
  
  ProcedureReturn sgxTopID
EndProcedure














; IDE Options = PureBasic 5.50 beta 1 (Windows - x64)
; CursorPosition = 251
; FirstLine = 229
; Folding = PBE9
; Markers = 180,320
; UseMainFile = demo.pb
; CompileSourceDirectory