
; ----------------------- basic tree list functionality



;** s sgx_llS_elem
;* type of the elements in an <a href=#s_sgx_lls_list>sgx_llS_list</a>. used for a basic tree list to represent the sgx element hierarchy. _
;* type of the field '*treeElem' of the structure <a href=#s_sgxs_gadget>sgxS_GADGET</a>. _
;* this type allows the use of lists as a tree, since it has a list it belongs to and a child list (branch) as well. _
;* lists are created via constructor macro <a href=#f_sgx_llnew>sgx_llnew</a>. _
;* elements are added to a list via <a href=#f_sgx_lladd>sgx_llAdd</a> and removed from a list via <a href=#f_sgx_lldelelem>sgx_llDelElem</a>. _
;* a list can be cleared via <a href=#f_sgx_llclearlist>sgx_llClearList</a>. _
;* an element can be moved to the end of its list via <a href=#f_sgx_llmoveend>sgx_llMoveEnd</a>. _
;* you can print a tree via <a href=#f_sgx_lldebug>sgx_llDebug</a>. _
;* a list element can be removed from one list and be recreated in another list using <a href=#f_sgx_lldelelem>sgx_llDelElem</a> in combination with <a href=#f_sgx_llreplaceelemnode>sgx_llReplaceElemNode</a>. _
;* _
;* there are several iterator macros to traverse a list/tree: _
;* be carefull with Static iterator variables in recursively called functions (even with the flat iterators) or better use Protected for your iterator variable. _
;* also do not use Break or Continue in such loops! _
;* flat iteration
;* <pre>
;* <span></span>sgx_llIterator(Define *it, *sgxLayeredView) _
;* <span></span>  Debug sgx(*it\id)\text _
;* <span></span>sgx_llNext(*it) _
;* </pre>
;* flat reverse iteration (from last to first element)
;* <pre>
;* <span></span>sgx_llIteratorReverse(Define *it, *sgxLayeredView) _
;* <span></span>  Debug sgx(*it\id)\text _
;* <span></span>sgx_llPrev(*it) _
;* </pre>
;* recursive iteration
;* (here we use protected - imagine we are inside a function - and we start at a window's child node instead of the sgx root node)
;* <pre>
;* <span></span>sgx_llIteratorRecursive(Protected *it, sgx(window)\treeElem\node) _
;* <span></span>  Debug sgx(*it\id)\text _
;* <span></span>sgx_llNextRecursive(*it) _
;* </pre>
Structure sgx_llS_elem ;{
  *list.sgx_llS_list  ;* pointer to the list the element belongs to
  *next.sgx_llS_elem  ;* pointer to the element's successor or 0 if the element is the last in its list
  *prev.sgx_llS_elem  ;* pointer to the element's predecessor or 0 if the element is the first in its list
  *node.sgx_llS_list  ;* pointer to the element's list of childs
  id.i                ;* the actual data of the element, an sgx element ID in our case
;}
EndStructure


;** s sgx_llS_list
;* the list structure of the (tree) list implementation. _
;* lists of this type are created via constructor macro <a href=#f_sgx_llnew>sgx_llnew</a>. _
;* type of variables '*sgxLayeredView' and '*sgx_CurrentParent'. _
;* _
;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
Structure sgx_llS_list ;{
  *curr .sgx_llS_elem  ;* pointer to the current element of the list or 0 if list is empty
  *first.sgx_llS_elem  ;* pointer to the first element in the list or 0 if list is empty
  *last .sgx_llS_elem  ;* pointer to the last element in the list or 0 if list is empty
;}
EndStructure


;** sgx_llnew
;* constructor macro for creation of <a href=#s_sgx_lls_list>sgx_llS_list</a> lists. _
;* for example: _
;* <pre>
;* Global *myList.sgx_llnew _
;* </pre>
;* will be expande to this: _
;* <pre>
;* Global *myList.sgx_llS_list = AllocateMemory(SizeOf(sgx_llS_list)) _
;* </pre>
;* 
;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
Macro sgx_llnew
  sgx_llS_list = AllocateMemory(SizeOf(sgx_llS_list))
EndMacro


;** s sgx_llS_IteratorStack
;* a list wrapper structure. type of 'sgx_llStackOfIteratorStacks'. _
;* _
;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
Structure sgx_llS_IteratorStack
  List *iteratorStack.sgx_llS_elem()
EndStructure

;** g sgx_llStackOfIteratorStacks
;* a stack of stacks of type <a href=#s_sgx_lls_iteratorstack>sgx_llS_IteratorStack</a> used by <a href=#f_sgx_lliteratorrecursive>sgx_llIteratorRecursive</a>. _
;* sgx_llStackOfIteratorStacks()\iteratorStack() is a list (stack) of <a href=#s_sgx_lls_elem>sgx_llS_elem</a> elements which get pushed and popped during a recursive iteration. _
;* sgx_llStackOfIteratorStacks() itself is a wrapper stack for the latter and gets pushed and popped with the beginning and end of each such a recursive iteration block (for example in case within the block a function gets called which uses a recursive iterator itself). _
;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list. _
Global NewList sgx_llStackOfIteratorStacks.sgx_llS_IteratorStack()

; ------------------------


;** sgx_llIterator
;* flat iterator macro (without recursion) to iterate over all list elements. _
;* use <a href=#f_sgx_llnext>sgx_llNext</a> to close such an iteration block. _
;* do not use Break or Continue in such loops! _
;* _
;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
Macro sgx_llIterator(it,List)
  If List\first
    it.sgx_llS_elem = List\first
    Repeat
EndMacro

;** sgx_llNext
;* the 'Next' command for an <a href=#f_sgx_lliterator>sgx_llIterator</a> block. _
;* _
;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
Macro sgx_llNext(it)
    it = it\next
    Until it=0
  EndIf
EndMacro

; ------------------------

;** sgx_llIteratorReverse
;* flat reverse iterator macro (without recursion) to iterate over all list elements backwards. _
;* use <a href=#f_sgx_llprev>sgx_llPrev</a> to close such an iteration block. _
;* do not use Break or Continue in such loops! _
;* _
;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
Macro sgx_llIteratorReverse(it,List)
  If List\last
    it.sgx_llS_elem = List\last
    Repeat
EndMacro

;** sgx_llPrev
;* the 'Next' (previous) command for an <a href=#f_sgx_lliteratorreverse>sgx_llIteratorReverse</a> block. _
;* _
;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
Macro sgx_llPrev(it)
    it = it\prev
    Until it=0
  EndIf
EndMacro

; ------------------------

;** sgx_llIteratorRecursive
;* recusrive iterator macro to iterate over all list elements including the elements child nodes. _
;* use <a href=#f_sgx_llnextrecursive>sgx_llNextRecursive</a> to close such an iteration block. _
;* do not use Break or Continue in such loops! _
;* _
;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
Macro sgx_llIteratorRecursive(it,List)
  If List\first
    ; push a new iterator stack in case of nested recursive iterator blocks
    AddElement(sgx_llStackOfIteratorStacks())
    it.sgx_llS_elem = List\first
    Repeat
EndMacro

;** sgx_llNextRecursive
;* the 'Next' command for an <a href=#f_sgx_lliteratorrecursive>sgx_llIteratorRecursive</a> block. _
;* with the parameter 'skipNode' the recursion can be controlled during the loop conditionally. used for example to constrain recursion to non-window sgx childs. _
;* _
;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
Macro sgx_llNextRecursive(it, skipNode=#False)
      it = sgx_llNextRecursive_proc(it, skipNode)
    Until it=0
    ; clear and pop our current iterator stack
    LastElement(sgx_llStackOfIteratorStacks())
    ClearList(sgx_llStackOfIteratorStacks()\iteratorStack())
    DeleteElement(sgx_llStackOfIteratorStacks())
  EndIf
EndMacro

;** sgx_llNextRecursive_proc
;* used by the <a href=#f_sgx_llnextrecursive>sgx_llNextRecursive</a> macro for recursion. _
;* _
;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
Procedure sgx_llNextRecursive_proc(*it.sgx_llS_elem, skipNode=#False)
;   Protected *iteratorStack.sgx_llS_elem
;   Protected *temp.sgx_llS_elem
  
;   *iteratorStack = @sgx_llStackOfIteratorStacks()\iteratorStack()
  
  If *it
    
    ; step into recursion
    If *it\node And *it\node\first And skipNode=#False
      ; push current iterator
      AddElement(sgx_llStackOfIteratorStacks()\iteratorStack())
      sgx_llStackOfIteratorStacks()\iteratorStack()=*it
      ; and return sublist's first element as new iterator
      ProcedureReturn *it\node\first
      
    ; just proceed in current list
    ElseIf *it\next
      ProcedureReturn *it\next
      
    ; step out of recursion
    ; and make shure the last pushed element is the current one
    ElseIf LastElement(sgx_llStackOfIteratorStacks()\iteratorStack())
      ; proceed with next element on the previous level
      *it = sgx_llStackOfIteratorStacks()\iteratorStack()\next
      ; and pop that iterator from the stack
      DeleteElement(sgx_llStackOfIteratorStacks()\iteratorStack())
      ; continue to pop levels until we find one with a next element in it
      While ListSize(sgx_llStackOfIteratorStacks()\iteratorStack()) And *it=0
        LastElement(sgx_llStackOfIteratorStacks()\iteratorStack())
        *it = sgx_llStackOfIteratorStacks()\iteratorStack()\next
        DeleteElement(sgx_llStackOfIteratorStacks()\iteratorStack())
      Wend
      ProcedureReturn *it
    EndIf
  Else
    ProcedureReturn 0
  EndIf
EndProcedure

; ------------------------

Declare sgx_llDelElem(*elem.sgx_llS_elem, returnNode=#False)
Declare sgx_llDebug(*list.sgx_llS_list)


;** sgx_llAdd
;* creates a new element at the end of the list and sets its data field to 'id'. _
;* _
;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
Procedure sgx_llAdd(*list.sgx_llS_list, id)
  If *list\first = 0
    *list\curr = AllocateMemory(SizeOf(sgx_llS_elem))
    *list\curr\id = id
    *list\curr\list = *list
    *list\first = *list\curr
    *list\last = *list\curr
  Else
    ;!expects curr = last
    *list\curr=*list\last
    *list\curr\next = AllocateMemory(SizeOf(sgx_llS_elem))
    *list\curr\next\prev = *list\curr
    *list\curr = *list\curr\next
    *list\curr\id = id
    *list\curr\list = *list
    If *list\last = *list\curr\prev
      *list\last = *list\curr
    EndIf
  EndIf
  ProcedureReturn *list\curr
EndProcedure


;** sgx_llClearList
;* removes all elements from the list which will remove sublevels too. the list will still remain a valid but empty list.
;* _
;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
Procedure sgx_llClearList(*list.sgx_llS_list)
  If *list
    While *list\first
      sgx_llDelElem(*list\first)
    Wend
  EndIf
EndProcedure


;** sgx_llDelElem
;* remove the element from its list and free it and its childs. _
;* if 'returnNode' is #True, the subtree is preserved and returned. 
;* you can thus create a new element in another list and use the 
;* node with sgx_llReplaceElemNode, basically moving the element 
;* to another list. but remember to also transfer the data field 'id' which 
;* is not preserved. <a href=#f_sgxsetparent>sgxSetParent</a> makes use of this. _
;* _
;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
Procedure sgx_llDelElem(*elem.sgx_llS_elem, returnNode=#False)
  Protected *returnNode.sgx_llS_list
  
  If *elem
    
    ; handle the subtree (childs)
    If returnNode
      *returnNode = *elem\node
      *elem\node=0
    Else
      If *elem\node
        ; free childs
        sgx_llClearList(*elem\node)
        FreeMemory(*elem\node)
        *elem\node=0
      EndIf
    EndIf
    
    ; handle the list of which this element is part of
    If *elem\list\last = *elem\list\first ; only one element in the list
      ; reset all links in the list
      *elem\list\curr=0
      *elem\list\first=0
      *elem\list\last=0
    Else ; at least 2 elements in list
      If *elem = *elem\list\last ; if its the last element, reset the predecessor's link and make the predecessor the last
        *elem\prev\next = 0
        *elem\list\last = *elem\prev
      ElseIf *elem = *elem\list\first ; if its the first element, reset the successor's link and make the successor the first
        *elem\next\prev = 0
        *elem\list\first = *elem\next
      Else ; if its elsewhere in the list, link the predecessor and successor to each other
        *elem\next\prev = *elem\prev
        *elem\prev\next = *elem\next
      EndIf
    EndIf
    FreeMemory(*elem)
    *elem=0
    
    If returnNode
      ProcedureReturn *returnNode
    EndIf
  EndIf
  
  ProcedureReturn 0
EndProcedure


;** sgx_llReplaceElemNode
;* remove and replace the node (list of childs) of a list element. used in <a href=#f_sgxsetparent>sgxSetParent</a> for example. _
;* _
;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
Procedure sgx_llReplaceElemNode(*elem.sgx_llS_elem, *newNode.sgx_llS_list)
  
  ; free current node
  If *elem\node
    ; free childs
    sgx_llClearList(*elem\node)
    FreeMemory(*elem\node)
    *elem\node=0
  EndIf
  
  ; replace the node pointer
  *elem\node = *newNode
  
  ; update the list pointer of the elements in the node
  If *elem\node
    sgx_llIterator(Protected *it, *elem\node)
      *it\list = *elem\node
    sgx_llNext(*it)
  EndIf
EndProcedure


;** sgx_llMoveEnd
;* move the element to the end of its list. used in <a href=#f_sgxmoveontop>sgxMoveOnTop</a> for example. _
;* _
;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
Procedure sgx_llMoveEnd(*elem.sgx_llS_elem)
  If *elem <> *elem\list\last
    If *elem = *elem\list\first
      *elem\list\first = *elem\next
    Else
      *elem\prev\next = *elem\next
    EndIf
    *elem\next\prev = *elem\prev
    *elem\list\last\next = *elem
    *elem\prev = *elem\list\last
    *elem\list\last = *elem
    *elem\next = 0
  EndIf
  ProcedureReturn #True
EndProcedure



;** sgx_llDebug
;* prints the list/tree hierarchy recursively to the debug output window (directly, not return a string). _
;* _
;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
Procedure sgx_llDebug(*list.sgx_llS_list)
  Static level
  
  sgx_llIterator(Protected *it, *list)
    If *it\node
      Debug LSet("",level*4," ") + Str(*it\id) + " {"
      level+1
      sgx_llDebug(*it\node)
      level-1
      Debug LSet("",level*4," ")  + "}"
    Else
      Debug LSet("",level*4," ") + Str(*it\id) ; + " {}"
    EndIf
  sgx_llNext(*it)
  
EndProcedure

; [merge_end]









; (test-code)
                        CompilerIf 0
; --------------------------------


; d(SizeOf(sgx_llS_list))
; d(SizeOf(sgx_llS_elem))


*list.sgx_llnew ;  ; << list constructor

;{ create random tree
For i=1 To 4 ; create random tree
  ;add elem
  sgx_llAdd(*list,i) ; element constructor
  ;add node..
  If Random(1)=0
    *list\curr\node.sgx_llnew
    If Random(1)=0
      For k=1 To 3
        sgx_llAdd(*list\curr\node, i*100+k)
        If Random(1)=0
          *list\curr\node\curr\node.sgx_llnew
          For z=1 To 2
            sgx_llAdd(*list\curr\node\curr\node, i*10000+k*100+z)
          Next
        EndIf
      Next
    EndIf
  EndIf
Next
;}


Debug "---------------"
sgx_llDebug(*list)
; Debug "---------------"
; sgx_llIteratorRecursive(*it,*list)
;   Debug *it\id
; sgx_llNextRecursive(*it)
Debug "---------------"
sgx_llMoveEnd(*list\first\next) ; move the 2nd element to the end
sgx_llDebug(*list)
Debug "---------------"
sgx_llDelElem(*list\first\next)
sgx_llDebug(*list)
sgx_llClearList(*list)
FreeMemory(*list)


; --------------------------------
                     CompilerEndIf
; --------------------------------




; IDE Options = PureBasic 5.42 LTS (Windows - x64)
; CursorPosition = 6
; Folding = ----
; Executable = ll_test.exe
; HideErrorLog
; CompileSourceDirectory