﻿
; - compile as uncidode
; - outputfile is always utf-8 with BOM
; - supports:
;   - XIncludeFile and IncludeFile (absolute paths and source-relative paths)
;   - [merge_off] inside a comment disables merging of following content
;   - [merge_on]  inside a comment enables merging of following content
;   - [merge_end] inside a comment skips the rest of the current file
; - no support for IncludePath
; - IDE Options will be skipped in include files but not in the main file (unless you use [merge_off] at the end of the main file)
; - commandline parameters:
;   - \indent <n>
;     - number of spaces for indentation of merged files (default is 8). example: "merger.exe \indent 0 mainfile.pb" to disable indentation.
;   - <filename>
;     - filename of the mainfile. if not specified a filerequester wil be opened.
;
; version 2.0 (2016-05-07, for updated sgx)

EnableExplicit

Global mainFilename.s
Global indentLength = 8

; ; testing
; Select 5
;   Case 1 : mainFilename = "C:\Users\user\Desktop\_sgx\merger\test1000.pb"
;   Case 2 : mainFilename = "C:\Users\user\Desktop\_sgx\merger\testBig.pb"
;   Case 3 : mainFilename = "C:\Users\user\Desktop\_sgx\sgx_.pbi"
;   Case 4 : mainFilename = "C:\Users\user\Desktop\_sgx\merger\testing\main.pb"
;   Case 5 : mainFilename = "..\..\sgx_.pbi"
;   Case 6 : mainFilename = "testing\main.pb"
; EndSelect

Declare.s processFile(filename.s, path.s="")
Declare writeStringToFile(s.s, filename.s)
  
Procedure main()
  Global mainFilename.s, path.s
  Protected result.s, outfilename.s
  Protected cpp, pp.s
  Protected i, time
  
  OpenConsole()
  
  If mainFilename = ""
    cpp = CountProgramParameters()
    For i=0 To cpp-1
      If ProgramParameter(i) = "/indent"
      ElseIf i>0 And ProgramParameter(i-1) = "/indent"
        indentLength = Val(ProgramParameter(i))
      Else
        mainFilename = ProgramParameter(i)
      EndIf
    Next
  EndIf
  
  If mainFilename = ""
    mainFilename = OpenFileRequester("merger - choose a main file", "", "*.*|*.*", 0)
  EndIf
  
  If mainFilename = ""
    End
  EndIf
  
  PrintN("merging..")
  time = ElapsedMilliseconds()
  
  result = processFile(mainFilename)
  
  If result
    outfilename = mainFilename + ".MERGED.pb"
    If writeStringToFile(result, outfilename)
    EndIf
    time = ElapsedMilliseconds() - time
    PrintN("finished in "+Str(time/1000)+"s ("+Str(time)+"ms)")
    ;PrintN("press return to continue")
  Else
    PrintN("error: result empty: "+mainFilename)
  EndIf
  
  ;Input()
  
EndProcedure

; -------------
; origin: http://www.purebasic.fr/english/viewtopic.php?p=277703#p277703
CompilerIf #PB_Compiler_OS = #PB_OS_Windows
  Macro isRelativePath(file)
    (Mid(file, 2, 1) <> ":")
  EndMacro
CompilerElse
  Macro isRelativePath(file)
    (Left(file, 1) <> "/")
  EndMacro
CompilerEndIf
; -------------

Macro putC(buff, offset, c)
  PokeC(buff + offset, c)
  offset + SizeOf(Character)
EndMacro

Macro putS(buff, offset, s)
  PokeS(buff + offset, s)
  offset + Len(s) * SizeOf(Character)
EndMacro

Procedure.s processFile(filename.s, path.s="")
  Protected filepath.s, file, c.c, c2.c
  Protected sourceBom
  Protected sourceContent.s
  Protected *sourcePtr, sourceLen, sourceByteLen, sourceOffset
  Protected *resultPtr, resultLen, resultByteLen, resultOffset
  Protected resultContent.s
  Protected chunkSize
  Protected comment, dquote, include, includeFilename.s, includeMerge, includeContent.s, includeByteLen, includeX, includeCheckPath.s, skip
  Protected mergeOn = #True, mergeEnd = #False
  Global    indentLength
  Protected indent.s, indentSub.s
  Static NewMap includedFiles()
  Static call
  call + 1
  
  indent    = LSet("", (call-1)*4,   " ") ; indentation for info output
  If indentLength
    indentSub = LSet("", indentLength, " ") ; indentation for merged files
  EndIf
  
  filepath = filename
  If isRelativePath(filename)
    filepath = path + filename ; note: filename may contain directory patterns too, like "C:\mydir\" + "..\myfile.pb"
  EndIf
  
  file = ReadFile(#PB_Any, filepath)
  If file
    ;AddGadgetItem(1, -1, indent+filename)
    PrintN(indent+filename)
    
    sourceBom = ReadStringFormat(file)
;     Select sourceBom
;       Case #PB_Ascii  : Debug "Kein BOM gefunden. Dies kennzeichnet üblicherweise eine normale Textdatei."
;       Case #PB_UTF8   : Debug "UTF-8 BOM gefunden."
;       Case #PB_Unicode: Debug "UTF-16 (Little Endian) BOM gefunden."
;       Case #PB_UTF16BE: Debug "UTF-16 (Big Endian) BOM gefunden."
;       Case #PB_UTF32  : Debug "UTF-32 (Little Endian) BOM gefunden."
;       Case #PB_UTF32BE: Debug "UTF-32 (Big Endian) BOM gefunden."
;     EndSelect
    
    sourceContent.s = ReadString(file, sourceBom | #PB_File_IgnoreEOL)
    CloseFile(file)
    sourceLen = Len(sourceContent)
    ;Debug "sourceLen: "+Str(sourceLen)
    sourceByteLen = StringByteLength(sourceContent)
    *sourcePtr = @sourceContent
    sourceOffset = 0
    
    resultContent = ""
    resultLen = Len(resultContent)
    resultByteLen = StringByteLength(resultContent)
    *resultPtr = #Null
    resultOffset = 0
    
    ; should at least be able to contain additional characters we put into the result at once like the folding
    ; comments we put around merged files etc. something like 200 would work but is slower due to more reallocations
    chunkSize = SizeOf(Character) * 10000
    
    While (sourceOffset < sourceByteLen)  ;And  (Not quit)  And  (Not cancel)
      
      c = PeekC(*sourcePtr+sourceOffset)
      
      ; make shure we have 1 empty chunk space left
      If resultOffset >= (resultByteLen - chunkSize)
        ;Debug "realloc at offset "+Str(resultOffset)
        resultByteLen + 2 * chunkSize
        *resultPtr = ReAllocateMemory(*resultPtr, resultByteLen)
      EndIf
      
      If c=Asc(#LF$)
        comment = #False
      ElseIf c = ';'  And  dquote = #False
        comment = #True
        If call > 1  And  PeekS(*sourcePtr+sourceOffset, 25) = "; IDE Options = PureBasic"
          mergeEnd = #True
          ; go back to end of previous line
          If resultOffset >= SizeOf(Character)
            resultOffset - SizeOf(Character)
            c2 = PeekC(*resultPtr+resultOffset)
            While (resultOffset >= SizeOf(Character))  And  (c2 = ' ' Or c2 = #TAB)
              resultOffset - SizeOf(Character)
              c2 = PeekC(*resultPtr+resultOffset)
            Wend
            While (resultOffset >= SizeOf(Character))  And  (c2 = #CR Or c2 = #LF)
              resultOffset - SizeOf(Character)
              c2 = PeekC(*resultPtr+resultOffset)
            Wend
          EndIf
        EndIf
      ElseIf c = '"'  And  comment = #False
        If dquote
          dquote = #False
          If include
            includeMerge = #True
          EndIf
        Else
          dquote = #True
        EndIf
      ElseIf c = '"' And dquote
        dquote = #True
      ElseIf dquote And include
        includeFilename + Chr(c)
      ElseIf comment = #False  And  dquote = #False  And  mergeOn = #True
        If (c = 'X'  Or  c = 'x')  And  include = #False  And  LCase(PeekS(*sourcePtr+sourceOffset, 12)) = "xincludefile"
          include = #True
          includeFilename = ""
          includeX = #True
          ;putC(*resultPtr, resultOffset, ';')
          putS(*resultPtr, resultOffset, "; ")
        ElseIf (c = 'I'  Or  c = 'i')  And  include = #False  And  LCase(PeekS(*sourcePtr+sourceOffset, 11)) = "includefile"
          include = #True
          includeFilename = ""
          ;putC(*resultPtr, resultOffset, ';')
          putS(*resultPtr, resultOffset, "; ")
        EndIf
      ElseIf comment = #True  And  c = '['
        If LCase(PeekS(*sourcePtr+sourceOffset, 11)) = "[merge_off]"
          mergeOn = #False
          putS(*resultPtr, resultOffset, "[merge_off]..")
        ElseIf LCase(PeekS(*sourcePtr+sourceOffset, 11)) = "[merge_end]"
          mergeEnd = #True
          putS(*resultPtr, resultOffset, "[merge_end]"+#CRLF$)
          comment = #False
        ElseIf LCase(PeekS(*sourcePtr+sourceOffset, 10)) = "[merge_on]"
          mergeOn = #True
        EndIf
      EndIf
      
      If mergeOn = #True  And  mergeEnd = #False
        
        putC(*resultPtr, resultOffset, c)
        
        If includeMerge
          
          skip = #False
          
          includeCheckPath = includeFilename
          ; path evaluation as done at the beginning of processFile()
          If isRelativePath(includeFilename)
            includeCheckPath = GetPathPart(filepath) + includeFilename ; note: includeFilename may contain directory patterns too, like "C:\mydir\" + "..\myfile.pb"
          EndIf
          If FindMapElement(includedFiles(), includeCheckPath)
            If includeX = #True
              skip = #True
            EndIf
          Else
            AddMapElement(includedFiles(), includeCheckPath)
          EndIf
          
          If skip
            putS(*resultPtr, resultOffset, #CRLF$+"; [file already included]" + #CRLF$)
          Else
            putS(*resultPtr, resultOffset, #CRLF$+";{ >>>>>>>>>>>>>>>>>>>>>>>> merging file "+includeFilename+#CRLF$)
            includeContent = processFile(includeFilename, GetPathPart(filepath))
            
            If indentLength
              includeContent = indentSub + ReplaceString(includeContent, #CRLF$, #CRLF$+indentSub) + #CRLF$
            EndIf
            
            ; make shure we have 1 empty chunk space left
            includeByteLen = StringByteLength(includeContent)
            If (resultOffset + includeByteLen) >= (resultByteLen - chunkSize)
              resultByteLen + includeByteLen + chunkSize
              *resultPtr = ReAllocateMemory(*resultPtr, resultByteLen)
            EndIf
            
            putS(*resultPtr, resultOffset, #CRLF$+includeContent+#CRLF$)
            putS(*resultPtr, resultOffset, ";}"+#CRLF$)
            putS(*resultPtr, resultOffset, ";  <<<<<<<<<<<<<<<<<<<<<<<< merging end  "+includeFilename+#CRLF$)
          EndIf
          includeMerge = #False
          include = #False
          includeFilename = ""
          includeX = #False
        EndIf
      EndIf
      
      sourceOffset + SizeOf(Character)
      
      If mergeEnd = #True
        Break
      EndIf
      
    Wend
    
    resultByteLen = resultOffset + SizeOf(Character) ; length is one more than last valid offset
    *resultPtr = ReAllocateMemory(*resultPtr, resultByteLen)
    
    resultContent = PeekS(*resultPtr, resultByteLen/SizeOf(Character) );, #PB_UTF16);), sourceBom)
    
    FreeMemory(*resultPtr)
    *resultPtr = #Null
    *sourcePtr = #Null
  Else
    PrintN(indent+"can't open file: "+filename)
  EndIf
  
  call - 1
  ProcedureReturn resultContent.s
EndProcedure

Procedure writeStringToFile(s.s, filename.s)
  Protected file
  file = CreateFile(#PB_Any, filename)
  If file
    WriteStringFormat(file, #PB_UTF8)
    WriteString(file, s, #PB_UTF8)
    CloseFile(file)
    PrintN("ouput file: ")
    PrintN(filename)
    ProcedureReturn 1
  Else
    PrintN("can't create file: " + filename)
    ProcedureReturn 0
  EndIf
EndProcedure

main()





; IDE Options = PureBasic 5.42 LTS (Windows - x64)
; ExecutableFormat = Console
; CursorPosition = 31
; Folding = --
; EnableUnicode
; EnableXP
; Executable = merger.exe
; Warnings = Error
; DisablePurifier = 1,1,1,1