﻿


;{ #################### sgx

;**
;* sgx is a screen-gadget library for PureBasic programmers to create and manage simple GUI-elements 
;* in a fullscreen or windowed screen. _
;* _
;* it's available as sourcecode to be included in your projects. it's currently only available for 
;* Windows Operating Systems (though the code is pretty straight PB code). there are two 
;* versions: a single-file version that only contains the necessary files to just use sgx and 
;* a multi-file version in case you want to make changes to the sgx code, including the gimp files 
;* for the image files and some tools to build a single-file version from that. _
;* _
;* with this library you can create simple GUI objects statically or dynamically of different 
;* type and appearance, modify them, display them easily, examine user actions and process events. 
;* sgx elements are accessed by ID (index-like, starting with index 1). the appearance is defined by 
;* designs, whereas you can create own designs with sets of images (there is also an instant design 
;* "lowbudget" which does not need any image files). currently there is no event queue, you will 
;* recognize events by checking for fields of your sgx elements. _
;* _
;* currently available gadget types: _
;* - <a href=#f_sgxcursor>sgxCursor</a> _
;* - <a href=#f_sgxbutton>sgxButton</a> (including toggle buttons and buttons with a user sprite) _
;* - <a href=#f_sgxtext>sgxText</a> _
;* - <a href=#f_sgxcheckbox>sgxCheckbox</a> _
;* - <a href=#f_sgxcombo>sgxCombo</a> _
;* - <a href=#f_sgxscrollbar>sgxScrollbar</a> _
;* - <a href=#f_sgxtrackbar>sgxTrackbar</a> _
;* - <a href=#f_sgxedit>sgxEdit</a> _
;* - <a href=#f_sgxwin>sgxWin</a> _
;* - <a href=#f_sgxtable>sgxTable</a> _
;* - <a href=#f_sgxspritegadget>sgxSpriteGadget</a> _
;* - <a href=#f_sgxarea>sgxArea</a> _
;* - <a href=#f_sgxsetgroup>option gadgets / radio buttons are available through grouping</a> _
;* _
;* _
;* the documentation includes _
;* - a quick start (sgxDoc\sgx.html) _
;* - a full reference (sgxDoc\sgxDocFull.html) _
;* - a smaller reference (sgxDoc\sgxDoc.html) without sgx internals (some links to sgx internals may not work in that
;* document thought!) _
;* - examples in the directory 'sgxDoc\' _
;* - _README.TXT contains further informations _
;* _
;* the references are rather technical, so use the quickstart and the examples to get the basic mechanics of sgx. _
;* _
;* _
;* all global identifiers start with "sgx.." or "*sgx.." or "#sgx.." _
;* globals, functions, macros, constants (#..) and pointer (*..) starting with "sgx_.." (with an underscore) as are 
;* basically used internally. _
;* structures (sgxS_..) are also basically used internally but the documentation of <a href=#s_sgxs_gadget>sgxS_GADGET</a> 
;* for example might be usefull nonetheless. _
;* constants are only documented by the documentation of the functions etc. whith which they are to be used. _
;* sgx uses implicit integer type definitions, so don't change the default type before including sgx. _
;* if you used DisableDebugger before including sgx you should repeat it after the include statement because the debugger
;* will be enabled. _
;* _
;* _
;* thanks to the PureBasic team and the community / the forums. _
;* _
;* [this document was blithely generated with jaPBdoc :) ] _


; heeere
; Procedure dummy()
;   ProcedureReturn 0
; EndProcedure
; Macro DrawText(x,y,t,fc=0,bc=0)
;   dummy()
; EndMacro



;  *** SGX ***

UsePNGImageDecoder()

;OpenConsole("sgx console")

;EnableExplicit

; XIncludeFile "sgx_ll.pbi"
;{ >>>>>>>>>>>>>>>>>>>>>>>> merging file sgx_ll.pbi

        
        ; ----------------------- basic tree list functionality
        
        
        
        ;** s sgx_llS_elem
        ;* type of the elements in an <a href=#s_sgx_lls_list>sgx_llS_list</a>. used for a basic tree list to represent the sgx element hierarchy. _
        ;* type of the field '*treeElem' of the structure <a href=#s_sgxs_gadget>sgxS_GADGET</a>. _
        ;* this type allows the use of lists as a tree, since it has a list it belongs to and a child list (branch) as well. _
        ;* lists are created via constructor macro <a href=#f_sgx_llnew>sgx_llnew</a>. _
        ;* elements are added to a list via <a href=#f_sgx_lladd>sgx_llAdd</a> and removed from a list via <a href=#f_sgx_lldelelem>sgx_llDelElem</a>. _
        ;* a list can be cleared via <a href=#f_sgx_llclearlist>sgx_llClearList</a>. _
        ;* an element can be moved to the end of its list via <a href=#f_sgx_llmoveend>sgx_llMoveEnd</a>. _
        ;* you can print a tree via <a href=#f_sgx_lldebug>sgx_llDebug</a>. _
        ;* a list element can be removed from one list and be recreated in another list using <a href=#f_sgx_lldelelem>sgx_llDelElem</a> in combination with <a href=#f_sgx_llreplaceelemnode>sgx_llReplaceElemNode</a>. _
        ;* _
        ;* there are several iterator macros to traverse a list/tree: _
        ;* be carefull with Static iterator variables in recursively called functions (even with the flat iterators) or better use Protected for your iterator variable. _
        ;* also do not use Break or Continue in such loops! _
        ;* flat iteration
        ;* <pre>
        ;* <span></span>sgx_llIterator(Define *it, *sgxLayeredView) _
        ;* <span></span>  Debug sgx(*it\id)\text _
        ;* <span></span>sgx_llNext(*it) _
        ;* </pre>
        ;* flat reverse iteration (from last to first element)
        ;* <pre>
        ;* <span></span>sgx_llIteratorReverse(Define *it, *sgxLayeredView) _
        ;* <span></span>  Debug sgx(*it\id)\text _
        ;* <span></span>sgx_llPrev(*it) _
        ;* </pre>
        ;* recursive iteration
        ;* (here we use protected - imagine we are inside a function - and we start at a window's child node instead of the sgx root node)
        ;* <pre>
        ;* <span></span>sgx_llIteratorRecursive(Protected *it, sgx(window)\treeElem\node) _
        ;* <span></span>  Debug sgx(*it\id)\text _
        ;* <span></span>sgx_llNextRecursive(*it) _
        ;* </pre>
        Structure sgx_llS_elem ;{
          *list.sgx_llS_list  ;* pointer to the list the element belongs to
          *next.sgx_llS_elem  ;* pointer to the element's successor or 0 if the element is the last in its list
          *prev.sgx_llS_elem  ;* pointer to the element's predecessor or 0 if the element is the first in its list
          *node.sgx_llS_list  ;* pointer to the element's list of childs
          id.i                ;* the actual data of the element, an sgx element ID in our case
        ;}
        EndStructure
        
        
        ;** s sgx_llS_list
        ;* the list structure of the (tree) list implementation. _
        ;* lists of this type are created via constructor macro <a href=#f_sgx_llnew>sgx_llnew</a>. _
        ;* type of variables '*sgxLayeredView' and '*sgx_CurrentParent'. _
        ;* _
        ;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
        Structure sgx_llS_list ;{
          *curr .sgx_llS_elem  ;* pointer to the current element of the list or 0 if list is empty
          *first.sgx_llS_elem  ;* pointer to the first element in the list or 0 if list is empty
          *last .sgx_llS_elem  ;* pointer to the last element in the list or 0 if list is empty
        ;}
        EndStructure
        
        
        ;** sgx_llnew
        ;* constructor macro for creation of <a href=#s_sgx_lls_list>sgx_llS_list</a> lists. _
        ;* for example: _
        ;* <pre>
        ;* Global *myList.sgx_llnew _
        ;* </pre>
        ;* will be expande to this: _
        ;* <pre>
        ;* Global *myList.sgx_llS_list = AllocateMemory(SizeOf(sgx_llS_list)) _
        ;* </pre>
        ;* 
        ;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
        Macro sgx_llnew
          sgx_llS_list = AllocateMemory(SizeOf(sgx_llS_list))
        EndMacro
        
        
        ;** s sgx_llS_IteratorStack
        ;* a list wrapper structure. type of 'sgx_llStackOfIteratorStacks'. _
        ;* _
        ;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
        Structure sgx_llS_IteratorStack
          List *iteratorStack.sgx_llS_elem()
        EndStructure
        
        ;** g sgx_llStackOfIteratorStacks
        ;* a stack of stacks of type <a href=#s_sgx_lls_iteratorstack>sgx_llS_IteratorStack</a> used by <a href=#f_sgx_lliteratorrecursive>sgx_llIteratorRecursive</a>. _
        ;* sgx_llStackOfIteratorStacks()\iteratorStack() is a list (stack) of <a href=#s_sgx_lls_elem>sgx_llS_elem</a> elements which get pushed and popped during a recursive iteration. _
        ;* sgx_llStackOfIteratorStacks() itself is a wrapper stack for the latter and gets pushed and popped with the beginning and end of each such a recursive iteration block (for example in case within the block a function gets called which uses a recursive iterator itself). _
        ;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list. _
        Global NewList sgx_llStackOfIteratorStacks.sgx_llS_IteratorStack()
        
        ; ------------------------
        
        
        ;** sgx_llIterator
        ;* flat iterator macro (without recursion) to iterate over all list elements. _
        ;* use <a href=#f_sgx_llnext>sgx_llNext</a> to close such an iteration block. _
        ;* do not use Break or Continue in such loops! _
        ;* _
        ;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
        Macro sgx_llIterator(it,List)
          If List\first
            it.sgx_llS_elem = List\first
            Repeat
        EndMacro
        
        ;** sgx_llNext
        ;* the 'Next' command for an <a href=#f_sgx_lliterator>sgx_llIterator</a> block. _
        ;* _
        ;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
        Macro sgx_llNext(it)
            it = it\next
            Until it=0
          EndIf
        EndMacro
        
        ; ------------------------
        
        ;** sgx_llIteratorReverse
        ;* flat reverse iterator macro (without recursion) to iterate over all list elements backwards. _
        ;* use <a href=#f_sgx_llprev>sgx_llPrev</a> to close such an iteration block. _
        ;* do not use Break or Continue in such loops! _
        ;* _
        ;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
        Macro sgx_llIteratorReverse(it,List)
          If List\last
            it.sgx_llS_elem = List\last
            Repeat
        EndMacro
        
        ;** sgx_llPrev
        ;* the 'Next' (previous) command for an <a href=#f_sgx_lliteratorreverse>sgx_llIteratorReverse</a> block. _
        ;* _
        ;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
        Macro sgx_llPrev(it)
            it = it\prev
            Until it=0
          EndIf
        EndMacro
        
        ; ------------------------
        
        ;** sgx_llIteratorRecursive
        ;* recusrive iterator macro to iterate over all list elements including the elements child nodes. _
        ;* use <a href=#f_sgx_llnextrecursive>sgx_llNextRecursive</a> to close such an iteration block. _
        ;* do not use Break or Continue in such loops! _
        ;* _
        ;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
        Macro sgx_llIteratorRecursive(it,List)
          If List\first
            ; push a new iterator stack in case of nested recursive iterator blocks
            AddElement(sgx_llStackOfIteratorStacks())
            it.sgx_llS_elem = List\first
            Repeat
        EndMacro
        
        ;** sgx_llNextRecursive
        ;* the 'Next' command for an <a href=#f_sgx_lliteratorrecursive>sgx_llIteratorRecursive</a> block. _
        ;* with the parameter 'skipNode' the recursion can be controlled during the loop conditionally. used for example to constrain recursion to non-window sgx childs. _
        ;* _
        ;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
        Macro sgx_llNextRecursive(it, skipNode=#False)
              it = sgx_llNextRecursive_proc(it, skipNode)
            Until it=0
            ; clear and pop our current iterator stack
            LastElement(sgx_llStackOfIteratorStacks())
            ClearList(sgx_llStackOfIteratorStacks()\iteratorStack())
            DeleteElement(sgx_llStackOfIteratorStacks())
          EndIf
        EndMacro
        
        ;** sgx_llNextRecursive_proc
        ;* used by the <a href=#f_sgx_llnextrecursive>sgx_llNextRecursive</a> macro for recursion. _
        ;* _
        ;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
        Procedure sgx_llNextRecursive_proc(*it.sgx_llS_elem, skipNode=#False)
        ;   Protected *iteratorStack.sgx_llS_elem
        ;   Protected *temp.sgx_llS_elem
          
        ;   *iteratorStack = @sgx_llStackOfIteratorStacks()\iteratorStack()
          
          If *it
            
            ; step into recursion
            If *it\node And *it\node\first And skipNode=#False
              ; push current iterator
              AddElement(sgx_llStackOfIteratorStacks()\iteratorStack())
              sgx_llStackOfIteratorStacks()\iteratorStack()=*it
              ; and return sublist's first element as new iterator
              ProcedureReturn *it\node\first
              
            ; just proceed in current list
            ElseIf *it\next
              ProcedureReturn *it\next
              
            ; step out of recursion
            ; and make shure the last pushed element is the current one
            ElseIf LastElement(sgx_llStackOfIteratorStacks()\iteratorStack())
              ; proceed with next element on the previous level
              *it = sgx_llStackOfIteratorStacks()\iteratorStack()\next
              ; and pop that iterator from the stack
              DeleteElement(sgx_llStackOfIteratorStacks()\iteratorStack())
              ; continue to pop levels until we find one with a next element in it
              While ListSize(sgx_llStackOfIteratorStacks()\iteratorStack()) And *it=0
                LastElement(sgx_llStackOfIteratorStacks()\iteratorStack())
                *it = sgx_llStackOfIteratorStacks()\iteratorStack()\next
                DeleteElement(sgx_llStackOfIteratorStacks()\iteratorStack())
              Wend
              ProcedureReturn *it
            EndIf
          Else
            ProcedureReturn 0
          EndIf
        EndProcedure
        
        ; ------------------------
        
        Declare sgx_llDelElem(*elem.sgx_llS_elem, returnNode=#False)
        Declare sgx_llDebug(*list.sgx_llS_list)
        
        
        ;** sgx_llAdd
        ;* creates a new element at the end of the list and sets its data field to 'id'. _
        ;* _
        ;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
        Procedure sgx_llAdd(*list.sgx_llS_list, id)
          If *list\first = 0
            *list\curr = AllocateMemory(SizeOf(sgx_llS_elem))
            *list\curr\id = id
            *list\curr\list = *list
            *list\first = *list\curr
            *list\last = *list\curr
          Else
            ;!expects curr = last
            *list\curr=*list\last
            *list\curr\next = AllocateMemory(SizeOf(sgx_llS_elem))
            *list\curr\next\prev = *list\curr
            *list\curr = *list\curr\next
            *list\curr\id = id
            *list\curr\list = *list
            If *list\last = *list\curr\prev
              *list\last = *list\curr
            EndIf
          EndIf
          ProcedureReturn *list\curr
        EndProcedure
        
        
        ;** sgx_llClearList
        ;* removes all elements from the list which will remove sublevels too. the list will still remain a valid but empty list.
        ;* _
        ;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
        Procedure sgx_llClearList(*list.sgx_llS_list)
          If *list
            While *list\first
              sgx_llDelElem(*list\first)
            Wend
          EndIf
        EndProcedure
        
        
        ;** sgx_llDelElem
        ;* remove the element from its list and free it and its childs. _
        ;* if 'returnNode' is #True, the subtree is preserved and returned. 
        ;* you can thus create a new element in another list and use the 
        ;* node with sgx_llReplaceElemNode, basically moving the element 
        ;* to another list. but remember to also transfer the data field 'id' which 
        ;* is not preserved. <a href=#f_sgxsetparent>sgxSetParent</a> makes use of this. _
        ;* _
        ;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
        Procedure sgx_llDelElem(*elem.sgx_llS_elem, returnNode=#False)
          Protected *returnNode.sgx_llS_list
          
          If *elem
            
            ; handle the subtree (childs)
            If returnNode
              *returnNode = *elem\node
              *elem\node=0
            Else
              If *elem\node
                ; free childs
                sgx_llClearList(*elem\node)
                FreeMemory(*elem\node)
                *elem\node=0
              EndIf
            EndIf
            
            ; handle the list of which this element is part of
            If *elem\list\last = *elem\list\first ; only one element in the list
              ; reset all links in the list
              *elem\list\curr=0
              *elem\list\first=0
              *elem\list\last=0
            Else ; at least 2 elements in list
              If *elem = *elem\list\last ; if its the last element, reset the predecessor's link and make the predecessor the last
                *elem\prev\next = 0
                *elem\list\last = *elem\prev
              ElseIf *elem = *elem\list\first ; if its the first element, reset the successor's link and make the successor the first
                *elem\next\prev = 0
                *elem\list\first = *elem\next
              Else ; if its elsewhere in the list, link the predecessor and successor to each other
                *elem\next\prev = *elem\prev
                *elem\prev\next = *elem\next
              EndIf
            EndIf
            FreeMemory(*elem)
            *elem=0
            
            If returnNode
              ProcedureReturn *returnNode
            EndIf
          EndIf
          
          ProcedureReturn 0
        EndProcedure
        
        
        ;** sgx_llReplaceElemNode
        ;* remove and replace the node (list of childs) of a list element. used in <a href=#f_sgxsetparent>sgxSetParent</a> for example. _
        ;* _
        ;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
        Procedure sgx_llReplaceElemNode(*elem.sgx_llS_elem, *newNode.sgx_llS_list)
          
          ; free current node
          If *elem\node
            ; free childs
            sgx_llClearList(*elem\node)
            FreeMemory(*elem\node)
            *elem\node=0
          EndIf
          
          ; replace the node pointer
          *elem\node = *newNode
          
          ; update the list pointer of the elements in the node
          If *elem\node
            sgx_llIterator(Protected *it, *elem\node)
              *it\list = *elem\node
            sgx_llNext(*it)
          EndIf
        EndProcedure
        
        
        ;** sgx_llMoveEnd
        ;* move the element to the end of its list. used in <a href=#f_sgxmoveontop>sgxMoveOnTop</a> for example. _
        ;* _
        ;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
        Procedure sgx_llMoveEnd(*elem.sgx_llS_elem)
          If *elem <> *elem\list\last
            If *elem = *elem\list\first
              *elem\list\first = *elem\next
            Else
              *elem\prev\next = *elem\next
            EndIf
            *elem\next\prev = *elem\prev
            *elem\list\last\next = *elem
            *elem\prev = *elem\list\last
            *elem\list\last = *elem
            *elem\next = 0
          EndIf
          ProcedureReturn #True
        EndProcedure
        
        
        
        ;** sgx_llDebug
        ;* prints the list/tree hierarchy recursively to the debug output window (directly, not return a string). _
        ;* _
        ;* see <a href=#s_sgx_lls_elem>sgx_llS_elem</a> for an overview of the tree list.
        Procedure sgx_llDebug(*list.sgx_llS_list)
          Static level
          
          sgx_llIterator(Protected *it, *list)
            If *it\node
              Debug LSet("",level*4," ") + Str(*it\id) + " {"
              level+1
              sgx_llDebug(*it\node)
              level-1
              Debug LSet("",level*4," ")  + "}"
            Else
              Debug LSet("",level*4," ") + Str(*it\id) ; + " {}"
            EndIf
          sgx_llNext(*it)
          
        EndProcedure
        
        ; [merge_end]
        

;}
;  <<<<<<<<<<<<<<<<<<<<<<<< merging end  sgx_ll.pbi

; XIncludeFile "sgx_define.pbi"
;{ >>>>>>>>>>>>>>>>>>>>>>>> merging file sgx_define.pbi

        
        
        
        ;-# ENUM / CONST ###############################################################
        
        
        ;{- element types
          #sgxTypeButton       =  1<<0
          #sgxTypeButtonToggle =  1<<1
          #sgxTypeCursor       =  1<<2
          #sgxTypeCheckbox     =  1<<3
          #sgxTypeScrollbarVer =  1<<4
          #sgxTypeScrollbarHor =  1<<5
          #sgxTypeScrollbar    =  #sgxTypeScrollbarVer | #sgxTypeScrollbarHor
          #sgxTypeTrackbarVer  =  1<<6
          #sgxTypeTrackbarHor  =  1<<7
          #sgxTypeTrackbar     =  #sgxTypeTrackbarVer | #sgxTypeTrackbarHor
          #sgxTypeEdit         =  1<<8
          #sgxTypeWin          =  1<<9
          #sgxTypeSpriteGadget =  1<<10
          #sgxTypeCombo        =  1<<11
          #sgxTypeTable        =  1<<12
          #sgxTypeText         =  1<<13
          #sgxTypeArea         =  1<<14
        ;}
        ; -
        
        
        Enumeration 1 ;{ alignment constants
          #sgxAlignCheckboxTextRight
          #sgxAlignCheckboxTextLeft
          #sgxAlignTextLeft
          #sgxAlignTextRight
          #sgxAlignTextCenter
          #sgxAlignTextJustifySpaces
          #sgxAlignTextJustifyCharacters
        ;}
        EndEnumeration
        
        
        Enumeration ;{ constants for design images: <sgxS_Design>\img[i]\..
          #sgx_DesignBackgroundNormal
          #sgx_DesignBackgroundHover
        
          #sgx_DesignCloseGadgetNormal
          #sgx_DesignCloseGadgetHover
          
          #sgx_DesignCheckboxChecked
        
          #sgx_DesignBorderNormalCorner1
          #sgx_DesignBorderNormalCorner2
          #sgx_DesignBorderNormalCorner3
          #sgx_DesignBorderNormalCorner4
          
          #sgx_DesignBorderHoverCorner1
          #sgx_DesignBorderHoverCorner2
          #sgx_DesignBorderHoverCorner3
          #sgx_DesignBorderHoverCorner4
          
          #sgx_DesignBorderPushedCorner1
          #sgx_DesignBorderPushedCorner2
          #sgx_DesignBorderPushedCorner3
          #sgx_DesignBorderPushedCorner4
        
          #sgx_DesignBorderNormalHor1
          #sgx_DesignBorderNormalHor2
          #sgx_DesignBorderNormalVer1
          #sgx_DesignBorderNormalVer2
          
          #sgx_DesignBorderHoverHor1
          #sgx_DesignBorderHoverHor2
          #sgx_DesignBorderHoverVer1
          #sgx_DesignBorderHoverVer2
          
          #sgx_DesignBorderPushedHor1
          #sgx_DesignBorderPushedHor2
          #sgx_DesignBorderPushedVer1
          #sgx_DesignBorderPushedVer2
        
          #sgx_DesignMouseCursor
        ;}
        EndEnumeration
        
        
        Enumeration ;{ constants for different refresh tasks (see sgxRefresh())
          #sgx_refreshNormalWithTip
          #sgx_refreshOnlyTip
          #sgx_refreshNormalWithChildsOfTables
          #sgx_refreshSkipTableCheckboxRedraw
          #sgx_refreshEditMouseclickCursor
          #sgx_refreshEditMousewheelScroll
          ;}
        EndEnumeration
        
        
        ;{- constants for gadget sprites: sgx(id)\spr[sprType]
        #sgx_GadgetSprTip         = -1 ; (not a real sprite array index but used as sprType for custom draw callbacks)
        #sgx_GadgetSprNormal      = 0
        #sgx_GadgetSprHover       = 1
        #sgx_GadgetSprPushed      = 2  :  #sgx_GadgetSprChecked = 2  :  #sgx_GadgetSprTextCursor = 2
        #sgx_GadgetSprNormal2     = 3
        #sgx_GadgetSprHover2      = 4
        #sgx_GadgetSprHoverPushed = 5
        #sgx_GadgetSprResize      = 6
        ;}
        ; -
        
        
        
        ;{- style flags
        ;#sgxStyleWin        = 1 << 0
        ;#sgxStylePanel      = 1 << 1
        #sgxStyleThin       = 1 << 2
        #sgxStyleHover      = 1 << 3
        #sgxStyleHoverThin  = #sgxStyleHover | #sgxStyleThin
        #sgxStyleGridV      = 1 << 4
        #sgxStyleGridH      = 1 << 5
        #sgxStyleGrid       = #sgxStyleGridV | #sgxStyleGridH
        #sgxStyleFitCellW   = 1 << 6
        #sgxStyleFitCellH   = 1 << 7
        #sgxStyleNoBG       = 1 << 8
        #sgxStyleReverse    = 1 << 9
        #sgxStyleResizable  = 1 << 10
        ;}
        ; -
        
        
        ;{- auto adaptation flags (for element creation)
        #sgx_AutoW       = 1 << 1
        #sgx_AutoH       = 1 << 2
        #sgx_AutoNL      = 1 << 3
        ;}
        ; -
        
        Enumeration 1 ;{ constants for sgxRightclickUnset configuration
          #sgxRightclickUnsetNone
          #sgxRightclickUnsetNoGroup
          #sgxRightclickUnsetAll
        ;}
        EndEnumeration
        
        
        Enumeration 1 ;{ state constants (for element creation)
          #sgxStateNone
          #sgxStateOff
          #sgxStateOn
        ;}
        EndEnumeration
        
        
        Enumeration 1 ;{ constants for drawing/refreshing operations
          #sgx_AppearanceNormal
          #sgx_AppearancePushed
          #sgx_AppearanceHover
          #sgx_SideTopOrLeft
          #sgx_SideBottomOrRight
        ;}
        EndEnumeration
        
        
        
        Enumeration ;{ constants for window states
          #sgxWinStateNormal
          #sgxWinStateMinimizing
          #sgxWinStateMinimized
          #sgxWinStateRestoring
          #sgxWinStateClosing
          #sgxWinStatePresenting
        ;}
        EndEnumeration
        
        
        Enumeration ;{ constants for textcursor types
          #sgxTextcursorTypeIBeam
          #sgxTextcursorTypeTeeth
          #sgxTextcursorTypeLine
          #sgxTextcursorTypeLine2
          #sgxTextcursorTypeLine3
          #sgxTextcursorTypeCaret
        ;}
        EndEnumeration
        
        
        ;{ flags for window animation styles
        #sgxAnimFade       = 1 << 1
        #sgxAnimSizeW      = 1 << 2
        #sgxAnimSizeH      = 1 << 3
        #sgxAnimSize       = #sgxAnimSizeW | #sgxAnimSizeH
        #sgxAnimRoll       = 1 << 4
        #sgxAnimZoom       = 1 << 5
        ;}
        
        
        ;{ flags for drawing callback return values
        #sgxDrawNone         = 1 << 1
        #sgxDrawDesign       = 1 << 2
        #sgxDrawText         = 1 << 3
        #sgxDrawTextNoOffset = 1 << 4
        ;}
        
        
        Enumeration 0 ;{ constants for keyboard layouts
          #sgxKeyLayQwerty
          #sgxKeyLayQwertz
        ;}
        EndEnumeration
        
        
        
        
        
        CompilerIf 0=Defined(sgxMaxStaticGagdets, #PB_Constant)
          #sgxMaxStaticGagdets = 500 ;* the numbers of sgx element you can create with static IDs (maximum ID). dynamic IDs start from there on. you can set this constant before including sgx, or use the default 500.
        CompilerEndIf
        
        
        
        
        
        
        
        
        
        
        
        
        
        ;-# STRUCTURES ###############################################################
        
        
        ;** s sgx_S_Point
        ;* stores a x/y position _
        ;* type of field 'textcursor' of the structure <a href=#s_sgxs_gadget>sgxS_GADGET</a> _
        ;* type of variable sgx_vec _
        ;* type of variable sgx_textCursorClippingOffset _
        Structure sgx_S_Point ;{
          x.i  ;* x-position
          y.i  ;* y-position
          ;}
        EndStructure
        
        
        ;** s sgx_S_Size
        ;* type of variable sgx_dimensions wich stores the screen diemsions
        Structure sgx_S_Size ;{
          w.i ; width (of screen)
          h.i ; height (of screen)
          ;}
        EndStructure
        
        
        ;** s sgx_S_Image
        ;* type of the array elements of field 'img' in the design structure <a href=#s_sgxs_design>sgxS_Design</a>
        Structure sgx_S_Image ;{
          nr.i  ;* the PB image number
          w.i   ;* width of the image
          h.i   ;* height of the image
          ;}
        EndStructure
        
        
        Prototype sgx_onBeginDraw(id, sprType, w, h, disabled=#False)
        Prototype sgx_onEndDraw  (id, sprType, w, h, disabled=#False)
        
        ;** s sgxS_Design
        ;* the basic structure of an sgx design. _
        ;* stored in field '*design' of the structure <a href=#s_sgxs_gadget>sgxS_GADGET</a>. _
        ;* returned by <a href=#f_sgxloaddesign>sgxLoadDesign</a> if the loading succeeds. _
        ;* type of list sgxDesign(). _
        ;* type of variable *sgx_CurrentDesign and can be passed to <a href=#f_sgxsetdesign>sgxSetDesign</a>. _
        ;* _
        ;* the fields onBeginDraw and onEndDraw are of the following type:
        ;* <pre> 
        ;* Prototype sgx_onBeginDraw(id, sprType, w, h, disabled=#False) _
        ;* Prototype sgx_onEndDraw  (id, sprType, w, h, disabled=#False) 
        ;* </pre> 
        ;* you can assign function pointers directly to these fields if you have defined a function that should be called during
        ;* the drawing proccess of an sgx element. they will be called inside a StartDrawing/StopDrawing block and the current
        ;* drawing output will already have been cleared with transparent black when onBeginDraw() is called. the parameters
        ;* passed to the callbacks include the ID of the sgx element and the type of the sprite (#sgx_GadgetSpr..) that is currently
        ;* being redrawn as well as the width and height of the current drawing output and if the sprite should visualize a disabled
        ;* state, in which case the sprite will not have been cleared but instead contains the appearance of the enabled state of
        ;* that sprite type which can be modified to make it look disabled (or cleared and redrawn if you wish). _
        ;* make sure you use a drawing mode with alpha channel and use colors that include alpha information, otherwise your drawing
        ;* will stay invisible. _
        ;* _
        ;* onBeginDraw() can be used to replace steps in the redraw process. _
        ;* with the return value you can define what sgx should draw after the call of your callback: <ul> 
        ;* <li> #sgxDrawDesign | #sgxDrawText : sgx should draw everything as usual: border/background/text etc. </li> 
        ;* <li> #sgxDrawDesign                : draw only the design parts (border/background etc.) but no text </li> 
        ;* <li> #sgxDrawText                  : don't draw design parts (border/background etc.) but draw text </li> 
        ;* <li> #sgxDrawNone                  : don't draw anything </li> 
        ;* <li> you can use #sgxDrawTextNoOffset instead of #sgxDrawText to let the text be drawn without position offset for pushed buttons </li> 
        ;* </ul> _
        ;* onEndDraw() can be used to just add or overdraw something after sgx has finished it's drawing operations. 
        ;* it works just like onBeginDraw() except you don't have to return anything. _
        ;* _
        ;* the sprite types #sgx_GadgetSprChecked and #sgx_GadgetSprTextCursor have the same value as #sgx_GadgetSprPushed, so checking
        ;* for these two would make it necessary to check the element type also (checkbox or edit). _
        ;* all element types might also be called with a sprite type of #sgx_GadgetSprTip, to redraw the tooltip of the element. _
        ;* other sprite types can differ depending on the element type _
        ;* _
        ;* here is an example in form of an onBeginDraw() custom callback that shows how you could handle the different cases. _
        ;* it could be also much simpler though, for example by handling only the tooltip case and return the default in all other cases _
        ;* <pre>
        ;* <span></span>Procedure myOnBeginDraw(id, sprType, w, h, disabled=#False) _
        ;* <span></span>  Static retval _
        ;* <span></span>   _
        ;* <span></span>  ; our default returnvalue (sgx should draw everthing as usual after this function has finished) _
        ;* <span></span>  retval = #sgxDrawDesign | #sgxDrawText _
        ;* <span></span>   _
        ;* <span></span>  If sprType = #sgx_GadgetSprTip _
        ;* <span></span>    ; the element's tooltip sprite is currently redrawn. _
        ;* <span></span>    ; you might want to handle this as a generel case, independent from element type. _
        ;* <span></span>     _
        ;* <span></span>  ElseIf sprType = #sgx_GadgetSprResize _
        ;* <span></span>    ; the element's resize gadget is currently being redrawn. _
        ;* <span></span>    ; you might want to handle this as a generel case, independent from element type. _
        ;* <span></span>  Else _
        ;* <span></span>     _
        ;* <span></span>    ; the other sprite types are element type dependent _
        ;* <span></span>     _
        ;* <span></span>    If disabled _
        ;* <span></span>      ; the drawing output already contains the visuals for that sprite type and should be _
        ;* <span></span>      ; modified (or redrawn completely) to visualize a disabled state. you could also return _
        ;* <span></span>      ; the default returnvalue and do the modification in an onEndDraw callback. _
        ;* <span></span>    EndIf _
        ;* <span></span>     _
        ;* <span></span>    Select sgx(id)\type _
        ;* <span></span>         _
        ;* <span></span>      Case #sgxTypeCursor _
        ;* <span></span>        ; sprType: _
        ;* <span></span>        ; #sgx_GadgetSprNormal _
        ;* <span></span>         _
        ;* <span></span>      Case #sgxTypeButton, #sgxTypeButtonToggle _
        ;* <span></span>        ; sprType: _
        ;* <span></span>        ; #sgx_GadgetSprNormal _
        ;* <span></span>        ; #sgx_GadgetSprHover _
        ;* <span></span>        ; #sgx_GadgetSprPushed _
        ;* <span></span>        ; #sgx_GadgetSprHoverPushed _
        ;* <span></span>      Case #sgxTypeCheckbox _
        ;* <span></span>        ; sprType: _
        ;* <span></span>        ; #sgx_GadgetSprNormal _
        ;* <span></span>        ; #sgx_GadgetSprHover _
        ;* <span></span>        ; #sgx_GadgetSprChecked _
        ;* <span></span>      Case #sgxTypeScrollbarVer, #sgxTypeScrollbarHor, #sgxTypeTrackbarVer, #sgxTypeTrackbarHor _
        ;* <span></span>        ; sprType: _
        ;* <span></span>        ; #sgx_GadgetSprNormal  ; - bar _
        ;* <span></span>        ; #sgx_GadgetSprHover   ; / _
        ;* <span></span>        ; #sgx_GadgetSprNormal2 ; - knob _
        ;* <span></span>        ; #sgx_GadgetSprHover2  ; / _
        ;* <span></span>      Case #sgxTypeEdit _
        ;* <span></span>        ; sprType: _
        ;* <span></span>        ; #sgx_GadgetSprNormal _
        ;* <span></span>        ; #sgx_GadgetSprHover _
        ;* <span></span>        ; #sgx_GadgetSprTextCursor _
        ;* <span></span>      Case #sgxTypeWin _
        ;* <span></span>        ; sprType: _
        ;* <span></span>        ; #sgx_GadgetSprNormal _
        ;* <span></span>        ; #sgx_GadgetSprHover _
        ;* <span></span>        ; #sgx_GadgetSprPushed  ; displayed if the window is foreground or catched focus (i.e. when moving the window manually) _
        ;* <span></span>        ; #sgx_GadgetSprNormal2 ; - close gadget _
        ;* <span></span>        ; #sgx_GadgetSprHover2  ; / _
        ;* <span></span>      Case #sgxTypeTable _
        ;* <span></span>        ; sprType: _
        ;* <span></span>        ; #sgx_GadgetSprNormal _
        ;* <span></span>        ; #sgx_GadgetSprHover _
        ;* <span></span>      Case #sgxTypeCombo _
        ;* <span></span>        ; sprType: _
        ;* <span></span>        ; #sgx_GadgetSprNormal _
        ;* <span></span>        ; #sgx_GadgetSprHover _
        ;* <span></span>        ; #sgx_GadgetSprPushed ; the opened combo _
        ;* <span></span>        ; #sgx_GadgetSprHover2 ; open combo hover effect _
        ;* <span></span>      Case #sgxTypeText _
        ;* <span></span>        ; sprType: _
        ;* <span></span>        ; #sgx_GadgetSprNormal _
        ;* <span></span>        ; #sgx_GadgetSprHover _
        ;* <span></span>    EndSelect _
        ;* <span></span>  EndIf _
        ;* <span></span>   _
        ;* <span></span>  ; if you have actually drawn something in one of the branches above then your returnvalue _
        ;* <span></span>  ; should not include the flag #sgxDrawDesign, otherwise sgx will paint over your content. _
        ;* <span></span>  ProcedureReturn retval _
        ;* <span></span>EndProcedure _
        ;* </pre>
        Structure sgxS_Design ;{
          name.s                       ;* name or path of the design like "lowbudget" or "..\sgxDesigns\blue" for example
          basecolor.i                  ;* basecolor of the design as specified at <a href=#f_sgxloaddesign>sgxLoadDesign</a> or as defined in the design's 'parts' image file
          c.i[20]                      ;* the design pallete (populated in <a href=#f_sgx_makepalette>sgx_MakePalette</a>)
          img.sgx_S_Image[30]          ;* array of the design images of type <a href=#s_sgx_s_image>sgx_S_Image</a> that will be drawn on the sgx element sprites during a refresh like the parts of the border or the background for example; accessed with #sgx_Design.. constants as indices
          onBeginDraw.sgx_onBeginDraw  ;* optional custom draw callback called at the beginning of element redraw (see structure description)
          onEndDraw.sgx_onEndDraW      ;* optional custom draw callback called at the end of element redraw (see structure description)
          ;}
        EndStructure
        
        
        ;** s sgx_S_Box
        ;* type of the field 'sys0' of the structure <a href=#s_sgxs_gadget>sgxS_GADGET</a> to store the coordinates of the close gadget of windows; used in <a href=#f_sgx_mousehoverclosebutton>sgx_MousehoverClosebutton</a>
        Structure sgx_S_Box ;{
          x1.i  ;* x-position of close the gadgets left edge relative to the element's x-position
          y1.i  ;* y-position of close the gadgets top edge relative to the element's y-position
          x2.i  ;* x-position of close the gadgets right edge relative to the element's x-position
          y2.i  ;* y-position of close the gadgets bottom edge relative to the element's y-position
        ;}
        EndStructure
        
        
        ;** s sgx_S_metrics
        ;* stores temporary element metrics; used in <a href=#f_sgxrefresh>sgxRefresh</a> for refresh of childs of tables to 
        ;* avoid unnecessary refresh of child
        Structure sgx_S_metrics
          id.i  ;* ID of the sgx element
          w.i   ;* width
          h.i   ;* height
          w2.i  ;* secondary width (type dependent)
          h2.i  ;* secondary height (type dependent)
        EndStructure
        
        
        ;** s sgxS_Mouse
        ;* type of variable sgx_m to store various mouse states
        Structure sgxS_Mouse ;{
          x.i              ;* x-position of mouse on screen
          y.i              ;* y-position of mouse on screen
          dx.i             ;* delta x of the mouse position
          dy.i             ;* delta y of the mouse position
          dlimit.i         ;* the x/y delta of the mouse position that must occure to make an currently shown sgx tooltip disappear again
          
          leftclick.i      ;* 1 if the left mouse button has been pushed and was not pushed previously, otherwise 0
          leftdown.i       ;* 1 if the left mouse button is currently pushed, otherwise 0
          leftup.i         ;* 1 if the left mouse button has been released, otherwise 0
          
          rightclick.i     ;* 1 if the right mouse button has been pushed and was not pushed previously, otherwise 0
          rightdown.i      ;* 1 if the right mouse button is currently pushed, otherwise 0
          rightup.i        ;* 1 if the right mouse button has been released, otherwise 0
        
          leftdown_prev.i  ;* 1 if the left mouse button was pushed previously, otherwise 0
          rightdown_prev.i ;* 1 if the right mouse button was pushed previously, otherwise 0
        
          wheel.i          ;* the amount of steps the mousewheel has ben rotated
          wheelSign.i      ;* 1 if 'wheel' is positive or 0 (scrolling 'up' or not scrolling), -1 if 'wheel' is negative (scrolling 'down')
          cursor.i         ;* ID of the current sgx cursor element used as the mouse cursor or -1 if no mouse cursor is set (if none has been created or if it has been freed). will be set by <a href=#f_sgxcursor>sgxCursor</a>.
          ;}
        EndStructure
        
        
        ;** s sgxS_groupElem
        ;* data stored as elements of list sgxGroups; associates sgx element IDs to group IDs via <a href=#f_sgxsetgroup>sgxSetGroup</a>
        Structure sgxS_groupElem ;{
          id.i        ;* 
          groupID.i   ;* 
          ;}
        EndStructure
        
        
        ;** s sgxS_pendingEvents
        ;* temporary event data stored as elements of list sgx_pendingEvents to trigger the according event with the next call of <a href=#f_sgxexamine>sgxExamine</a>
        Structure sgxS_pendingEvents ;{
          id.i         ;* ID of the sgx element whose event field should be set
          changed.i    ;* the value that should be set for the 'changed' field
          resized.i    ;* the value that should be set for the 'resized' field (0 or 1)
          refreshed.i  ;* the value that should be set for the 'refreshed' field
          ;}
        EndStructure
        
        
        ;** s sgxS_Tip
        ;* allocated in <a href=#f_sgxtip>sgxTip</a> when a tooltip is added to an sgx element and is stored in field '*tip' of the structure <a href=#s_sgxs_gadget>sgxS_GADGET</a>
        Structure sgxS_Tip ;{
          spr.i    ;* the PB sprite that is displayed as the tooltip
          w.i      ;* width of the tooltip
          h.i      ;* height of the tooltip
          text.s   ;* the text of the tooltip
          timer.i  ;* a timer for the popup delay
          show.i   ;* 0=sleeping ; 1=show ; -1=hide (which means do not popup again before element mouseout. will be reset to 0 on 'no hover')
          ;}
        EndStructure
        
        
        ;** s sgxS_Animation
        ;* animation data temporarily used during window animations and stored as elements of map sgx_animations; is being populated in <a href=#f_sgx_addanimation>sgx_addAnimation</a>
        Structure sgxS_Animation ;{
          startTime.i
          startX.i
          startY.i
          startW.i
          startH.i
          startAlpha.i
          endX.i
          endY.i
          endW.i
          endH.i
          endAlpha.i
          ;}
        EndStructure
        
        
        ;** s sgxS_ListWrapper
        ;* just a linked list wrapped in a structure to use it as optional function parameter via pointer in <a href=#f_sgxsetstate>sgxSetState</a> (used by <a href=#f_sgxexamine>sgxExamine</a>)
        Structure sgxS_ListWrapper ;{
          List list_.i()
          ;}
        EndStructure
        
        
        ;** s sgxS_Table
        ;* will be allocated by sgx table elements and stored in field '*table' of the structure <a href=#s_sgxs_gadget>sgxS_GADGET</a>
        Structure sgxS_Table ;{
          w.i     ;* number of columns
          h.i     ;* number of lines
          *cells  ;* memory block to store cell ids
          *gridx  ;* memory block to store x-positions of vertical grid lines
          *gridy  ;* memory block to store y-positions of horizontal grid lines
          ;}
        EndStructure
        
        
        
        
        ;** s sgxS_GADGET
        ;* the basic structure of an sgx element. _
        ;* _
        ;* some fields are used differently depending on the element type: _
        ;* _
        ;* x2, y2, w2 and h2 are set to x, y, w and h as a default during element creation in sgx_PassStandardValues() and are used as follows: _
        ;* - sgxButton x2,y2: offset for optional button sprite. _
        ;* - sgxScrollbar/Trackbar x2,y2,w2,h2: position and size of knob. _
        ;* - sgxCombo h2: height of open hover frame (same as h actually). while combo is open, h is temporarily stored in h2 and becomes the expanded height itself. _
        ;* - sgxEdit x2,y2,w2,h2: 'client area' or text clipping area (area inside the element where text appears). _
        ;* - sgxCheckbox x2,y2,w2,h2: overall size and knob-relative position (see the notes on <a href=#f_sgxcheckbox>sgxCheckbox</a>). _
        ;* - sgxWin: x2,y2,w2,h2: hold the original position and size while minimized. _
        ;* _
        ;* width: _
        ;* - the height of the knob of a vertical scrollbar/trackbar and the width of the knob of a horizontal scrollbar/trackbar. used for scrollpos calculation and knob positioning. _
        ;* - sxgWin: the height of the titlebar. often checked as a bool value to know if a titlebar exists (but its usually not 1, so don't compare it to #True). _
        ;* _
        ;* max: _
        ;* - sgxScrollbar/sgxTrackbar: the maximum value (minimum is 0) _
        ;* - sgxEdit: the maximum number of lines allowed _
        ;* - sgxCombo: 0 if the combo is closed. 1 if it's opened ('maximized') _
        ;* _
        ;* state (see also <a href=#f_sgxsetstate>sgxSetState</a>): _
        ;* - sgxSpriteGadget: the PB Sprite number or 0. _
        ;* - sgxCombo: the currently selected entry number in the range of 1 to the number of entries (both inclusive). default is 1. _
        ;* - sgxWin: one of the #sgxWinState.. values _
        ;* - toggle buttons: 1 if the button is toggled/pushed, otherwise 0. _
        ;* - sgxCheckbox: 1 if the checkbox is checked, otherwise 0. _
        ;* - sgxScrollbar/sgxTrackbar: the current value in the range of 0..max (both inclusive) _
        ;* never compare the state field to the #sgxState.. values because these are used only during element creation.
        ;* _
        Structure sgxS_GADGET ;{ <<<<<<<
          xToParent.i             ;* x-position relative to parent position
          yToParent.i             ;* y-position relative to parent position
          x.i                     ;* x-position on screen (absolute)
          y.i                     ;* y-position on screen (absolute)
          w.i                     ;* width of the element
          h.i                     ;* height of the element
          type.i                  ;* type of the element (#sgxType...)
          parent.i                ;* parent element ID or -1 for top-level elements
          hasChilds.i             ;* (bool) true if the element has childs
          x2.i                    ;* type dependent. see structure description.
          y2.i                    ;* type dependent. see structure description.
          w2.i                    ;* type dependent. see structure description.
          h2.i                    ;* type dependent. see structure description.
          auto.i                  ;* auto adaptation flags (combination of #sgx_Auto.. constants)
          *treeElem.sgx_llS_elem  ;* element in the tree list. treeElem\node (list of childs) will 0 if the element does not have childs.
          
          *design.sgxS_Design     ;* the design used by the sgx element
          
          text.s                  ;* window title / button caption / text of sgxText or sgxEdit / checkbox label / etc. if you need the text unmodifed by automatic word wrap see 'origText'.
          width.i                 ;* type dependent. see structure description.
          textw.i                 ;* width of the element's text. for sgxText elements this is the text width of the longest line in case of multiline text.
          texth.i                 ;* height of the element's text. line height in case of multiline text.
          textpos.i               ;* used by sgxEdit elements as current cursor position in characters including linebreaks where 0 means at the beginning of text / before the first character.
          textcursor.sgx_S_Point  ;* position (in pixel) of textcursor of sgxEdit elements inside the text clipping area (as defined by x2,y2,w2 and h2)
          textalign.i             ;* like #sgxAlign.. for sgxCheckbox and sgxText elements. also used by sgxEdit elements for horizontal text scrolling.
          textfontID.i            ;* the PB font ID used for drawing text during element refresh. set to sgx_CurrentFontID on element creation.
          
          origText.s              ;* original text i.e. as specified at creation as opposed to field 'text' which might be modified due to automatic word wrap etc.
          origW.i                 ;* original width as specified at creation as opposed to field 'w' (real resulting width). used for example to store the -1 that might have been used for automatic sizing.
          origH.i                 ;* original height as specified at creation as opposed to field 'h' (real resulting height). used for example to store the -1 that might have been used for automatic sizing.
          
          *tip.sgxS_Tip           ;* tooltip data (will be allocated only if needed).
          *table.sgxS_Table       ;* will be allocated for tables only (#sgxTypeTable)
          
          scrollpos.i             ;* used for knob positioning by scrollbars/trackbars and for vertical text scrolling by sgxEdit elements.
          
          spr.i[8]                ;* the sprites used for displaying
          alpha.i                 ;* transparency. can be set via <a href=#f_sgxsetalpha>sgxSetAlpha</a>
          alpha2.i                ;* original transparency stored during animations
          sys0.sgx_S_Box          ;* some box coordinates relative to the elements origin/position defining the close button area of windows to detect mouse events there.
          
          hover.i                 ;* 1 if the mouse is hovering the element. 0 if it doesn't. an element might not get the hover event if some element (an sgxEdit element or an sgxScrollbar for example) has catched the focus.
          state.i                 ;* type dependent. see structure description.
          max.i                   ;* type dependent. see structure description.
          changed.i               ;* fired once if the elements state changed. for scrollbars and trackbars: the value by that the state changed. fired also for sgxEdit element if the text or the cursor position has changed.
          resizable.i             ;* 1 if the element can be resized with the mouse. otherwise 0. can be set via <a href=#f_sgxsetresizable>sgxSetResizeable</a>.
          minWidth.i              ;* the smallest width allowed for manually resizing elements via mouse. can be set via <a href=#f_sgxsetminsize>sgxSetMinSize</a>.
          minHeight.i             ;* the smallest height allowed for manually resizing elements via mouse. can be set via <a href=#f_sgxsetminsize>sgxSetMinSize</a>.
          resized.i               ;* fired once if the elements size has been changed by <a href=#f_sgxresize>sgxResize</a>.
          refreshed.i             ;* fired once if the element has been refreshed. this may include partial refresh like updating only the tooltip of an element and does not necessarily mean that the element has been redrawn but on the other hand any redraw will tigger this event.
          show.i                  ;* if #True then the element will be displayed and receives events, otherwise #False to hide the element. should be changed via <a href=#f_sgxsetshow>sgxSetShow</a>.
          show2.i                 ;* original visibility stored during animations
          style.i                 ;* type dependent. see documentation of the compose functions that support it like <a href=#f_sgxtext>sgxText</a> or <a href=#f_sgxcheckbox>sgxCheckbox</a>.
          passive.i               ;* 0 by default. if set to 1 the element won't catch events or move in front. usefull for background sprite gadgets.
          disabled.i              ;* 0 by default. if set to 1 the element won't catch events or move in front and will be displayed dimmed. can be set via <a href=#f_sgxsetdisabled>sgxSetDisabled</a>
          
          margin.i                ;* space in pixels between the elements boundary and it's content. used by <a href=#f_sgxtext>sgxText</a> and <a href=#f_sgxtable>sgxTable</a> elements. also used by sprite buttons to store the sprite offset for displaying when pushed.
          
          leftclick.i             ;* (bool) fired once with first left mouse button down
          rightclick.i            ;* (bool) fired once with first right mouse button down
          
          leftdown.i              ;* (bool) continuously fired while left mouse button down
          rightdown.i             ;* (bool) continuously fired while right mouse button down
          
          leftup.i                ;* (bool) fired once with left mouse button release
          rightup.i               ;* (bool) fired once with right mouse button release
          
          lefthold.i              ;* (bool) fired once each interval as configured via sgxHoldDelay and sgxHoldInterval
          lefthold_timer.i        ;* used for timing of hold events
        ;}
        EndStructure
        
        
        
        
        
        
        
        
        
        ;-# DECLARE ###############################################################
        ;{
        
        ; note:
        ; some initialization is done in sgxLoadDesign() when
        ; called for the first time
        
        ; [next line resets the jaPBDoc documentation target]
        ;** 
        
        Global  sgx_LIMIT_STATIC_GADGETS = #sgxMaxStaticGagdets                ;* static IDs can range from 1 to sgx_LIMIT_STATIC_GADGETS. see also the notes on #sgxMaxStaticGagdets.
        Global  sgx_CURRENT_MAX_STATIC_GADGETS = 0                             ;* highest static ID that has ever been used for element creation. used to shorten traversal through the elements array. note that this doesn't tell anything about validity of this ID or of lower IDs.
        Global  sgx_CURRENT_MAX_DYNAMIC_GADGETS = sgx_LIMIT_STATIC_GADGETS     ;* same as sgx_CURRENT_MAX_STATIC_GADGETS but for dynamic IDs instead of static ones.
        Global  Dim *sgxID.sgxS_GADGET(sgx_CURRENT_MAX_DYNAMIC_GADGETS)        ;* the actual array of pointers to all sgx elements (<a href=#s_sgxs_gadget>sgxS_GADGET</a>). the element IDs are used as indices here. this array is accessed via macro sgx() which includes some error checking when debugger is used.
        Global  Dim sgx.sgxS_GADGET(0)                                         ;* empty dummy array only to support autocompletion for fields of the sgx element structure <a href=#s_sgxs_gadget>sgxS_GADGET</a>. the PB-IDE uses this for autocompletion of fields, but sgx(id) is actually a macro and not an array. no other use of this one and it could also be commented out. also remember that when using this the debugger the tooltip in the editor will say 'index out of bounds' if you mark 'sgx(id)' in the IDE because it does not evaluate the macro.
        Global  Dim sgxCell.sgxS_GADGET(0,0,0)                                 ;* empty dummy array only to support autocompletion for macro sgxCell(). see documentation of the dummy array sgx().
        Global *sgxLayeredView.sgx_llnew                                       ;* the root parent node. basically a list of top the level list elements (the sgx element id is part of the list element structure). the root node of the element tree. this an sgx_llS_list, not an sgx element and it is analog to the child list \treeElem\node of an sgx element. sgx_llnew is the list constructor macro
        Global  NewList sgxGroups.sgxS_groupElem()                             ;* list of radio group elements
        Global  NewList sgxDesign.sgxS_Design()                                ;* list of loaded designs
        Global  sgx_m.sgxS_Mouse                                               ;* stores various mouse states like position, buttons and also the current sgxCursor element id.
        Global  sgx_vec.sgx_S_Point                                            ;* stores the position the mouse has inside the titlebar for proper positioning while moving windows. also being used in a similar way while resizing elements with the mouse.
        Global  sgx_textCursorClippingOffset.sgx_S_Point                       ;* stores the current clipping offset applied to the textcursor sprite of sgxEdit elements for proper textcursor positioning.
        Global  sgx_dimensions.sgx_S_Size                                      ;* w\h values defining the screen size. can be set via <a href=#f_sgxsetdimensions>sgxSetDimensions</a>.
        Global  sgx_focusCatchedBy = -1                                        ;* -1 if no element has currently catched the focused, otherwise an element ID. focus is catched on leftclick and released with a leftup, so that down/hold events and knob dragging etc. can be directed to the according element even if the mouse is not hovering it as well as preventing other elements from receiving events during that time.
        Global  sgxActiveWin = -1                                              ;* the currently active (foreground) window id or -1 if no window is active.
        Global  sgx_resizing = -1                                              ;* ID of the element currently being resized with the mouse or -1 if no element is currently being resized.
        Global  sgx_moving = -1                                                ;* ID of the (window) element currently being moved manually via mouse or -1 if no element is currently being moved manually.
        Global  sgx_editing = -1                                               ;* ID of the edit element currently being active for text editing or -1 if no edit element is active.
        Global  sgx_resizeGadgetSize = 15                                      ;* size of the resize handle at the bottom right corner of resizable elements. must be at least 1 because it's also used as minimum size during manual resize (via mouse).
        Global  sgxStdFont = LoadFont(#PB_Any,"Arial",8)                       ;* font used for caption of minimized windows and as a default for the current font.
        Global  sgx_fontMonoSmall = LoadFont(#PB_Any,"Courier",8)              ;* font used by 'sgx_debugInfos' for example.
        Global  sgx_CurrentFontID = FontID(sgxStdFont)                         ;* the font being assign during element creation. can be set via <a href=#f_sgxsetfont>sgxSetFont</a>
        Global *sgx_CurrentDesign.sgxS_Design                                  ;* can be set via <a href=#f_sgxsetdesign>sgxSetDesign</a> and is also set by <a href=#f_sgxloaddesign>sgxLoadDesign</a>
        Global  sgx_CurrentParent = -1                                         ;* the sgx element id of the current parent, or -1 if there is no current parent.
        Global *sgx_CurrentParent.sgx_llS_list = *sgxLayeredView               ;* the current parent node. basically a list of list elements (the sgx element id is part of the list element structure). this an sgx_llS_list, not an sgx element. if an element becomes the current parent, this points to the element's \treeElem\node, which might still be 0 if it doesn't have any childs.
        Global  sgx_defaultAlpha = 255                                         ;* can be set via <a href=#f_sgxsetalpha>sgxSetAlpha</a>
        Global  sgx_xMinimizedWins = 2                                         ;* the x-position for the possibly next coming taskbar button in pixels.
        Global  sgx_debugInfos = 0                                             ;* 0 per default. you can set it to 1 to display debug informations about sgx elements and also display current mouse position. set it to 2 to display frames for all elements. for simplicity this uses a StartDrawing()/StopDrawing() block and is very slow, especially if value is 2 (one drawing block per element).
        Global  sgxTopID = -1                                                  ;* ID of the element currently under the mouse or -1 if no element is there.
        Global  sgx_ems                                                        ;* holds the current value of EllapsedMilliseconds(), or as set via <a href=#f_sgxsetemscontainer>sgxSetEmsContainer</a>. used for animations, intervals etc.
        Global  sgx_mouseWheelDelta                                            ;* gets sum up in <a href=#f_sgx_windowcallback>sgx_windowCallback</a> and used and reset in <a href=#f_sgxexamine>sgxExamine</a>.
        Global *sgx_emsContainer.Integer                                       ;* can be set via <a href=#f_sgxsetemscontainer>sgxSetEmsContainer</a>.
        Global  sgxTipDelay = 500                                              ;* delay for tip popup in milliseconds
        Global  sgxHoldDelay = 400                                             ;* delay for first hold event in milliseconds
        Global  sgxHoldInterval = 30                                           ;* interval of hold events in milliseconds
        Global  sgxRightclickUnset = #sgxRightclickUnsetAll                    ;* per default (#sgxRightclickUnsetAll) rightclick on checkboxes and togglebuttons changes their state to unchecked / not pushed. if set to #sgxRightclickUnsetNoGroup then this will only be applied to elements that are not part of a radiogroup. #sgxRightclickUnsetNone completely disables this behaviour.
        Global  sgxAnimTime = 400                                              ;* duration of window animations in milliseconds. you can set this to 0 to effectively disable animations.
        Global  NewMap sgx_animations.sgxS_Animation()                         ;* this map contains animation data objects temporarily created and used while animating windows.
        Global  sgxWinCloseEnabled = 1                                         ;* (bool) adjusts if windows can be closed by rightclick
        Global  sgx_BuildRefresh = 1                                           ;* usually set to 1 to refresh elements after finishing creation/building. set to 0 while creating elements should not refresh them but only initialize them. temporarily deactivated (0) by tables to avoid double refreshes of table childs.
        Global  NewList sgx_BuildRefreshStack()                                ;* used to push/pop values for sgx_BuildRefresh
        Global  NewList sgx_pendingEvents.sgxS_pendingEvents()                 ;* list of pending events that will be triggered with the next call of <a href=#f_sgxexamine>sgxExamine</a>
        Global  sgxNumberOfElements = 0                                        ;* number of currently existing and valid sgx elements
        Global  sgx_animStyle_minimizing                                       ;* the animation styles used for minimizing windows
        Global  sgx_animStyle_restoring                                        ;* the animation styles used for restoring windows
        Global  sgx_animStyle_closing                                          ;* the animation styles used for closing windows
        Global  sgx_animStyle_presenting                                       ;* the animation styles used for presenting windows
        Global  sgx_textcursorType = #sgxTextcursorTypeLine                    ;* text cursor type used by sgxEdit elements. can be set via <a href=#f_sgxsettextcursortype>sgxSetTextcursorType</a>.
        Global  sgx_lowbudgetDistortion.f = 0.2                                ;* distortion applied to backgrounds of design "lowbudget", a float value in the range 0..1 (both inclusive)
        Global  sgx_lowbudgetDistortColors = #True                             ;* #True if variable sgx_lowbudgetDistorstion should be applied to brightness as well as color component distribution. #False to only distort brightness.
        Global  sgx_disabledColor1 = RGBA(255,255,255,5)                       ;* the first color applied to the final gadget sprites if the element is disabled (a thin white for desaturation).
        Global  sgx_disabledColor2 = RGBA(50,50,50,80)                         ;* the second color applied to the final gadget sprites if the element is disabled (a thin black for darkening).
        ;}
        ; #

;}
;  <<<<<<<<<<<<<<<<<<<<<<<< merging end  sgx_define.pbi

; XIncludeFile "sgx_access.pbi"
;{ >>>>>>>>>>>>>>>>>>>>>>>> merging file sgx_access.pbi

        
        
        
        
        ;-# MACROS / PROC ###############################################################
        
        
        ;  ---------------- MACRO: sgx()
        ;
        ; there are two versions: for debugger and without.
        ;
        ; this macro has its own sourcefile to get autocompletion of sgx element structure
        ; fields via dummy 'Dim sgx.sgxS_GADGET(0)' working in the other sourcefiles. structure
        ; field autocompletion won't work in the same sourcefile as where this macro is
        ; defined but in files which include it. this also means it won't work inside the
        ; single-file version (sgx.pbi).
        ; also adding this file to the pb project will break autocompletion for structure
        ; fields.
        
        CompilerIf #PB_Compiler_Debugger
          
          Import "Debugger.lib"
            CompilerIf #PB_Compiler_Processor = #PB_Processor_x86
              sgx_err(error.p-ascii) As "_PB_DEBUGGER_SendError@4"
            CompilerElse
              sgx_err(error.p-ascii) As "PB_DEBUGGER_SendError"
            CompilerEndIf  
          EndImport
          
          ;** sgx_cb
          ;* wrapper used in the macro <a href=#f_sgx>sgx</a> if compiled with debugger. will through an error on invalid indices.
          Procedure sgx_cb(index)
            If (index>=0) And (index<=sgx_CURRENT_MAX_DYNAMIC_GADGETS)
              ;If *sgxID(index)
                ProcedureReturn index
              ;Else
              ;  sgx_err("sgx element invalid: sgx("+Str(index)+")")
              ;EndIf
            Else
              sgx_err("sgx array index out of bounds: sgx("+Str(index)+")")
            EndIf
            ; return at least the valid 0 index so checking 'sgx(id)' will work (accessing structure fields won't though!)
            ProcedureReturn 0
          EndProcedure
          
          ; (see CompilerElse section)
          Macro sgx(index)
            *sgxID(sgx_cb(index))
          EndMacro
          
        ;  ----------------
        CompilerElse
          
          ;** sgx_err
          ;* used to throw sgx errors when compiled with debugger.
          Macro sgx_err(_message_)
            ;empty dummy (see PB_DEBUGGER_SendError import above)
          EndMacro
          
          ;** sgx
          ;* use this macro to access the structure of your sgx elements. _
          ;* _
          ;* if you try to access invalid sgx objects you might get one of the following errors: _
          ;* (use Menu > Debugger > Callstack to find out where the error might have been caused) _
          ;* <pre>
          ;* <span></span>; this will cause an error: _
          ;* <span></span>;Debug sgx(-123)\text   ; "[ERROR] sgx array index out of bounds: sgx(-123)" _
          ;* <span></span> _
          ;* <span></span>; this will cause an error if we did not create an element with this ID: _
          ;* <span></span>;Debug sgx(345)\text     ; "[ERROR] Pointer is null." _
          ;* <span></span> _
          ;* <span></span>; this will NOT cause an error since the index is ok and we check if that element exists: _
          ;* <span></span>If sgx(345) _
          ;* <span></span>  ; but we might not get here _
          ;* <span></span>  Debug sgx(345)\text _
          ;* <span></span>EndIf 
          ;* <pre>
          ;** .index : ID of a valid and existing sgx element (if structure fields will be accessed), where index <b></b>
          ;*           can be in the range of 1 .. sgx_CURRENT_MAX_DYNAMIC_GADGETS. you can check if 
          ;*           the ID is valid by a nonzero value of 'sgx(index)' (without structure field access). 
          Macro sgx(index)
            *sgxID(index)
          EndMacro
          
        CompilerEndIf
        ;  ----------------

;}
;  <<<<<<<<<<<<<<<<<<<<<<<< merging end  sgx_access.pbi

; XIncludeFile "sgx_misc.pbi"
;{ >>>>>>>>>>>>>>>>>>>>>>>> merging file sgx_misc.pbi

        
        
        
        
        
        ;-# DECLARE ###############################################################
        ;{
        
        ; (don't use this as an overview, since it will be incomplete)
        
        Declare sgxLoadDesign(design.s="lowbudget", basecolor=-1)
        Declare sgxRefreshDesign(*design.sgxS_Design)
        
        Declare sgxCursor(id)
        Declare sgxButton(id, x, y, w, h, text.s, toggle=#sgxStateNone , sprNr=-1, x2=5, y2=5)
        Declare sgxCheckbox(id, x, y, w, h, checked=#sgxStateOff, text.s="", align=#sgxAlignCheckboxTextRight)
        Declare sgxScrollbar(id, x, y, w, h, w2, h2, max, state=0, style=0)
        Declare sgxEdit(id, x, y, w, h, text.s="", maxLines=-1)
        Declare sgxWin(id, x, y, w, h, text.s="", style=0)
        Declare sgxTip(id, text.s)
        Declare sgxCombo(id, x, y, w, h, text.s="")
        
        Declare sgxRefresh(id, mode=#sgx_refreshNormalWithTip)
        Declare sgxRefreshAll()
        Declare sgx_DrawCursor(id, sprType)
        Declare sgx_DrawButton(id, sprType)
        Declare sgx_DrawCheckbox(id, sprType)
        Declare sgx_DrawScrollbar(id, sprType)
        Declare sgx_DrawEdit(id, sprType, updateMetricsOnly=#False, moveTextpos=#False, scrollToCursor=#True)
        Declare sgx_DrawWin(id, sprType)
        Declare sgx_DrawTip(id)
        Declare sgx_DrawResizeGadget(id)
        Declare sgx_DrawCombo(id, sprType)
        Declare sgx_DrawText(id, sprType)
        Declare sgx_DrawDisabled(id, sprType)
        
        Declare sgx_Fill(*design.sgxS_Design, x, y, w, h, sprType)
        
        Declare sgxExamine( mousex=-1, mousey=-1 )
        Declare sgxDisplay();onlyOfParent=-1)
        Declare sgxDisplayID(id)
        Declare sgxDisplayCursor()
        Declare sgxMoveOnTop(id)
        Declare sgxGetElementAt(x, y)
        Declare sgxGetTopLevel(id)
        Declare sgx_GetGadgetX(id)
        Declare sgx_GetGadgetY(id)
        Declare sgx_updateToParentPos(id)
        Declare sgxMove(id, x, y, relative=#True)
        Declare sgx_MoveChildsOf(id)
        Declare sgx_IsParentage(parent, child)
        Declare sgxAddToRadioGroup(id, groupID)
        Declare sgxGetGroup(id)
        Declare sgxFree(id)
        Declare sgxFreeChildsOf(parent)
        Declare sgx_RestoreWin(id)
        Declare sgx_RefreshWinBar()
        Declare sgxSetCurrentParent(id)
        Declare sgxSetAlpha(alpha, id=-1, recursive=#False)
        Declare sgx_animate(id, *animation.sgxS_Animation)
        Declare sgx_HideChildsOf(parent, show=#False, ignoreWindows=#True)
        Declare sgxSetState(id, state, handleRadioGroups=#True, *changed.sgxS_ListWrapper=0)
        
        ;}
        ; #
        
        
        
        
        
        
        
        
        ;** sgxNew
        ;* create a new sgx object. _
        ;* returns the ID of that newly created object, or 0 if creation failed. _
        ;* you should use the compose functions like <a href=#f_sgxbutton>sgxButton</a> etc. to create actual sgx elements. _
        ;** .id : the ID to be used for the new object, where id can be in the range of 1 .. sgx_LIMIT_STATIC_GADGETS, 
        ;*        <span></span> or -1 to dynamically generate an ID. _
        ;*        if the ID is already in use, the according element is freed (including childs) and replaced.
        Procedure sgxNew(id)
          
          CompilerIf #PB_Compiler_Debugger
            If *sgx_CurrentDesign = 0
              sgx_err("there is no current design")
            EndIf
            
            If id=0
              sgx_err("id 0 should not be used for valid sgx object. start by id 1 instead")
            ElseIf id > sgx_LIMIT_STATIC_GADGETS
              sgx_err("id too big (" + Str(id) + "). see documentation for #sgxMaxStaticGagdets")
            ElseIf id < -1
              sgx_err("id is negative (" + Str(id) + ").")
            EndIf
          CompilerEndIf
          
          If id = -1
            sgx_CURRENT_MAX_DYNAMIC_GADGETS + 1
            ReDim *sgxID.sgxS_GADGET(sgx_CURRENT_MAX_DYNAMIC_GADGETS)
            id = sgx_CURRENT_MAX_DYNAMIC_GADGETS
          ElseIf id > 0  And  id <= sgx_LIMIT_STATIC_GADGETS  And  id > sgx_CURRENT_MAX_STATIC_GADGETS
            sgx_CURRENT_MAX_STATIC_GADGETS = id
          EndIf
          
          If sgx(id)
            sgxFree(id)
          EndIf
          sgx(id) = AllocateMemory(SizeOf(sgxS_GADGET))
          
          ; the array sgx(id)\spr[] will contain zeros now wich might be valid sprite IDs generally and IsSprite() would say
          ; fine, but it's not valid as a gadget sprite. sgx will check for non-zero values before IsSprite() and
          ; sgx_FreeSpriteChecked() won't free sprites with id 0.
          
          ; (*obsolete*)
          ; init sprites with -1 to not interfere with possibly used constant 0 as sprite id ouside of sgx.
          ;sgx(id)\spr[0] = -1
          ;sgx(id)\spr[1] = -1
          ;sgx(id)\spr[2] = -1
          ;[...]
          
          If sgx(id)
            sgxNumberOfElements + 1
            ProcedureReturn id
          Else
            ProcedureReturn 0
          EndIf
        EndProcedure
        
        
        ;** sgx_isInElement
        ;* checks if the specified absolute screen position is within the element's dimensions.
        Procedure sgx_isInElement(id, x, y)
          If sgx(id)\type = #sgxTypeCheckbox
            ProcedureReturn Bool( x>=(sgx(id)\x+sgx(id)\x2) And x<(sgx(id)\x+sgx(id)\x2+sgx(id)\w2) And y>=(sgx(id)\y+sgx(id)\y2) And y<(sgx(id)\y+sgx(id)\y2+sgx(id)\h2) )
          Else
            ProcedureReturn Bool( x>=sgx(id)\x And x<(sgx(id)\x+sgx(id)\w) And y>=sgx(id)\y And y<(sgx(id)\y+sgx(id)\h) )
          EndIf
        EndProcedure
        
        
        ;** sgx_MousehoverClosebutton
        ;* checks if the mouse position is within the dimensions of the element's close button (windows only).
        Macro sgx_MousehoverClosebutton(id)
          ( sgx_m\x>(sgx(id)\x+sgx(id)\sys0\x1) And sgx_m\x<sgx(id)\x+sgx(id)\sys0\x2 And sgx_m\y>sgx(id)\y+sgx(id)\sys0\y1 And sgx_m\y<sgx(id)\y+sgx(id)\sys0\y2 )
        EndMacro
        
        
        ;** sgx_MousehoverResize
        ;* checks if the mouse position is within the dimensions of the element's resize button in hte bottom right corner.
        Procedure sgx_MousehoverResize(id)
          Static x,y
          ; get mouse cursor distance from bottom right element corner, [-2,-1] for example.
          x = sgx_m\x - (sgx(id)\x + sgx(id)\w)
          y = sgx_m\y - (sgx(id)\y + sgx(id)\h)
          ; the "x+y < size" simulates a triangular area
          If x<0 And y<0 And (-x + -y) < sgx_resizeGadgetSize
            ProcedureReturn #True
          EndIf
          ProcedureReturn #False
        EndProcedure
        
        
        ;** sgx_FreeImageChecked
        ;* frees the image with the passed number if it's a valid PB image. used for easy recreation of images. _
        ;* images with id 0 will not be freed even if they are valid images, because sgx uses #PB_Any and interprets
        ;* 0 as uninitialized.
        ;** .img : a variable containing the PB image number. the variable will be set to -1 after freeing the image, so you cannot use this macro with constants for example.
        Macro sgx_FreeImageChecked(img)
          ;If img=0
          ;  sgxERROR("develinfo: an internal sgx image shouldn't be 0")
          ;EndIf
          If img And IsImage(img)
            FreeImage(img)
          EndIf
          img = -1 ; works with variables passed only (not constants) !
        EndMacro
        
        
        ;** sgx_FreeSpriteChecked
        ;* frees the sprite with the passed number if it's a valid PB sprite. used for easy recreation of sprites. _
        ;* sprites with id 0 will not be freed even if they are valid sprites, because sgx uses #PB_Any and interprets
        ;* 0 as uninitialized.
        ;** .spr : a variable containing the PB sprite number. the variable will be set to -1 after freeing the sprite, so you cannot use this macro with constants for example.
        Macro sgx_FreeSpriteChecked(spr)
          ;If spr=0
          ;  sgxERROR("develinfo: an internal sgx sprite shouldn't be 0")
          ;EndIf
          If spr And IsSprite(spr)
            FreeSprite(spr)
          EndIf
          spr = -1 ; works with variables passed only (not with constants) !
        EndMacro
        
        
        ;** sgx_SkipUnused
        ;* skips the gap of irrelevant IDs between the currently highest used static ID and the first dynamic one. used for more efficient element traversal for example in <a href=#f_sgxeach>sgxEACH</a>.
        ;** .iterator : the identifier used as ID in the loop.
        Macro sgx_SkipUnused(iterator)
          If iterator = sgx_CURRENT_MAX_STATIC_GADGETS+1
            iterator = sgx_LIMIT_STATIC_GADGETS
          EndIf
        EndMacro
        
        
        ;** sgxInBounds
        ;* returns #True if 'index' is within bounds of the basic sgx element array, otherwise it returns #False. _
        ;* a return value of '#True' does not say anything about the existence or validity of a possible element at that position.
        ;** .index : the index that is to be checked.
        Macro sgxInBounds(index)
          Bool( (index>=0) And (index<=sgx_CURRENT_MAX_DYNAMIC_GADGETS) )
        EndMacro
        
        
        ;** sgx_SetParentList
        ;* used in <a href=#f_sgxsetcurrentparent>sgxSetCurrentParent</a> to keep *sgx_CurrentParent (the current parent tree node) in sync with sgx_CurrentParent (the current parent ID).
        Procedure sgx_SetParentList(id)
          If id = -1
            *sgx_CurrentParent = *sgxLayeredView
          Else
            *sgx_CurrentParent = sgx(id)\treeElem\node
          EndIf
        EndProcedure
        
        
        ;** sgxSetCurrentParent
        ;* set the parent for coming creations of new sgx elements. _
        ;* needed if you want to add childs later.
        ;* calls of this function in combination with <a href=#f_sgxcloselevel>sgxCloseLevel</a> should not be nested.
        ;** .id : ID of the element to become the current parent
        Procedure sgxSetCurrentParent(id)
          If id <> sgx_CurrentParent
            sgx_CurrentParent = id
            sgx_SetParentList(id)
          EndIf
        EndProcedure
        
        
        ;** sgxSetParent
        ;* sets or changes the parent of an sgx element. _
        ;* used at element creation and also in <a href=#f_sgxcellassign>sgxCellAssign</a>. _
        ;* the child is not automatically repositioned, use <a href=#f_sgxmove>sgxMove</a> if necessary.
        ;** .id     : ID of the child element which parent is to be changed.
        ;** .parent : ID of the element to become the new parent
        Procedure sgxSetParent(id, parent)
          Protected *node.sgx_llS_list
          
          ; remove the tree element from its current parent's node but preserve it's own childs (the node of the treeElem)
          *node = sgx_llDelElem(sgx(id)\treeElem, #True)
          
          sgx(id)\parent=parent
          
          If parent <> -1
            ; initialize the parent's child list if not done
            If sgx(parent)\hasChilds = 0
              sgx(parent)\hasChilds = 1
              sgx(parent)\treeElem\node.sgx_llnew ; <-- list constructor
              
              ; update *sgx_CurrentParent from 0 to our newly created node
              ; note: we can not simply use sgxSetCurrentParent(parent) here because
              ; the condition 'If id <> sgx_CurrentParent' inside that function would
              ; fail and thus *sgx_CurrentParent would remain 0.
              sgx_SetParentList(parent)
            EndIf
          EndIf
          
          ; add the element to the parent's element tree node.
          If parent = -1
            ; add the element to *sgxLayeredView (the root node)
            ; this happens if sgx elements are created normally as top level childs, or if
            ; the parent is changed later to top level, independent from the current parent.
            sgx(id)\treeElem=sgx_llAdd(*sgxLayeredView, id)
            sgx_llReplaceElemNode(sgx(id)\treeElem, *node) ; restore the childs list
          ElseIf parent = sgx_CurrentParent
            ; this happens if sgx elements are created normally as the current parent's child.
            ; here *sgx_CurrentParent is the actually the same as the parent's node in the Else
            ; branch but i kept this separate to be clear.
            sgx(id)\treeElem=sgx_llAdd(*sgx_CurrentParent, id)
            sgx_llReplaceElemNode(sgx(id)\treeElem, *node) ; restore the childs list
          Else
            ; this happens if the parent is changed later, independent from the current parent.
            sgx(id)\treeElem=sgx_llAdd(sgx(parent)\treeElem\node, id)
            sgx_llReplaceElemNode(sgx(id)\treeElem, *node) ; restore the childs list
          EndIf
        EndProcedure
        
        
        ;** sgxEach
        ;* use this macro to iterate through the elements. the order in which the elements are iterated is undefined. _
        ;* usage example: _
        ;* <pre>
        ;* ;Define id  ; needed if EnableExplicit is used _
        ;* _
        ;* For sgxEach(id) _
        ;* <span></span>  Debug sgx(id)\text _
        ;* Next _
        ;* </pre>
        ;** .it : choose your iterator variable here that will contain the ID of the sgx elements. this variable is used in the for/next loop as is and must have been declared previously if EnableExplicit is used.
        Macro sgxEach(it)
          it=1 To sgx_CURRENT_MAX_DYNAMIC_GADGETS
            sgx_SkipUnused(it)
            If Not sgx(it)
              Continue
            EndIf
        EndMacro
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        ;-# MISC PROC ###############################################################
        
        
        
        
        
        ;** sgxMoveOnTop
        ;* make an sgx window (#sgxTypeWin) active and foreground or move an element above its siblings. if the element is not a window then the parent window will be activated additionally.
        ;** .id : the ID of the sgx element
        Procedure sgxMoveOnTop(id)
          Protected done
          
          ; reset possible window focus for top level elements
          If sgx(id)\parent = -1 And sgxActiveWin
            sgxActiveWin = -1
          EndIf
          
          While id <> -1
            ;If sgx(id)\type = #sgxTypeWin  ;Or  sgx(id)\type = #sgxTypeTable
              sgx_llmoveend(sgx(id)\treeElem)
              If sgx(id)\type = #sgxTypeWin  And  Not done
                ; activate only the direct parent window, not the topmost parent window
                done=1
                sgxActiveWin = id
              EndIf
            ;EndIf
            id=sgx(id)\parent
          Wend
        EndProcedure
        
        
        ;** sgx_PushBuildRefresh
        ;* temporarily changes the value of sgx_BuildRefresh.
        Procedure sgx_PushBuildRefresh(state)
          If ListSize(sgx_BuildRefreshStack())
            LastElement(sgx_BuildRefreshStack())
          EndIf
          AddElement( sgx_BuildRefreshStack() )
          If state
            sgx_BuildRefresh = 1
          Else
            sgx_BuildRefresh = 0
          EndIf
          sgx_BuildRefreshStack() = sgx_BuildRefresh
        EndProcedure
        
        
        ;** sgx_PopBuildRefresh
        ;* restores sgx_BuildRefresh to the previous value.
        Procedure sgx_PopBuildRefresh()
          If ListSize(sgx_BuildRefreshStack())
            LastElement(sgx_BuildRefreshStack())
            DeleteElement(sgx_BuildRefreshStack())
            LastElement(sgx_BuildRefreshStack())
            If ListSize(sgx_BuildRefreshStack())
              sgx_BuildRefresh = sgx_BuildRefreshStack()
            Else
              sgx_BuildRefresh = 1
            EndIf
          EndIf
        EndProcedure
        
        
        ;** sgxGetElementAt
        ;* returns the ID of the topmost sgx element at the given position on screen, or -1 if no sgx element is found there. passive elements are ignored.
        ;** .x : x-position on screen in pixels
        ;** .y : y-position on screen in pixels
        Procedure sgxGetElementAt(x, y)
          Protected ret=-1
          Protected id
          sgx_llIteratorRecursive(Protected *it, *sgxLayeredView)
            If sgx(*it\id)\show  And  (Not sgx(*it\id)\passive)
              If sgx_isInElement(*it\id, x, y)
                ret = *it\id
              EndIf
            EndIf
            sgx_llNextRecursive(*it)
          ProcedureReturn ret
        EndProcedure
        
        
        ;** sgxGetTopLevel
        ;* returns the ID of the topmost parent.
        ;** .id : ID of child element. if this element does not have parents, it is the top level itself and thus returned.
        Procedure sgxGetTopLevel(id)
          While sgx(id)\parent <> -1
            id = sgx(id)\parent
          Wend
          ProcedureReturn id
        EndProcedure
        
        
        ;** sgxMove
        ;* change the position of an sgx element. _
        ;* the positioning is either in absolute screen coordinates or in absolute toParent coordinates. _
        ;* if you want to position the element relative to its own current position (by offset) you would calculate the offset as follows (both methods would have the same effect): _
        ;* <pre> 
        ;* ; moves the element 10 pixels to the right as well as down by specifiying toParent coordinates. _
        ;* sgxMove(id, sgx(id)\xToParent+10, sgx(id)\yToParent+10); _
        ;* _
        ;* ; moves the element 10 pixels to the right as well as down, using absolute screen coordinates. it's toParent-position will be updated accordingly. _
        ;* sgxMove(id, sgx(id)\x+10, sgx(id)\y+10, #False); _
        ;* </pre> 
        ;** .id : ID of the sgx element to be moved
        ;** .x  : new x-position relative to parent element
        ;** .y  : new y-position relative to parent element
        ;** .relative  : by default (#True) the element will be positioned relative to it's parent. specify #False if x/y are absolute screen coordinates.
        Procedure sgxMove(id, x, y, relative=#True)
          Static parent
          
          If relative
            ; set realitve position
            sgx(id)\xToParent = x
            sgx(id)\yToParent = y
            ; update absolute position
            sgx(id)\x=sgx_GetGadgetX(id)
            sgx(id)\y=sgx_GetGadgetY(id)
          Else
            ; set absolute position
            sgx(id)\x=x
            sgx(id)\y=y
            ; update relative position
            sgx_updateToParentPos(id)
          EndIf
          
          ; handle child positions
          sgx_MoveChildsOf(id)
          
        EndProcedure
        
        
        ;** sgx_updateToParentPos
        ;* updates the element's to-parent-position if it's absolute screen position has been modified.
        ;** .id : ID of the sgx element.
        Procedure sgx_updateToParentPos(id)
          If sgx(id)\parent <> -1
            sgx(id)\xToParent = sgx(id)\x - sgx(sgx(id)\parent)\x
            sgx(id)\yToParent = sgx(id)\y - sgx(sgx(id)\parent)\y
          Else
            sgx(id)\xToParent = sgx(id)\x
            sgx(id)\yToParent = sgx(id)\y
          EndIf
        EndProcedure
        
        ;** sgxSetDisabled
        ;* disable or enable an sgx element. _
        ;* if the disabled/enabled state changes then the element will be refreshed. _
        ;** .id : ID of the sgx element to be disabled or enabled
        ;** .disabled  : #True (default) to disable the element or #False to enable it. _
        Procedure sgxSetDisabled(id, disabled=#True)
          If sgx(id)
            If (Not sgx(id)\disabled And disabled)  Or  (sgx(id)\disabled And Not disabled)
              If disabled
                sgx(id)\disabled = 1
              Else
                sgx(id)\disabled = 0
              EndIf
              If id = sgxActiveWin       : sgxActiveWin       = -1 : EndIf
              If id = sgx_focusCatchedBy : sgx_focusCatchedBy = -1 : EndIf
              If id = sgx_resizing       : sgx_resizing       = -1 : EndIf
              If id = sgx_moving         : sgx_moving         = -1 : EndIf
              If id = sgx_editing        : sgx_editing        = -1 : EndIf
              sgxRefresh(id)
            EndIf
          EndIf
        EndProcedure
        
        
        ;** sgxResize
        ;* change the size of an sgx element. _
        ;* sgxTable elements determine their size by their childs and cannot be resized directly.
        ;* sgxCheckbox elements resize only their knob (w,h) but their overall size (w2,h2) might be influenced by this too. _
        ;** .id : ID of the sgx element to be resized
        ;** .w  : the new width. one of these values: _
        ;*        - a positive value > 0 for an explicit width. _
        ;*        - a value of -2 to ignore that parameter and not change the width or to use the automatic size adjustment as specified at creation for elements which support it. _
        ;*        - a value of -1 to explicitly use automatic size adjustment for elements which support it. _
        ;** .h  : the new height. one of these values: _
        ;*        - a positive value > 0 for an explicit height. _
        ;*        - a value of -2 to ignore that parameter and not change the height or to use the automatic size adjustment as specified at creation for elements which support it. _
        ;*        - a value of -1 to explicitly use automatic size adjustment for elements which support it. _
        Procedure sgxResize(id, w, h)
          Static wPrev, hPrev, wNew, hNew
          Static topmostTable, parent
          Static dw, dh, fw.f, fh.f, switched, tmp
          
          If sgx(id) And sgx(id)\type <> #sgxTypeTable ; (skip tables)
            wPrev = sgx(id)\w
            hPrev = sgx(id)\h
            
            ; 'ignore' parameter
            If w = -2
              If sgx(id)\origW = -1
                w = -1
              Else
                w = sgx(id)\w
              EndIf
            EndIf
            If h = -2
              If sgx(id)\origH = -1
                h = -1
              Else
                h = sgx(id)\h
              EndIf
            EndIf
            
            ; handle auto size for buttons with sprite
            If sgx(id)\type = #sgxTypeButton Or sgx(id)\type = #sgxTypeButtonToggle
              If w = -1
                If IsSprite(sgx(id)\spr[#sgx_GadgetSprNormal2])
                  w = SpriteWidth (sgx(id)\spr[#sgx_GadgetSprNormal2]) + 2 * sgx(id)\x2 + 1
                Else
                  ; at least get a safe size if no valid sprite
                  w = 1
                EndIf
              EndIf
              If h = -1
                If IsSprite(sgx(id)\spr[#sgx_GadgetSprNormal2])
                  h = SpriteHeight(sgx(id)\spr[#sgx_GadgetSprNormal2]) + 2 * sgx(id)\y2 + 1
                Else
                  ; at least get a safe size if no valid sprite
                  h = 1
                EndIf
              EndIf
              
            ElseIf sgx(id)\type = #sgxTypeText
              ; restore text that might have been modified by auto word wrap
              If sgx(id)\auto & #sgx_AutoNL
                sgx(id)\text = sgx(id)\origText
              EndIf
              ; set auto flags
              If (w=-1 Or h=-1)
                If w=-1
                  sgx(id)\auto | #sgx_AutoW
                  ; auto word wrap is only supported for fixed width
                  sgx(id)\auto & ~#sgx_AutoNL
                Else
                  sgx(id)\auto & ~#sgx_AutoW
                EndIf
                If h=-1
                  sgx(id)\auto | #sgx_AutoH
                Else
                  sgx(id)\auto & ~#sgx_AutoH
                EndIf
              EndIf
              If w>0
                ; remove possible auto width flag if fixed width is specified
                sgx(id)\auto & ~#sgx_AutoW
              EndIf
              If h>0
                ; remove possible auto height flag if fixed height is specified
                sgx(id)\auto & ~#sgx_AutoH
              EndIf
              
            ElseIf w=-1 Or h=-1
              sgx_err("automatic size adjustment with -1 is not supported for this element type: sgx("+Str(id)+").")
            EndIf
            
            If (w = -1 Or w > 0) And (h = -1 Or h > 0)
              If w <> wPrev  Or  w = -1  Or  h <> hPrev  Or  h = -1
                If sgx(id)\type & (#sgxTypeScrollbar|#sgxTypeTrackbar)
                  
                  ; detect change between from/to horizontal/vertical
                  switched = #False
                  ; (was ver and is hor) or (was hor and is ver)
                  If ((hPrev > wPrev) And Not (h > w))  Or  (Not (hPrev > wPrev) And (h > w))
                    switched = #True
                  EndIf
                  
                  ; ; keep ratio between knob size and overall size
                  ;If Not switched
                  ;  fw = w / wPrev   ; ratio 'new width' / 'old width' ..
                  ;  fh = h / hPrev
                  ;  sgx(id)\w2 * fw  ; ..is also applied to 'knob width'
                  ;  sgx(id)\h2 * fh
                  ;Else
                  ;  ; if switched then..
                  ;  fw = w / hPrev   ; ..ratio 'new width' / 'old height' ..
                  ;  fh = h / wPrev
                  ;  sgx(id)\w2 * fh
                  ;  sgx(id)\h2 * fw  ; ..is also applied to 'old knob height'
                  ;  ; and knob size is swapped
                  ;  tmp = sgx(id)\w2
                  ;  sgx(id)\w2 = sgx(id)\h2
                  ;  sgx(id)\h2 = tmp
                  ;EndIf
                  
                  ; ensure valid knob size for later CreateSprite()
                  If sgx(id)\w2 < 1
                    sgx(id)\w2 = 1
                  EndIf
                  If sgx(id)\h2 < 1
                    sgx(id)\h2 = 1
                  EndIf
                  
                  sgx(id)\w = w
                  sgx(id)\h = h
                Else
                  sgx(id)\w = w
                  sgx(id)\h = h
                EndIf
                sgx(id)\origW = w
                sgx(id)\origH = h
                sgxRefresh(id)
                If sgx(id)\type & (#sgxTypeScrollbar|#sgxTypeTrackbar)
                  ; correcting knob position
                  sgxSetState(id, sgx(id)\state)
                EndIf
                
                ; check if size changed
                If sgx(id)\w <> wPrev Or sgx(id)\h <> hPrev
                  ; prepare event triggering
                  AddElement(sgx_pendingEvents())
                  sgx_pendingEvents()\id = id
                  sgx_pendingEvents()\resized = 1
                EndIf
                
                ; refresh the topmost parent table we may find
                topmostTable = -1
                parent = sgx(id)\parent
                While parent <> -1
                  If sgx(parent)\type = #sgxTypeTable
                    topmostTable = parent
                  EndIf
                  parent = sgx(parent)\parent
                Wend
                If topmostTable <> -1
                  ; refesh the table and possible sub tables
                  sgxRefresh(topmostTable, #sgx_refreshNormalWithChildsOfTables)
                EndIf
                
              EndIf
            EndIf
          EndIf
        EndProcedure
        
        
        ;** sgx_MoveChildsOf
        ;* updates the position of child elements if their parent element has moved, keeping their to-parent-position. _
        ;* child windows are not moved if their parent window moves, but instead their to-parent-position is updated accordingly. _
        ;* called in <a href=#f_sgxmove>sgxMove</a> And recursively whithin itself.
        ;** .id : ID of the sgx element which childs are to be repositioned.
        Procedure sgx_MoveChildsOf(id)
          Static parent
          If sgx(id)\hasChilds
            sgx_llIterator(Protected *it, sgx(id)\treeElem\node)
            
              ; don't move child windows at all, only update their toParent-position
              If sgx(*it\id)\type = #sgxTypeWin
                parent = sgx(*it\id)\parent
                If parent <> -1
                  sgx(*it\id)\xToParent = sgx(*it\id)\x - sgx(parent)\x
                  sgx(*it\id)\yToParent = sgx(*it\id)\y - sgx(parent)\y
                EndIf
              Else
                
                ; move other childs using their toParent-position
                sgx(*it\id)\x = sgx_GetGadgetX(*it\id)
                sgx(*it\id)\y = sgx_GetGadgetY(*it\id)
                
                ; handle the child's childs
                If sgx(*it\id)\hasChilds
                  sgx_MoveChildsOf(*it\id)
                EndIf
              EndIf
            sgx_llNext(*it)
          EndIf
        EndProcedure
        
        
        ;** sgx_GetGadgetX
        ;* returns (and is used to update) the elements absolute position on screen (x, y) together with <a href=#f_sgx_getgadgety>sgx_GetGadgetY</a> by summarizing all to-parent-positions up the parent hierarchy. _
        ;** .id : ID of the sgx element.
        Procedure sgx_GetGadgetX(id)
          Static ret
          Static parent
          ret = sgx(id)\xToParent
          parent = sgx(id)\parent
          While parent <> -1
            ret + sgx(parent)\xToParent
            parent = sgx(parent)\parent
          Wend
          ProcedureReturn ret
        EndProcedure
        
        
        ;** sgx_GetGadgetY
        ;* returns (and is used to update) the elements absolute position on screen (x, y) together with <a href=#f_sgx_getgadgetx>sgx_GetGadgetX</a> by summarizing all to-parent-positions up the parent hierarchy. _
        ;** .id : ID of the sgx element.
        Procedure sgx_GetGadgetY(id)
          Static ret
          Static parent
          ret = sgx(id)\yToParent
          parent = sgx(id)\parent
          While parent <> -1
            ret + sgx(parent)\yToParent
            parent = sgx(parent)\parent
          Wend
          ProcedureReturn ret
        EndProcedure
        
        
        ;** sgxCloseLevel
        ;* call this function when you finished the creation of a window or table and it's childs. _
        ;* note that this function will always set the current parent to its own parent and NOT necessarily to the value
        ;* it had before a call of <a href=#f_sgxsetcurrentparent>sgxSetCurrentParent</a>.
        ;** .closeAllLevels : if set to #True, there will be no current parent after the call and new creations will be global. _
        ;* if set to #False (default), the above parent level will be the new current parent.
        Procedure sgxCloseLevel(closeAllLevels=#False)
          If sgx_CurrentParent <> -1
            If sgx(sgx_CurrentParent)\type = #sgxTypeTable
              sgxRefresh(sgx_CurrentParent, #sgx_refreshNormalWithChildsOfTables)
              sgx_PopBuildRefresh()
            EndIf
            sgxSetCurrentParent(sgx(sgx_CurrentParent)\parent)
            
            If closeAllLevels = #True
              While sgx_CurrentParent <> -1
                sgxCloseLevel()
              Wend
            EndIf
          EndIf
        EndProcedure
        
        
        ;** sgx_IsParentage
        ;* returns #True if 'parent' is found anywhere up the parent hierarchy of 'child', otherwise returns #False. _
        ;* used to highlight parent windows as hovered if one of their child windows is the active window.
        ;** .parent : the parent elements ID to be searched for.
        ;** .child  : the child elements ID.
        Procedure sgx_IsParentage(parent, child)
          Static tempParent
        
          If sgxInBounds(parent) And sgxInBounds(child)
            tempParent = sgx(child)\parent
            While tempParent <> -1
              If tempParent = parent 
                ProcedureReturn #True
              EndIf
              tempParent = sgx(tempParent)\parent
            Wend
          EndIf
          ProcedureReturn #False
        EndProcedure
        
        
        ;** sgx_getTypeString
        ;* returns a string informing about the type of an sgx element for debugging purposes.
        ;** .id : ID of the sgx element.
        Procedure.s sgx_getTypeString(id)
          If sgx(id)
            Select sgx(id)\type
              Case #sgxTypeCursor       : ProcedureReturn "cursor"
              Case #sgxTypeButton       : ProcedureReturn "button"
              Case #sgxTypeButtonToggle : ProcedureReturn "buttonToggle"
              Case #sgxTypeSpriteGadget : ProcedureReturn "spriteGadet"
              Case #sgxTypeCheckbox     : ProcedureReturn "checkbox"
              Case #sgxTypeCombo        : ProcedureReturn "combo"
              Case #sgxTypeText         : ProcedureReturn "text"
              Case #sgxTypeEdit         : ProcedureReturn "edit"
              Case #sgxTypeScrollbarHor : ProcedureReturn "scrollbarHor"
              Case #sgxTypeScrollbarVer : ProcedureReturn "scrollbarVer"
              Case #sgxTypeTrackbarHor  : ProcedureReturn "trackbarHor"
              Case #sgxTypeTrackbarVer  : ProcedureReturn "trackbarVer"
              Case #sgxTypeTable        : ProcedureReturn "table"
              Case #sgxTypeWin          : ProcedureReturn "window"
              Case #sgxTypeArea         : ProcedureReturn "area"
              Default                   : ProcedureReturn "?"
            EndSelect
          EndIf
          ProcedureReturn "-"
        EndProcedure
        
        
        ;** sgxDebugElem
        ;* returns a string with information about the element's attributes.
        ;** .id : ID of the sgx element.
        ;** .multiline : #False (default) to return a single line. #True to format with newlines (#LF$ / Chr(10)).
        Procedure.s sgxDebugElem(id, multiline=#False)
          Protected ret.s, lf.s, text.s
          lf = ""
          If multiline
            lf = #LF$
          EndIf
          ret = ""
          ret + "[id=" + Str(id) + ", " + sgx_getTypeString(id) + "] " + lf
          ret + "(x,y=" + sgx(id)\x + "," + sgx(id)\y + ") " + lf
          ret + "(w,h=" + sgx(id)\w + "," + sgx(id)\h + ") " + lf
          ret + "(xToParent,yToParent=" + sgx(id)\xToParent + "," + sgx(id)\yToParent + ") " + lf
          ret + "(x2,y2=" + sgx(id)\x2 + "," + sgx(id)\y2 + ") " + lf
          ret + "(w2,h2=" + sgx(id)\w2 + "," + sgx(id)\h2 + ") " + lf
          ret + "(alpha=" + sgx(id)\alpha + ") " + lf
          ret + "(state=" + sgx(id)\state + ") " + lf
          ret + "(show=" + sgx(id)\show + ") " + lf
          ret + "(disabled=" + sgx(id)\disabled + ") " + lf
          ret + "(passive=" + sgx(id)\passive + ") " + lf
          text = Left(sgx(id)\text,100)
          If Len(text) > 20
            text = Left(text,20) + ".."
          EndIf
          ret + "(text=" + ReplaceString(text,#LF$,"\n") + ") " ; + lf
          ProcedureReturn ret
        EndProcedure
        
        
        ;** sgxDebugTree
        ;* returns a string with information about the elements attributes and their hierarchy.
        ;** .id : -1 (default) to debug the whole element hierarchy. specify an element ID to only debug that element
        ;*        <span></span> and its childs recursively.
        Procedure.s sgxDebugTree(id = -1)
          Protected ret.s
          Protected *list.sgx_llS_list
          Static level
        
          If id = -1
            ret = "[*sgxLayeredView]" + #LF$
            *list = *sgxLayeredView
          ElseIf sgxInBounds(id) And sgx(id)
            ret = LSet("",(level)*2,".")
            ret + sgxDebugElem(id)
            ret + #LF$
            *list = sgx(id)\treeElem\node
          Else
            ret = "sgxDebugTree: invalid id: " + Str(id) + #LF$
          EndIf
          
          If *list
            level+1
            sgx_llIterator(Protected *it, *list)
              ret + sgxDebugTree(*it\id)
            sgx_llNext(*it)
            level-1
          EndIf
          
          ProcedureReturn ret
        EndProcedure
        
        
        ;** sgxSetFont
        ;* set the font for coming creations of new sgx elements.
        ;** .font : a valid PB #Font. if you pass sgxStdFont or an invalid font then the current font will be reset to the default.
        Procedure sgxSetFont(font)
          If IsFont(font)
            sgx_CurrentFontID = FontID(font)
          Else
            sgx_CurrentFontID = FontID(sgxStdFont)
          EndIf
        EndProcedure
        
        
        ;** sgxSetGroup
        ;* add an sgx element to a radio group. _
        ;* checkboxes and/or toggle buttons can be grouped using this function so that if one element gets checked/pushed, all
        ;* others will be unchecked/untoggled. _
        ;* those elements can still be unchecked via righclick per default setting of variable sgxRightclickUnset, so that no 
        ;* element in that group would be checked/pushed. _
        ;* see also <a href=#f_sgxgetgroup>sgxGetGroup</a>.
        ;** .id      : ID of the sgx element to be added
        ;** .groupID : any number. use same numbers for elements you want to group
        Procedure sgxSetGroup(id, groupID)
          ForEach sgxGroups()
            If sgxGroups()\id=id
              DeleteElement(sgxGroups())
            EndIf
          Next
          AddElement(sgxGroups())
          sgxGroups()\id = id
          sgxGroups()\groupID = groupID
        EndProcedure
        
        
        ;** sgxGetGroup
        ;* returns the groupID of an sgx element, or -1 if the element does not belong to a radio group. _
        ;* see also <a href=#f_sgxsetgroup>sgxSetGroup</a>.
        ;** .id : ID of the sgx element to be inquired
        Procedure sgxGetGroup(id)
          ForEach sgxGroups()
            If sgxGroups()\id = id
              ProcedureReturn sgxGroups()\groupID
            EndIf
          Next
          ProcedureReturn -1
        EndProcedure
        
        
        ;** sgxSetDimensions
        ;* tell sgx about the screen dimensions you use. _
        ;* this isn't necessary if a screen has been opened before the first call of <a href=#f_sgxloaddesign>sgxLoadDesign</a>.
        ;** .width  : screenwidth
        ;** .height : screenheight
        Procedure sgxSetDimensions(width, height)
          sgx_dimensions\w = width
          sgx_dimensions\w = height
        EndProcedure
        
        
        ;** sgxFree
        ;* free / delete an sgx element, including childs. the ID of the element and all its child element IDs will become invalid.
        ;** .id : ID of the sgx element to be freed
        Procedure sgxFree(id)
          Protected i
          Protected parent
          If sgx(id)
            
            ;free childs
            If sgx(id)\hasChilds
              For i=0 To sgx_CURRENT_MAX_DYNAMIC_GADGETS
                sgx_SkipUnused(i)
                If sgx(i)  And  sgx(i)\parent = id
                  sgxFree(i)
                EndIf
              Next
              sgx(id)\hasChilds = 0
            EndIf
            
            ; the freed children should have removed themself from the node actually, so it should be empty already
            If sgx(id)\treeElem And sgx(id)\treeElem\node
              sgx_llClearList(sgx(id)\treeElem\node)
              FreeMemory(sgx(id)\treeElem\node)
              sgx(id)\treeElem\node = 0;
            EndIf
            
            ;free gadget sprites
            For i=0 To 4
              If i = #sgx_GadgetSprNormal2  And  (sgx(id)\type = #sgxTypeButton Or sgx(id)\type = #sgxTypeButtonToggle)
                ; dont't free user sprites
                Continue
              EndIf
              sgx_FreeSpriteChecked(sgx(id)\spr[i])
            Next
            
            ;free tip sprite and memory
            If sgx(id)\tip
              sgx_FreeSpriteChecked(sgx(id)\tip\spr)
              FreeMemory(sgx(id)\tip)
              sgx(id)\tip = 0
            EndIf
            
            ;free table data
            If sgx(id)\table
              If sgx(id)\table\cells
                FreeMemory(sgx(id)\table\cells)
                sgx(id)\table\cells=0
              EndIf
              If sgx(id)\table\gridx
                FreeMemory(sgx(id)\table\gridx)
                sgx(id)\table\gridx=0
              EndIf
              If sgx(id)\table\gridy
                FreeMemory(sgx(id)\table\gridy)
                sgx(id)\table\gridy=0
              EndIf
              FreeMemory(sgx(id)\table)
              sgx(id)\table = 0
            EndIf
            
            ;remove entries in sgxGroups()
            ForEach sgxGroups()
              If sgxGroups()\id = id
                DeleteElement(sgxGroups())
              EndIf
            Next
            
            ;clean environment
            If sgx_CurrentParent = id
              sgxSetCurrentParent( sgx(id)\parent )
            EndIf
            
            If id = sgxActiveWin       : sgxActiveWin       = -1 : EndIf
            If id = sgx_focusCatchedBy : sgx_focusCatchedBy = -1 : EndIf
            If id = sgx_resizing       : sgx_resizing       = -1 : EndIf
            If id = sgx_moving         : sgx_moving         = -1 : EndIf
            If id = sgx_editing        : sgx_editing        = -1 : EndIf
            
            If id = sgx_m\cursor
              sgx_m\cursor = -1
            EndIf
            
            ; remove window from taskbar
            If sgx(id)\type = #sgxTypeWin 
              sgx(id)\state = #sgxWinStateNormal
              sgx_RefreshWinBar()
            EndIf
            
            ; remove the tree list element from its current parent's node and free it (deleting the treeElem will remove it from it's list too)
            sgx_llDelElem(sgx(id)\treeElem)
            sgx(id)\treeElem = 0
            
            ; remove the node of the parent (the parent's child list) if empty and update the hasChilds flag
            parent = sgx(id)\parent
            If parent <> -1
              If Not sgx(parent)\treeElem\node\first
                FreeMemory(sgx(parent)\treeElem\node)
                sgx(parent)\treeElem\node = 0
                sgx(parent)\hasChilds = 0
              EndIf
            Else
              ; the root node *sgxLayeredView stays valid even if
              ; empty and there is no 'hasChilds' on the root level
            EndIf
            
            ; free the basic sgxS_GADGET memory
            FreeMemory(sgx(id))
            sgx(id) = 0
            
            sgxNumberOfElements - 1
          EndIf
        EndProcedure
        
        
        ;** sgxFreeChildsOf
        ;* free / delete all childs of an sgx element, including their own childs.
        ;** .parent : ID of the sgx element whose childs are to be freed. this parent will remain valid, the IDs of all 
        ;*            <span></span> child elements will become invald.
        Procedure sgxFreeChildsOf(parent)
          Protected id
          For id=0 To sgx_CURRENT_MAX_DYNAMIC_GADGETS
            sgx_SkipUnused(id)
            If sgx(id) And sgx(id)\parent = parent
              sgxFree(id)
            EndIf
          Next
        EndProcedure
        
        
        ;** sgx_HideChildsOf
        ;* hides or shows all childs of an element (but not the element itself) recursively but ignores child windows (per default).
        ;** .parent        : ID of the element whose childs are to be hidden/shown.
        ;** .show          : #False (default) to hide the childs, #True to make them visible.
        ;** .ignoreWindows : #True (default) to not modify child windows. #False if you want to hide/show also child windows (if 'parent' is a window).
        Procedure sgx_HideChildsOf(parent, show=#False, ignoreWindows=#True)
          Protected id
          For id=0 To sgx_CURRENT_MAX_DYNAMIC_GADGETS
            sgx_SkipUnused(id)
            If sgx(id)  And  (sgx(id)\parent = parent)  And  (sgx(id)\type <> #sgxTypeWin Or Not ignoreWindows)
              If sgx(id)\hasChilds
                sgx_HideChildsOf(id,show)
              EndIf
              sgx(id)\show = show
            EndIf
          Next
        EndProcedure
        
        
        ;** sgxSetShow
        ;* shows or hides an element. _
        ;* if you hide a window including its childs and later show it again then previously hidden childs will be all visible and
        ;* not restored to their actual previous visibility.
        ;** .id            : ID of the sgx element.
        ;** .show          : #True to show the element, or #False to hide it.
        ;** .recursively   : #True (default) to show/hide child elements too recursively. #False to only show/hide the element itself.
        ;** .ignoreWindows : if parameter 'recursively' is #True then recursion will not traverse into child windows per default (#True). set it to #False for complete recursion including child windows.
        Procedure sgxSetShow(id, show, recursively=#True, ignoreWindows=#True)
          Static oldShow
          If sgxInBounds(id) And sgx(id)
            oldShow = sgx(id)\show
            sgx(id)\show = Bool(show)
            ; taskbar refresh for windows
            If sgx(id)\type = #sgxTypeWin  And  oldShow <> sgx(id)\show
              sgx_RefreshWinBar()
            EndIf
            If recursively
              sgx_HideChildsOf(id, sgx(id)\show, ignoreWindows)
            EndIf
          EndIf
        EndProcedure
        
        
        
        ;** sgx_addAnimation
        ;* add an animation data object to the animation list. _
        ;* used by <a href=#f_sgxminimizewin>sgxMinimizeWin</a>, <a href=#f_sgxrestorewin>sgxRestoreWin</a>, <a href=#f_sgxclosewin>sgxCloseWin</a> and <a href=#f_sgxpresentwin>sgxPresentWin</a>
        ;** .id : ID of the element that should be animated.
        ;** .animationType : #sgxWinStateMinimizing, #sgxWinStateRestoring, #sgxWinStateClosing or #sgxWinStatePresenting.
        Procedure sgx_addAnimation(id, animationType)
          If sgx(id)
            AddMapElement(sgx_animations(),Str(id))
            sgx_animations()\startTime = sgx_ems
            sgx_animations()\startX = sgx(id)\x
            sgx_animations()\startY = sgx(id)\y
            sgx_animations()\startW = sgx(id)\w
            sgx_animations()\startH = sgx(id)\h
            sgx_animations()\startAlpha = sgx(id)\alpha
            sgx_animations()\endX = sgx_animations()\startX
            sgx_animations()\endY = sgx_animations()\startY
            sgx_animations()\endW = sgx_animations()\startW
            sgx_animations()\endH = sgx_animations()\startH
            sgx_animations()\endAlpha = sgx_animations()\startAlpha
            
            ; the alpha value of childs of windows may be animated/interpolated
            ; directly between alpha and alpha2 of the element not using this
            ; general animation data
            
            If animationType = #sgxWinStateMinimizing
              
              If sgx_animStyle_minimizing & #sgxAnimFade
                sgx_animations()\endAlpha = 0
              EndIf
              If sgx_animStyle_minimizing & #sgxAnimRoll
                sgx_animations()\endY = sgx_dimensions\h-2-20
              EndIf
              If sgx_animStyle_minimizing & #sgxAnimZoom
                sgx_animations()\endX = sgx_xMinimizedWins
                sgx_animations()\endY = sgx_dimensions\h-2-20
                sgx_animations()\endW = 100
                sgx_animations()\endH = 20
              EndIf
              
            ElseIf animationType = #sgxWinStateRestoring
              
              If sgx_animStyle_restoring & #sgxAnimFade
                sgx_animations()\startAlpha = 0
                sgx_animations()\endAlpha = sgx(id)\alpha2
              Else
                sgx_animations()\startAlpha = sgx(id)\alpha2
                sgx_animations()\endAlpha = sgx(id)\alpha2
              EndIf
              If sgx_animStyle_restoring & #sgxAnimRoll
                sgx_animations()\startX = sgx(id)\x2
                sgx_animations()\startY = sgx_dimensions\h
                sgx_animations()\startW = sgx(id)\w2
                sgx_animations()\startH = sgx(id)\h2
                sgx_animations()\endY = sgx(id)\y2
              ElseIf sgx_animStyle_restoring & #sgxAnimZoom
                sgx_animations()\endX = sgx(id)\x2
                sgx_animations()\endY = sgx(id)\y2
                sgx_animations()\endW = sgx(id)\w2
                sgx_animations()\endH = sgx(id)\h2
              Else
                sgx_animations()\startX = sgx(id)\x2
                sgx_animations()\startY = sgx(id)\y2
                sgx_animations()\startW = sgx(id)\w2
                sgx_animations()\startH = sgx(id)\h2
              EndIf
              
            ElseIf animationType = #sgxWinStateClosing
              
              If sgx_animStyle_closing & #sgxAnimFade
                sgx_animations()\endAlpha = 0
              EndIf
              If sgx_animStyle_closing & #sgxAnimSizeW
                sgx_animations()\endW = 0
              EndIf
              If sgx_animStyle_closing & #sgxAnimSizeH
                sgx_animations()\endH = 0
              EndIf
              
            ElseIf animationType = #sgxWinStatePresenting
              
              If sgx_animStyle_presenting & #sgxAnimFade
                sgx_animations()\startAlpha = 0
                sgx_animations()\endAlpha = sgx(id)\alpha
              Else
                sgx_animations()\startAlpha = sgx(id)\alpha
                sgx_animations()\endAlpha = sgx(id)\alpha
              EndIf
              If sgx_animStyle_presenting & #sgxAnimSizeW
                sgx_animations()\startW = 1
                sgx_animations()\endW = sgx(id)\w
              EndIf
              If sgx_animStyle_presenting & #sgxAnimSizeH
                sgx_animations()\startH = 1
                sgx_animations()\endH = sgx(id)\h
              EndIf
              
            EndIf
          EndIf
        EndProcedure
        
        
        ;** sgxMinimizeWin
        ;* initializes minimizing of window. _
        ;* the animation might finish immediately.
        Procedure sgxMinimizeWin(id)
          Protected *it.sgx_llS_elem
          Static skipNode
          If sgx(id) And sgx(id)\type = #sgxTypeWin And sgx(id)\state = #sgxWinStateNormal
            
            ; backup current properties
            sgx(id)\x2 = sgx(id)\x
            sgx(id)\y2 = sgx(id)\y
            sgx(id)\w2 = sgx(id)\w
            sgx(id)\h2 = sgx(id)\h
            sgx(id)\alpha2 = sgx(id)\alpha
            ; backup child alpha/show individually
            If sgx(id)\hasChilds
              skipNode=#False
              sgx_llIteratorRecursive(*it, sgx(id)\treeElem\node)
                ; constrain recursion to non-window childs
                If sgx(*it\id)\type = #sgxTypeWin
                  skipNode = #True
                Else
                  skipNode = #False
                  sgx(*it\id)\alpha2 = sgx(*it\id)\alpha
                  sgx(*it\id)\show2 = sgx(*it\id)\show
                EndIf
              sgx_llNextRecursive(*it, skipNode)
            EndIf
            
            ; create animation
            sgx_addAnimation(id, #sgxWinStateMinimizing)
            sgx(id)\state = #sgxWinStateMinimizing
            If sgx_animStyle_minimizing = 0
              ; if no animation style then expire the animation immediately
              sgx_animations()\startTime = sgx_ems - sgxAnimTime
            EndIf
            sgx_animate(id, sgx_animations())
            
            ; hide childs if size is animated and if animation has not finished immediately (happens if zero anim styles or a small/zero sgxAnimTime)
            If (sgx_animStyle_minimizing & #sgxAnimZoom) And sgx(id)\state = #sgxWinStateMinimizing
              sgx_HideChildsOf(id)
            EndIf
            
            If id = sgxActiveWin       : sgxActiveWin       = -1 : EndIf
            If id = sgx_focusCatchedBy : sgx_focusCatchedBy = -1 : EndIf
            If id = sgx_resizing       : sgx_resizing       = -1 : EndIf
            If id = sgx_moving         : sgx_moving         = -1 : EndIf
            ;If id = sgx_editing        : sgx_editing        = -1 : EndIf
          EndIf
        EndProcedure
        
        
        ;** sgxRestoreWin
        ;* initializes restoring of window. _
        ;* the window's 'resized' event will be triggered when the animation has finished which can be received after 
        ;* the next call of <a href=#f_sgxexamine>sgxExamine</a>. _
        ;* the animation might finish immediately.
        Procedure sgxRestoreWin(id)
          Protected *it.sgx_llS_elem ; use Protected when using recursion!
          Static skipNode
          If sgx(id) And sgx(id)\type = #sgxTypeWin And sgx(id)\state = #sgxWinStateMinimized
            
            ; set state and restore pos/size
            sgx_addAnimation(id, #sgxWinStateRestoring)
            sgx(id)\state = #sgxWinStateRestoring
            
            ; how restoring begins depends on animation styles
            sgx(id)\x = sgx_animations()\startX
            sgx(id)\y = sgx_animations()\startY
            sgx(id)\w = sgx_animations()\startW
            sgx(id)\h = sgx_animations()\startH
            sgxSetAlpha(sgx_animations()\startAlpha, id, #True)
            
            sgxRefresh(id)
            sgxMove(id, sgx(id)\x, sgx(id)\y, #False)
            sgx_RefreshWinBar()
            sgxActiveWin=id
            
            If sgx_animStyle_restoring = 0
              ; if no animation style then expire the animation immediately
              sgx_animations()\startTime = sgx_ems - sgxAnimTime
            EndIf
            sgx_animate(id, sgx_animations())
            
            ; hide childs if size is animated and if animation has not finished immediately (happens if zero anim styles or a small/zero sgxAnimTime)
            If (sgx_animStyle_restoring & #sgxAnimZoom)  And  sgx(id)\state = #sgxWinStateRestoring
              sgx_HideChildsOf(id)
            Else
              ; otherwise show
              ;sgx_HideChildsOf(id, #True)
              ; restore child show individually
              If sgx(id)\hasChilds
                skipNode=#False
                sgx_llIteratorRecursive(*it, sgx(id)\treeElem\node)
                  ; constrain recursion to non-window childs
                  If sgx(*it\id)\type = #sgxTypeWin
                    skipNode = #True
                  Else
                    skipNode = #False
                    sgx(*it\id)\show = sgx(*it\id)\show2
                  EndIf
                sgx_llNextRecursive(*it, skipNode)
              EndIf
            EndIf
            
          EndIf
        EndProcedure
        
        
        ;** sgxCloseWin
        ;* initializes closing of a window and it's child windows. _
        ;* the animation might finish immediately in wich case the element and its childs will have been deleted
        ;* and their IDs become invalid. in that case the returnvalue is 1, otherwise 0.
        Procedure sgxCloseWin(id)
          Protected *it.sgx_llS_elem ; use Protected when using recursion!
          Static destroyed
          If sgx(id) And sgx(id)\type = #sgxTypeWin ;And sgx(id)\state = #sgxWinStateNormal
            sgx(id)\state = #sgxWinStateClosing
            sgx_addAnimation(id, #sgxWinStateClosing)
            
            ; backup current properties
            ; (though not used at the moment since the element will die anyway)
            sgx(id)\x2 = sgx(id)\x
            sgx(id)\y2 = sgx(id)\y
            sgx(id)\w2 = sgx(id)\w
            sgx(id)\h2 = sgx(id)\h
            sgx(id)\alpha2 = sgx(id)\alpha
            ; backup child alpha/show individually
            If sgx(id)\hasChilds
              sgx_llIteratorRecursive(*it, sgx(id)\treeElem\node)
                sgx(*it\id)\alpha2 = sgx(*it\id)\alpha
                sgx(*it\id)\show2 = sgx(*it\id)\show
              sgx_llNextRecursive(*it)
            EndIf
            
            ; withdraw focus..
            If id = sgxActiveWin       : sgxActiveWin       = -1 : EndIf
            If id = sgx_focusCatchedBy : sgx_focusCatchedBy = -1 : EndIf
            If id = sgx_resizing       : sgx_resizing       = -1 : EndIf
            If id = sgx_moving         : sgx_moving         = -1 : EndIf
            If id = sgx_editing        : sgx_editing        = -1 : EndIf
            ; ..also from childs
            If sgx(id)\hasChilds
              sgx_llIteratorRecursive(*it, sgx(id)\treeElem\node)
                If *it\id = sgxActiveWin       : sgxActiveWin       = -1 : EndIf
                If *it\id = sgx_focusCatchedBy : sgx_focusCatchedBy = -1 : EndIf
                If *it\id = sgx_resizing       : sgx_resizing       = -1 : EndIf
                If *it\id = sgx_moving         : sgx_moving         = -1 : EndIf
                If *it\id = sgx_editing        : sgx_editing        = -1 : EndIf
              sgx_llNextRecursive(*it)
            EndIf
            
            If sgx_animStyle_closing = 0
              ; if no animation style then expire the animation immediately
              sgx_animations()\startTime = sgx_ems - sgxAnimTime
            EndIf
            destroyed = sgx_animate(id, sgx_animations())
            
            ; animation might have finished immediately (happens if zero anim styles or a small/zero sgxAnimTime)
            If Not destroyed
              
              ; hide childs if size is animated
              If (sgx_animStyle_closing & #sgxAnimSize)
                sgx_HideChildsOf(id)
              EndIf
              
              ; close child windows as well
              If sgx(id)\hasChilds
                sgx_llIterator(*it, sgx(id)\treeElem\node)
                  If sgx(*it\id)  And  (sgx(*it\id)\type = #sgxTypeWin)  ;And  (sgx(*it\id)\state = #sgxWinStateNormal)
                    sgxCloseWin(*it\id)
                  EndIf
                sgx_llNext(*it)
              EndIf
              
            Else
              ; tell the caller the element has been destroyed
              ProcedureReturn 1
            EndIf
          EndIf
          ProcedureReturn 0
        EndProcedure
        
        
        ;** sgxPresentWin
        ;* initializes presenting of a window and it's child windows. _
        ;* the window's 'resized' event will be triggered when the animation has finished which can be received after 
        ;* the next call of <a href=#f_sgxexamine>sgxExamine</a>. _
        ;* the animation might finish immediately.
        Procedure sgxPresentWin(id)
          Protected *it.sgx_llS_elem ; use Protected when using recursion!
          Protected skipNode
          
          If sgx(id) And sgx(id)\type = #sgxTypeWin And sgx(id)\state = #sgxWinStateNormal
            sgx(id)\state = #sgxWinStatePresenting
            sgx_addAnimation(id, #sgxWinStatePresenting)
            
            ; backup current properties
            sgx(id)\x2 = sgx(id)\x
            sgx(id)\y2 = sgx(id)\y
            sgx(id)\w2 = sgx(id)\w
            sgx(id)\h2 = sgx(id)\h
            sgx(id)\alpha2 = sgx(id)\alpha
            ; backup child alpha/show individually
            If sgx(id)\hasChilds
              sgx_llIteratorRecursive(*it, sgx(id)\treeElem\node)
                skipNode=#False
                ; constrain recursion to non-window childs (child windows handled later)
                If sgx(*it\id)\type = #sgxTypeWin
                  skipNode = #True
                Else
                  skipNode = #False
                  ; backup alpha
                  sgx(*it\id)\alpha2 = sgx(*it\id)\alpha
                  
                  ; backup 'show' only if window (and childs) has not been hidden (i.e. after creation for later presenting)
                  If sgx(id)\show
                    sgx(*it\id)\show2 = sgx(*it\id)\show
                  Else
                    ; other wise we just backup #True for later restoring of visibility.
                    sgx(*it\id)\show2 = #True
                  EndIf
                EndIf
              sgx_llNextRecursive(*it, skipNode)
            EndIf
            
            ; how presenting begins depends on animation styles
            sgx(id)\x = sgx_animations()\startX 
            sgx(id)\y = sgx_animations()\startY
            sgx(id)\w = sgx_animations()\startW
            sgx(id)\h = sgx_animations()\startH
            sgxSetAlpha(sgx_animations()\startAlpha, id, #True)
            
            sgxRefresh(id)
            sgxMove(id, sgx(id)\x, sgx(id)\y, #False)
            
            If sgx_animStyle_presenting = 0
              ; if no animation style then expire the animation immediately
              sgx_animations()\startTime = sgx_ems - sgxAnimTime
            EndIf
            
            sgx_animate(id, sgx_animations())
            
            ; hide childs and show window if size is animated and if animation has not finished immediately (happens if zero anim styles or a small/zero sgxAnimTime)
            If sgx_animStyle_presenting & #sgxAnimSize And sgx(id)\state = #sgxWinStatePresenting
              sgx_HideChildsOf(id)
              sgxSetShow(id, #True, #False)
            Else
              ; show element and childs if window is hidden (i.e. has been hidden after creation for later presenting).
              ; if not hidden then keep current visibility including child visibility
              If Not sgx(id)\show
                sgxSetShow(id, #True)
              EndIf
            EndIf
            
            ; present child windows as well
            If sgx(id)\hasChilds
              sgx_llIterator(*it, sgx(id)\treeElem\node)
                If sgx(*it\id)\type = #sgxTypeWin  And  sgx(*it\id)\state = #sgxWinStateNormal
                  sgxPresentWin(*it\id)
                EndIf
              sgx_llNext(*it)
            EndIf
            
          EndIf
        EndProcedure
        
        
        ;** sgx_RefreshWinBar
        ;* determines visible minimized windows and repositions them as buttons in the taskbar. _
        ;* used after a window's state or its visibility has changed for example.
        Procedure sgx_RefreshWinBar()
          Static *it.sgx_llS_elem ; use Protected when using recursion!
          sgx_xMinimizedWins = 2
          sgx_llIteratorRecursive(*it, *sgxLayeredView)
            If sgx(*it\id)  And  (sgx(*it\id)\type = #sgxTypeWin)  And  sgx(*it\id)\show  And  (sgx(*it\id)\state = #sgxWinStateMinimized)
              sgxMove(*it\id, sgx_xMinimizedWins, sgx(*it\id)\y, #False)
              sgx_xMinimizedWins + sgx(*it\id)\w + 2
            EndIf
          sgx_llNextRecursive(*it)
        EndProcedure
        
        
        ;** sgx_animate
        ;* animate an element, that is to say, update its attributes according to the currently reach animation progress.  _
        ;* returns 1 if the element has been destroyed due to a finished closing animation.
        ;** .id         : ID of the element to be animated
        ;** .*animation : an animtaion data object from the animation map 'sgx_animations'
        Procedure sgx_animate(id, *animation.sgxS_Animation)
          Static time
          Static fraction.f
          Static *it.sgx_llS_elem ; use Protected when using recursion!
          Static skipNode
          
          time = 0
          fraction.f = 0
          
          If *animation
            
            ; time passed since animation start
            time = (sgx_ems - *animation\startTime)
            
            ; how much we have progressed in the animation (0.0 at the beginning, 1.0 when animation time reached sgxAnimTime)
            fraction.f = time/sgxAnimTime
            If fraction > 1
              ; fraction gets much to big if sgxAnimTime is very small or 0, for example time=60ms and sgxAnimTime=1
              fraction = 1
            EndIf
            
            ; ------------------------------------------------------
            ; minimizing window
            If sgx(id)\state = #sgxWinStateMinimizing
              
              ; animate
              If sgx_animStyle_minimizing & #sgxAnimRoll
                sgxMove(id, sgx(id)\x, *animation\startY + fraction * (*animation\endY - *animation\startY), #False )
              ElseIf sgx_animStyle_minimizing & #sgxAnimZoom
                sgx(id)\x = *animation\startX - fraction * (*animation\startX - *animation\endX)
                sgx(id)\y = *animation\startY + fraction * (*animation\endY - *animation\startY)
                
                sgx(id)\w = *animation\startW - fraction * (*animation\startW - *animation\endW)
                If sgx(id)\w < 1
                  sgx(id)\w = 1
                EndIf
                sgx(id)\h = *animation\startH - fraction * (*animation\startH - *animation\endH)
                If sgx(id)\h < 1
                  sgx(id)\h = 1
                EndIf
                sgxRefresh(id)
                sgxMove(id, sgx(id)\x, sgx(id)\y, #False)
              EndIf
              If sgx_animStyle_minimizing & #sgxAnimFade
                ;sgxSetAlpha(*animation\startAlpha - fraction * (*animation\startAlpha - *animation\endAlpha), id, #True)
                
                ; animating alpha individually
                sgx(id)\alpha = sgx(id)\alpha2 - fraction * sgx(id)\alpha2
                If sgx(id)\hasChilds
                  sgx_llIteratorRecursive(*it, sgx(id)\treeElem\node)
                    skipNode=#False
                    ; constrain recursion to non-window childs
                    If sgx(*it\id)\type = #sgxTypeWin
                      skipNode = #True
                    Else
                      skipNode = #False
                      sgx(*it\id)\alpha = sgx(*it\id)\alpha2 - fraction * sgx(*it\id)\alpha2
                    EndIf
                  sgx_llNextRecursive(*it, skipNode)
                EndIf
              EndIf
              
              ; finish minimizing
              If time >= sgxAnimTime
                
                ; set state
                sgx(id)\state = #sgxWinStateMinimized
                
                ; withdraw any focus
                If id = sgxActiveWin       : sgxActiveWin       = -1 : EndIf
                If id = sgx_focusCatchedBy : sgx_focusCatchedBy = -1 : EndIf
                If id = sgx_resizing       : sgx_resizing       = -1 : EndIf
                If id = sgx_moving         : sgx_moving         = -1 : EndIf
                If id = sgx_editing        : sgx_editing        = -1 : EndIf
                
                ; transform window to taskbar button
                sgx(id)\w = 100
                sgx(id)\h = 20
                sgx(id)\x = sgx_xMinimizedWins
                sgx(id)\y = sgx_dimensions\h-2-20
                sgxMove(id, sgx(id)\x, sgx(id)\y, #False)
                sgx_RefreshWinBar()
                sgxRefresh(id)
                
                ; restore alpha
                sgx(id)\alpha = sgx(id)\alpha2
                ; restore child alpha/show individually
                If sgx(id)\hasChilds
                  skipNode=#False
                  sgx_llIteratorRecursive(*it, sgx(id)\treeElem\node)
                    ; constrain recursion to non-window childs
                    If sgx(*it\id)\type = #sgxTypeWin
                      skipNode = #True
                    Else
                      skipNode = #False
                      sgx(*it\id)\alpha = sgx(*it\id)\alpha2
                      sgx(*it\id)\show = sgx(*it\id)\show2
                    EndIf
                  sgx_llNextRecursive(*it, skipNode)
                EndIf
                
                ; hide childs
                sgx_HideChildsOf(id, #False, #True)
                
                ; remove animation
                DeleteMapElement(sgx_animations(), Str(id))
                *animation = 0
              EndIf
              
            ; ------------------------------------------------------
            ; restoring window
            ElseIf sgx(id)\state = #sgxWinStateRestoring
              
              ; animate
              If sgx_animStyle_restoring & #sgxAnimRoll
                sgxMove(id, sgx(id)\x, *animation\startY + fraction * (*animation\endY - *animation\startY), #False )
              ElseIf sgx_animStyle_restoring & #sgxAnimZoom
                sgx(id)\x = *animation\startX + fraction * (*animation\endX - *animation\startX)
                sgx(id)\y = *animation\startY - fraction * (*animation\startY - *animation\endY)
                
                sgx(id)\w = *animation\startW + fraction * (*animation\endW - *animation\startW)
                If sgx(id)\w < 1
                  sgx(id)\w = 1
                EndIf
                sgx(id)\h = *animation\startH + fraction * (*animation\endH - *animation\startH)
                If sgx(id)\h < 1
                  sgx(id)\h = 1
                EndIf
                sgxRefresh(id)
                sgxMove(id, sgx(id)\x, sgx(id)\y, #False)
              EndIf
              If sgx_animStyle_restoring & #sgxAnimFade
                ;sgxSetAlpha(*animation\startAlpha + fraction * (*animation\endAlpha - *animation\startAlpha), id, #True)
                
                ; animating alpha individually
                sgx(id)\alpha = fraction * sgx(id)\alpha2
                If sgx(id)\hasChilds
                  skipNode=#False
                  sgx_llIteratorRecursive(*it, sgx(id)\treeElem\node)
                    ; constrain recursion to non-window childs
                    If sgx(*it\id)\type = #sgxTypeWin
                      skipNode = #True
                    Else
                      skipNode = #False
                      sgx(*it\id)\alpha = fraction * sgx(*it\id)\alpha2
                    EndIf
                  sgx_llNextRecursive(*it, skipNode)
                EndIf
              EndIf
              
              ; finish restoring
              If time >= sgxAnimTime
                
                ; set state
                sgx(id)\state = #sgxWinStateNormal
                
                ; restore properties (resetting x,w,h and refresh might be redundant for 'roll')
                sgx(id)\x = sgx(id)\x2
                sgx(id)\y = sgx(id)\y2
                sgx(id)\w = sgx(id)\w2
                sgx(id)\h = sgx(id)\h2
                sgxMove(id, sgx(id)\x, sgx(id)\y, #False)
                sgxRefresh(id)
                
                ; restore alpha
                sgx(id)\alpha = sgx(id)\alpha2
                ; restore child alpha/show individually
                If sgx(id)\hasChilds
                  skipNode=#False
                  sgx_llIteratorRecursive(*it, sgx(id)\treeElem\node)
                    ; constrain recursion to non-window childs
                    If sgx(*it\id)\type = #sgxTypeWin
                      skipNode = #True
                    Else
                      skipNode = #False
                      sgx(*it\id)\alpha = sgx(*it\id)\alpha2
                      sgx(*it\id)\show = sgx(*it\id)\show2
                    EndIf
                  sgx_llNextRecursive(*it, skipNode)
                EndIf
                
                ; show childs (should be already if size is not animated)
                ;sgx_HideChildsOf(id, #True)
                
                ; prepare event triggering
                AddElement(sgx_pendingEvents())
                sgx_pendingEvents()\id = id
                sgx_pendingEvents()\resized = 1
                
                sgxActiveWin = id
                
                ; remove animation
                DeleteMapElement(sgx_animations(), Str(id))
                *animation = 0
              EndIf
              
            ; ------------------------------------------------------
            ; closing window
            ElseIf sgx(id)\state = #sgxWinStateClosing
              
              ; animate
              If sgx_animStyle_closing & #sgxAnimSizeW
                sgx(id)\w = *animation\startW - fraction * (*animation\startW - *animation\endW)
                If sgx(id)\w < 1
                  sgx(id)\w = 1
                EndIf
                sgx(id)\x = *animation\startX + *animation\startW/2 - sgx(id)\w/2
              EndIf
              If sgx_animStyle_closing & #sgxAnimSizeH
                sgx(id)\h = *animation\startH - fraction * (*animation\startH - *animation\endH)
                If sgx(id)\h < 1
                  sgx(id)\h = 1
                EndIf
                sgx(id)\y = *animation\startY + *animation\startH/2 - sgx(id)\h/2
              EndIf
              If sgx_animStyle_closing & #sgxAnimSize
                sgxRefresh(id)
                sgxMove(id, sgx(id)\x, sgx(id)\y, #False)
              EndIf
              If sgx_animStyle_closing & #sgxAnimFade
                ;sgxSetAlpha(*animation\startAlpha - fraction * (*animation\startAlpha - *animation\endAlpha), id, #True)
                
                ; animating alpha individually
                sgx(id)\alpha = sgx(id)\alpha2 - fraction * sgx(id)\alpha2
                If sgx(id)\hasChilds
                  sgx_llIteratorRecursive(*it, sgx(id)\treeElem\node)
                    sgx(*it\id)\alpha = sgx(*it\id)\alpha2 - fraction * sgx(*it\id)\alpha2
                  sgx_llNextRecursive(*it)
                EndIf
              EndIf
              
              ; finish closing
              If time >= sgxAnimTime
                
                ; remove element
                sgxFree(id)
                
                ; remove animation
                DeleteMapElement(sgx_animations(), Str(id))
                *animation = 0
                
                ; tell the caller the element has been destroyed
                ProcedureReturn 1
              EndIf
              
            ; ------------------------------------------------------
            ; presenting window
            ElseIf sgx(id)\state = #sgxWinStatePresenting
              
              ;Debug "id:"+Str(id)+" x,y:"+Str(sgx(id)\x)+","+Str(sgx(id)\y)+" w,h:"+Str(sgx(id)\w)+","+Str(sgx(id)\h)+", a:"+Str(sgx(id)\alpha)+", sh:"+Str(sgx(id)\show)+", ss:"+Str(sgx(id)\state)
              
              ; animate
              If sgx_animStyle_presenting & #sgxAnimSizeW
                sgx(id)\w = *animation\startW - fraction * (*animation\startW - *animation\endW)
                If sgx(id)\w < 1
                  sgx(id)\w = 1
                EndIf
                sgx(id)\x = *animation\startX + *animation\endW/2 - sgx(id)\w/2
              EndIf
              If sgx_animStyle_presenting & #sgxAnimSizeH
                sgx(id)\h = *animation\startH - fraction * (*animation\startH - *animation\endH)
                If sgx(id)\h < 1
                  sgx(id)\h = 1
                EndIf
                sgx(id)\y = *animation\startY + *animation\endH/2 - sgx(id)\h/2
              EndIf
              If sgx_animStyle_presenting & #sgxAnimSize ; if size is animated
                sgxRefresh(id)
                sgxMove(id, sgx(id)\x, sgx(id)\y, #False)
              EndIf
              If sgx_animStyle_presenting & #sgxAnimFade
                ;sgxSetAlpha(*animation\startAlpha - fraction * (*animation\startAlpha - *animation\endAlpha), id, #True)
                
                ; animating alpha individually
                sgx(id)\alpha = fraction * sgx(id)\alpha2
                If sgx(id)\hasChilds
                  skipNode=#False
                  sgx_llIteratorRecursive(*it, sgx(id)\treeElem\node)
                    ; constrain recursion to non-window childs
                    If sgx(*it\id)\type = #sgxTypeWin
                      skipNode = #True
                    Else
                      skipNode = #False
                      sgx(*it\id)\alpha = fraction * sgx(*it\id)\alpha2
                    EndIf
                  sgx_llNextRecursive(*it, skipNode)
                EndIf
              EndIf
              
              ;Debug "id:"+Str(id)+" x,y:"+Str(sgx(id)\x)+","+Str(sgx(id)\y)+" w,h:"+Str(sgx(id)\w)+","+Str(sgx(id)\h)+", a:"+Str(sgx(id)\alpha)+", sh:"+Str(sgx(id)\show)+", ss:"+Str(sgx(id)\state)
              
              ; finish presenting
              If time >= sgxAnimTime
                
                ; set state
                sgx(id)\state = #sgxWinStateNormal
                
                ; restore properties
                sgx(id)\x = sgx(id)\x2
                sgx(id)\y = sgx(id)\y2
                sgx(id)\w = sgx(id)\w2
                sgx(id)\h = sgx(id)\h2
                sgxMove(id, sgx(id)\x, sgx(id)\y, #False)
                sgxRefresh(id)
                
                ; restore alpha
                sgx(id)\alpha = sgx(id)\alpha2
                ; restore child alpha/show individually
                sgx(id)\alpha = sgx(id)\alpha2
                If sgx(id)\hasChilds
                  skipNode=#False
                  sgx_llIteratorRecursive(*it, sgx(id)\treeElem\node)
                    ; constrain recursion to non-window childs
                    If sgx(*it\id)\type = #sgxTypeWin
                      skipNode = #True
                    Else
                      skipNode = #False
                      sgx(*it\id)\alpha = sgx(*it\id)\alpha2
                      sgx(*it\id)\show = sgx(*it\id)\show2
                    EndIf
                  sgx_llNextRecursive(*it, skipNode)
                EndIf
                
                ; prepare event triggering
                AddElement(sgx_pendingEvents())
                sgx_pendingEvents()\id = id
                sgx_pendingEvents()\resized = 1
                
                sgxActiveWin = id
                
                ; show childs  
                ;sgx_HideChildsOf(id, #True)  
                
                ; remove animation
                DeleteMapElement(sgx_animations(), Str(id))
                *animation = 0
              EndIf
            EndIf
          EndIf
          
          ; everything is normal
          ProcedureReturn 0;
        EndProcedure
        
        
        ;** sgxSetWinAnimStyle
        ;* configures window animations. _
        ;* animation of size will make a window hide its childs during the animation. animations 'zoom' and 'roll' only work for minimizing and restoring of windows.
        ;** .winState     : the type of animation you want to configure: #sgxWinStateMinimizing, #sgxWinStateRestoring, #sgxWinStateClosing or #sgxWinStatePresenting
        ;** .winAnimStyle : combination of animation styles (you can combine them with '|' (bitwise OR)): _
        ;*                  #sgxAnimFade : animate the elements alpha value (transparency) _ 
        ;*                  #sgxAnimSizeW : collapse / expand the width _ 
        ;*                  #sgxAnimSizeH : collapse / expand the height _ 
        ;*                  #sgxAnimSize : collapse / expand width and height _ 
        ;*                  #sgxAnimZoom : zoom windows into / out of the taskbar _
        ;*                  #sgxAnimRoll : roll windows down to / up from the taskbar _
        Procedure sgxSetWinAnimStyle(winState, winAnimStyle)
          Select winState
            Case #sgxWinStateMinimizing
              sgx_animStyle_minimizing = winAnimStyle
            Case #sgxWinStateRestoring
              sgx_animStyle_restoring = winAnimStyle
            Case #sgxWinStateClosing
              sgx_animStyle_closing = winAnimStyle
            Case #sgxWinStatePresenting
              sgx_animStyle_presenting = winAnimStyle
          EndSelect
        EndProcedure
        
        
        ;** sgxSetTextcursorType
        ;* changes the type of the textcursor used by sgxEdit elements. call this function before creating sgxEdit elements, otherwise the change takes effect with next element refresh.
        ;** .textcursorType   : one of these values: _
        ;*                      #sgxTextcursorTypeIBeam (I-Beam type cursor) _
        ;*                      #sgxTextcursorTypeTeeth (something like an open I-Beam cursor) _
        ;*                      #sgxTextcursorTypeLine (vertical line) _
        ;*                      #sgxTextcursorTypeLine2 (thicker vertical line/bar) _
        ;*                      #sgxTextcursorTypeLine3 (vertical line/bar with some alpha blending) _
        ;*                      #sgxTextcursorTypeCaret (caret type cursor) _
        Procedure sgxSetTextcursorType(textcursorType)
          Select textcursorType
            Case #sgxTextcursorTypeIBeam, #sgxTextcursorTypeTeeth, 
                 #sgxTextcursorTypeLine, #sgxTextcursorTypeLine2, #sgxTextcursorTypeLine3,
                 #sgxTextcursorTypeCaret
              sgx_textcursorType = textcursorType
          EndSelect
        EndProcedure
        
        
        ;** sgxSetText
        ;* changes the text of an sgx element and refreshes it. _
        ;* this function may trigger the 'changed' event for sgxEdit elements which can be received after 
        ;* the next call of <a href=#f_sgxexamine>sgxExamine</a>. _
        ;** .id   : ID of the sgx element whose text is to be changed.
        ;** .text : new text.
        Procedure sgxSetText(id, text.s)
          Static textPrevious.s
          If sgx(id)
            If sgx(id)\type = #sgxTypeText
              ; we don't check text change for event triggering, since text could be change by style/wordwrap etc.
              ; we _could_ check the new text against the current origText though i guess.
              sgx(id)\origText = text
            ElseIf sgx(id)\type = #sgxTypeEdit
              textPrevious = sgx(id)\text
            EndIf
            sgx(id)\text = text
            sgxRefresh(id)
            If sgx(id)\type = #sgxTypeEdit
              If sgx(id)\text <> textPrevious
                AddElement(sgx_pendingEvents())
                sgx_pendingEvents()\id = id
                sgx_pendingEvents()\changed = 1
              EndIf
            EndIf
          EndIf
          textPrevious = ""
        EndProcedure
        
        
        ;** sgxSetResizable
        ;* make an element resizeable via the mouse. _
        ;* sgxTable elements determine their size by their childs and cannot be resized directly. _
        ;* sgxCheckbox elements resize only their knob (w,h) but their overall size (w2,h2) might be influenced by this too. _
        ;* scrollbars/trackbars will resize only their bar and not their knob. _
        ;* use <a href=#f_sgxsetminsize>sgxSetMinSize</a> to ensure a minimum size. _
        ;* the element's 'resized' field will be triggered every time the size changes.
        ;** .id        : ID of the sgx element.
        ;** .resizable : #True (default) to make the element resizable. #False to disable resizing via mouse.
        Procedure sgxSetResizable(id, resizable=#True)
          If sgx(id) And sgx(id)\type <> #sgxTypeTable
            If resizable
              sgx(id)\resizable = 1
            Else
              sgx(id)\resizable = 0
            EndIf
          EndIf
        EndProcedure
        
        
        ;** sgxSetMinSize
        ;* set the minimum size allowed for manually resizing elements via mouse. _
        ;* the default minimum size (if you don't call this function) is the size of the resize gadget.
        ;** .id          : ID of the sgx element.
        ;** .minWidth.i  : the smallest width allowed for resizing. must be greater 0.
        ;** .minHeight.i : the smallest height allowed for resizing. must be greater 0.
        Procedure sgxSetMinSize(id, minWidth, minHeight)
          If sgx(id)
            If minWidth < 1
              minWidth = 1
            EndIf
            If minHeight < 1
              minHeight = 1
            EndIf
            sgx(id)\minWidth = minWidth
            sgx(id)\minHeight = minHeight
          EndIf
        EndProcedure
        
        
        ;** sgxSetScrollMax
        ;* change the range of a scrollbar/trackbar. _
        ;* this will update the element's state if the new maximum is lower than its current state and updates the knob position.
        ;* the element's 'changed' event might be triggered if the state changes.
        ;** .id  : ID of the sgx scrollbar/trackbar element.
        ;** .max : new maximum possible scroll state (minimum is 0). this must be greater than 0 and will be corrected to 1 
        ;*         <span></span> automatically if necessary.
        Procedure sgxSetScrollMax(id, max)
          Static state
          If sgx(id) And sgx(id)\type & (#sgxTypeScrollbar|#sgxTypeTrackbar)
            If max < 1
              max = 1
            EndIf
            state = sgx(id)\state
            If state > max
              state = max
            EndIf
            sgx(id)\scrollpos=0
            sgx(id)\max = max
            sgxSetState(id, state)
          EndIf
        EndProcedure
        
        
        ;** sgx_SetScrollPos
        ;* translates a pixel position on scrollbar/trackbar into its state and updates the knob position and the 'state' and 'changed' fields (directly without using pending events, so it should only be called during <a href=#f_sgxexamine>sgxExamine</a>).
        ;** .id        : ID of the sgx element.
        ;** .scrollpos : pixel position on the bar. will be automatically clipped into the valid range.
        Procedure sgx_SetScrollPos(id, scrollpos)
          Protected tmp
          If sgx(id)
            Select sgx(id)\type
              Case #sgxTypeScrollbarVer, #sgxTypeTrackbarVer
                sgx(id)\scrollpos = scrollpos
                If sgx(id)\scrollpos < sgx(id)\width/2
                  sgx(id)\scrollpos = sgx(id)\width/2
                ElseIf sgx(id)\scrollpos > sgx(id)\h-sgx(id)\width/2
                  sgx(id)\scrollpos = sgx(id)\h-sgx(id)\width/2
                EndIf
                tmp = sgx(id)\max * (sgx(id)\scrollpos-sgx(id)\width/2) / (sgx(id)\h-sgx(id)\width)
                ; trackbar default is 0 at the bottom. for scrollbars only when reversed.
                If (sgx(id)\type = #sgxTypeTrackbarVer)  XOr  (sgx(id)\style & #sgxStyleReverse)
                  tmp = sgx(id)\max - tmp
                EndIf
                sgx(id)\changed = tmp - sgx(id)\state
                sgx(id)\state = tmp
                sgx(id)\y2 = sgx(id)\scrollpos - sgx(id)\width/2
        
              Case #sgxTypeScrollbarHor, #sgxTypeTrackbarHor
                sgx(id)\scrollpos = scrollpos
                If sgx(id)\scrollpos < sgx(id)\width/2
                  sgx(id)\scrollpos = sgx(id)\width/2
                ElseIf sgx(id)\scrollpos > sgx(id)\w-sgx(id)\width/2
                  sgx(id)\scrollpos = sgx(id)\w-sgx(id)\width/2
                EndIf
                tmp = sgx(id)\max * (sgx(id)\scrollpos-sgx(id)\width/2) / (sgx(id)\w-sgx(id)\width)
                ; default is 0 at the left. at the right when reversed.
                If (sgx(id)\style & #sgxStyleReverse)
                  tmp = sgx(id)\max - tmp
                EndIf
                sgx(id)\changed = tmp - sgx(id)\state
                sgx(id)\state = tmp
                sgx(id)\x2 = sgx(id)\scrollpos - sgx(id)\width/2
            EndSelect
          EndIf
        EndProcedure
        
        
        ;** sgxSetState
        ;* changes the state of an sgx element and refreshes it if necessary. _
        ;* this function may trigger the 'changed' event for the element (or for other elements in a radiogroup) which can be
        ;* received after the next call of <a href=#f_sgxexamine>sgxExamine</a>.
        ;** .id    : ID of the sgx element whose state is to be changed.
        ;** .state : the new state (don't ever use #sgxState.. values here because these are used only during element creation.). _
        ;* for scrollbars and trackbars the state will not be changed if it's lower than 0 or greater than the maximum. _
        ;* for toggle buttons and checkboxes specify a state of 1 for toggled/checked or 0 for not toggled/checked. _
        ;* for combos the state must be in the range of 1 to the number of entries (both inclusive) otherwise the state will not be changed. _
        ;* for sprite gadgets the parameter 'state' must be a valid PB sprite number or 0 to detach the current sprite. this will also update the element's size according to the size of the sprite. if you specify 0 as the sprite then the element size will be zero and you won't receive any mouse events! _
        ;* to change the state of a window use <a href=#f_sgxpresentwin>sgxPresentWin</a>, <a href=#f_sgxminimizewin>sgxMinimizeWin</a>, <a href=#f_sgxrestorewin>sgxRestoreWin</a> and <a href=#f_sgxclosewin>sgxCloseWin</a>. _
        ;* _
        ;** .handleRadioGroups : #True (default) to also handle other element's state in a possibly assigned radiogroup accordingly.
        ;** .*changed : used internally by <a href=#f_sgxexamine>sgxExamine</a> to correctly set the 'changed' flag for the other radiogroup members.
        Procedure sgxSetState(id, state, handleRadioGroups=#True, *changed.sgxS_ListWrapper=0)
          Static groupId
          Static scrollpos
          Static previousState
          If sgx(id)
            Select sgx(id)\type
                
                ; SCROLLBARS, TRACKBARS
              Case #sgxTypeTrackbarHor, #sgxTypeScrollbarHor
                If state>=0 And state<=sgx(id)\max
                  ; default is 0 at the left. at the right when reversed.
                  If (sgx(id)\style & #sgxStyleReverse)
                    scrollpos = sgx(id)\width/2 + (sgx(id)\w-sgx(id)\width) * (sgx(id)\max - state) / (sgx(id)\max)
                  Else
                    scrollpos = sgx(id)\width/2 + (sgx(id)\w-sgx(id)\width) * (              state) / (sgx(id)\max)
                  EndIf
                  previousState = sgx(id)\state
                  ; update knob position
                  sgx_SetScrollPos(id, scrollpos)
                  sgx(id)\state = state
                  ; prepare event triggering
                  If sgx(id)\state <> previousState
                    AddElement(sgx_pendingEvents())
                    sgx_pendingEvents()\id = id
                    sgx_pendingEvents()\changed = sgx(id)\state - previousState
                  EndIf
                EndIf
              Case #sgxTypeTrackbarVer, #sgxTypeScrollbarVer
                If state>=0 And state<=sgx(id)\max
                  ; trackbar default is 0 at the bottom. for scrollbars only when reversed.
                  If (sgx(id)\type = #sgxTypeTrackbarVer)  XOr  (sgx(id)\style & #sgxStyleReverse)
                    scrollpos = sgx(id)\width/2 + (sgx(id)\h-sgx(id)\width) * (sgx(id)\max - state) / (sgx(id)\max)
                  Else
                    scrollpos = sgx(id)\width/2 + (sgx(id)\h-sgx(id)\width) * (              state) / (sgx(id)\max)
                  EndIf
                  previousState = sgx(id)\state
                  ; update knob position
                  sgx_SetScrollPos(id, scrollpos)
                  sgx(id)\state = state
                  ; prepare event triggering
                  If sgx(id)\state <> previousState
                    AddElement(sgx_pendingEvents())
                    sgx_pendingEvents()\id = id
                    sgx_pendingEvents()\changed = sgx(id)\state - previousState
                  EndIf
                EndIf
                
                ; TOGGLEBUTTONS, CHECKBOXES
              Case #sgxTypeButtonToggle, #sgxTypeCheckbox
                
                If state >= 1
                  state = 1
                Else
                  state = 0
                EndIf
                
                If handleRadioGroups
                  
                  ; get possibly attached group
                  groupId = sgxGetGroup(id)
                  If groupId = -1 ; no group
                    
                    ; set state
                    previousState = sgx(id)\state
                    sgx(id)\state = state
                    
                    ; set 'changed'
                    If sgx(id)\state <> previousState
                      ; add a pending 'changed' event or add it to the list passed by sgxExamine
                      If *changed
                        AddElement(*changed\list_())
                        *changed\list_() = id
                      Else
                        AddElement(sgx_pendingEvents())
                        sgx_pendingEvents()\id = id
                        sgx_pendingEvents()\changed = 1
                      EndIf
                    EndIf
                    
                  ElseIf groupId >= 0
                    ForEach sgxGroups()
                      If sgxGroups()\groupID = groupId
                        previousState = sgx(sgxGroups()\id)\state
                        ; set state
                        If sgxGroups()\id = id
                          sgx(sgxGroups()\id)\state = state   ; our element
                        Else
                          sgx(sgxGroups()\id)\state = state!1 ; other elements
                        EndIf
                        If previousState <> sgx(sgxGroups()\id)\state
                          ; add a pending 'changed' event or add it to the list passed by sgxExamine
                          If *changed
                            AddElement(*changed\list_())
                            *changed\list_() = sgxGroups()\id
                          Else
                            AddElement(sgx_pendingEvents())
                            sgx_pendingEvents()\id = sgxGroups()\id
                            sgx_pendingEvents()\changed = 1
                          EndIf
                        EndIf
                      EndIf
                    Next
                  EndIf
                Else
                  ; only set element's state but do not modify other group members
                  
                  ; set state
                  previousState = sgx(id)\state
                  sgx(id)\state = state
                  
                  ; set 'changed'
                  If sgx(id)\state <> previousState
                    ; add a pending 'changed' event or add it to the list passed by sgxExamine
                    If *changed
                      AddElement(*changed\list_())
                      *changed\list_() = id
                    Else
                      AddElement(sgx_pendingEvents())
                      sgx_pendingEvents()\id = id
                      sgx_pendingEvents()\changed = 1
                    EndIf
                  EndIf
                EndIf
                
              ; SPRITEGADGET
            Case #sgxTypeSpriteGadget
              previousState = sgx(id)\state
              ; spritegadget's state is a sprite number
              If IsSprite(state)
                sgx(id)\w = SpriteWidth(state)
                sgx(id)\h = SpriteHeight(state)
                sgx(id)\state = state
                sgx(id)\w2 = sgx(id)\w ; - not really needed but thats the expected default
                sgx(id)\h2 = sgx(id)\h ; /
              ElseIf state=0
                sgx(id)\w = 0
                sgx(id)\h = 0
                sgx(id)\state = 0
                sgx(id)\w2 = sgx(id)\w ; - not really needed but thats the expected default
                sgx(id)\h2 = sgx(id)\h ; /
              Else
                ; no change
              EndIf
              ; prepare event triggering
              If sgx(id)\state <> previousState
                AddElement(sgx_pendingEvents())
                sgx_pendingEvents()\id = id
                sgx_pendingEvents()\changed = 1
              EndIf
              
              ; COMBO
            Case #sgxTypeCombo
              previousState = sgx(id)\state
              If state >= 1 And state <= (CountString(sgx(id)\text,#LF$) + 1)
                sgx(id)\state = state
              EndIf
              ; prepare event triggering
              If sgx(id)\state <> previousState
                AddElement(sgx_pendingEvents())
                sgx_pendingEvents()\id = id
                sgx_pendingEvents()\changed = 1
                sgxRefresh(id)
              EndIf
              
              ; OTHER TYPES
            Default
              ;previousState = sgx(id)\state
              ;sgx(id)\state = state
              ;If sgx(id)\state <> previousState
              ;AddElement(sgx_pendingEvents())
              ;sgx_pendingEvents()\id = id
              ;sgx_pendingEvents()\changed = 1
              ;EndIf
            EndSelect
            
          EndIf
        EndProcedure
        
        
        ;** sgx_setParentsHovered
        ;* makes all parents of the element up to the first window hover. _
        ;* this includes tables elements and it stops with the window type element.
        ;** .id : ID of the sgx element whichs parents are to be set hovered.
        Procedure sgx_setParentWindowHovered(id)
          Static parent
          parent = sgx(id)\parent
          While parent <> -1
            sgx(parent)\hover = 1
            ; stop when first window reached
            If sgx(id)\type = #sgxTypeWin
              Break
            EndIf
            parent = sgx(parent)\parent
          Wend
        EndProcedure
        
        
        ;** sgxSetEmsContainer
        ;* tell sgx about your 'ems' variable.
        ;* since it's likely that you've got already a variable holding some 'ElapsedMilliseconds()'-like value, _
        ;* you can let sgx access that data. usefull if you have better timing resolution. otherwise if you don't call this procedure then sgx will inquire PB's ElapsedMilliseconds() by itself (default). _
        ;* you need to call that procedure once only.
        ;** .*emsContainer : a pointer to your long-variable (use @).
        Procedure sgxSetEmsContainer(*emsContainer.Integer)
          *sgx_emsContainer = *emsContainer
        EndProcedure
        
        
        ;** sgxSetDesign
        ;* set the design for coming creations of new sgx elements, or change the design of an sgx element.
        ;** .*design : pointer to a loaded design (return value of <a href=#f_sgxloaddesign>sgxLoadDesign</a> or one of the sgxDesign() list elements).
        ;** .id      : per default (-1), the from now on used design is set to *design. if an ID is passed, only the design of the sgx element is changed wich takes effect with the next element refresh which is not done automatically.
        Procedure sgxSetDesign(*design.sgxS_Design, id=-1)
          If *design
            If id=-1
              *sgx_CurrentDesign = *design
            ElseIf sgxInBounds(id) And sgx(id)
              sgx(id)\design = *design
            EndIf
          EndIf
        EndProcedure
        
        
        ;** sgxSetAlpha
        ;* set the alpha value for coming creations of new sgx elements (default alpha is 255, no transparency), or change the alpha value of an sgx element.
        ;** .alpha     : alpha value in the range of 0 (invisible) to 255 (fully opaque / no tranparency)
        ;** .id        : if ignored (default/-1), the from now on used default value is set to alpha. if an ID is passed, only the alpha value of the sgx element is changed. no refresh of that element is necessary, since the value is only used for display.
        ;** .recursive : #False per default. if set to #True, the alpha value of the childs of the element will also be changed unless they are windows.
        Procedure sgxSetAlpha(alpha, id=-1, recursive=#False)
          If alpha < 0
            alpha = 0
          ElseIf alpha > 255
            alpha = 255
          EndIf
          If id=-1
            sgx_defaultAlpha = alpha
          ElseIf sgxInBounds(id) And sgx(id)
            sgx(id)\alpha = alpha
            If recursive And sgx(id)\hasChilds
              sgx_llIterator(Protected *it, sgx(id)\treeElem\node)
                If sgx(*it\id)\type <> #sgxTypeWin
                  sgxSetAlpha(alpha, *it\id, #True)
                EndIf
              sgx_llNext(*it)
            EndIf
          EndIf
        EndProcedure
        
        
        ;** sgxCell
        ;* use this macro to access the structure of your sgx elements if they are cells of a table and you want to access them via their position in the table. you can also access them the normal way via <a href=#f_sgx>sgx</a> though. _
        ;** .table : ID of an sgx table element (#sgxTypeTable)
        ;** .i     : column (starting with 0)
        ;** .k     : line (starting with 0)
        Macro sgxCell(table, i, k)
          sgx( sgxCellId(table, i, k) )
        EndMacro
        
        
        ;** sgxCellId
        ;* returns the ID of a table-cell, or 0 for empty cells or if it fails, which happens if table is not a valid sgx table element or if the coordinates are out of the table's dimensions.
        ;** .table : ID of an sgx table element (#sgxTypeTable)
        ;** .i     : column (starting with 0)
        ;** .k     : line (starting with 0)
        Procedure sgxCellId(table, i, k)
          Static offset, *p
          If sgx(table)  And  sgx(table)\type = #sgxTypeTable
            If i>=0 And i<sgx(table)\table\w And k>=0 And k<sgx(table)\table\h
              ; note: gives an 'too complex expression' error if written as a one-liner
              offset = (i + k*sgx(table)\table\w) * SizeOf(Long)
              *p = sgx(table)\table\cells + offset
              ProcedureReturn PeekL( *p )
            EndIf
          EndIf
          ProcedureReturn 0
        EndProcedure
        
        
        
        
        ;** sgxCellAssign
        ;* assigns an sgx element to a cell of a table or removes it. that table must either be the current parent or must be specified by parameter. the element will be removed from its current parent's child list and becomes a child of the table. it returns the same ID as passed, or 0 if assignment failed. it fails if the table is not a valid sgx table element, or if i/k are out of the bounds of the table's columns/lines. _
        ;* it's recommended to create and assign the table's child elements between the table creation with <a href=#f_sgxtable>sgxTable</a> and it's associated call of <a href=#f_sgxcloselevel>sgxCloseLevel</a>. since table cells and their assigned elements can influence each other's dimensions, they are all refreshed once with the call of sgxCloseLevel. but it's generally possible to create the childs before or after and only assign them during the creation of the table or later, though there might happen redundant double refreshing of elements. _
        ;* if an id of 0 is specified, the cell will be cleared and becomes empty. in any case a possibly already assigned element at that position will be removed and freed. _
        ;* _
        ;* sgxCellAssign should be used in one of the following ways: _
        ;*  _
        ;* table creation -> child creation and assignment -> parent level closing: _
        ;* <pre> 
        ;* t=sgxTable(-1, 10,10, 4,4, 6) _
        ;* <span></span>  b=sgxButton(-1, 4,4, 100,20, "button") _
        ;* <span></span>  sgxCellAssign(0,0, b) _
        ;* sgxCloseLevel() _
        ;* </pre> 
        ;* or if you need to create childs later then like this.. _
        ;* table creation and parent level closing -> child creation and assignment with table parameter -> table refresh _
        ;* <pre> 
        ;* t=sgxTable(-1, 10,10, 4,4, 6) _
        ;* sgxCloseLevel() _
        ;* b=sgxButton(-1, 4,4, 100,20, "button") _
        ;* sgxCellAssign(0,0, b, t) ; <- table parameter specified _
        ;* sgxRefresh(t) _
        ;* </pre> 
        ;* ..or like this.. _
        ;*   table creation and parent level closing -> child creation -> setting the current parent -> cell assignment -> parent level closing _
        ;* <pre> 
        ;* t=sgxTable(-1, 10,10, 4,4, 6) _
        ;* sgxCloseLevel() _
        ;* b=sgxButton(-1, 4,4, 100,20, "button") _
        ;* sgxSetCurrentParent(t) _
        ;* <span></span>  sgxCellAssign(0,0, b) _
        ;* sgxCloseLevel() _
        ;* </pre> 
        ;*
        ;** .i  : column. (starting with 0)
        ;** .k  : line (starting with 0)
        ;** .id : ID of a valid and existing sgx element or 0 if you want to clear the cell. the x/y-position of that element will be overwritten according to the position in table. the w/h dimensions of the element might be overwritten according to the table style.
        ;** .table : -1 (default) if the current parent should be used as the table. otherwise specify the id of the table you want to assign to.
        Procedure sgxCellAssign(i, k, id, table=-1)
          Static *p, currentCellId
          If table = -1
            table = sgx_CurrentParent
          EndIf
          If table<>-1  And  sgx(table)  And  sgx(table)\type = #sgxTypeTable
            If i>=0 And i<sgx(table)\table\w And k>=0 And k<sgx(table)\table\h
              *p = sgx(table)\table\cells + i*SizeOf(Long) + k*sgx(table)\table\w*SizeOf(Long)
              If sgx(id)
                ; make the element a child of the table
                sgxSetParent(id, table)
                currentCellId = PeekL(*p)
                ; free the old element of the cell, if its not the same
                If id <> currentCellId
                  sgxFree(currentCellId)
                  ; store the id
                  PokeL( *p , id)
                EndIf
              ElseIf id=0
                ; free the old element of the cell, if one has been assigned
                currentCellId = PeekL(*p)
                If sgx(currentCellId)
                    sgxFree(currentCellId)
                EndIf
                ; reset the stored id
                PokeL( *p , 0)
              EndIf
              ProcedureReturn id
            Else
              sgx_err("the table (ID="+Str(table)+") does not have that cell (i="+Str(i)+" k="+Str(k)+")")
            EndIf
          EndIf
          ProcedureReturn 0
        EndProcedure
        
        
        ;** sgx_ArrangeTable
        ;* scans the cells of a table to calculate size of columns, rows as well as the final table's size and may overwrite
        ;* cell element positions and sizes according to the table style but does not refresh elements. _
        ;** .id : a valid sgx table element id.
        Procedure sgx_ArrangeTable(id) ; valid data asumed! no checks
          Static i, k, cellid, cellw, cellh, temp, *tmp
          Static wColumn, hLine, currentW, currentH, x
          
          ; we will add each colum's width to this to get the full table width
          currentW = sgx(id)\margin
          
          ; foreach colum go through all column cells/lines to get max column width
          For i=0 To sgx(id)\table\w-1
            wColumn=0
            For k=0 To sgx(id)\table\h-1
              cellid = sgxCellId(id,i,k)
              If sgx(cellid)
                
                If sgx(cellid)\type = #sgxTypeCheckbox
                  cellw = sgx(cellid)\w2
                Else
                  cellw = sgx(cellid)\w
                EndIf
                
                ; just use cell width if cell element is currently resized manually
                If cellid = sgx_resizing  And  sgx(id)\style & #sgxStyleFitCellW
                  wColumn = cellw
                  Break
                ; get max column width
                ElseIf cellw>wColumn
                  wColumn = cellw
                EndIf
                
              EndIf
            Next
            
            For k=0 To sgx(id)\table\h-1 ; (reusing k here is not a problem)
              cellid = sgxCellId(id,i,k)
              If sgx(cellid)
                
                x = currentW
                If sgx(cellid)\type = #sgxTypeCheckbox And sgx(cellid)\textalign = #sgxAlignCheckboxTextLeft
                  ; get position of knob, aligned to right cell edge (colwidth - knobwidth)
                  x = currentW + ( (sgx(id)\margin + wColumn) - (sgx(id)\margin + sgx(cellid)\w) )
                EndIf
                
                ; positioning cell element x (y ignored for now)
                sgxMove(cellid, x, sgx(cellid)\yToParent)
                
              EndIf
            Next
            
            If wColumn
              currentW + sgx(id)\margin + wColumn + sgx(id)\margin
              
              ; store current x for later vertical grid drawing
              PokeL( sgx(id)\table\gridx + i*SizeOf(Long), currentW - sgx(id)\margin)
              
              ; overwrite cell element width
              If sgx(id)\style & #sgxStyleFitCellW
                For k=0 To sgx(id)\table\h-1 ; (reusing k here is not a problem)
                  cellid = sgxCellId(id,i,k)
                  If sgx(cellid)
                    If sgx(cellid)\type & (#sgxTypeScrollbarVer | #sgxTypeTrackbarVer)
                      ; don't change with of vertical scrollbars/trackbars
                    ElseIf sgx(cellid)\type = #sgxTypeCheckbox
                      ; don't change with of checkboxes (they are sized by their label; and their knob size should not change)
                    ElseIf sgx(cellid)\type = #sgxTypeText And sgx(cellid)\origW = -1
                      ; don't change with of text elements if they have autowidth
                    Else
                      sgx(cellid)\w = wColumn
                      ;sgxRefresh(cellid) ; (done later since we still might overwrite cell h)
                    EndIf
                  EndIf
                Next
              EndIf
            Else
              ; store x=0 to skip this vertical grid line drawing later if column is empty
              PokeL( sgx(id)\table\gridx + i*SizeOf(Long), 0 )
            EndIf
          Next
          
          ; we will add each line's height to this to get the full table height
          currentH = sgx(id)\margin
          
          ; foreach row go through all row cells to get max row height
          For k=0 To sgx(id)\table\h-1
            hLine=0
            For i=0 To sgx(id)\table\w-1
              cellid = sgxCellId(id,i,k)
              If sgx(cellid)
                
                If sgx(cellid)\type = #sgxTypeCheckbox
                  cellh = sgx(cellid)\h2
                Else
                  cellh = sgx(cellid)\h
                EndIf
                
                ; positioning cell element y (x ignored since we've set that already)
                sgxMove(cellid, sgx(cellid)\xToParent, currentH)
                
                ; just use cell height if cell element is currently resized manually
                If cellid = sgx_resizing  And  sgx(id)\style & #sgxStyleFitCellH
                  hLine = cellh
                  Break
                ; get max column width
                ElseIf cellh>hLine
                  hLine = cellh
                EndIf
              EndIf
            Next
            
            If hLine
              currentH + sgx(id)\margin + hLine + sgx(id)\margin
              
              ; store current y for later horizontal grid drawing
              PokeL( sgx(id)\table\gridy + k*SizeOf(Long), currentH - sgx(id)\margin)
              
              ; overwrite cell element height
              If sgx(id)\style & #sgxStyleFitCellH
                For i=0 To sgx(id)\table\w-1 ; (reusing i here is not a problem)
                  cellid = sgxCellId(id,i,k)
                  If sgx(cellid)
                    If sgx(cellid)\type & (#sgxTypeScrollbarHor | #sgxTypeTrackbarHor)
                      ; don't change height of horizontal scrollbars/trackbars
                    ElseIf sgx(cellid)\type = #sgxTypeCheckbox
                      ; don't change height of checkboxes (they are sized by their label; and their knob size should not change)
                    ElseIf sgx(cellid)\type = #sgxTypeText And sgx(cellid)\origH = -1
                      ; don't change height of text elements if they have autoheight
                    Else
                      sgx(cellid)\h = hLine
                      ;sgxRefresh(cellid) ; (done later)
                    EndIf
                  EndIf
                Next
              EndIf
            Else
              ; store y=0 to skip this horizontal grid line drawing later if row is empty
              PokeL( sgx(id)\table\gridy + k*SizeOf(Long), 0 )
            EndIf
          Next
          
          ; set table size to the sum of column widths / row heights
          sgx(id)\w = currentW - sgx(id)\margin
          If sgx(id)\w < 1
            sgx(id)\w = 1
          EndIf
          sgx(id)\h = currentH - sgx(id)\margin
          If sgx(id)\h < 1
            sgx(id)\h = 1
          EndIf
        EndProcedure
        
        
        ;** sgx_textWidthLF
        ;* returns the width of the longest line in the text, that is to say the width of the complete textblock. _
        ;* must be called inside a StartDrawing()/StopDrawing() block.
        ;** .text : (multiline) text. use #LF$ / Chr(10) for linebreaks.
        Procedure sgx_textWidthLF(text.s)
          Protected max,i,tw,maxtw
          max = CountString(text,#LF$)+1
          For i=1 To max
            tw = TextWidth(StringField(text,i,#LF$))
            If tw > maxtw
              maxtw = tw
            EndIf
          Next
          ProcedureReturn maxtw
        EndProcedure
        
        
        ; [next line resets the jaPBDoc documentation target]
        ;** 
        Global sgx_drawTextLF_EndX ;* x-position where the last call of <a href=#f_sgx_drawtextlf>sgx_drawTextLF</a> has finished (right edge of drawn text).
        Global sgx_drawTextLF_EndY ;* y-position where the last call of <a href=#f_sgx_drawtextlf>sgx_drawTextLF</a> has finished (top edge of drawn text).
        
        ;** sgx_drawTextLF
        ;* draws multiline text to the current drawing output. _
        ;* the variables sgx_drawTextLF_EndX and sgx_drawTextLF_EndY can be used to get the position where the drawing finished in case you want to continue drawing some text later.
        ;** .x               : starting x-position.
        ;** .y               : starting y-position.
        ;** .text            : (multiline) text to be drawn.
        ;** .color           : textcolor.
        ;** .xWrapTo         : x-position of further drawing after occurrence of linebreaks. per default (-1) the specified 'x' parameter is used for this.
        ;** .lineSeperator$  : the string that is used in 'text' as line seperator (default value is #LF$ / Chr(10))
        ;** .textAlign       : one of these values (aligments that need a width will use the longest line for that): _
        ;*                    #sgxAlignTextLeft _
        ;*                    #sgxAlignTextRight _
        ;*                    #sgxAlignTextCenter _
        ;*                    #sgxAlignTextJustifySpaces _
        ;*                    #sgxAlignTextJustifyCharacters _
        Procedure sgx_drawTextLF(x, y, text.s, color, xWrapTo=-1, lineSeperator$=#LF$, textAlign=#sgxAlignTextLeft)
          Static yPitch
          Static xPitch
          Static line.s
          Static l, nbLines, c, lineLen, *c.Character, charWidth.f, xf.f
          Static maxLinewWidth.f, lineWidth, lineWidthNoSpaces, nbSpaces.f, spaceWidth.f
          
          Select textAlign
            Case #sgxAlignTextRight, #sgxAlignTextCenter, #sgxAlignTextJustifySpaces, #sgxAlignTextJustifyCharacters
              ; get longest line width if needed
              maxLinewWidth = sgx_textWidthLF(text)
          EndSelect
          If xWrapTo=-1
            xWrapTo=x
          EndIf
          xPitch = x
          yPitch = TextHeight("X")
          nbLines = CountString(text,lineSeperator$)
          For l=0 To nbLines
            line = StringField(text,l+1,lineSeperator$)
            lineWidth = 0;
            
            ; right
            If textAlign = #sgxAlignTextRight
              lineWidth = TextWidth(line)
              xPitch = DrawText( x+maxLinewWidth-lineWidth, y+l*yPitch, line, color)
              
            ; center
            ElseIf textAlign = #sgxAlignTextCenter
              lineWidth = TextWidth(line)
              xPitch = DrawText( x+maxLinewWidth/2-lineWidth/2, y+l*yPitch, line, color)
              
            ; justify (spaces)
            ElseIf textAlign = #sgxAlignTextJustifySpaces
              ; auto word wrap may have put a newline after a space
              line = RTrim(line)
              
              lineWidth = TextWidth(line)
              lineLen = Len(Line)
              *c = @line
              ; get factor to stretch/shrink current line
              xf = maxLinewWidth/lineWidth
              charWidth = 0
              ; only before last line
              If l<nbLines
                
                ; get line width without spaces
                lineWidthNoSpaces = TextWidth(ReplaceString(line," ", ""))
                nbSpaces = CountString(Line, " ")
                ; get necessary width per space
                spaceWidth = (maxLinewWidth - lineWidthNoSpaces) / nbSpaces
                
                For c=0 To lineLen-1
                  If (*c\c = 32)
                    ; draw space and add translated width
                    xPitch = DrawText( x + spaceWidth, y+l*yPitch, Chr(*c\c), color)
                    xPitch = x + spaceWidth
                    charWidth + spaceWidth
                  Else
                    ; draw char and add real width
                    xPitch = DrawText( x + charWidth , y+l*yPitch, Chr(*c\c), color)
                    charWidth + TextWidth(Chr(*c\c))
                  EndIf
                  *c + SizeOf(Character)
                Next
              Else
                ; no hassle with last line
                xPitch = DrawText( x, y+l*yPitch, line, color)
              EndIf
              
            ; justify (chars)
            ElseIf textAlign = #sgxAlignTextJustifyCharacters
              ; auto word wrap may have put a newline after a space
              line = RTrim(line)
              
              lineWidth = TextWidth(line)
              lineLen = Len(Line)
              *c = @line
              ; get factor to stretch/shrink current line
              xf = maxLinewWidth/lineWidth
              charWidth = 0
              ; only before last line
              If l<nbLines
                For c=0 To lineLen-1
                  ; draw current char at translated x position
                  xPitch = DrawText( x + charWidth * xf , y+l*yPitch, Chr(*c\c), color)
                  charWidth + TextWidth(Chr(*c\c))
                  *c + SizeOf(Character)
                Next
              Else
                ; no hassle with last line
                xPitch = DrawText( x, y+l*yPitch, line, color)
              EndIf
              
            ; left
            Else
              xPitch = DrawText( x, y+l*yPitch, line, color)
              If l=0
                x=xWrapTo
              EndIf
            EndIf
        
          Next
          sgx_drawTextLF_EndX = xPitch
          sgx_drawTextLF_EndY = y+(l-1)*yPitch
        EndProcedure
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        ;- *** COMMON ***
        
        
        
        
        ;** sgx_max
        ;* returns 'a' if its greater that 'b', otherwise returns 'b'.
        ;** .a : an integer value
        ;** .b : an integer value
        Procedure sgx_max(a, b)
          If a>b
            ProcedureReturn a
          Else
            ProcedureReturn b
          EndIf
        EndProcedure
        
        
        ;** sgx_fade
        ;* returns a new color with different brightness derived from 'color' according to the value of 'fade'. the alpha will not change.
        ;** .color : some RGBA value
        ;** .fade  :  0: no change _
        ;*           negative value: darken (value -1000 = black) _
        ;*           positive value: brighten (value 1000 = white) _
        Procedure sgx_fade(color, fade)
          Protected r
          Protected g
          Protected b
          Protected a
          If fade<0
            If fade<-1000 : fade=0
            Else          : fade+1000
            EndIf
            If fade>1000 : fade=1000 : EndIf
            r= (color & $ff) *fade/1000
            If r>255 : r=255 : EndIf
            g= ((color & $ff00) >>8) *fade/1000
            If g>255 : g=255 : EndIf
            b= ((color & $ff0000) >>16) *fade/1000
            If b>255 : b=255 : EndIf
            ;a= ((color & $ff000000) >>24) *fade/1000
            ;If a>255 : a=255 : EndIf
            a= (color & $ff000000)
            ProcedureReturn ( (r) | (g<<8) | (b<<16) | (a) )
          ElseIf fade>0
            If fade>1000 : fade=0
            Else         : fade=1000-fade
            EndIf
            r= 255- ( 255-(color & $ff) ) *fade/1000
            g= 255- ( 255-((color & $ff00) >>8) ) *fade/1000
            b= 255- ( 255-((color & $ff0000) >>16) ) *fade/1000
            ;a= 255- ( 255-((color & $ff000000) >>24) ) *fade/1000
            a= (color & $ff000000)
            ProcedureReturn ( (r) | (g<<8) | (b<<16) | (a) )
          Else;[if fade=0]
            ProcedureReturn color
          EndIf
        EndProcedure
        
        
        ; Procedure sgx_debugColorP(text.s, x,y)
        ;   Debug text + ": " + Red(Point(x,y)) + ","+Green(Point(x,y))+","+Blue(Point(x,y))+","+Alpha(Point(x,y))
        ; EndProcedure
        
        
        ; Procedure sgx_debugColorC(text.s, color)
        ;   Debug text + ": " + Red(color) + ","+Green(color)+","+Blue(color)+","+Alpha(color)
        ; EndProcedure
        
        
        ; Procedure sgx_debugColorSD(text.s, img, x,y)
        ;   StartDrawing(ImageOutput(img))
        ;     DrawingMode(#PB_2DDrawing_AllChannels)
        ;     Debug text + ": " + Red(Point(x,y)) + ","+Green(Point(x,y))+","+Blue(Point(x,y))+","+Alpha(Point(x,y))
        ;   StopDrawing()
        ; EndProcedure
        
        
        
        
        
        
        
        
        ; *** SPECIAL ***
        
        
        
        
        ;{ --- defines
        
        ;** s sgx_S_key
        ;* used per key in an keyboard layout (<a href=#s_sgx_s_keyboardlayout>sgx_S_keyboardLayout</a>)
        Structure sgx_S_key
          pbid.i         ;* a PB key like #PB_Key_Up or #PB_Key_Escape
          chr.s[3]       ;* the characters that should result from the key per key combination: 0 = normal, 1 = shift, 2 = alt-gr
          timer.i        ;* individual timer per key for keystroke delays
          delaySwitch.i  ;* a switch between first keystroke delay and continuous keypush interval
        EndStructure
        
        ;** s sgx_S_keyboardLayout
        ;* a keyboard layout consisting of key definitions for all the supported keys
        Structure sgx_S_keyboardLayout
          key.sgx_S_key[101]  ;* array of key definitions
        EndStructure
        
        ; [next line resets the jaPBDoc documentation target]
        ;** 
        
        Global Dim sgx_keyLay.sgx_S_keyboardLayout(1)  ;* array of keyboard layouts
        Global sgx_keyLayout=0                  ;* currently used keyboard layout. can be set via <a href=#f_sgxsetkeyboardlayout>sgxSetKeyboardLayout</a>
        Global sgx_keystrokeDelay_1=280         ;* the keystroke delay after a first key-push. can be set via <a href=#f_sgxsetkeyboardlayout>sgxSetKeyboardLayout</a>
        Global sgx_keystrokeDelay_2=40          ;* the keystroke delay for continuous key-push. can be set via <a href=#f_sgxsetkeyboardlayout>sgxSetKeyboardLayout</a>
        Global sgx_keyboardEdit_lastTextPos     ;* see <a href=#f_sgx_keyboardedit>sgx_keyboardEdit</a>
        Global sgx_keyboardEdit_TextModified    ;* see <a href=#f_sgx_keyboardedit>sgx_keyboardEdit</a>
        
        ;}
        ;  ----------
        
        
        ; there seems to be no way to get the
        ; pipe symbol/key triggered with the
        ; pb keyboard lib with a german keyboard
        
        
        ;{ ---------- define table 0 - qwerty / us ???
        ;sgx_keyLay(0)         pbid                       ---    chr[0] / mode 0 / normal       ---  chr[1] / mode 1 / shift          ---  chr[2] / mode 2 / alt-gr
        
        sgx_keyLay(0)\key[000]\pbid=#PB_Key_1             :  sgx_keyLay(0)\key[000]\chr[0] = "1"  :  sgx_keyLay(0)\key[000]\chr[1] = "!"  :  sgx_keyLay(0)\key[000]\chr[2] = ""
        sgx_keyLay(0)\key[001]\pbid=#PB_Key_2             :  sgx_keyLay(0)\key[001]\chr[0] = "2"  :  sgx_keyLay(0)\key[001]\chr[1] = "@"  :  sgx_keyLay(0)\key[001]\chr[2] = ""
        sgx_keyLay(0)\key[002]\pbid=#PB_Key_3             :  sgx_keyLay(0)\key[002]\chr[0] = "3"  :  sgx_keyLay(0)\key[002]\chr[1] = "#"  :  sgx_keyLay(0)\key[002]\chr[2] = ""
        sgx_keyLay(0)\key[003]\pbid=#PB_Key_4             :  sgx_keyLay(0)\key[003]\chr[0] = "4"  :  sgx_keyLay(0)\key[003]\chr[1] = "$"  :  sgx_keyLay(0)\key[003]\chr[2] = ""
        sgx_keyLay(0)\key[004]\pbid=#PB_Key_5             :  sgx_keyLay(0)\key[004]\chr[0] = "5"  :  sgx_keyLay(0)\key[004]\chr[1] = "%"  :  sgx_keyLay(0)\key[004]\chr[2] = ""
        sgx_keyLay(0)\key[005]\pbid=#PB_Key_6             :  sgx_keyLay(0)\key[005]\chr[0] = "6"  :  sgx_keyLay(0)\key[005]\chr[1] = "^"  :  sgx_keyLay(0)\key[005]\chr[2] = ""
        sgx_keyLay(0)\key[006]\pbid=#PB_Key_7             :  sgx_keyLay(0)\key[006]\chr[0] = "7"  :  sgx_keyLay(0)\key[006]\chr[1] = "&"  :  sgx_keyLay(0)\key[006]\chr[2] = ""
        sgx_keyLay(0)\key[007]\pbid=#PB_Key_8             :  sgx_keyLay(0)\key[007]\chr[0] = "8"  :  sgx_keyLay(0)\key[007]\chr[1] = "*"  :  sgx_keyLay(0)\key[007]\chr[2] = ""
        sgx_keyLay(0)\key[008]\pbid=#PB_Key_9             :  sgx_keyLay(0)\key[008]\chr[0] = "9"  :  sgx_keyLay(0)\key[008]\chr[1] = "("  :  sgx_keyLay(0)\key[008]\chr[2] = ""
        sgx_keyLay(0)\key[009]\pbid=#PB_Key_0             :  sgx_keyLay(0)\key[009]\chr[0] = "0"  :  sgx_keyLay(0)\key[009]\chr[1] = ")"  :  sgx_keyLay(0)\key[009]\chr[2] = ""
        
        sgx_keyLay(0)\key[010]\pbid=#PB_Key_A             :  sgx_keyLay(0)\key[010]\chr[0] = "a"  :  sgx_keyLay(0)\key[010]\chr[1] = "A"  :  sgx_keyLay(0)\key[010]\chr[2] = ""
        sgx_keyLay(0)\key[011]\pbid=#PB_Key_B             :  sgx_keyLay(0)\key[011]\chr[0] = "b"  :  sgx_keyLay(0)\key[011]\chr[1] = "B"  :  sgx_keyLay(0)\key[011]\chr[2] = ""
        sgx_keyLay(0)\key[012]\pbid=#PB_Key_C             :  sgx_keyLay(0)\key[012]\chr[0] = "c"  :  sgx_keyLay(0)\key[012]\chr[1] = "C"  :  sgx_keyLay(0)\key[012]\chr[2] = ""
        sgx_keyLay(0)\key[013]\pbid=#PB_Key_D             :  sgx_keyLay(0)\key[013]\chr[0] = "d"  :  sgx_keyLay(0)\key[013]\chr[1] = "D"  :  sgx_keyLay(0)\key[013]\chr[2] = ""
        sgx_keyLay(0)\key[014]\pbid=#PB_Key_E             :  sgx_keyLay(0)\key[014]\chr[0] = "e"  :  sgx_keyLay(0)\key[014]\chr[1] = "E"  :  sgx_keyLay(0)\key[014]\chr[2] = ""
        sgx_keyLay(0)\key[015]\pbid=#PB_Key_F             :  sgx_keyLay(0)\key[015]\chr[0] = "f"  :  sgx_keyLay(0)\key[015]\chr[1] = "F"  :  sgx_keyLay(0)\key[015]\chr[2] = ""
        sgx_keyLay(0)\key[016]\pbid=#PB_Key_G             :  sgx_keyLay(0)\key[016]\chr[0] = "g"  :  sgx_keyLay(0)\key[016]\chr[1] = "G"  :  sgx_keyLay(0)\key[016]\chr[2] = ""
        sgx_keyLay(0)\key[017]\pbid=#PB_Key_H             :  sgx_keyLay(0)\key[017]\chr[0] = "h"  :  sgx_keyLay(0)\key[017]\chr[1] = "H"  :  sgx_keyLay(0)\key[017]\chr[2] = ""
        sgx_keyLay(0)\key[018]\pbid=#PB_Key_I             :  sgx_keyLay(0)\key[018]\chr[0] = "i"  :  sgx_keyLay(0)\key[018]\chr[1] = "I"  :  sgx_keyLay(0)\key[018]\chr[2] = ""
        sgx_keyLay(0)\key[019]\pbid=#PB_Key_J             :  sgx_keyLay(0)\key[019]\chr[0] = "j"  :  sgx_keyLay(0)\key[019]\chr[1] = "J"  :  sgx_keyLay(0)\key[019]\chr[2] = ""
        sgx_keyLay(0)\key[020]\pbid=#PB_Key_K             :  sgx_keyLay(0)\key[020]\chr[0] = "k"  :  sgx_keyLay(0)\key[020]\chr[1] = "K"  :  sgx_keyLay(0)\key[020]\chr[2] = ""
        sgx_keyLay(0)\key[021]\pbid=#PB_Key_L             :  sgx_keyLay(0)\key[021]\chr[0] = "l"  :  sgx_keyLay(0)\key[021]\chr[1] = "L"  :  sgx_keyLay(0)\key[021]\chr[2] = ""
        sgx_keyLay(0)\key[022]\pbid=#PB_Key_M             :  sgx_keyLay(0)\key[022]\chr[0] = "m"  :  sgx_keyLay(0)\key[022]\chr[1] = "M"  :  sgx_keyLay(0)\key[022]\chr[2] = ""
        sgx_keyLay(0)\key[023]\pbid=#PB_Key_N             :  sgx_keyLay(0)\key[023]\chr[0] = "n"  :  sgx_keyLay(0)\key[023]\chr[1] = "N"  :  sgx_keyLay(0)\key[023]\chr[2] = ""
        sgx_keyLay(0)\key[024]\pbid=#PB_Key_O             :  sgx_keyLay(0)\key[024]\chr[0] = "o"  :  sgx_keyLay(0)\key[024]\chr[1] = "O"  :  sgx_keyLay(0)\key[024]\chr[2] = ""
        sgx_keyLay(0)\key[025]\pbid=#PB_Key_P             :  sgx_keyLay(0)\key[025]\chr[0] = "p"  :  sgx_keyLay(0)\key[025]\chr[1] = "P"  :  sgx_keyLay(0)\key[025]\chr[2] = ""
        sgx_keyLay(0)\key[026]\pbid=#PB_Key_Q             :  sgx_keyLay(0)\key[026]\chr[0] = "q"  :  sgx_keyLay(0)\key[026]\chr[1] = "Q"  :  sgx_keyLay(0)\key[026]\chr[2] = ""
        sgx_keyLay(0)\key[027]\pbid=#PB_Key_R             :  sgx_keyLay(0)\key[027]\chr[0] = "r"  :  sgx_keyLay(0)\key[027]\chr[1] = "R"  :  sgx_keyLay(0)\key[027]\chr[2] = ""
        sgx_keyLay(0)\key[028]\pbid=#PB_Key_S             :  sgx_keyLay(0)\key[028]\chr[0] = "s"  :  sgx_keyLay(0)\key[028]\chr[1] = "S"  :  sgx_keyLay(0)\key[028]\chr[2] = ""
        sgx_keyLay(0)\key[029]\pbid=#PB_Key_T             :  sgx_keyLay(0)\key[029]\chr[0] = "t"  :  sgx_keyLay(0)\key[029]\chr[1] = "T"  :  sgx_keyLay(0)\key[029]\chr[2] = ""
        sgx_keyLay(0)\key[030]\pbid=#PB_Key_U             :  sgx_keyLay(0)\key[030]\chr[0] = "u"  :  sgx_keyLay(0)\key[030]\chr[1] = "U"  :  sgx_keyLay(0)\key[030]\chr[2] = ""
        sgx_keyLay(0)\key[031]\pbid=#PB_Key_V             :  sgx_keyLay(0)\key[031]\chr[0] = "v"  :  sgx_keyLay(0)\key[031]\chr[1] = "V"  :  sgx_keyLay(0)\key[031]\chr[2] = ""
        sgx_keyLay(0)\key[032]\pbid=#PB_Key_W             :  sgx_keyLay(0)\key[032]\chr[0] = "w"  :  sgx_keyLay(0)\key[032]\chr[1] = "W"  :  sgx_keyLay(0)\key[032]\chr[2] = ""
        sgx_keyLay(0)\key[033]\pbid=#PB_Key_X             :  sgx_keyLay(0)\key[033]\chr[0] = "x"  :  sgx_keyLay(0)\key[033]\chr[1] = "X"  :  sgx_keyLay(0)\key[033]\chr[2] = ""
        sgx_keyLay(0)\key[034]\pbid=#PB_Key_Y             :  sgx_keyLay(0)\key[034]\chr[0] = "y"  :  sgx_keyLay(0)\key[034]\chr[1] = "Y"  :  sgx_keyLay(0)\key[034]\chr[2] = ""
        sgx_keyLay(0)\key[035]\pbid=#PB_Key_Z             :  sgx_keyLay(0)\key[035]\chr[0] = "z"  :  sgx_keyLay(0)\key[035]\chr[1] = "Z"  :  sgx_keyLay(0)\key[035]\chr[2] = ""
        
        sgx_keyLay(0)\key[036]\pbid=#PB_Key_Escape        :  sgx_keyLay(0)\key[036]\chr[0] = ""   :  sgx_keyLay(0)\key[036]\chr[1] = ""   :  sgx_keyLay(0)\key[036]\chr[2] = ""
        sgx_keyLay(0)\key[037]\pbid=#PB_Key_Minus         :  sgx_keyLay(0)\key[037]\chr[0] = "-"  :  sgx_keyLay(0)\key[037]\chr[1] = "_"  :  sgx_keyLay(0)\key[037]\chr[2] = ""
        sgx_keyLay(0)\key[038]\pbid=#PB_Key_Equals        :  sgx_keyLay(0)\key[038]\chr[0] = "="  :  sgx_keyLay(0)\key[038]\chr[1] = "+"  :  sgx_keyLay(0)\key[038]\chr[2] = ""
        sgx_keyLay(0)\key[039]\pbid=#PB_Key_Back          :  sgx_keyLay(0)\key[039]\chr[0] = ""   :  sgx_keyLay(0)\key[039]\chr[1] = ""   :  sgx_keyLay(0)\key[039]\chr[2] = ""
        sgx_keyLay(0)\key[040]\pbid=#PB_Key_Tab           :  sgx_keyLay(0)\key[040]\chr[0] = ""   :  sgx_keyLay(0)\key[040]\chr[1] = ""   :  sgx_keyLay(0)\key[040]\chr[2] = ""
        sgx_keyLay(0)\key[041]\pbid=#PB_Key_LeftBracket   :  sgx_keyLay(0)\key[041]\chr[0] = "["  :  sgx_keyLay(0)\key[041]\chr[1] = "{"  :  sgx_keyLay(0)\key[041]\chr[2] = ""
        sgx_keyLay(0)\key[042]\pbid=#PB_Key_RightBracket  :  sgx_keyLay(0)\key[042]\chr[0] = "]"  :  sgx_keyLay(0)\key[042]\chr[1] = "}"  :  sgx_keyLay(0)\key[042]\chr[2] = ""
        sgx_keyLay(0)\key[043]\pbid=#PB_Key_Return        :  sgx_keyLay(0)\key[043]\chr[0] = ""   :  sgx_keyLay(0)\key[043]\chr[1] = ""   :  sgx_keyLay(0)\key[043]\chr[2] = ""
        sgx_keyLay(0)\key[044]\pbid=#PB_Key_LeftControl   :  sgx_keyLay(0)\key[044]\chr[0] = ""   :  sgx_keyLay(0)\key[044]\chr[1] = ""   :  sgx_keyLay(0)\key[044]\chr[2] = ""
        sgx_keyLay(0)\key[045]\pbid=#PB_Key_SemiColon     :  sgx_keyLay(0)\key[045]\chr[0] = ";"  :  sgx_keyLay(0)\key[045]\chr[1] = ":"  :  sgx_keyLay(0)\key[045]\chr[2] = ""
        sgx_keyLay(0)\key[046]\pbid=#PB_Key_Apostrophe    :  sgx_keyLay(0)\key[046]\chr[0] = "'"  :  sgx_keyLay(0)\key[046]\chr[1]=Chr(34):  sgx_keyLay(0)\key[046]\chr[2] = ""
        sgx_keyLay(0)\key[047]\pbid=#PB_Key_Grave         :  sgx_keyLay(0)\key[047]\chr[0] = "`"  :  sgx_keyLay(0)\key[047]\chr[1] = "~"  :  sgx_keyLay(0)\key[047]\chr[2] = ""
        sgx_keyLay(0)\key[048]\pbid=#PB_Key_LeftShift     :  sgx_keyLay(0)\key[048]\chr[0] = ""   :  sgx_keyLay(0)\key[048]\chr[1] = ""   :  sgx_keyLay(0)\key[048]\chr[2] = ""
        sgx_keyLay(0)\key[049]\pbid=#PB_Key_BackSlash     :  sgx_keyLay(0)\key[049]\chr[0] = "\"  :  sgx_keyLay(0)\key[049]\chr[1] = "|"  :  sgx_keyLay(0)\key[049]\chr[2] = ""
        sgx_keyLay(0)\key[050]\pbid=#PB_Key_Comma         :  sgx_keyLay(0)\key[050]\chr[0] = ","  :  sgx_keyLay(0)\key[050]\chr[1] = "<"  :  sgx_keyLay(0)\key[050]\chr[2] = ""
        sgx_keyLay(0)\key[051]\pbid=#PB_Key_Period        :  sgx_keyLay(0)\key[051]\chr[0] = "."  :  sgx_keyLay(0)\key[051]\chr[1] = ">"  :  sgx_keyLay(0)\key[051]\chr[2] = ""
        sgx_keyLay(0)\key[052]\pbid=#PB_Key_Slash         :  sgx_keyLay(0)\key[052]\chr[0] = "/"  :  sgx_keyLay(0)\key[052]\chr[1] = "?"  :  sgx_keyLay(0)\key[052]\chr[2] = ""
        sgx_keyLay(0)\key[053]\pbid=#PB_Key_RightShift    :  sgx_keyLay(0)\key[053]\chr[0] = ""   :  sgx_keyLay(0)\key[053]\chr[1] = ""   :  sgx_keyLay(0)\key[053]\chr[2] = ""
        sgx_keyLay(0)\key[054]\pbid=#PB_Key_Multiply      :  sgx_keyLay(0)\key[054]\chr[0] = "*"  :  sgx_keyLay(0)\key[054]\chr[1] = "*"  :  sgx_keyLay(0)\key[054]\chr[2] = ""
        sgx_keyLay(0)\key[055]\pbid=#PB_Key_LeftAlt       :  sgx_keyLay(0)\key[055]\chr[0] = ""   :  sgx_keyLay(0)\key[055]\chr[1] = ""   :  sgx_keyLay(0)\key[055]\chr[2] = ""
        sgx_keyLay(0)\key[056]\pbid=#PB_Key_Space         :  sgx_keyLay(0)\key[056]\chr[0] = " "  :  sgx_keyLay(0)\key[056]\chr[1] = " "  :  sgx_keyLay(0)\key[056]\chr[2] = ""
        sgx_keyLay(0)\key[057]\pbid=#PB_Key_Capital       :  sgx_keyLay(0)\key[057]\chr[0] = ""   :  sgx_keyLay(0)\key[057]\chr[1] = ""   :  sgx_keyLay(0)\key[057]\chr[2] = ""
        
        sgx_keyLay(0)\key[058]\pbid=#PB_Key_F1            :  sgx_keyLay(0)\key[058]\chr[0] = ""   :  sgx_keyLay(0)\key[058]\chr[1] = ""   :  sgx_keyLay(0)\key[058]\chr[2] = ""
        sgx_keyLay(0)\key[059]\pbid=#PB_Key_F2            :  sgx_keyLay(0)\key[059]\chr[0] = ""   :  sgx_keyLay(0)\key[059]\chr[1] = ""   :  sgx_keyLay(0)\key[059]\chr[2] = ""
        sgx_keyLay(0)\key[060]\pbid=#PB_Key_F3            :  sgx_keyLay(0)\key[060]\chr[0] = ""   :  sgx_keyLay(0)\key[060]\chr[1] = ""   :  sgx_keyLay(0)\key[060]\chr[2] = ""
        sgx_keyLay(0)\key[061]\pbid=#PB_Key_F4            :  sgx_keyLay(0)\key[061]\chr[0] = ""   :  sgx_keyLay(0)\key[061]\chr[1] = ""   :  sgx_keyLay(0)\key[061]\chr[2] = ""
        sgx_keyLay(0)\key[062]\pbid=#PB_Key_F5            :  sgx_keyLay(0)\key[062]\chr[0] = ""   :  sgx_keyLay(0)\key[062]\chr[1] = ""   :  sgx_keyLay(0)\key[062]\chr[2] = ""
        sgx_keyLay(0)\key[063]\pbid=#PB_Key_F6            :  sgx_keyLay(0)\key[063]\chr[0] = ""   :  sgx_keyLay(0)\key[063]\chr[1] = ""   :  sgx_keyLay(0)\key[063]\chr[2] = ""
        sgx_keyLay(0)\key[064]\pbid=#PB_Key_F7            :  sgx_keyLay(0)\key[064]\chr[0] = ""   :  sgx_keyLay(0)\key[064]\chr[1] = ""   :  sgx_keyLay(0)\key[064]\chr[2] = ""
        sgx_keyLay(0)\key[065]\pbid=#PB_Key_F8            :  sgx_keyLay(0)\key[065]\chr[0] = ""   :  sgx_keyLay(0)\key[065]\chr[1] = ""   :  sgx_keyLay(0)\key[065]\chr[2] = ""
        sgx_keyLay(0)\key[066]\pbid=#PB_Key_F9            :  sgx_keyLay(0)\key[066]\chr[0] = ""   :  sgx_keyLay(0)\key[066]\chr[1] = ""   :  sgx_keyLay(0)\key[066]\chr[2] = ""
        sgx_keyLay(0)\key[067]\pbid=#PB_Key_F10           :  sgx_keyLay(0)\key[067]\chr[0] = ""   :  sgx_keyLay(0)\key[067]\chr[1] = ""   :  sgx_keyLay(0)\key[067]\chr[2] = ""
        sgx_keyLay(0)\key[068]\pbid=#PB_Key_F11           :  sgx_keyLay(0)\key[068]\chr[0] = ""   :  sgx_keyLay(0)\key[068]\chr[1] = ""   :  sgx_keyLay(0)\key[068]\chr[2] = ""
        sgx_keyLay(0)\key[069]\pbid=#PB_Key_F12           :  sgx_keyLay(0)\key[069]\chr[0] = ""   :  sgx_keyLay(0)\key[069]\chr[1] = ""   :  sgx_keyLay(0)\key[069]\chr[2] = ""
        
        sgx_keyLay(0)\key[070]\pbid=#PB_Key_NumLock       :  sgx_keyLay(0)\key[070]\chr[0] = ""   :  sgx_keyLay(0)\key[070]\chr[1] = ""   :  sgx_keyLay(0)\key[070]\chr[2] = ""
        sgx_keyLay(0)\key[071]\pbid=#PB_Key_Scroll        :  sgx_keyLay(0)\key[071]\chr[0] = ""   :  sgx_keyLay(0)\key[071]\chr[1] = ""   :  sgx_keyLay(0)\key[071]\chr[2] = ""
        sgx_keyLay(0)\key[072]\pbid=#PB_Key_Pad0          :  sgx_keyLay(0)\key[072]\chr[0] = "0"  :  sgx_keyLay(0)\key[072]\chr[1] = "0"  :  sgx_keyLay(0)\key[072]\chr[2] = ""
        sgx_keyLay(0)\key[073]\pbid=#PB_Key_Pad1          :  sgx_keyLay(0)\key[073]\chr[0] = "1"  :  sgx_keyLay(0)\key[073]\chr[1] = "1"  :  sgx_keyLay(0)\key[073]\chr[2] = ""
        sgx_keyLay(0)\key[074]\pbid=#PB_Key_Pad2          :  sgx_keyLay(0)\key[074]\chr[0] = "2"  :  sgx_keyLay(0)\key[074]\chr[1] = "2"  :  sgx_keyLay(0)\key[074]\chr[2] = ""
        sgx_keyLay(0)\key[075]\pbid=#PB_Key_Pad3          :  sgx_keyLay(0)\key[075]\chr[0] = "3"  :  sgx_keyLay(0)\key[075]\chr[1] = "3"  :  sgx_keyLay(0)\key[075]\chr[2] = ""
        sgx_keyLay(0)\key[076]\pbid=#PB_Key_Pad4          :  sgx_keyLay(0)\key[076]\chr[0] = "4"  :  sgx_keyLay(0)\key[076]\chr[1] = "4"  :  sgx_keyLay(0)\key[076]\chr[2] = ""
        sgx_keyLay(0)\key[077]\pbid=#PB_Key_Pad5          :  sgx_keyLay(0)\key[077]\chr[0] = "5"  :  sgx_keyLay(0)\key[077]\chr[1] = "5"  :  sgx_keyLay(0)\key[077]\chr[2] = ""
        sgx_keyLay(0)\key[078]\pbid=#PB_Key_Pad6          :  sgx_keyLay(0)\key[078]\chr[0] = "6"  :  sgx_keyLay(0)\key[078]\chr[1] = "6"  :  sgx_keyLay(0)\key[078]\chr[2] = ""
        sgx_keyLay(0)\key[079]\pbid=#PB_Key_Pad7          :  sgx_keyLay(0)\key[079]\chr[0] = "7"  :  sgx_keyLay(0)\key[079]\chr[1] = "7"  :  sgx_keyLay(0)\key[079]\chr[2] = ""
        sgx_keyLay(0)\key[080]\pbid=#PB_Key_Pad8          :  sgx_keyLay(0)\key[080]\chr[0] = "8"  :  sgx_keyLay(0)\key[080]\chr[1] = "8"  :  sgx_keyLay(0)\key[080]\chr[2] = ""
        sgx_keyLay(0)\key[081]\pbid=#PB_Key_Pad9          :  sgx_keyLay(0)\key[081]\chr[0] = "9"  :  sgx_keyLay(0)\key[081]\chr[1] = "9"  :  sgx_keyLay(0)\key[081]\chr[2] = ""
        
        sgx_keyLay(0)\key[082]\pbid=#PB_Key_Add           :  sgx_keyLay(0)\key[082]\chr[0] = "+"  :  sgx_keyLay(0)\key[082]\chr[1] = "+"  :  sgx_keyLay(0)\key[082]\chr[2] = ""
        sgx_keyLay(0)\key[083]\pbid=#PB_Key_Subtract      :  sgx_keyLay(0)\key[083]\chr[0] = "-"  :  sgx_keyLay(0)\key[083]\chr[1] = "-"  :  sgx_keyLay(0)\key[083]\chr[2] = ""
        sgx_keyLay(0)\key[084]\pbid=#PB_Key_Decimal       :  sgx_keyLay(0)\key[084]\chr[0] = "."  :  sgx_keyLay(0)\key[084]\chr[1] = ""   :  sgx_keyLay(0)\key[084]\chr[2] = ""
        sgx_keyLay(0)\key[085]\pbid=#PB_Key_PadEnter      :  sgx_keyLay(0)\key[085]\chr[0] = ""   :  sgx_keyLay(0)\key[085]\chr[1] = ""   :  sgx_keyLay(0)\key[085]\chr[2] = ""
        sgx_keyLay(0)\key[086]\pbid=#PB_Key_RightControl  :  sgx_keyLay(0)\key[086]\chr[0] = ""   :  sgx_keyLay(0)\key[086]\chr[1] = ""   :  sgx_keyLay(0)\key[086]\chr[2] = ""
        sgx_keyLay(0)\key[087]\pbid=#PB_Key_PadComma      :  sgx_keyLay(0)\key[087]\chr[0] = "."  :  sgx_keyLay(0)\key[087]\chr[1] = ""   :  sgx_keyLay(0)\key[087]\chr[2] = ""
        sgx_keyLay(0)\key[088]\pbid=#PB_Key_Divide        :  sgx_keyLay(0)\key[088]\chr[0] = "/"  :  sgx_keyLay(0)\key[088]\chr[1] = "/"  :  sgx_keyLay(0)\key[088]\chr[2] = ""
        sgx_keyLay(0)\key[089]\pbid=#PB_Key_RightAlt      :  sgx_keyLay(0)\key[089]\chr[0] = ""   :  sgx_keyLay(0)\key[089]\chr[1] = ""   :  sgx_keyLay(0)\key[089]\chr[2] = ""
        sgx_keyLay(0)\key[090]\pbid=#PB_Key_Pause         :  sgx_keyLay(0)\key[090]\chr[0] = ""   :  sgx_keyLay(0)\key[090]\chr[1] = ""   :  sgx_keyLay(0)\key[090]\chr[2] = ""
        sgx_keyLay(0)\key[091]\pbid=#PB_Key_Home          :  sgx_keyLay(0)\key[091]\chr[0] = ""   :  sgx_keyLay(0)\key[091]\chr[1] = ""   :  sgx_keyLay(0)\key[091]\chr[2] = ""
        sgx_keyLay(0)\key[092]\pbid=#PB_Key_Up            :  sgx_keyLay(0)\key[092]\chr[0] = ""   :  sgx_keyLay(0)\key[092]\chr[1] = ""   :  sgx_keyLay(0)\key[092]\chr[2] = ""
        sgx_keyLay(0)\key[093]\pbid=#PB_Key_Down          :  sgx_keyLay(0)\key[093]\chr[0] = ""   :  sgx_keyLay(0)\key[093]\chr[1] = ""   :  sgx_keyLay(0)\key[093]\chr[2] = ""
        sgx_keyLay(0)\key[094]\pbid=#PB_Key_Left          :  sgx_keyLay(0)\key[094]\chr[0] = ""   :  sgx_keyLay(0)\key[094]\chr[1] = ""   :  sgx_keyLay(0)\key[094]\chr[2] = ""
        sgx_keyLay(0)\key[095]\pbid=#PB_Key_Right         :  sgx_keyLay(0)\key[095]\chr[0] = ""   :  sgx_keyLay(0)\key[095]\chr[1] = ""   :  sgx_keyLay(0)\key[095]\chr[2] = ""
        sgx_keyLay(0)\key[096]\pbid=#PB_Key_End           :  sgx_keyLay(0)\key[096]\chr[0] = ""   :  sgx_keyLay(0)\key[096]\chr[1] = ""   :  sgx_keyLay(0)\key[096]\chr[2] = ""
        sgx_keyLay(0)\key[097]\pbid=#PB_Key_PageUp        :  sgx_keyLay(0)\key[097]\chr[0] = ""   :  sgx_keyLay(0)\key[097]\chr[1] = ""   :  sgx_keyLay(0)\key[097]\chr[2] = ""
        sgx_keyLay(0)\key[098]\pbid=#PB_Key_PageDown      :  sgx_keyLay(0)\key[098]\chr[0] = ""   :  sgx_keyLay(0)\key[098]\chr[1] = ""   :  sgx_keyLay(0)\key[098]\chr[2] = ""
        sgx_keyLay(0)\key[099]\pbid=#PB_Key_Insert        :  sgx_keyLay(0)\key[099]\chr[0] = ""   :  sgx_keyLay(0)\key[099]\chr[1] = ""   :  sgx_keyLay(0)\key[099]\chr[2] = ""
        sgx_keyLay(0)\key[100]\pbid=#PB_Key_Delete        :  sgx_keyLay(0)\key[100]\chr[0] = ""   :  sgx_keyLay(0)\key[100]\chr[1] = ""   :  sgx_keyLay(0)\key[100]\chr[2] = ""
        ;}
        ;  ----------
        
        
        ;{ ---------- define table 1 - qwertz / german
        ;sgx_keyLay(1)             pbid                       ---    chr[0] / mode 0 / normal           ---  chr[1] / mode 1 / shift              ---  chr[2] / mode 2 / alt-gr
        
        
        sgx_keyLay(1)\key[000]\pbid=#PB_Key_1             :  sgx_keyLay(1)\key[000]\chr[0] = "1"  :  sgx_keyLay(1)\key[000]\chr[1] = "!"  :  sgx_keyLay(1)\key[000]\chr[2] = ""
        sgx_keyLay(1)\key[001]\pbid=#PB_Key_2             :  sgx_keyLay(1)\key[001]\chr[0] = "2"  :  sgx_keyLay(1)\key[001]\chr[1]=Chr(34):  sgx_keyLay(1)\key[001]\chr[2] = "²"
        sgx_keyLay(1)\key[002]\pbid=#PB_Key_3             :  sgx_keyLay(1)\key[002]\chr[0] = "3"  :  sgx_keyLay(1)\key[002]\chr[1] = "§"  :  sgx_keyLay(1)\key[002]\chr[2] = "³"
        sgx_keyLay(1)\key[003]\pbid=#PB_Key_4             :  sgx_keyLay(1)\key[003]\chr[0] = "4"  :  sgx_keyLay(1)\key[003]\chr[1] = "$"  :  sgx_keyLay(1)\key[003]\chr[2] = ""
        sgx_keyLay(1)\key[004]\pbid=#PB_Key_5             :  sgx_keyLay(1)\key[004]\chr[0] = "5"  :  sgx_keyLay(1)\key[004]\chr[1] = "%"  :  sgx_keyLay(1)\key[004]\chr[2] = ""
        sgx_keyLay(1)\key[005]\pbid=#PB_Key_6             :  sgx_keyLay(1)\key[005]\chr[0] = "6"  :  sgx_keyLay(1)\key[005]\chr[1] = "&"  :  sgx_keyLay(1)\key[005]\chr[2] = ""
        sgx_keyLay(1)\key[006]\pbid=#PB_Key_7             :  sgx_keyLay(1)\key[006]\chr[0] = "7"  :  sgx_keyLay(1)\key[006]\chr[1] = "/"  :  sgx_keyLay(1)\key[006]\chr[2] = "{"
        sgx_keyLay(1)\key[007]\pbid=#PB_Key_8             :  sgx_keyLay(1)\key[007]\chr[0] = "8"  :  sgx_keyLay(1)\key[007]\chr[1] = "("  :  sgx_keyLay(1)\key[007]\chr[2] = "["
        sgx_keyLay(1)\key[008]\pbid=#PB_Key_9             :  sgx_keyLay(1)\key[008]\chr[0] = "9"  :  sgx_keyLay(1)\key[008]\chr[1] = ")"  :  sgx_keyLay(1)\key[008]\chr[2] = "]"
        sgx_keyLay(1)\key[009]\pbid=#PB_Key_0             :  sgx_keyLay(1)\key[009]\chr[0] = "0"  :  sgx_keyLay(1)\key[009]\chr[1] = "="  :  sgx_keyLay(1)\key[009]\chr[2] = "}"
        
        sgx_keyLay(1)\key[010]\pbid=#PB_Key_A             :  sgx_keyLay(1)\key[010]\chr[0] = "a"  :  sgx_keyLay(1)\key[010]\chr[1] = "A"  :  sgx_keyLay(1)\key[010]\chr[2] = ""
        sgx_keyLay(1)\key[011]\pbid=#PB_Key_B             :  sgx_keyLay(1)\key[011]\chr[0] = "b"  :  sgx_keyLay(1)\key[011]\chr[1] = "B"  :  sgx_keyLay(1)\key[011]\chr[2] = ""
        sgx_keyLay(1)\key[012]\pbid=#PB_Key_C             :  sgx_keyLay(1)\key[012]\chr[0] = "c"  :  sgx_keyLay(1)\key[012]\chr[1] = "C"  :  sgx_keyLay(1)\key[012]\chr[2] = ""
        sgx_keyLay(1)\key[013]\pbid=#PB_Key_D             :  sgx_keyLay(1)\key[013]\chr[0] = "d"  :  sgx_keyLay(1)\key[013]\chr[1] = "D"  :  sgx_keyLay(1)\key[013]\chr[2] = ""
        sgx_keyLay(1)\key[014]\pbid=#PB_Key_E             :  sgx_keyLay(1)\key[014]\chr[0] = "e"  :  sgx_keyLay(1)\key[014]\chr[1] = "E"  :  sgx_keyLay(1)\key[014]\chr[2] = ""
        sgx_keyLay(1)\key[015]\pbid=#PB_Key_F             :  sgx_keyLay(1)\key[015]\chr[0] = "f"  :  sgx_keyLay(1)\key[015]\chr[1] = "F"  :  sgx_keyLay(1)\key[015]\chr[2] = ""
        sgx_keyLay(1)\key[016]\pbid=#PB_Key_G             :  sgx_keyLay(1)\key[016]\chr[0] = "g"  :  sgx_keyLay(1)\key[016]\chr[1] = "G"  :  sgx_keyLay(1)\key[016]\chr[2] = ""
        sgx_keyLay(1)\key[017]\pbid=#PB_Key_H             :  sgx_keyLay(1)\key[017]\chr[0] = "h"  :  sgx_keyLay(1)\key[017]\chr[1] = "H"  :  sgx_keyLay(1)\key[017]\chr[2] = ""
        sgx_keyLay(1)\key[018]\pbid=#PB_Key_I             :  sgx_keyLay(1)\key[018]\chr[0] = "i"  :  sgx_keyLay(1)\key[018]\chr[1] = "I"  :  sgx_keyLay(1)\key[018]\chr[2] = ""
        sgx_keyLay(1)\key[019]\pbid=#PB_Key_J             :  sgx_keyLay(1)\key[019]\chr[0] = "j"  :  sgx_keyLay(1)\key[019]\chr[1] = "J"  :  sgx_keyLay(1)\key[019]\chr[2] = ""
        sgx_keyLay(1)\key[020]\pbid=#PB_Key_K             :  sgx_keyLay(1)\key[020]\chr[0] = "k"  :  sgx_keyLay(1)\key[020]\chr[1] = "K"  :  sgx_keyLay(1)\key[020]\chr[2] = ""
        sgx_keyLay(1)\key[021]\pbid=#PB_Key_L             :  sgx_keyLay(1)\key[021]\chr[0] = "l"  :  sgx_keyLay(1)\key[021]\chr[1] = "L"  :  sgx_keyLay(1)\key[021]\chr[2] = ""
        sgx_keyLay(1)\key[022]\pbid=#PB_Key_M             :  sgx_keyLay(1)\key[022]\chr[0] = "m"  :  sgx_keyLay(1)\key[022]\chr[1] = "M"  :  sgx_keyLay(1)\key[022]\chr[2] = "µ"
        sgx_keyLay(1)\key[023]\pbid=#PB_Key_N             :  sgx_keyLay(1)\key[023]\chr[0] = "n"  :  sgx_keyLay(1)\key[023]\chr[1] = "N"  :  sgx_keyLay(1)\key[023]\chr[2] = ""
        sgx_keyLay(1)\key[024]\pbid=#PB_Key_O             :  sgx_keyLay(1)\key[024]\chr[0] = "o"  :  sgx_keyLay(1)\key[024]\chr[1] = "O"  :  sgx_keyLay(1)\key[024]\chr[2] = ""
        sgx_keyLay(1)\key[025]\pbid=#PB_Key_P             :  sgx_keyLay(1)\key[025]\chr[0] = "p"  :  sgx_keyLay(1)\key[025]\chr[1] = "P"  :  sgx_keyLay(1)\key[025]\chr[2] = ""
        sgx_keyLay(1)\key[026]\pbid=#PB_Key_Q             :  sgx_keyLay(1)\key[026]\chr[0] = "q"  :  sgx_keyLay(1)\key[026]\chr[1] = "Q"  :  sgx_keyLay(1)\key[026]\chr[2] = "@"
        sgx_keyLay(1)\key[027]\pbid=#PB_Key_R             :  sgx_keyLay(1)\key[027]\chr[0] = "r"  :  sgx_keyLay(1)\key[027]\chr[1] = "R"  :  sgx_keyLay(1)\key[027]\chr[2] = ""
        sgx_keyLay(1)\key[028]\pbid=#PB_Key_S             :  sgx_keyLay(1)\key[028]\chr[0] = "s"  :  sgx_keyLay(1)\key[028]\chr[1] = "S"  :  sgx_keyLay(1)\key[028]\chr[2] = ""
        sgx_keyLay(1)\key[029]\pbid=#PB_Key_T             :  sgx_keyLay(1)\key[029]\chr[0] = "t"  :  sgx_keyLay(1)\key[029]\chr[1] = "T"  :  sgx_keyLay(1)\key[029]\chr[2] = ""
        sgx_keyLay(1)\key[030]\pbid=#PB_Key_U             :  sgx_keyLay(1)\key[030]\chr[0] = "u"  :  sgx_keyLay(1)\key[030]\chr[1] = "U"  :  sgx_keyLay(1)\key[030]\chr[2] = ""
        sgx_keyLay(1)\key[031]\pbid=#PB_Key_V             :  sgx_keyLay(1)\key[031]\chr[0] = "v"  :  sgx_keyLay(1)\key[031]\chr[1] = "V"  :  sgx_keyLay(1)\key[031]\chr[2] = ""
        sgx_keyLay(1)\key[032]\pbid=#PB_Key_W             :  sgx_keyLay(1)\key[032]\chr[0] = "w"  :  sgx_keyLay(1)\key[032]\chr[1] = "W"  :  sgx_keyLay(1)\key[032]\chr[2] = ""
        sgx_keyLay(1)\key[033]\pbid=#PB_Key_X             :  sgx_keyLay(1)\key[033]\chr[0] = "x"  :  sgx_keyLay(1)\key[033]\chr[1] = "X"  :  sgx_keyLay(1)\key[033]\chr[2] = ""
        sgx_keyLay(1)\key[034]\pbid=#PB_Key_Y             :  sgx_keyLay(1)\key[034]\chr[0] = "z"  :  sgx_keyLay(1)\key[034]\chr[1] = "Z"  :  sgx_keyLay(1)\key[034]\chr[2] = ""
        sgx_keyLay(1)\key[035]\pbid=#PB_Key_Z             :  sgx_keyLay(1)\key[035]\chr[0] = "y"  :  sgx_keyLay(1)\key[035]\chr[1] = "Y"  :  sgx_keyLay(1)\key[035]\chr[2] = ""
        
        sgx_keyLay(1)\key[036]\pbid=#PB_Key_Escape        :  sgx_keyLay(1)\key[036]\chr[0] = ""   :  sgx_keyLay(1)\key[036]\chr[1] = ""   :  sgx_keyLay(1)\key[036]\chr[2] = ""
        sgx_keyLay(1)\key[037]\pbid=#PB_Key_Minus         :  sgx_keyLay(1)\key[037]\chr[0] = "ß"  :  sgx_keyLay(1)\key[037]\chr[1] = "?"  :  sgx_keyLay(1)\key[037]\chr[2] = "\"
        sgx_keyLay(1)\key[038]\pbid=#PB_Key_Equals        :  sgx_keyLay(1)\key[038]\chr[0] = "´"  :  sgx_keyLay(1)\key[038]\chr[1] = "`"  :  sgx_keyLay(1)\key[038]\chr[2] = ""
        sgx_keyLay(1)\key[039]\pbid=#PB_Key_Back          :  sgx_keyLay(1)\key[039]\chr[0] = ""   :  sgx_keyLay(1)\key[039]\chr[1] = ""   :  sgx_keyLay(1)\key[039]\chr[2] = ""
        sgx_keyLay(1)\key[040]\pbid=#PB_Key_Tab           :  sgx_keyLay(1)\key[040]\chr[0] = ""   :  sgx_keyLay(1)\key[040]\chr[1] = ""   :  sgx_keyLay(1)\key[040]\chr[2] = ""
        sgx_keyLay(1)\key[041]\pbid=#PB_Key_LeftBracket   :  sgx_keyLay(1)\key[041]\chr[0] = "ü"  :  sgx_keyLay(1)\key[041]\chr[1] = "Ü"  :  sgx_keyLay(1)\key[041]\chr[2] = ""
        sgx_keyLay(1)\key[042]\pbid=#PB_Key_RightBracket  :  sgx_keyLay(1)\key[042]\chr[0] = "+"  :  sgx_keyLay(1)\key[042]\chr[1] = "*"  :  sgx_keyLay(1)\key[042]\chr[2] = "~"
        sgx_keyLay(1)\key[043]\pbid=#PB_Key_Return        :  sgx_keyLay(1)\key[043]\chr[0] = ""   :  sgx_keyLay(1)\key[043]\chr[1] = ""   :  sgx_keyLay(1)\key[043]\chr[2] = ""
        sgx_keyLay(1)\key[044]\pbid=#PB_Key_LeftControl   :  sgx_keyLay(1)\key[044]\chr[0] = ""   :  sgx_keyLay(1)\key[044]\chr[1] = ""   :  sgx_keyLay(1)\key[044]\chr[2] = ""
        sgx_keyLay(1)\key[045]\pbid=#PB_Key_SemiColon     :  sgx_keyLay(1)\key[045]\chr[0] = "ö"  :  sgx_keyLay(1)\key[045]\chr[1] = "Ö"  :  sgx_keyLay(1)\key[045]\chr[2] = ""
        sgx_keyLay(1)\key[046]\pbid=#PB_Key_Apostrophe    :  sgx_keyLay(1)\key[046]\chr[0] = "ä"  :  sgx_keyLay(1)\key[046]\chr[1] = "Ä"  :  sgx_keyLay(1)\key[046]\chr[2] = ""
        sgx_keyLay(1)\key[047]\pbid=#PB_Key_Grave         :  sgx_keyLay(1)\key[047]\chr[0] = "^"  :  sgx_keyLay(1)\key[047]\chr[1] = "°"  :  sgx_keyLay(1)\key[047]\chr[2] = ""
        sgx_keyLay(1)\key[048]\pbid=#PB_Key_LeftShift     :  sgx_keyLay(1)\key[048]\chr[0] = ""   :  sgx_keyLay(1)\key[048]\chr[1] = ""   :  sgx_keyLay(1)\key[048]\chr[2] = ""
        sgx_keyLay(1)\key[049]\pbid=#PB_Key_BackSlash     :  sgx_keyLay(1)\key[049]\chr[0] = "#"  :  sgx_keyLay(1)\key[049]\chr[1] = "'"  :  sgx_keyLay(1)\key[049]\chr[2] = ""
        sgx_keyLay(1)\key[050]\pbid=#PB_Key_Comma         :  sgx_keyLay(1)\key[050]\chr[0] = ","  :  sgx_keyLay(1)\key[050]\chr[1] = ";"  :  sgx_keyLay(1)\key[050]\chr[2] = "<"
        sgx_keyLay(1)\key[051]\pbid=#PB_Key_Period        :  sgx_keyLay(1)\key[051]\chr[0] = "."  :  sgx_keyLay(1)\key[051]\chr[1] = ":"  :  sgx_keyLay(1)\key[051]\chr[2] = ">"
        sgx_keyLay(1)\key[052]\pbid=#PB_Key_Slash         :  sgx_keyLay(1)\key[052]\chr[0] = "-"  :  sgx_keyLay(1)\key[052]\chr[1] = "_"  :  sgx_keyLay(1)\key[052]\chr[2] = "|"
        sgx_keyLay(1)\key[053]\pbid=#PB_Key_RightShift    :  sgx_keyLay(1)\key[053]\chr[0] = ""   :  sgx_keyLay(1)\key[053]\chr[1] = ""   :  sgx_keyLay(1)\key[053]\chr[2] = ""
        sgx_keyLay(1)\key[054]\pbid=#PB_Key_Multiply      :  sgx_keyLay(1)\key[054]\chr[0] = "*"  :  sgx_keyLay(1)\key[054]\chr[1] = "*"  :  sgx_keyLay(1)\key[054]\chr[2] = ""
        sgx_keyLay(1)\key[055]\pbid=#PB_Key_LeftAlt       :  sgx_keyLay(1)\key[055]\chr[0] = ""   :  sgx_keyLay(1)\key[055]\chr[1] = ""   :  sgx_keyLay(1)\key[055]\chr[2] = ""
        sgx_keyLay(1)\key[056]\pbid=#PB_Key_Space         :  sgx_keyLay(1)\key[056]\chr[0] = " "  :  sgx_keyLay(1)\key[056]\chr[1] = " "  :  sgx_keyLay(1)\key[056]\chr[2] = ""
        sgx_keyLay(1)\key[057]\pbid=#PB_Key_Capital       :  sgx_keyLay(1)\key[057]\chr[0] = ""   :  sgx_keyLay(1)\key[057]\chr[1] = ""   :  sgx_keyLay(1)\key[057]\chr[2] = ""
        
        sgx_keyLay(1)\key[058]\pbid=#PB_Key_F1            :  sgx_keyLay(1)\key[058]\chr[0] = ""   :  sgx_keyLay(1)\key[058]\chr[1] = ""   :  sgx_keyLay(1)\key[058]\chr[2] = ""
        sgx_keyLay(1)\key[059]\pbid=#PB_Key_F2            :  sgx_keyLay(1)\key[059]\chr[0] = ""   :  sgx_keyLay(1)\key[059]\chr[1] = ""   :  sgx_keyLay(1)\key[059]\chr[2] = ""
        sgx_keyLay(1)\key[060]\pbid=#PB_Key_F3            :  sgx_keyLay(1)\key[060]\chr[0] = ""   :  sgx_keyLay(1)\key[060]\chr[1] = ""   :  sgx_keyLay(1)\key[060]\chr[2] = ""
        sgx_keyLay(1)\key[061]\pbid=#PB_Key_F4            :  sgx_keyLay(1)\key[061]\chr[0] = ""   :  sgx_keyLay(1)\key[061]\chr[1] = ""   :  sgx_keyLay(1)\key[061]\chr[2] = ""
        sgx_keyLay(1)\key[062]\pbid=#PB_Key_F5            :  sgx_keyLay(1)\key[062]\chr[0] = ""   :  sgx_keyLay(1)\key[062]\chr[1] = ""   :  sgx_keyLay(1)\key[062]\chr[2] = ""
        sgx_keyLay(1)\key[063]\pbid=#PB_Key_F6            :  sgx_keyLay(1)\key[063]\chr[0] = ""   :  sgx_keyLay(1)\key[063]\chr[1] = ""   :  sgx_keyLay(1)\key[063]\chr[2] = ""
        sgx_keyLay(1)\key[064]\pbid=#PB_Key_F7            :  sgx_keyLay(1)\key[064]\chr[0] = ""   :  sgx_keyLay(1)\key[064]\chr[1] = ""   :  sgx_keyLay(1)\key[064]\chr[2] = ""
        sgx_keyLay(1)\key[065]\pbid=#PB_Key_F8            :  sgx_keyLay(1)\key[065]\chr[0] = ""   :  sgx_keyLay(1)\key[065]\chr[1] = ""   :  sgx_keyLay(1)\key[065]\chr[2] = ""
        sgx_keyLay(1)\key[066]\pbid=#PB_Key_F9            :  sgx_keyLay(1)\key[066]\chr[0] = ""   :  sgx_keyLay(1)\key[066]\chr[1] = ""   :  sgx_keyLay(1)\key[066]\chr[2] = ""
        sgx_keyLay(1)\key[067]\pbid=#PB_Key_F10           :  sgx_keyLay(1)\key[067]\chr[0] = ""   :  sgx_keyLay(1)\key[067]\chr[1] = ""   :  sgx_keyLay(1)\key[067]\chr[2] = ""
        sgx_keyLay(1)\key[068]\pbid=#PB_Key_F11           :  sgx_keyLay(1)\key[068]\chr[0] = ""   :  sgx_keyLay(1)\key[068]\chr[1] = ""   :  sgx_keyLay(1)\key[068]\chr[2] = ""
        sgx_keyLay(1)\key[069]\pbid=#PB_Key_F12           :  sgx_keyLay(1)\key[069]\chr[0] = ""   :  sgx_keyLay(1)\key[069]\chr[1] = ""   :  sgx_keyLay(1)\key[069]\chr[2] = ""
        
        sgx_keyLay(1)\key[070]\pbid=#PB_Key_NumLock       :  sgx_keyLay(1)\key[070]\chr[0] = ""   :  sgx_keyLay(1)\key[070]\chr[1] = ""   :  sgx_keyLay(1)\key[070]\chr[2] = ""
        sgx_keyLay(1)\key[071]\pbid=#PB_Key_Scroll        :  sgx_keyLay(1)\key[071]\chr[0] = ""   :  sgx_keyLay(1)\key[071]\chr[1] = ""   :  sgx_keyLay(1)\key[071]\chr[2] = ""
        sgx_keyLay(1)\key[072]\pbid=#PB_Key_Pad0          :  sgx_keyLay(1)\key[072]\chr[0] = "0"  :  sgx_keyLay(1)\key[072]\chr[1] = "0"  :  sgx_keyLay(1)\key[072]\chr[2] = ""
        sgx_keyLay(1)\key[073]\pbid=#PB_Key_Pad1          :  sgx_keyLay(1)\key[073]\chr[0] = "1"  :  sgx_keyLay(1)\key[073]\chr[1] = "1"  :  sgx_keyLay(1)\key[073]\chr[2] = ""
        sgx_keyLay(1)\key[074]\pbid=#PB_Key_Pad2          :  sgx_keyLay(1)\key[074]\chr[0] = "2"  :  sgx_keyLay(1)\key[074]\chr[1] = "2"  :  sgx_keyLay(1)\key[074]\chr[2] = ""
        sgx_keyLay(1)\key[075]\pbid=#PB_Key_Pad3          :  sgx_keyLay(1)\key[075]\chr[0] = "3"  :  sgx_keyLay(1)\key[075]\chr[1] = "3"  :  sgx_keyLay(1)\key[075]\chr[2] = ""
        sgx_keyLay(1)\key[076]\pbid=#PB_Key_Pad4          :  sgx_keyLay(1)\key[076]\chr[0] = "4"  :  sgx_keyLay(1)\key[076]\chr[1] = "4"  :  sgx_keyLay(1)\key[076]\chr[2] = ""
        sgx_keyLay(1)\key[077]\pbid=#PB_Key_Pad5          :  sgx_keyLay(1)\key[077]\chr[0] = "5"  :  sgx_keyLay(1)\key[077]\chr[1] = "5"  :  sgx_keyLay(1)\key[077]\chr[2] = ""
        sgx_keyLay(1)\key[078]\pbid=#PB_Key_Pad6          :  sgx_keyLay(1)\key[078]\chr[0] = "6"  :  sgx_keyLay(1)\key[078]\chr[1] = "6"  :  sgx_keyLay(1)\key[078]\chr[2] = ""
        sgx_keyLay(1)\key[079]\pbid=#PB_Key_Pad7          :  sgx_keyLay(1)\key[079]\chr[0] = "7"  :  sgx_keyLay(1)\key[079]\chr[1] = "7"  :  sgx_keyLay(1)\key[079]\chr[2] = ""
        sgx_keyLay(1)\key[080]\pbid=#PB_Key_Pad8          :  sgx_keyLay(1)\key[080]\chr[0] = "8"  :  sgx_keyLay(1)\key[080]\chr[1] = "8"  :  sgx_keyLay(1)\key[080]\chr[2] = ""
        sgx_keyLay(1)\key[081]\pbid=#PB_Key_Pad9          :  sgx_keyLay(1)\key[081]\chr[0] = "9"  :  sgx_keyLay(1)\key[081]\chr[1] = "9"  :  sgx_keyLay(1)\key[081]\chr[2] = ""
        
        sgx_keyLay(1)\key[082]\pbid=#PB_Key_Add           :  sgx_keyLay(1)\key[082]\chr[0] = "+"  :  sgx_keyLay(1)\key[082]\chr[1] = "+"  :  sgx_keyLay(1)\key[082]\chr[2] = ""
        sgx_keyLay(1)\key[083]\pbid=#PB_Key_Subtract      :  sgx_keyLay(1)\key[083]\chr[0] = "-"  :  sgx_keyLay(1)\key[083]\chr[1] = "-"  :  sgx_keyLay(1)\key[083]\chr[2] = ""
        sgx_keyLay(1)\key[084]\pbid=#PB_Key_Decimal       :  sgx_keyLay(1)\key[084]\chr[0] = ","  :  sgx_keyLay(1)\key[084]\chr[1] = ","  :  sgx_keyLay(1)\key[084]\chr[2] = ""
        sgx_keyLay(1)\key[085]\pbid=#PB_Key_PadEnter      :  sgx_keyLay(1)\key[085]\chr[0] = ""   :  sgx_keyLay(1)\key[085]\chr[1] = ""   :  sgx_keyLay(1)\key[085]\chr[2] = ""
        sgx_keyLay(1)\key[086]\pbid=#PB_Key_RightControl  :  sgx_keyLay(1)\key[086]\chr[0] = ""   :  sgx_keyLay(1)\key[086]\chr[1] = ""   :  sgx_keyLay(1)\key[086]\chr[2] = ""
        sgx_keyLay(1)\key[087]\pbid=#PB_Key_PadComma      :  sgx_keyLay(1)\key[087]\chr[0] = ","  :  sgx_keyLay(1)\key[087]\chr[1] = ""   :  sgx_keyLay(1)\key[087]\chr[2] = ""
        sgx_keyLay(1)\key[088]\pbid=#PB_Key_Divide        :  sgx_keyLay(1)\key[088]\chr[0] = "/"  :  sgx_keyLay(1)\key[088]\chr[1] = "/"  :  sgx_keyLay(1)\key[088]\chr[2] = ""
        sgx_keyLay(1)\key[089]\pbid=#PB_Key_RightAlt      :  sgx_keyLay(1)\key[089]\chr[0] = ""   :  sgx_keyLay(1)\key[089]\chr[1] = ""   :  sgx_keyLay(1)\key[089]\chr[2] = ""
        sgx_keyLay(1)\key[090]\pbid=#PB_Key_Pause         :  sgx_keyLay(1)\key[090]\chr[0] = ""   :  sgx_keyLay(1)\key[090]\chr[1] = ""   :  sgx_keyLay(1)\key[090]\chr[2] = ""
        sgx_keyLay(1)\key[091]\pbid=#PB_Key_Home          :  sgx_keyLay(1)\key[091]\chr[0] = ""   :  sgx_keyLay(1)\key[091]\chr[1] = ""   :  sgx_keyLay(1)\key[091]\chr[2] = ""
        sgx_keyLay(1)\key[092]\pbid=#PB_Key_Up            :  sgx_keyLay(1)\key[092]\chr[0] = ""   :  sgx_keyLay(1)\key[092]\chr[1] = ""   :  sgx_keyLay(1)\key[092]\chr[2] = ""
        sgx_keyLay(1)\key[093]\pbid=#PB_Key_Down          :  sgx_keyLay(1)\key[093]\chr[0] = ""   :  sgx_keyLay(1)\key[093]\chr[1] = ""   :  sgx_keyLay(1)\key[093]\chr[2] = ""
        sgx_keyLay(1)\key[094]\pbid=#PB_Key_Left          :  sgx_keyLay(1)\key[094]\chr[0] = ""   :  sgx_keyLay(1)\key[094]\chr[1] = ""   :  sgx_keyLay(1)\key[094]\chr[2] = ""
        sgx_keyLay(1)\key[095]\pbid=#PB_Key_Right         :  sgx_keyLay(1)\key[095]\chr[0] = ""   :  sgx_keyLay(1)\key[095]\chr[1] = ""   :  sgx_keyLay(1)\key[095]\chr[2] = ""
        sgx_keyLay(1)\key[096]\pbid=#PB_Key_End           :  sgx_keyLay(1)\key[096]\chr[0] = ""   :  sgx_keyLay(1)\key[096]\chr[1] = ""   :  sgx_keyLay(1)\key[096]\chr[2] = ""
        sgx_keyLay(1)\key[097]\pbid=#PB_Key_PageUp        :  sgx_keyLay(1)\key[097]\chr[0] = ""   :  sgx_keyLay(1)\key[097]\chr[1] = ""   :  sgx_keyLay(1)\key[097]\chr[2] = ""
        sgx_keyLay(1)\key[098]\pbid=#PB_Key_PageDown      :  sgx_keyLay(1)\key[098]\chr[0] = ""   :  sgx_keyLay(1)\key[098]\chr[1] = ""   :  sgx_keyLay(1)\key[098]\chr[2] = ""
        sgx_keyLay(1)\key[099]\pbid=#PB_Key_Insert        :  sgx_keyLay(1)\key[099]\chr[0] = ""   :  sgx_keyLay(1)\key[099]\chr[1] = ""   :  sgx_keyLay(1)\key[099]\chr[2] = ""
        sgx_keyLay(1)\key[100]\pbid=#PB_Key_Delete        :  sgx_keyLay(1)\key[100]\chr[0] = ""   :  sgx_keyLay(1)\key[100]\chr[1] = ""   :  sgx_keyLay(1)\key[100]\chr[2] = ""
        
        ;}
        ;  ----------
        
                
        
        ;** sgxSetKeyboardLayout:
        ;* configure keyboard layout and keystroke delay used by sgxEdit elements (via <a href=#f_sgx_keyboardedit>sgx_keyboardEdit</a>). _
        ;* when using qwerty layout the characters '<', '>' (angle brackets) and '|' (pipe) can be accessed via 'alt-gr' + ',' / '.' / '-' (comma, dot and hyphen).
        ;** .layout           : #sgxKeyLayQwerty (default) for qwerty / US keyboard layout. #sgxKeyLayQwertz for qwertz / DE
        ;** .keystrokeDelay_1 : the keystroke delay after a first key-push in milliseconds (default 280)
        ;** .keystrokeDelay_2 : the keystroke delay for continuous key-push in milliseconds (default 40)
        Procedure sgxSetKeyboardLayout(layout=#sgxKeyLayQwerty, keystrokeDelay_1=280, keystrokeDelay_2=40)
          If layout >= 0 And layout <= ArraySize(sgx_keyLay())
            sgx_keyLayout = layout
          EndIf
          sgx_keystrokeDelay_1 = keystrokeDelay_1
          sgx_keystrokeDelay_2 = keystrokeDelay_2
        EndProcedure
        
        
        ;** sgx_keyboardEdit
        ;* defines string operations on keyboard data. (text editing) _
        ;* basic usage: _
        ;* <pre>
        ;* sgxSetKeyboardLayout() _
        ;* Repeat _
        ;* <span></span>  ExamineKeyboard() _
        ;* <span></span>  text = sgx_keyboardEdit(text, textpos) _
        ;* <span></span>  textpos = keyboardEdit_lastTextPos _
        ;* <span></span>  if sgx_keyboardEdit_TextModified _
        ;* <span></span>    ; text or textpos changed _
        ;* <span></span>  ;... _
        ;* </pre>
        ;* if after a call of this function 'sgx_keyboardEdit_TextModified' is #True then the text or the textpos has changed. it will be #False if no change occured. _
        ;* the possibly changed text is returned and 'sgx_keyboardEdit_lastTextPos' will be set to the current textpos. _
        ;* you can configure keyboard layout and keystroke delay via <a href=#f_sgxsetkeyboardlayout>sgxSetKeyboardLayout</a>
        ;** .text            : the text to be edited.
        ;** .textpos         : the initial cursor position in characters including linebreaks. -1 (default) means beginning of text. the position might change during the call i.e. if cursor keys have been used ot text has been entered. use sgx_keyboardEdit_lastTextPos to get the updated value after a call.
        ;** .maxLines        : the maximum lines allowed. -1 (default) means no limit.
        Procedure.s sgx_keyboardEdit(text.s, textpos=-1, maxLines=-1)
          Static mode
          Static caps
          Static cntrl
          Static textPartA.s
          Static textPartB.s
          Static ems
          Static k
          Static i
          Static timeout
          Static temp
          Static textlen
          
          DisableDebugger
          
          sgx_keyboardEdit_TextModified = #False
          
          If textpos<0
            textpos=0
          EndIf
          If textpos>Len(text)
            textpos=Len(text)
          EndIf
          If maxLines>0 And CountString(text,#LF$)+1>maxLines
            textPartA=""
            For i=1 To maxLines
              textPartA + StringField(text,i,#LF$) + #LF$
            Next
            text = Left(textPartA, Len(textPartA)-1)
            sgx_keyboardEdit_TextModified = #True
          EndIf
          
          textPartA = Mid(text,1,textpos)
          textPartB = Mid(text,textpos+1,Len(text))
          
          ems=ElapsedMilliseconds()
          
          
          For k=0 To 100
            If KeyboardReleased(sgx_keyLay(sgx_keyLayout)\key[k]\pbid)
              sgx_keyLay(sgx_keyLayout)\key[k]\timer = 0
              sgx_keyLay(sgx_keyLayout)\key[k]\delaySwitch = 0
            EndIf
          Next
          
          caps = #False
          If (KeyboardPushed(#PB_Key_LeftShift) Or KeyboardPushed(#PB_Key_RightShift))  And  (Not KeyboardPushed(#PB_Key_Capital))
            caps = #True
          ElseIf Not (KeyboardPushed(#PB_Key_LeftShift) Or KeyboardPushed(#PB_Key_RightShift))  And  KeyboardPushed(#PB_Key_Capital)
            caps = #True
          EndIf
          
          If KeyboardPushed(#PB_Key_All)
            
            If KeyboardPushed(#PB_Key_RightAlt)
              mode = 2
            ElseIf caps
              mode = 1
            Else
              mode = 0
            EndIf
        
            If KeyboardPushed(#PB_Key_LeftControl) Or KeyboardPushed(#PB_Key_RightControl)
              cntrl = 1
            Else
              cntrl = 0
            EndIf
            
          
            For k=0 To 100
              
              If KeyboardPushed(sgx_keyLay(sgx_keyLayout)\key[k]\pbid)
                
                ;[-keystroke-delayed-]
                
                If sgx_keyLay(sgx_keyLayout)\key[k]\timer < ems             ; <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
                  timeout=1
                  
                  ;{ BACK
                  If sgx_keyLay(sgx_keyLayout)\key[k]\pbid = #PB_Key_Back
                    textPartA = LSet( textPartA, Len(textPartA)-1 )
                    textpos = Len(textPartA)
                    If sgx_keyLay(sgx_keyLayout)\key[k]\delaySwitch = 0
                      sgx_keyLay(sgx_keyLayout)\key[k]\timer = ems + sgx_keystrokeDelay_1
                      sgx_keyLay(sgx_keyLayout)\key[k]\delaySwitch = 1
                    ElseIf sgx_keyLay(sgx_keyLayout)\key[k]\delaySwitch = 1
                      sgx_keyLay(sgx_keyLayout)\key[k]\timer = ems + sgx_keystrokeDelay_2
                    EndIf
                    ;}
                  
                  ;{ DEL  
                  ElseIf sgx_keyLay(sgx_keyLayout)\key[k]\pbid = #PB_Key_Delete
                    textPartB = Right( textPartB, Len(textPartB)-1 )
                    If sgx_keyLay(sgx_keyLayout)\key[k]\delaySwitch = 0
                      sgx_keyLay(sgx_keyLayout)\key[k]\timer = ems + sgx_keystrokeDelay_1
                      sgx_keyLay(sgx_keyLayout)\key[k]\delaySwitch = 1
                    ElseIf sgx_keyLay(sgx_keyLayout)\key[k]\delaySwitch = 1
                      sgx_keyLay(sgx_keyLayout)\key[k]\timer = ems + sgx_keystrokeDelay_2
                    EndIf
                    ;}
                    
                  ;{ RETURN
                  ElseIf sgx_keyLay(sgx_keyLayout)\key[k]\pbid = #PB_Key_Return Or sgx_keyLay(sgx_keyLayout)\key[k]\pbid = #PB_Key_PadEnter
                    If maxLines<1  Or  CountString(text,#LF$)+1<maxLines
                      textPartA + #LF$
                      textpos = Len(textPartA)
                      If sgx_keyLay(sgx_keyLayout)\key[k]\delaySwitch = 0
                        sgx_keyLay(sgx_keyLayout)\key[k]\timer = ems + sgx_keystrokeDelay_1
                        sgx_keyLay(sgx_keyLayout)\key[k]\delaySwitch = 1
                      ElseIf sgx_keyLay(sgx_keyLayout)\key[k]\delaySwitch = 1
                        sgx_keyLay(sgx_keyLayout)\key[k]\timer = ems + sgx_keystrokeDelay_2
                      EndIf
                    EndIf
                    ;}
                    
                  ;{ LEFT
                  ElseIf sgx_keyLay(sgx_keyLayout)\key[k]\pbid = #PB_Key_Left
                    textpos-1
                    If textpos<0 : textpos=0 : EndIf
                    textPartA = Mid(text,1,textpos)
                    textPartB = Mid(text,textpos+1,Len(text))
                    If sgx_keyLay(sgx_keyLayout)\key[k]\delaySwitch = 0
                      sgx_keyLay(sgx_keyLayout)\key[k]\timer = ems + sgx_keystrokeDelay_1
                      sgx_keyLay(sgx_keyLayout)\key[k]\delaySwitch = 1
                    ElseIf sgx_keyLay(sgx_keyLayout)\key[k]\delaySwitch = 1
                      sgx_keyLay(sgx_keyLayout)\key[k]\timer = ems + sgx_keystrokeDelay_2
                    EndIf
                    ;}
                    
                  ;{ RIGHT
                  ElseIf sgx_keyLay(sgx_keyLayout)\key[k]\pbid = #PB_Key_Right
                    textpos+1
                    If textpos>=Len(text) : textpos=Len(text) : EndIf
                    textPartA = Mid(text,1,textpos)
                    textPartB = Mid(text,textpos+1,Len(text))
                    If sgx_keyLay(sgx_keyLayout)\key[k]\delaySwitch = 0
                      sgx_keyLay(sgx_keyLayout)\key[k]\timer = ems + sgx_keystrokeDelay_1
                      sgx_keyLay(sgx_keyLayout)\key[k]\delaySwitch = 1
                    ElseIf sgx_keyLay(sgx_keyLayout)\key[k]\delaySwitch = 1
                      sgx_keyLay(sgx_keyLayout)\key[k]\timer = ems + sgx_keystrokeDelay_2
                    EndIf
                    ;}
                  
                  ;{ UP
                  ElseIf sgx_keyLay(sgx_keyLayout)\key[k]\pbid = #PB_Key_Up
                    If Mid(text,textpos,1) = #LF$
                      textpos-1
                      While textpos>0  And  Mid(text,textpos,1) <> #LF$
                        textpos-1
                      Wend
                    Else
                      temp = textpos
                      While textpos>0  And  Mid(text,textpos,1) <> #LF$
                        textpos-1
                      Wend
                      If textpos=0
                        textpos=temp
                      Else
                        temp - textpos
                        textpos-1
                        While textpos>0  And  Mid(text,textpos,1) <> #LF$
                          textpos-1
                        Wend
                        i=1
                        While (Mid(text,textpos+i,1) <> #LF$) And (i < temp)
                          i+1
                        Wend
                        textpos + i
                        If Mid(text, textpos, 1) = #LF$
                          textpos-1
                        EndIf
                      EndIf
                    EndIf
                    
                    textPartA = Mid(text,1,textpos)
                    textPartB = Mid(text,textpos+1,Len(text))
                    If sgx_keyLay(sgx_keyLayout)\key[k]\delaySwitch = 0
                      sgx_keyLay(sgx_keyLayout)\key[k]\timer = ems + sgx_keystrokeDelay_1
                      sgx_keyLay(sgx_keyLayout)\key[k]\delaySwitch = 1
                    ElseIf sgx_keyLay(sgx_keyLayout)\key[k]\delaySwitch = 1
                      sgx_keyLay(sgx_keyLayout)\key[k]\timer = ems + sgx_keystrokeDelay_2
                    EndIf
                    ;}
                  
                  ;{ DOWN
                  ElseIf sgx_keyLay(sgx_keyLayout)\key[k]\pbid = #PB_Key_Down
                    i = FindString(text,#LF$,textpos)
                    If i>0
                      textlen = Len(text)
                      If Mid(text,textpos,1) = #LF$
                        textpos+1
        
                        While textpos<textlen  And  Mid(text,textpos,1) <> #LF$
                          textpos+1
                        Wend
                      Else
                        temp = textpos
                        While textpos>0  And  Mid(text,textpos,1) <> #LF$
                          textpos-1
                        Wend
                        temp - textpos
                        textpos = i
                        While (i-textpos)<temp And Mid(text,i+1,1) <> #LF$
                          i+1
                        Wend
                        textpos = i
                      EndIf
                      textPartA = Mid(text,1,textpos)
                      textPartB = Mid(text,textpos+1,textlen)
                      If sgx_keyLay(sgx_keyLayout)\key[k]\delaySwitch = 0
                        sgx_keyLay(sgx_keyLayout)\key[k]\timer = ems + sgx_keystrokeDelay_1
                        sgx_keyLay(sgx_keyLayout)\key[k]\delaySwitch = 1
                      ElseIf sgx_keyLay(sgx_keyLayout)\key[k]\delaySwitch = 1
                        sgx_keyLay(sgx_keyLayout)\key[k]\timer = ems + sgx_keystrokeDelay_2
                      EndIf
                    EndIf
                    ;}
                  
                    
                  ;{ <CHARACTERS>
                  ElseIf sgx_keyLay(sgx_keyLayout)\key[k]\chr[mode]<>""
                    textPartA + sgx_keyLay(sgx_keyLayout)\key[k]\chr[mode]
                    textpos + Len( sgx_keyLay(sgx_keyLayout)\key[k]\chr[mode] )
                    If sgx_keyLay(sgx_keyLayout)\key[k]\delaySwitch = 0
                      sgx_keyLay(sgx_keyLayout)\key[k]\timer = ems + sgx_keystrokeDelay_1
                      sgx_keyLay(sgx_keyLayout)\key[k]\delaySwitch = 1
                    ElseIf sgx_keyLay(sgx_keyLayout)\key[k]\delaySwitch = 1
                      sgx_keyLay(sgx_keyLayout)\key[k]\timer = ems + sgx_keystrokeDelay_2
                    EndIf
                  EndIf
                  ;}
                EndIf;
          
          
                ;[-delay-independent-]
                
                ;{ HOME
                If sgx_keyLay(sgx_keyLayout)\key[k]\pbid = #PB_Key_Home
                  If cntrl=1
                    textpos=0
                  EndIf
                  While textpos>0  And  Mid(text,textpos,1) <> #LF$
                    textpos-1
                  Wend
                  textPartA = Mid(text,1,textpos)
                  textPartB = Mid(text,textpos+1,Len(text))
                  ;}
                  
                ;{ END
                ElseIf sgx_keyLay(sgx_keyLayout)\key[k]\pbid = #PB_Key_End
                  textpos = FindString(text,#LF$,textpos+1)-1
                  If textpos=-1 Or cntrl=1
                    textpos=Len(text)
                  EndIf
                  textPartA = Mid(text,1,textpos)
                  textPartB = Mid(text,textpos+1,Len(text))
                EndIf ;}
                
                
                If text <> textPartA+textPartB
                  text = textPartA+textPartB
                  sgx_keyboardEdit_TextModified = #True
                EndIf
                
              EndIf
            Next;k
            
          EndIf;KeyPushed_ALL
          
          
          If sgx_keyboardEdit_lastTextPos <> textpos
            sgx_keyboardEdit_TextModified = #True
            sgx_keyboardEdit_lastTextPos = textpos
          EndIf
          ProcedureReturn (textPartA + textPartB)
          
          EnableDebugger
        EndProcedure
        
        
        ; init defaults
        sgxSetKeyboardLayout()

;}
;  <<<<<<<<<<<<<<<<<<<<<<<< merging end  sgx_misc.pbi

; XIncludeFile "sgx_lowbudget.pbi"
;{ >>>>>>>>>>>>>>>>>>>>>>>> merging file sgx_lowbudget.pbi

        
        ; created by lowbudget_toDataSection
        DataSection
          sgx_lowbudget_png_start:
          Data.b $89,$50,$4E,$47,$0D,$0A,$1A,$0A,$00,$00,$00,$0D,$49,$48,$44,$52,$00,$00,$00,$1B
          Data.b $00,$00,$00,$20,$08,$06,$00,$00,$00,$E3,$69,$03,$3B,$00,$00,$00,$06,$62,$4B,$47
          Data.b $44,$00,$FF,$00,$FF,$00,$FF,$A0,$BD,$A7,$93,$00,$00,$00,$09,$70,$48,$59,$73,$00
          Data.b $00,$0E,$C4,$00,$00,$0E,$C4,$01,$95,$2B,$0E,$1B,$00,$00,$00,$07,$74,$49,$4D,$45
          Data.b $07,$E0,$05,$0A,$0C,$04,$06,$F9,$D8,$47,$63,$00,$00,$02,$9E,$49,$44,$41,$54,$48
          Data.b $C7,$C5,$96,$B1,$4A,$EC,$40,$14,$86,$BF,$B3,$AE,$DD,$BE,$41,$1E,$60,$F3,$00,$F7
          Data.b $11,$B6,$13,$21,$85,$58,$09,$BA,$AE,$36,$82,$8D,$B2,$A0,$85,$22,$A8,$20,$B2,$95
          Data.b $85,$A2,$16,$01,$5B,$15,$B2,$8D,$55,$F2,$10,$56,$9A,$46,$6C,$44,$58,$10,$2D,$16
          Data.b $14,$9B,$73,$AB,$99,$3B,$89,$D9,$5D,$8B,$90,$3B,$CD,$24,$33,$E7,$CC,$37,$E7,$9C
          Data.b $3F,$93,$11,$55,$55,$2A,$6A,$35,$00,$11,$91,$E3,$E3,$63,$00,$CE,$CF,$CF,$6D,$2F
          Data.b $22,$62,$0C,$45,$44,$2E,$2F,$2F,$01,$E8,$F7,$FB,$B6,$CF,$DB,$6C,$6F,$6F,$03,$60
          Data.b $FA,$AD,$AD,$2D,$6B,$53,$07,$38,$3D,$3D,$D5,$EF,$EF,$6F,$00,$5E,$5F,$5F,$6D,$DF
          Data.b $E9,$74,$6C,$D4,$D7,$D7,$D7,$FA,$F5,$F5,$05,$40,$9A,$A6,$B6,$5F,$5E,$5E,$B6,$36
          Data.b $9B,$9B,$9B,$3A,$18,$0C,$00,$78,$7F,$7F,$07,$E0,$F3,$F3,$93,$FD,$FD,$7D,$05,$90
          Data.b $CA,$D3,$78,$76,$76,$C6,$CE,$CE,$4E,$66,$62,$65,$65,$85,$F9,$F9,$79,$FB,$7E,$73
          Data.b $73,$C3,$E1,$E1,$61,$C6,$A6,$D3,$E9,$30,$3B,$3B,$6B,$DF,$37,$36,$36,$58,$5D,$5D
          Data.b $CD,$D8,$9C,$9C,$9C,$70,$70,$70,$C0,$FF,$89,$EC,$EA,$EA,$8A,$DD,$DD,$5D,$00,$96
          Data.b $96,$96,$00,$58,$5B,$5B,$63,$6E,$6E,$CE,$1A,$26,$49,$C2,$D1,$D1,$11,$00,$0B,$0B
          Data.b $0B,$00,$AC,$AF,$AF,$33,$33,$33,$63,$6D,$BA,$DD,$2E,$7B,$7B,$7B,$98,$6C,$01,$84
          Data.b $61,$48,$AF,$D7,$FB,$07,$5B,$5C,$5C,$94,$E7,$E7,$67,$00,$A6,$A6,$A6,$00,$18,$0C
          Data.b $06,$DC,$DE,$DE,$5A,$A5,$B5,$5A,$2D,$B9,$BF,$BF,$07,$60,$7A,$7A,$1A,$80,$A7,$A7
          Data.b $27,$EE,$EE,$EE,$AC,$4D,$AF,$D7,$93,$7A,$BD,$0E,$C0,$DB,$DB,$1B,$00,$1F,$1F,$1F
          Data.b $74,$BB,$5D,$A9,$3C,$8D,$18,$58,$18,$86,$1A,$86,$E1,$58,$B0,$99,$37,$3E,$71,$1C
          Data.b $6B,$1C,$C7,$63,$7D,$CC,$BC,$AA,$6A,$CD,$2C,$E2,$FB,$3E,$BE,$EF,$33,$0A,$68,$6C
          Data.b $CC,$7C,$1C,$C7,$EA,$79,$1E,$9E,$E7,$31,$0A,$68,$6C,$CC,$BC,$A8,$AA,$8A,$88,$98
          Data.b $C5,$00,$1E,$1F,$1F,$69,$B7,$DB,$E2,$46,$E3,$FB,$BE,$1D,$37,$3E,$66,$31,$80,$97
          Data.b $97,$17,$5A,$AD,$96,$B8,$D1,$78,$9E,$67,$C7,$55,$55,$AD,$A3,$BB,$7B,$03,$34,$90
          Data.b $FC,$06,$5C,$9F,$3C,$D0,$40,$F2,$1B,$F8,$01,$CB,$03,$5D,$A8,$01,$E5,$61,$79,$A0
          Data.b $0B,$35,$A0,$4C,$CD,$26,$35,$17,$F4,$DB,$E6,$82,$32,$1F,$75,$51,$54,$6E,$1A,$C7
          Data.b $A9,$74,$54,$1A,$8B,$44,$53,$1B,$05,$6A,$B7,$DB,$32,$09,$58,$24,$90,$71,$C0,$1F
          Data.b $69,$74,$85,$F0,$1B,$60,$5E,$08,$E3,$80,$B5,$22,$51,$E4,$EB,$55,$04,$CC,$8B,$22
          Data.b $5F,$AF,$22,$60,$6D,$54,$54,$A3,$80,$E3,$A2,$1A,$05,$2C,$3C,$AE,$26,$29,$AC,$E8
          Data.b $B8,$9A,$E4,$E3,$1E,$57,$D5,$FF,$CF,$92,$24,$B1,$97,$17,$B7,$F5,$FB,$7D,$92,$24
          Data.b $29,$17,$36,$1C,$0E,$69,$36,$9B,$44,$51,$64,$A3,$8C,$A2,$48,$9B,$CD,$26,$C3,$E1
          Data.b $B0,$34,$98,$3D,$AE,$CC,$E2,$E6,$E6,$64,$9E,$83,$20,$28,$2D,$D5,$99,$B3,$D1,$00
          Data.b $CD,$35,$2D,$08,$02,$71,$05,$51,$4A,$1A,$2B,$15,$88,$1B,$55,$9A,$A6,$A4,$69,$FA
          Data.b $A3,$86,$A5,$5D,$0B,$A2,$28,$D2,$87,$87,$07,$CD,$0B,$C4,$8C,$95,$D5,$6A,$00,$8D
          Data.b $46,$23,$53,$23,$80,$20,$08,$24,$4D,$53,$1A,$8D,$46,$B9,$6A,$AC,$AC,$66,$17,$17
          Data.b $7F,$2A,$13,$C8,$5F,$9A,$56,$53,$7B,$87,$1E,$BF,$C4,$00,$00,$00,$00,$49,$45,$4E
          Data.b $44,$AE,$42,$60,$82
          sgx_lowbudget_png_end:
        EndDataSection
        

;}
;  <<<<<<<<<<<<<<<<<<<<<<<< merging end  sgx_lowbudget.pbi

; XIncludeFile "sgx_design.pbi"
;{ >>>>>>>>>>>>>>>>>>>>>>>> merging file sgx_design.pbi

        
        
        
        
        ;** sgx_RemakeDesignImage
        ;* frees an design image (in case it is a valid one) and recreates it with the specified dimensions
        ;** ._img : index for the *design\img[] array
        ;** ._w   : new width
        ;** ._w   : new height
        Macro sgx_RemakeDesignImage(_img, _w, _h)
          sgx_FreeImageChecked(*design\img[_img]\nr)
          *design\img[_img]\nr = CreateImage(#PB_Any,_w,_h, 32)
          *design\img[_img]\w = _w
          *design\img[_img]\h = _h
        EndMacro
        
        
        ; Macro Save_(_img)
        ;   If IsImage(*design\img[_img]\nr)
        ;     SaveImage(*design\img[_img]\nr, RSet(Str(_img),2,"0") +"_"+ Mid( M_quote#_img#M_quote+".bmp", 11, 999))
        ;   ElseIf IsSprite(*design\img[_img]\nr)
        ;     SaveSprite(*design\img[_img]\nr, RSet(Str(_img),2,"0") +"_"+ Mid( M_quote#_img#M_quote+".bmp", 11, 999))
        ;   EndIf
        ; EndMacro
        
        
        ; -------------------------------------- init
        
        
        ;** sgx_MakePalette
        ;* creates a color pallete for the design derived from a basecolor. _
        ;* basically an array of some colors derived from a base color and to be used as text color when hovered or for thin borders of text elements etc. _
        ;* called by <a href=#f_sgxrefreshdesign>sgxRefreshDesign</a>
        ;** .*design   : the design to create the pallete for
        ;** .basecolor : the base color to derive the pallete from
        Procedure sgx_MakePalette(*design.sgxS_Design, basecolor)
          Protected i
          
          ; search for '\c[' to see where colors are used
          
          *design\basecolor = basecolor
          *design\c[00] = sgx_fade(*design\basecolor,-500)
          *design\c[01] = sgx_fade(*design\basecolor,-700)
          *design\c[02] = sgx_fade(*design\basecolor,-300)
          *design\c[03] = sgx_fade(*design\basecolor,-200)
          *design\c[04] = sgx_fade(*design\basecolor,-400)
          *design\c[05] = sgx_fade(*design\basecolor,-150)
          *design\c[06] = sgx_fade(*design\basecolor,-050)
          
          *design\c[07] = *design\basecolor
          
          *design\c[08] = sgx_fade(*design\basecolor, 050)
          *design\c[09] = sgx_fade(*design\basecolor, 150)
          *design\c[10] = sgx_fade(*design\basecolor, 200)
          *design\c[11] = sgx_fade(*design\basecolor, 300)
          *design\c[12] = sgx_fade(*design\basecolor, 400)
          *design\c[13] = sgx_fade(*design\basecolor, 500)
          *design\c[14] = sgx_fade(*design\basecolor, 600)
          *design\c[15] = sgx_fade(*design\basecolor, 300)
          
          *design\c[16] = sgx_fade(*design\basecolor,-600) ; table grid color 1 normal
          *design\c[17] = sgx_fade(*design\basecolor,-300) ; table grid color 2 normal
          *design\c[18] = sgx_fade(*design\basecolor,-600) ; table grid color 1 hover
          *design\c[19] = sgx_fade(*design\basecolor,-350) ; table grid color 2 hover
          
          ; redo zero-fades (avoid black)
          For i=0 To 15
            If RGB(Red(*design\c[i]),Green(*design\c[i]),Blue(*design\c[i])) = $000000
              *design\c[i] = RGBA(Red(0),Green(0),Blue(1), Alpha(*design\basecolor))
            EndIf
          Next
          
          ;save pallette (debugging)
          If 00
            Protected h = 30, w = 400
            Protected pal = CreateSprite(#PB_Any,w,20*h+20, #PB_Sprite_AlphaBlending)
            Protected font = LoadFont(#PB_Any, "courier",8)
            Protected color, y
            TransparentSpriteColor(pal, RGB(0,0,0))
            StopDrawing()
            StartDrawing(SpriteOutput(pal))
              DrawingMode(#PB_2DDrawing_AllChannels)
              Box(0,0,SpriteWidth(pal),SpriteHeight(pal),RGBA(0,0,0,0))
              Box(100,0,20,SpriteHeight(pal),RGBA($00,$00,$00,$ff));black
              Box(120,0,20,SpriteHeight(pal),RGBA($77,$77,$77,$ff));gray
              Box(140,0,20,SpriteHeight(pal),RGBA($ff,$ff,$ff,$ff));white
              Box(160,0,20,SpriteHeight(pal),RGBA($ff,$00,$00,$ff));red
              Box(180,0,20,SpriteHeight(pal),RGBA($00,$ff,$00,$ff));green
              Box(200,0,20,SpriteHeight(pal),RGBA($00,$00,$ff,$ff));blue
              For i=0 To 19
                y = i*h+20
                color = *design\c[i]
                DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Transparent)
                Box(1,y+1,w-2,h-2, color)
                DrawingFont(FontID(font))
                DrawText(20,y+2, Str(i)+" : $"+Hex(color)+" / RGBA("+Red(color) + ","+Green(color)+","+Blue(color)+","+Alpha(color)+")" ,RGBA($ff,$ff,$ff,$ff))
                DrawingMode(#PB_2DDrawing_AllChannels | #PB_2DDrawing_Outlined)
                Box(3,y+3,h-7,h-7, RGBA(Red(color),Green(color),Blue(color),$ff))
              Next
              DrawingMode(#PB_2DDrawing_AllChannels | #PB_2DDrawing_Outlined)
              ;Debug SpriteWidth(pal)
              ;Debug SpriteHeight(pal)
              Box(0,0,SpriteWidth(pal)-1,SpriteHeight(pal)-1,RGBA($ff,$66,$66,$ff))
            StopDrawing()
            UsePNGImageEncoder()
            UsePNGImageEncoder()
            Define sav
            sav = SaveSprite(pal,"palette_" + *design\name + ".png", #PB_ImagePlugin_PNG)
            ;Debug sav
            FreeSprite(pal)
            ;RunProgram("palette_" + *design\name + ".png")
            ;Delay(5000)
            End
          EndIf
        EndProcedure
        
        
        ;** sgxLoadDesign:
        ;* loads a design. _
        ;* returns the loaded design or 0 if loading fails. _
        ;* you don't necessarily need the return value, but you can store it for later use with <a href=#f_sgxsetdesign>sgxSetDesign</a> in which case an integer variable is sufficient. _
        ;* if you want to access structure fields of the design you should use a pointer of type <a href=#s_sgxs_design>sgxS_Design</a> as variable. _
        ;* if you call this procedure before you opened a screen, make use of <a href=#f_sgxsetdimensions>sgxSetDimensions</a>.
        ;** .design : the path / directory that contains the design images, or "lowbudget" (default) for an instant design
        ;** .basecolor : basecolor to be used for the design, or -1 (default) to use the color as found in the image data. 'lowbudget' will be fully colored. file-designs only support this for color of text and thin borders.
        Procedure sgxLoadDesign(design.s="lowbudget", basecolor=-1)
          Protected ret
          Protected i
          If Not ( design="lowbudget" Or FileSize(design)=-2 )
            sgx_err("can't load design. no such directory: " + design + " (currentDirectory: "+GetCurrentDirectory()+")")
            ProcedureReturn 0
          EndIf
          
          Static sgx_is_initialized = 0
          If Not sgx_is_initialized
            sgx_is_initialized = 1
            sgx_m\cursor=-1
            If sgx_dimensions\w = 0
              sgx_dimensions\w = ScreenWidth()
            EndIf
            If sgx_dimensions\h = 0
              sgx_dimensions\h = ScreenHeight()
            EndIf
            sgx_m\dlimit = 3
            
            ; default anim styles
            sgx_animStyle_minimizing = #sgxAnimZoom | #sgxAnimFade 
            sgx_animStyle_restoring  = #sgxAnimZoom | #sgxAnimFade
            sgx_animStyle_closing    = #sgxAnimSize | #sgxAnimFade
            sgx_animStyle_presenting = #sgxAnimSize | #sgxAnimFade
            
            ; call sgxExamine() once for other initializations.
            ; (sgx_ems needs to be non-zero for window animation to work, in case you would
            ; call sgxPresentWin() before the first call of sgxExamine() for example.)
            sgxExamine()
          EndIf
        
          AddElement(sgxDesign())
          sgxDesign()\name = design
          sgxDesign()\basecolor = basecolor
          sgxRefreshDesign(sgxDesign())
          *sgx_CurrentDesign = @sgxDesign()
          ret = @sgxDesign()
          
          ProcedureReturn ret
        EndProcedure
        
        
        ;** sgxRefreshDesign
        ;* refresh the design. i.e. after you changed the basecolor of the design. the images will be regenerated / reloaded from disk.
        ;** .*design : pointer to a valid design
        Procedure sgxRefreshDesign(*design.sgxS_Design)
          
          Protected designPath.s = *design\name + Mid("\",1,Bool(0 Or Right(*design\name,1)<>"\")) ; (ensure trailing backslash)
          Protected isLowbudget = Bool(designPath = "lowbudget\")
          Protected imgParts
          Protected imgPartsW
          Protected imgPartsH
          Protected noColor
          Protected p
          Protected m
          Protected isBlank
          Protected isBlankPrevious
          Protected i
          Protected k
          Protected w
          Protected h
          Protected r, g, b, a, mean
          Protected c1, c2, c3
          Protected bgColor
          Protected rnd.f
          Protected Dim edgesY(9)
          Protected Dim edgesX(17)
          Protected Dim edgesX2(3)
          Protected Dim edgesX3(1)
          
          ; reset design images
          For i=#sgx_DesignBackgroundNormal To #sgx_DesignMouseCursor
            sgx_FreeImageChecked(*design\img[i]\nr)
          Next
          
          
          
          
          
          
          If isLowbudget
          
            ; loading lowbudget parts image from file during sgx development, or from datasection otherwise.
            ; (use lowbudget_toDataSection.exe to update datasection from image file)
            If 01
              imgParts = CatchImage(#PB_Any, ?sgx_lowbudget_png_start, ?sgx_lowbudget_png_end - ?sgx_lowbudget_png_start)
            Else
              imgParts = LoadImage(#PB_Any, "lowbudget.png")
            EndIf
          
          ElseIf FileSize(designPath) = -2 ; is directory?
              imgParts = LoadImage(#PB_Any, designPath + "parts" + ".png")
              *design\img[#sgx_DesignBackgroundNormal]\nr = LoadImage(#PB_Any,designPath + "BackgroundNormal" + ".png")
              *design\img[#sgx_DesignBackgroundHover ]\nr = LoadImage(#PB_Any,designPath + "BackgroundHover"  + ".png")
              *design\img[#sgx_DesignMouseCursor]\nr = LoadImage(#PB_Any, designPath + "MouseCursor.png")
          EndIf
          
          ;{ -- parts
          
          If IsImage(imgParts)
            imgPartsW = ImageWidth(imgParts)
            imgPartsH = ImageHeight(imgParts)
            
            StartDrawing(ImageOutput(imgParts))
              DrawingMode(#PB_2DDrawing_AllChannels)
              
              ; get the separating color from top left pixel
              noColor = Point(0,0)
              
              ; if no basecolor is specified explicitly, then get the base color from bottom left pixel
              If *design\basecolor = -1
                ;Debug "..picking"
                *design\basecolor = Point(0,imgPartsH-1)
              EndIf
              
              ;sgx_debugColorC("noColor", noColor)
              ;sgx_debugColorC("*design\basecolor", *design\basecolor)
              
              If isLowbudget
                ; apply basecolor to every pixel that is not black and not the separating color
                For i=0 To imgPartsW-1
                  For k=0 To imgPartsH-1
                    c1 = Point(i,k)
                    If c1 And c1<>noColor
                      r = (c1&$000000ff)
                      g = (c1&$0000ff00) >> 8
                      b = (c1&$00ff0000) >> 16
                      a = (c1&$ff000000) >> 24
                      c2 =    (*design\basecolor&$000000ff)      * r/255
                      c2 | ( ((*design\basecolor&$0000ff00)>>8)  * g/255 ) << 8
                      c2 | ( ((*design\basecolor&$00ff0000)>>16) * b/255 ) << 16
                      c2 | ( ((*design\basecolor&$ff000000)>>24) * a/255 ) << 24
                      Plot(i,k, c2)
                    Else
                      Plot(i,k, c1)
                    EndIf
                  Next
                Next
              EndIf
              
              ; use the separating color to get the coordinates of the empty parts in the image (horizontal/vertical gaps/lines)
              ; (see section below to debug these)
              
              ;{ -- Y-gaps
              isBlank = 1
              m = 0
              
              For k=0 To imgPartsH-1
                isBlankPrevious = isBlank
                isBlank = 1
                For i=0 To imgPartsW-1
                  p = Point(i,k)
                  If p<>noColor
                    isBlank = 0
                    Break
                  EndIf
                Next
                If (isBlank<>isBlankPrevious)
                  If isBlank
                    edgesY(m) = k-1
                  ElseIf isBlankPrevious
                    edgesY(m) = k
                  EndIf
                  m+1
                  If m>9
                    Break
                  EndIf
                EndIf
              Next
              ;}
              ;  --
              
              ;{ -- X-gaps (parts)
              isBlank = 1
              m = 0
              
              For i=0 To imgPartsW-1
                isBlankPrevious = isBlank
                isBlank = 1
                For k=0 To edgesY(5)
                  p = Point(i,k)
                  If p<>noColor
                    isBlank = 0
                    Break
                  EndIf
                Next
                If (isBlank<>isBlankPrevious)
                  If isBlank
                    edgesX(m) = i-1
                  ElseIf isBlankPrevious
                    edgesX(m) = i
                  EndIf
                  m+1
                  If m>17
                    Break
                  EndIf
                EndIf
              Next
              ;}
              ;  --
              
              ;{ -- X2-gaps (closegadget)
              isBlank = 1
              m = 0
              
              For i=0 To imgPartsW-1
                isBlankPrevious = isBlank
                isBlank = 1
                For k=edgesY(5)+1 To edgesY(7)
                  p = Point(i,k)
                  If p<>noColor
                    isBlank = 0
                    Break
                  EndIf
                Next
                If (isBlank<>isBlankPrevious)
                  If isBlank
                    edgesX2(m) = i-1
                  ElseIf isBlankPrevious
                    edgesX2(m) = i
                  EndIf
                  m+1
                  If m>4
                    Break
                  EndIf
                EndIf
              Next
              ;}
              ;  --
              
              ;{ -- X3-gaps (checkboxChecked)
              isBlank = 1
              m = 0
              
              For i=0 To imgPartsW-1
                isBlankPrevious = isBlank
                isBlank = 1
                For k=edgesY(7)+1 To edgesY(9)
                  p = Point(i,k)
                  If p<>noColor
                    isBlank = 0
                    Break
                  EndIf
                Next
                If (isBlank<>isBlankPrevious)
                  If isBlank
                    edgesX3(m) = i-1
                  ElseIf isBlankPrevious
                    edgesX3(m) = i
                  EndIf
                  m+1
                  If m>1
                    Break
                  EndIf
                EndIf
              Next
              ;}
              ;  --
              
              ;{ debug edges
              If 00
                Protected _i
                Debug "edgesY: "
                For _i=0 To ArraySize(edgesY())
                  Debug Str(_i)+": y="+Str(edgesY(_i))
                Next
                Debug "edgesX: "
                For _i=0 To ArraySize(edgesX())
                  Debug Str(_i)+": x="+Str(edgesX(_i))
                Next
                Debug "edgesX2: "
                For _i=0 To ArraySize(edgesX2())
                  Debug Str(_i)+": x="+Str(edgesX2(_i))
                Next
                Debug "edgesX3: "
                For _i=0 To ArraySize(edgesX3())
                  Debug Str(_i)+": x="+Str(edgesX3(_i))
                Next
              EndIf
              ;}
              
            StopDrawing()
            
            ; create some other colors from the basecolor
            sgx_MakePalette(*design, *design\basecolor )
            
            ;{ -- grab parts
            i=0  :  k=0  :  *design\img[#sgx_DesignBorderNormalCorner1]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
            i=2  :  k=0  :  *design\img[#sgx_DesignBorderNormalHor1   ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
            i=4  :  k=0  :  *design\img[#sgx_DesignBorderNormalCorner2]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
            
            i=6  :  k=0  :  *design\img[#sgx_DesignBorderHoverCorner1 ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
            i=8  :  k=0  :  *design\img[#sgx_DesignBorderHoverHor1    ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
            i=10 :  k=0  :  *design\img[#sgx_DesignBorderHoverCorner2 ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
            
            i=12 :  k=0  :  *design\img[#sgx_DesignBorderPushedCorner1]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
            i=14 :  k=0  :  *design\img[#sgx_DesignBorderPushedHor1   ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
            i=16 :  k=0  :  *design\img[#sgx_DesignBorderPushedCorner2]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
            
        
            i=0  :  k=2  :  *design\img[#sgx_DesignBorderNormalVer1   ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
            i=4  :  k=2  :  *design\img[#sgx_DesignBorderNormalVer2   ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
            
            i=6  :  k=2  :  *design\img[#sgx_DesignBorderHoverVer1    ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
            i=10 :  k=2  :  *design\img[#sgx_DesignBorderHoverVer2    ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
            
            i=12 :  k=2  :  *design\img[#sgx_DesignBorderPushedVer1   ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
            i=16 :  k=2  :  *design\img[#sgx_DesignBorderPushedVer2   ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
            
        
            i=0  :  k=4  :  *design\img[#sgx_DesignBorderNormalCorner3]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
            i=2  :  k=4  :  *design\img[#sgx_DesignBorderNormalHor2   ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
            i=4  :  k=4  :  *design\img[#sgx_DesignBorderNormalCorner4]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
            
            i=6  :  k=4  :  *design\img[#sgx_DesignBorderHoverCorner3 ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
            i=8  :  k=4  :  *design\img[#sgx_DesignBorderHoverHor2    ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
            i=10 :  k=4  :  *design\img[#sgx_DesignBorderHoverCorner4 ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
            
            i=12 :  k=4  :  *design\img[#sgx_DesignBorderPushedCorner3]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
            i=14 :  k=4  :  *design\img[#sgx_DesignBorderPushedHor2   ]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
            i=16 :  k=4  :  *design\img[#sgx_DesignBorderPushedCorner4]\nr = GrabImage(imgParts,#PB_Any, edgesX(i),edgesY(k), edgesX(i+1)-edgesX(i)+1, edgesY(k+1)-edgesY(k)+1 )
            
            i=0  :  k=6  :  *design\img[#sgx_DesignCloseGadgetNormal  ]\nr = GrabImage(imgParts,#PB_Any, edgesX2(i),edgesY(k), edgesX2(i+1)-edgesX2(i)+1, edgesY(k+1)-edgesY(k)+1 )
            i=2  :  k=6  :  *design\img[#sgx_DesignCloseGadgetHover   ]\nr = GrabImage(imgParts,#PB_Any, edgesX2(i),edgesY(k), edgesX2(i+1)-edgesX2(i)+1, edgesY(k+1)-edgesY(k)+1 )
            
            i=0  :  k=8  :  *design\img[#sgx_DesignCheckboxChecked    ]\nr = GrabImage(imgParts,#PB_Any, edgesX3(i),edgesY(k), edgesX3(i+1)-edgesX3(i)+1, edgesY(k+1)-edgesY(k)+1 )
            
            ;}
            ;  --
            
            sgx_FreeImageChecked(imgParts)
            ;}
            
          Else
            sgx_err("invalid parts image (design: "+designPath +")")
          EndIf
          
          
          If isLowbudget
        
            ;{ -- (lowbudget) BackgroundNormal
            w = 80
            h = 50
            sgx_RemakeDesignImage(#sgx_DesignBackgroundNormal,w,h)
            StartDrawing(ImageOutput(*design\img[#sgx_DesignBackgroundNormal]\nr))
              DrawingMode(#PB_2DDrawing_AllChannels)
              bgColor = *design\c[0]
              Box(0,0,w,h,bgColor)
              If sgx_lowbudgetDistortion
                ;a = Alpha(bgColor)
                For i=0 To w-1
                  For k=0 To h-1
                    ; get a small variance of each component of the bgcolor and add the variances to the slightly 
                    ; reduced bgcolor basically something like 90% bgcolor + random(20% bgcolor) so we get random 
                    ; pixels in the range 90%-110% bgcolor brightness (and color distribution)
                    If sgx_lowbudgetDistortColors
                      ; random per component
                      r = Random(  Red(sgx_fade(bgColor,-1000 + sgx_lowbudgetDistortion * 1000 )))
                      g = Random(Green(sgx_fade(bgColor,-1000 + sgx_lowbudgetDistortion * 1000 )))
                      b = Random( Blue(sgx_fade(bgColor,-1000 + sgx_lowbudgetDistortion * 1000 )))
                    Else
                      ; keep component distribution
                      rnd = Random(1000) / 1000
                      r = rnd * Red  (sgx_fade(bgColor,-1000 + sgx_lowbudgetDistortion * 1000 ))
                      g = rnd * Green(sgx_fade(bgColor,-1000 + sgx_lowbudgetDistortion * 1000 ))
                      b = rnd * Blue (sgx_fade(bgColor,-1000 + sgx_lowbudgetDistortion * 1000 ))
                    EndIf
                    Plot(i,k, sgx_fade(bgColor,-(1000 * sgx_lowbudgetDistortion/2)) + r | g<<8 | b<<16 ); | a<<24)  alpha stays untouched
                  Next
                Next
              EndIf
            StopDrawing() ;}
            ;  --
            
            ;{ -- (lowbudget) BackgroundHover (copy and brighten up a bit)
            w = *design\img[#sgx_DesignBackgroundNormal]\w
            h = *design\img[#sgx_DesignBackgroundNormal]\h
            *design\img[#sgx_DesignBackgroundHover]\w = w
            *design\img[#sgx_DesignBackgroundHover]\h = h
            sgx_FreeImageChecked(*design\img[#sgx_DesignBackgroundHover]\nr)
            *design\img[#sgx_DesignBackgroundHover]\nr = CopyImage(*design\img[#sgx_DesignBackgroundNormal]\nr, #PB_Any)
            StartDrawing(ImageOutput(*design\img[#sgx_DesignBackgroundHover]\nr))
              DrawingMode(#PB_2DDrawing_AlphaBlend)
              Box(0,0, w,h, RGBA(255,255,255,5))
            StopDrawing()
            ;}
            ;  --
            
            ;{ -- (lowbudget) cursor 
            sgx_FreeImageChecked(*design\img[#sgx_DesignMouseCursor]\nr)
            *design\img[#sgx_DesignMouseCursor]\nr = CreateImage(#PB_Any, 32, 32, 32)
            StartDrawing(ImageOutput(*design\img[#sgx_DesignMouseCursor]\nr))
              DrawingMode(#PB_2DDrawing_AllChannels)
              Box(0,0,32,32, RGBA(0,0,0,0))
              DrawingMode(#PB_2DDrawing_AlphaBlend)
              c1 = RGBA(Red(*design\basecolor), Green(*design\basecolor), Blue(*design\basecolor), 0.3*Alpha(*design\basecolor))
              c2 = RGBA(Red(*design\basecolor), Green(*design\basecolor), Blue(*design\basecolor), 0.8*Alpha(*design\basecolor))
              c3 = RGBA(Red(*design\basecolor), Green(*design\basecolor), Blue(*design\basecolor), 0.1*Alpha(*design\basecolor))
              LineXY( 0, 1,  5,25, c1)
              LineXY( 6,25,  9,18, c1)
              LineXY(10,18, 16,17, c1)
              LineXY(16,16,  0, 0, c1)
              FillArea(4,6, -1, c2)
              LineXY( 1, 2,  8,17, c3)
            StopDrawing()
            
            ;}
            ;  --
            
          Else    
          
            ;{ -- BackgroundNormal
            If IsImage( *design\img[#sgx_DesignBackgroundNormal]\nr )
              *design\img[#sgx_DesignBackgroundNormal]\w = ImageWidth(*design\img[#sgx_DesignBackgroundNormal]\nr)
              *design\img[#sgx_DesignBackgroundNormal]\h = ImageHeight(*design\img[#sgx_DesignBackgroundNormal]\nr) ;}
            EndIf
            ;  --
          
            ;{ -- BackgroundHover
            If IsImage( *design\img[#sgx_DesignBackgroundHover]\nr )
              *design\img[#sgx_DesignBackgroundHover]\w = ImageWidth(*design\img[#sgx_DesignBackgroundHover]\nr)
              *design\img[#sgx_DesignBackgroundHover]\h = ImageHeight(*design\img[#sgx_DesignBackgroundHover]\nr) ;}
            EndIf
            ;  --
            
            ;{ -- cursor
            If IsImage( *design\img[#sgx_DesignMouseCursor]\nr )
              *design\img[#sgx_DesignMouseCursor]\w = ImageWidth(*design\img[#sgx_DesignMouseCursor]\nr)
              *design\img[#sgx_DesignMouseCursor]\h = ImageHeight(*design\img[#sgx_DesignMouseCursor]\nr) ;}
            EndIf
            ;  --
            
          EndIf
          
          ; get images width and height
          For i=#sgx_DesignBackgroundNormal To #sgx_DesignBorderPushedVer2
            *design\img[i]\w = ImageWidth(*design\img[i]\nr)
            *design\img[i]\h = ImageHeight(*design\img[i]\nr)
          Next
          
        EndProcedure

;}
;  <<<<<<<<<<<<<<<<<<<<<<<< merging end  sgx_design.pbi

; XIncludeFile "sgx_compose.pbi"
;{ >>>>>>>>>>>>>>>>>>>>>>>> merging file sgx_compose.pbi

        
        
        ;** sgx_PassStandardValues
        ;* macro to initalize element property default for most element type during creation. used by the compose functions like <a href=#f_sgxbutton>sgxButton</a> etc.
        Macro sgx_PassStandardValues()
          sgxSetParent(id, sgx_CurrentParent)
          sgx(id)\design=*sgx_CurrentDesign
          sgx(id)\alpha = sgx_defaultAlpha
          sgx(id)\xToParent=x
          sgx(id)\yToParent=y
          sgx(id)\x=sgx_GetGadgetX(id)
          sgx(id)\y=sgx_GetGadgetY(id)
          sgx(id)\w=w
          sgx(id)\h=h
          sgx(id)\origW=w
          sgx(id)\origH=h
          sgx(id)\show=1
          sgx(id)\x2 = sgx(id)\x
          sgx(id)\y2 = sgx(id)\y
          sgx(id)\w2 = sgx(id)\w
          sgx(id)\h2 = sgx(id)\h
          sgx(id)\minWidth = sgx_resizeGadgetSize
          sgx(id)\minHeight = sgx_resizeGadgetSize
        EndMacro
        
        
        ;** sgx_infoTail
        ;* used to add debug information to the text of sgx elements (must be activated in the code)
        Macro sgx_infoTail()
          ""
          ;" {"+Str(id)+"} "
          ;"["+Str(id)+"|"+Str(sgx(id)\parent)+"]"
          ;" #"+Str( sgx_fontID )
        EndMacro
        
        
        
        
        ; -------------------------------------- compose
        
        
        
        
        
        ; --------- ELEMENTS
        
        
        ;** sgxCursor
        ;* creates an sgx cursor element and returns it's ID. _
        ;* the notes on <a href=#f_sgxnew>sgxNew</a> apply to return value and parameter 'id'.
        ;** .id : ID for the new sgx element.
        Procedure sgxCursor(id)
          id = sgxNew(id)
          If id
            sgx(id)\parent=-1
            sgx(id)\design=*sgx_CurrentDesign
            sgx(id)\type=#sgxTypeCursor
            If sgx_BuildRefresh
              sgxRefresh(id)
            EndIf
            sgx(id)\show=1
            sgx_m\cursor=id
          EndIf
          ProcedureReturn id
        EndProcedure
        
        
        ;** sgxButton
        ;* creates an sgx button element and returns it's ID. _
        ;* the notes on <a href=#f_sgxnew>sgxNew</a> apply to return value and parameter 'id'. _
        ;* if a sprite is specified then you are responsible for the sprites lifetime, <a href=#f_sgxfree>sgxFree</a> will not
        ;* free the sprite. also you should not free the sprite before you have freed the sgxButton, otherwise the sprite
        ;* simply won't be displayed, but doesn't generate an error. _
        ;* a togglebutton's state will change when leftclicked, or will be set to pushed if it is part of a radiogroup.
        ;* rightclick behaviour can be configured via variable sgxRightclickUnset. _
        ;* you can use <a href=#f_sgxsetstate>sgxSetState</a> with toggle buttons. _
        ;** .id     : ID for the new sgx element.
        ;** .x      : x-position relative to parent element
        ;** .y      : y-position relative to parent element
        ;** .w      : width
        ;** .h      : height
        ;** .text   : caption. use #LF$ / Chr(10) for line breaks
        ;** .toggle : one of these values: _
        ;*            #sgxStateNone (default) for a standard button _
        ;*            #sgxStateOff for a toggle button _
        ;*            #sgxStateOn for a toggle button initialized as 'pushed' _
        ;** .sprNr  : an optional PB #sprite to be displayed on the button. you can then also use -1 for the w/h-parameters to adjust the button to the size of the sprite.
        ;** .x2     : x-offset for the additional sprite
        ;** .y2     : y-offset for the additional sprite
        Procedure sgxButton(id, x, y, w, h, text.s, toggle=#sgxStateNone , sprNr=-1, x2=5, y2=5)
          id = sgxNew(id)
          If id
            sgx_PassStandardValues()
            If toggle = #sgxStateOff  Or  toggle = #sgxStateOn
              sgx(id)\type=#sgxTypeButtonToggle
              If toggle = #sgxStateOn
                sgxSetState(id, 1)
              EndIf
            Else
              sgx(id)\type=#sgxTypeButton
            EndIf
            sgx(id)\text=text + sgx_infoTail()
            sgx(id)\textfontID=sgx_CurrentFontID
            If IsSprite(sprNr)
              sgx(id)\x2 = x2
              sgx(id)\y2 = y2
              sgx(id)\spr[#sgx_GadgetSprNormal2] = sprNr
              If sgx(id)\w = -1
                sgx(id)\w = SpriteWidth (sgx(id)\spr[#sgx_GadgetSprNormal2]) + 2 * sgx(id)\x2 + 1
              EndIf
              If sgx(id)\h = -1
                sgx(id)\h = SpriteHeight(sgx(id)\spr[#sgx_GadgetSprNormal2]) + 2 * sgx(id)\y2 + 1
              EndIf
            ElseIf w=-1 Or h=-1
              sgx_err("you have to specify a sprite to use automatic size adjustment for id " + id + ".")
            EndIf
            If sgx_BuildRefresh
              sgxRefresh(id)
            EndIf
            sgxMoveOnTop(id)
          EndIf
          ProcedureReturn id
        EndProcedure
        
        
        ;** sgxCheckbox
        ;* creates an sgx checkbox element and returns it's ID. _
        ;* the notes on <a href=#f_sgxnew>sgxNew</a> apply to return value and parameter 'id'. _
        ;* _
        ;* the specified position and size will define the knob. the overall size including the label will be stored
        ;* in the fields w2 and h2. the fields x2 and y2 store the display position of the elements sprite (the visual size including
        ;* the label) relative to the knob position. with #sgxAlignCheckboxTextLeft for example the label will start on screen
        ;* at (x+x2),(y+y2) where x2 will be negative and y2 will be negative if the text is larger than the knob height. _
        ;* _
        ;* a checkbox's state will change when leftclicked, or will be set to checked if it is part of a radiogroup.
        ;* rightclick behaviour can be configured via variable sgxRightclickUnset. _
        ;* you can use <a href=#f_sgxsetstate>sgxSetState</a> with this element. _
        ;** .id    : ID for the new sgx element.
        ;** .x     : knob x-position relative to parent element
        ;** .y     : knob y-position relative to parent element
        ;** .w     : knob width
        ;** .h     : knob height
        ;** .checked  : #sgxStateOff (default) for unchecked or #sgxStateOn for checked
        ;** .text  : label
        ;** .align : can be #sgxAlignCheckboxTextRight to have the label on the right (default), or #sgxAlignCheckboxTextLeft to have it on the left.
        Procedure sgxCheckbox(id, x, y, w, h, checked=#sgxStateOff, text.s="", align=#sgxAlignCheckboxTextRight)
          id = sgxNew(id)
          If id
            sgx_PassStandardValues()
            sgx(id)\type=#sgxTypeCheckbox
            sgx(id)\text=text + sgx_infoTail()
            sgx(id)\textalign=align
            sgx(id)\textfontID=sgx_CurrentFontID
            If checked = #sgxStateOn
              sgxSetState(id, 1)
            EndIf
            
            ; for checkboxes we ne need at least to update size and position according to the label
            ; so a possible parent table can rely on it. but we will skip the drawing in sgxRefresh
            ; in case sgx_BuildRefresh is set
            If sgx_BuildRefresh
              sgxRefresh(id)
            Else
              sgxRefresh(id, #sgx_refreshSkipTableCheckboxRedraw)
            EndIf
            
            sgxMoveOnTop(id)
          EndIf
          ProcedureReturn id
        EndProcedure
        
        
        ;** sgxScrollbar
        ;* creates an sgx scrollbar element and returns it's ID. _
        ;* the notes on <a href=#f_sgxnew>sgxNew</a> apply to return value and parameter 'id'. _
        ;* if height is greater than width, the scrollbar will be vertical, otherwise horizontal. _
        ;* you can use <a href=#f_sgxsetstate>sgxSetState</a> and <a href=#f_sgxsetscrollmax>sgxSetScrollMax</a> with this element. _
        ;* note that for example for a scrollbar length of 10 pixel and a max state value of 100 you will only be able to receive every 10th state. _
        ;* likewise for a scrollbar length of 100 pixel and a max state value of 10 you will only receive state changes every 10th pixel. _
        ;** .id    : ID for the new sgx element.
        ;** .x     : x-position relative to parent element
        ;** .y     : y-position relative to parent element
        ;** .w     : width (bar)
        ;** .h     : height (bar)
        ;** .w2    : width (knob)
        ;** .h2    : height (knob)
        ;** .max   : maximum possible scroll state (minimum is 0). this must be greater than 0 and will be corrected to 1 
        ;*           <span></span> automatically if necessary.
        ;** .state : initial state (default is 0)
        ;** .style : possible values (you can combine flags with '|' (bitwise OR)): _
        ;*           0 (scrollbar default) _
        ;*           #sgxStyleGrid: draws a pseudo-grid. _
        ;*           #sgxStyleReverse : the state 0 will be at the left for horizontal and at the top for vertical scrollbars per default. by specifying the reverse flag the 0 state will be at the other end of the scrollbar. _
        Procedure sgxScrollbar(id, x, y, w, h, w2, h2, max, state=0, style=0)
          id = sgxNew(id)
          If id
            sgx_PassStandardValues()
            sgx(id)\w2 = w2
            sgx(id)\h2 = h2
            If sgx(id)\h > sgx(id)\w
              sgx(id)\type=#sgxTypeScrollbarVer
              sgx(id)\width = h2
            Else
              sgx(id)\type=#sgxTypeScrollbarHor
              sgx(id)\width = w2
            EndIf
            sgx(id)\scrollpos=0
            If max < 1
              max = 1
            EndIf
            sgx(id)\max=max
            sgx(id)\style=style
            sgx(id)\textfontID=sgx_CurrentFontID
            sgxSetState(id, state)
            If sgx_BuildRefresh
              sgxRefresh(id)
            EndIf
            sgxMoveOnTop(id)
          EndIf
          ProcedureReturn id
        EndProcedure
        
        
        ;** sgxTrackbar
        ;* creates an sgx trackbar element and returns it's ID. _
        ;* the notes on <a href=#f_sgxnew>sgxNew</a> apply to return value and parameter 'id'. _
        ;* if height is greater than width, the trackbar will be vertical, otherwise horizontal. _
        ;* you can use <a href=#f_sgxsetstate>sgxSetState</a> and <a href=#f_sgxsetscrollmax>sgxSetScrollMax</a> with this element. _
        ;* note that for example for a trackbar length of 10 pixel and a max state value of 100 you will only be able to receive every 10th state. _
        ;* likewise for a trackbar length of 100 pixel and a max state value of 10 you will only receive state changes every 10th pixel. _
        ;** .id    : ID for the new sgx element.
        ;** .x     : x-position relative to parent element
        ;** .y     : y-position relative to parent element
        ;** .w     : width (bar)
        ;** .h     : height (bar)
        ;** .w2    : width (knob)
        ;** .h2    : height (knob)
        ;** .max   : maximum possible scroll state (minimum is 0). this must be greater than 0 and will be corrected to 1  
        ;*           <span></span> automatically if necessary.
        ;** .state : initial state (default is 0)
        ;** .style : possible values (you can combine flags with '|' (bitwise OR)): _
        ;*           0 _
        ;*           #sgxStyleGrid: draws a pseudo-grid (trackbar default) _
        ;*           #sgxStyleReverse : the state 0 will be at the left for horizontal and at the bottom for vertical trackbars per default. by specifying the reverse flag the 0 state will be at the other end of the trackbar. _
        Procedure sgxTrackbar(id, x, y, w, h, w2, h2, max, state=0, style=#sgxStyleGrid)
          id = sgxNew(id)
          If id
            sgx_PassStandardValues()
            sgx(id)\w2 = w2
            sgx(id)\h2 = h2
            If sgx(id)\h > sgx(id)\w
              sgx(id)\type=#sgxTypeTrackbarVer
              sgx(id)\width=h2
            Else
              sgx(id)\type=#sgxTypeTrackbarHor
              sgx(id)\width=w2
            EndIf
            sgx(id)\scrollpos=0
            If max < 1
              max = 1
            EndIf
            sgx(id)\max=max
            sgx(id)\style=style
            sgx(id)\textfontID=sgx_CurrentFontID
            sgxSetState(id, state)
            If sgx_BuildRefresh
              sgxRefresh(id)
            EndIf
            sgxMoveOnTop(id)
          EndIf
          ProcedureReturn id
        EndProcedure
        
        
        ;** sgxEdit
        ;* creates an sgx edit element and returns it's ID. _
        ;* the notes on <a href=#f_sgxnew>sgxNew</a> apply to return value and parameter 'id'. _
        ;* this is for user editable text only. for static labels use <a href=#f_sgxtext>sgxText</a>.
        ;** .id       : ID for the new sgx element.
        ;** .x        : x-position relative to parent element
        ;** .y        : y-position relative to parent element
        ;** .w        : width
        ;** .h        : height
        ;** .text     : inital text. use #LF$ / Chr(10) for line breaks
        ;** .maxLines : you can limit the amount of lines that can be entered by the user. use -1 (default) to not limit the amount of lines.
        Procedure sgxEdit(id, x, y, w, h, text.s="", maxLines=-1)
          Protected i
          id = sgxNew(id)
          If id
            sgx_PassStandardValues()
            sgx(id)\type=#sgxTypeEdit
            sgx(id)\textfontID=sgx_CurrentFontID
            sgx(id)\text=text + sgx_infoTail()
            sgx(id)\textpos=0
            sgx(id)\textalign=0
            sgx(id)\scrollpos=0
            sgx(id)\max=maxLines
            ExamineKeyboard()
            sgx(id)\text = sgx_keyboardEdit( sgx(id)\text , sgx(id)\textpos)
            sgx(id)\textpos = sgx_keyboardEdit_lastTextPos
            
            If sgx_BuildRefresh
              sgxRefresh(id)
            EndIf
            sgxMoveOnTop(id)
          EndIf
          ProcedureReturn id
        EndProcedure
        
        
        ;** sgxWin
        ;* creates an sgx window element and returns it's ID. _
        ;* the notes on <a href=#f_sgxnew>sgxNew</a> apply to return value and parameter 'id'. _
        ;* this element will be the parent for new sgx elements until you call <a href=#f_sgxcloselevel>sgxCloseLevel</a> or until
        ;* you create another parent element. _
        ;* _
        ;* a window's state can be changed via the functions
        ;* <a href=#f_sgxpresentwin>sgxPresentWin</a>, <a href=#f_sgxminimizewin>sgxMinimizeWin</a>, 
        ;* <a href=#f_sgxrestorewin>sgxRestoreWin</a>, <a href=#f_sgxclosewin>sgxCloseWin</a> and can be inquired by checking the
        ;* 'state' field for #sgxWinState.. values. _
        ;* a closed window does not exists anymore and thus has no state. use the macro <a href=#f_sgx>sgx()</a> without structure
        ;* field access to test the vailidty in that case. _
        ;* _
        ;* you can use <a href=#f_sgxpresentwin>sgxPresentWin</a> directly after <a href=#f_sgxcloselevel>sgxCloseLevel</a>
        ;* to present the window instead of showing it immeditately. _
        ;* if you want the window to be created hidden, use <a href=#f_sgxsetshow>sgxSetShow</a> (see notes about child visibility)
        ;* to hide the window after you finished the creation with <a href=#f_sgxcloselevel>sgxCloseLevel</a>. you can then use
        ;* <a href=#f_sgxpresentwin>sgxPresentWin</a> later to show the window. _
        ;** .id    : ID for the new sgx element.
        ;** .x     : x-position relative to parent element
        ;** .y     : y-position relative to parent element
        ;** .w     : width
        ;** .h     : height
        ;** .text  : caption in titlebar. no text will cause the window to have no titlebar. use a space for an empty titlebar.
        ;** .style : 0 (default) _
        ;*           #sgxStyleResizable to make the window resizable via mouse.
        Procedure sgxWin(id, x, y, w, h, text.s="", style=0)
          id = sgxNew(id)
          If id
            sgx_PassStandardValues()
            sgx(id)\type=#sgxTypeWin
            sgx(id)\text=text + sgx_infoTail()
            ;sgx(id)\style=style
            ;sgx(id)\textalign=align
            sgx(id)\textfontID=sgx_CurrentFontID
            If style & #sgxStyleResizable
              sgxSetResizable(id, #True)
            EndIf
            If sgx_BuildRefresh
              sgxRefresh(id)
            EndIf
            sgxMoveOnTop(id)
            sgxSetCurrentParent(id)
          EndIf
          ProcedureReturn id
        EndProcedure
        
        
        ;** sgxSpriteGadget
        ;* creates an sgx sprite container and returns it's ID. _
        ;* the notes on <a href=#f_sgxnew>sgxNew</a> apply to return value and parameter 'id'. _
        ;* you are responsible for the sprites lifetime, <a href=#f_sgxfree>sgxFree</a> will not free the sprite. also you should not free the sprite before you have freed the sgxSpriteGadget, otherwise the sprite simply won't be displayed, but doesn't generate an error. _
        ;* you can use <a href=#f_sgxsetstate>sgxSetState</a> with this element. _
        ;** .id    : ID for the new sgx element.
        ;** .x     : x-position relative to parent element
        ;** .y     : y-position relative to parent element
        ;** .sprNr : a valid PB Sprite number, or 0. you can change the sprite via <a href=#f_sgxsetstate>sgxSetState</a>.
        Procedure sgxSpriteGadget(id, x, y, sprNr)
          Protected w
          Protected h
          id = sgxNew(id)
          If id
            If IsSprite(sprNr)
              w = SpriteWidth(sprNr)  ; - w and h are needed in macro sgx_PassStandardValues()
              h = SpriteHeight(sprNr) ; /
            Else
              w = 0
              h = 0
            EndIf
            sgx_PassStandardValues()
            sgx(id)\type = #sgxTypeSpriteGadget
            If IsSprite(sprNr)
              sgx(id)\state = sprNr
            Else
              sgx(id)\state = 0
            EndIf
            sgx(id)\textfontID=sgx_CurrentFontID
            If sgx_BuildRefresh
              sgxRefresh(id)
            EndIf
            sgxMoveOnTop(id)
            ProcedureReturn id
          EndIf
          ProcedureReturn 0
        EndProcedure
        
        
        ;** sgxArea
        ;* creates an sgx area container and returns it's ID. _
        ;* the notes on <a href=#f_sgxnew>sgxNew</a> apply to return value and parameter 'id'. _
        ;* use sgx area elements to get mouse events without using any visuals. _
        ;* areas dont't move up in z-order when clicked as other types do but other elements may move in front of an area. you can use <a href=#f_sgxmoveontop>sgxMoveOnTop</a> to move it up manually if needed. _
        ;** .id    : ID for the new sgx element.
        ;** .x     : x-position relative to parent element
        ;** .y     : y-position relative to parent element
        ;** .w    : width
        ;** .h    : height
        Procedure sgxArea(id, x, y, w, h)
          id = sgxNew(id)
          If id
            sgx_PassStandardValues()
            sgx(id)\type = #sgxTypeArea
            If sgx_BuildRefresh
              sgxRefresh(id)
            EndIf
            sgxMoveOnTop(id)
            ProcedureReturn id
          EndIf
          ProcedureReturn 0
        EndProcedure
        
        
        ;** sgxTip
        ;* creates a tooltip for an sgx element.
        ;** .id   : ID of a valid and existing sgx element
        ;** .text : inital text of the tooltip. use #LF$ / Chr(10) for line breaks
        Procedure sgxTip(id, text.s)
          If sgx(id)
            If sgx(id)\tip = 0
              sgx(id)\tip = AllocateMemory(SizeOf(sgxS_Tip))
              If sgx(id)\tip = 0
                ProcedureReturn 0
              EndIf
            EndIf
            sgx(id)\tip\text = text
            ;sgx(id)\tip\timer = 0
            ;sgx(id)\tip\show = 0
            ;sgx(id)\tip\spr = -1
            If sgx_BuildRefresh
              sgxRefresh(id, #sgx_refreshOnlyTip)
            EndIf
            ProcedureReturn id
          EndIf
          ProcedureReturn 0
        EndProcedure
        
        
        ;** sgxCombo
        ;* creates an sgx combo element and returns it's ID. _
        ;* the notes on <a href=#f_sgxnew>sgxNew</a> apply to return value and parameter 'id'. _
        ;* you can use <a href=#f_sgxsetstate>sgxSetState</a> with this element. _
        ;** .id   : ID for the new sgx element.
        ;** .x    : x-position relative to parent element
        ;** .y    : y-position relative to parent element
        ;** .w    : width
        ;** .h    : height
        ;** .text : the possible entries separated by #LF$ / Chr(10)
        Procedure sgxCombo(id, x, y, w, h, text.s="")
          id = sgxNew(id)
          If id
            sgx_PassStandardValues()
            sgx(id)\type=#sgxTypeCombo
            sgx(id)\text=text + sgx_infoTail()
            sgx(id)\textfontID=sgx_CurrentFontID
            sgx(id)\state = 1
            If sgx_BuildRefresh
              sgxRefresh(id)
            EndIf
            sgxMoveOnTop(id)
          EndIf
          ProcedureReturn id
        EndProcedure
        
        
        ;** sgxText
        ;* creates an sgx text element and returns it's ID. _
        ;* the notes on <a href=#f_sgxnew>sgxNew</a> apply to return value and parameter 'id'. _
        ;* this is for static labels only. for user editable text use <a href=#f_sgxedit>sgxEdit</a>. _
        ;* take in consideration that changing the text via <a href=#f_sgxsettext>sgxSetText</a> will cause an element 
        ;* refresh, including drawing of design images as well as drawing of text onto the element's sprites, possibly also recreation 
        ;* of sprites if the size changes too. this will have an impact on performance, so don't take it lightly by changing it's
        ;* text every frame for example. _
        ;** .id        : ID for the new sgx element.
        ;** .x         : x-position relative to parent element
        ;** .y         : y-position relative to parent element
        ;** .w         : width. -1 (default) will adjust width to textwidth
        ;** .h         : height. -1 (default) will adjust height to textheigt
        ;** .text      : inital text. use #LF$ / Chr(10) for line breaks
        ;** .style     : with #sgxStyleThin the element will have a frame. #sgxStyleHover will cause it to have a mouseover effect. you can combine flags with '|' (bitwise OR), or use #sgxStyleHoverThin.
        ;** .margin    : space between text and border
        ;** .textalign : one of these values: _
        ;*               #sgxAlignTextLeft (Default) _
        ;*               #sgxAlignTextRight _
        ;*               #sgxAlignTextCenter _
        ;*               #sgxAlignTextJustifySpaces (text justified per word) _
        ;*               #sgxAlignTextJustifyCharacters (text justified per character)
        ;** .wordwrap  : #False (Default), #True for automatic word wrap. works only if width (parameter w) is not set to -1.
        Procedure sgxText(id, x, y, w=-1, h=-1, text.s="", style=0, margin=3, textalign=#sgxAlignTextLeft, wordwrap=#False)
          id = sgxNew(id)
          If id
            sgx_PassStandardValues()
            If w=-1
              sgx(id)\auto | #sgx_AutoW
            EndIf
            If h=-1
              sgx(id)\auto | #sgx_AutoH
            EndIf
            sgx(id)\type = #sgxTypeText
            sgx(id)\text=text + sgx_infoTail()
            sgx(id)\origText = sgx(id)\text
            sgx(id)\textfontID=sgx_CurrentFontID
            sgx(id)\textalign=textalign
            sgx(id)\style=style
            sgx(id)\margin=margin
            If wordwrap  And  Not (sgx(id)\auto & #sgx_AutoW)
              sgx(id)\auto | #sgx_AutoNL
            EndIf
            If sgx_BuildRefresh Or (sgx(id)\auto & (#sgx_AutoW|#sgx_AutoH)) ; bypass refresh deactivation to get proper w/h of element according to text
              sgxRefresh(id)
            EndIf
            sgxMoveOnTop(id)
          EndIf
          ProcedureReturn id
        EndProcedure
        
        
        ;** sgxTable
        ;* creates an sgx table element and returns it's ID. _
        ;* the notes on <a href=#f_sgxnew>sgxNew</a> apply to return value and parameter 'id'. _
        ;* this element will be the parent for new sgx elements until you call <a href=#f_sgxcloselevel>sgxCloseLevel</a> or 
        ;* until you create another parent element. _
        ;* use <a href=#f_sgxcellassign>sgxCellAssign</a> to place elements in the table, <a href=#f_sgxcellid>sgxCellId</a> and
        ;* <a href=#f_sgxcell>sgxCell</a> to access the cell elements. _
        ;* each column will be as wide as the widest element in it. _
        ;* each line will be as high as the highest element in it. _
        ;* the size of the table will adapt to the sum of column widths / line heights.
        ;* child elements of the table might not be completely refreshed until table creation is finished
        ;* with <a href=#f_sgxcloselevel>sgxCloseLevel</a>. _
        ;* _
        ;* the parameters 'columns' and 'lines' specify the amount, so cell indices will range from 0 to amount-1. it does not
        ;* necessarily hurt to oversize these dimensions if you're not shure how big you need the table to be later. each cell
        ;* will need 4 bytes of memory, no matter if assigned or left empty and each colum as well as each lines needs
        ;* additional 4 bytes of memory. _
        ;* _
        ;* big tables with a lot of elements, possibly also subtables and especially in combination with #sgxStyleFitCellW and/or 
        ;* #sgxStyleFitCellH can impact performance if cell elements need to be refreshed often since this could cause refresh of
        ;* other cell elements and the table itself. using #sgxStyleNoBG will also make a table refresh faster. _
        ;* tables are desinged to be a quick way for simple layouts and in some cases it might be better
        ;* to position, resize and refesh elements on you own without using a table.
        ;** .id      : ID for the new sgx table element.
        ;** .x       : x-position relative to parent element
        ;** .y       : y-position relative to parent element
        ;** .columns : amount of maximum columns (see notes above)
        ;** .lines   : amount of maximum lines (see notes above)
        ;** .margin  : space between cells
        ;** .style   : possible values (you can combine flags with '|' (bitwise OR)): _
        ;*             0 _
        ;*             #sgxStyleGridV    : draw vertical grid lines _
        ;*             #sgxStyleGridH    : draw horizontal grid lines _
        ;*             #sgxStyleGrid (default) : draw vertical and horizontal grid lines _
        ;*             #sgxStyleFitCellW : width of cell elements will be overwritten according to column width except for vertical scrollbars/trackbars, checkboxes and sgxText elements with automatic width adjustment. _
        ;*             #sgxStyleFitCellH : height of cell elements will be overwritten according to line height except for horizontal scrollbars/trackbars, checkboxes and sgxText elements with automatic height adjustment. _
        ;*             #sgxStyleNoBG     : no background / only cells will be visible. (this will disable the grid flags) _
        Procedure sgxTable(id, x, y, columns, lines, margin, style=#sgxStyleGrid)
          id = sgxNew(id)
          If id
            ;If margin < 1
            ;  margin = 1
            ;EndIf
            Protected w = 1
            Protected h = 1
            sgx_PassStandardValues()
            sgx(id)\margin = margin
            sgx(id)\type=#sgxTypeTable
            sgx(id)\text="";text + sgx_infoTail()
            sgx(id)\textfontID=sgx_CurrentFontID
            sgx(id)\style=style
            sgx(id)\table = AllocateMemory(SizeOf(sgxS_Table))
            sgx(id)\table\w = columns
            sgx(id)\table\h = lines
            sgx(id)\table\cells = AllocateMemory(columns*lines*SizeOf(Long))
            sgx(id)\table\gridx = AllocateMemory(columns*SizeOf(Long))
            sgx(id)\table\gridy = AllocateMemory(lines*SizeOf(Long))
            If style & #sgxStyleNoBG
              sgxSetShow(id, #False, #False)
            EndIf
            
            ; from now on only initialize child elements on creation without refresh
            ; and refresh them later during table refresh in sgxCloseLevel, since
            ; table and child dimensions can influence each other by #sgxStyleFitCellW/H
            ; and we want avoid double refreshes
            sgx_PushBuildRefresh(0)
            
            sgxMoveOnTop(id)
            sgxSetCurrentParent(id)
          EndIf
          ProcedureReturn id
        EndProcedure

;}
;  <<<<<<<<<<<<<<<<<<<<<<<< merging end  sgx_compose.pbi

; XIncludeFile "sgx_draw.pbi"
;{ >>>>>>>>>>>>>>>>>>>>>>>> merging file sgx_draw.pbi

        
        
        ;** sgx_prepareSprite
        ;* creates an element sprite it not done yet or frees and recreates the sprite if the size is different.
        ;** .spr : variable that contains the pb sprite number.
        ;** .w   : new width
        ;** .h   : new height
        ;** .id  : ID of the sgx element that is shown in a debugger error if the sprite creation fails.
        Macro sgx_prepareSprite(spr, w, h, id)
          ; values of 0 are ignored, handled as unititialized and are not freed (sprite number 0 might be a valid sprite outside of sgx)
          If Not (spr)  Or  Not IsSprite(spr)  Or  (w) <> SpriteWidth(spr)  Or  (h) <> SpriteHeight(spr)
            sgx_FreeSpriteChecked(spr)
            spr = CreateSprite(#PB_Any, w, h, #PB_Sprite_AlphaBlending)
            If Not spr
              sgx_err("sprite creation failed with size "+Str(w)+", "+Str(h)+": sgx("+id+")")
            Else
              TransparentSpriteColor(spr, RGB(0,0,0))
            EndIf
          EndIf
        EndMacro
        
        
        
        ;** sgxRefreshAll
        ;* refresh all existing sgx elements, if you modified the design for instance.
        Procedure sgxRefreshAll()
          Protected id
          For id=0 To sgx_CURRENT_MAX_DYNAMIC_GADGETS
            sgx_SkipUnused(id)
            If sgx(id)
              sgxRefresh(id)
            EndIf
          Next
        EndProcedure
        
        
        ;** sgxRefresh
        ;* refresh an sgx element.
        ;** .id   : ID of the sgx element to be refreshed.
        ;** .mode : #sgx_refreshNormalWithTip           : normal refresh (including sgx-tip). _
        ;*          #sgx_refreshOnlyTip                 : only the sgx-tip will be refreshed. _
        ;*          #sgx_refreshNormalWithChildsOfTables: normal refresh, but includes childs of tables. _
        ;*          #sgx_refreshSkipTableCheckboxRedraw : skip redraw for checkboxes created in tables. _
        ;*          #sgx_refreshEditMouseclickCursor    : for edit elements: update the character textpos from the current cursor pixel position (as opposed to other way round). used when positioning cursor by mouseclick. _
        ;*          #sgx_refreshEditMousewheelScroll    : for edit elements: disable the correction of textalign/scrollpos if cursor is outside clientarea. used when scrolling by mousewheel.
        Procedure sgxRefresh(id, mode=#sgx_refreshNormalWithTip)
          Protected w, h
          Protected i, nbChars
          Protected i2
          Protected temp.s
          Protected char.s, text.s
          Protected lastwhitespace, linestart, inserted, maxLineWidth
          Protected textcursorX_prev, textcursorY_prev
          Protected sprW, sprH, xClip, yClip, wClip, hClip
          Protected wPrev, hPrev, w2Prev, h2Prev
          Protected NewList sizes.sgx_S_metrics(), _id_, doRefresh
          
          
          If sgx(id)
            
            AddElement(sgx_pendingEvents())
            sgx_pendingEvents()\id = id
            sgx_pendingEvents()\refreshed = 1
            
            ; (debug)
            ; Static indent.s, parent
            ; indent = ""
            ; parent = sgx(id)\parent
            ; While parent <> -1
            ;   indent + "    "
            ;   parent = sgx(parent)\parent
            ; Wend
            ; indent + "sgxRefresh("+id+", "+mode+")" + sgx_getTypeString(id)
            ; Debug indent
            
            wPrev = sgx(id)\w
            hPrev = sgx(id)\h
            w2Prev = sgx(id)\w2
            h2Prev = sgx(id)\h2
            
            ;- refresh tip
            If sgx(id)\tip
              sgx_DrawTip(id)
            EndIf
            
            sgx_DrawResizeGadget(id)
            
            If mode <> #sgx_refreshOnlyTip
              
              Select sgx(id)\type
                  
                  ;- refresh cursor
                Case #sgxTypeCursor ;{
                  sgx_DrawCursor(id, #sgx_GadgetSprNormal)
                  ;}
                  
                  ;- refresh button/togglebutton
                Case #sgxTypeButton, #sgxTypeButtonToggle ;{
                  StartDrawing(ScreenOutput())
                    DrawingFont(sgx(id)\textfontID)
                    sgx(id)\texth = TextHeight("X")
                  StopDrawing()
                  sgx_DrawButton(id, #sgx_GadgetSprNormal)
                  sgx_DrawButton(id, #sgx_GadgetSprHover)
                  sgx_DrawButton(id, #sgx_GadgetSprPushed)
                  sgx_DrawButton(id, #sgx_GadgetSprHoverPushed)
                  ;}
                  
                  ;- refresh checkbox
                Case #sgxTypeCheckbox ;{
                  
                  ; get text width and line height
                  StartDrawing(ScreenOutput())
                    DrawingFont(sgx(id)\textfontID)
                    sgx(id)\textw = TextWidth(sgx(id)\text)
                    sgx(id)\texth = TextHeight("X")
                  StopDrawing()
                  
                  ; overall width = knob width + label width
                  sgx(id)\w2 = sgx(id)\w + sgx(id)\textw+5
                  ; overall height = knob height or label height
                  sgx(id)\h2 = sgx_max(sgx(id)\h, sgx(id)\texth)
                  
                  ; knob-relative display x-position..
                  If sgx(id)\textalign = #sgxAlignCheckboxTextLeft
                    ; ..is shifted 'labelwidth' the left
                    sgx(id)\x2 = -(sgx(id)\textw+5)
                  Else
                    ; ..is same as knob position
                    sgx(id)\x2 = 0
                  EndIf
                  
                  ; knob-relative display y-position will be 0 (if h=h2) or negative (if text is larger then knob height)
                  sgx(id)\y2 = sgx(id)\h/2 - sgx(id)\h2/2
                  
                  ; skip redraw for now for checkboxes created in tables (for checkboxes created during table creation
                  ; we only got here to update the metrics above. the table will trigger the elements redraw later)
                  If Not mode=#sgx_refreshSkipTableCheckboxRedraw
                    sgx_DrawCheckbox(id, #sgx_GadgetSprNormal)
                    sgx_DrawCheckbox(id, #sgx_GadgetSprHover)
                    sgx_DrawCheckbox(id, #sgx_GadgetSprChecked)
                  EndIf
                  ;}
                  
                  ;- refresh scrollbar/trackbar
                Case #sgxTypeScrollbarVer, #sgxTypeScrollbarHor, #sgxTypeTrackbarVer, #sgxTypeTrackbarHor ;{
                  If sgx(id)\h > sgx(id)\w ; ### VER |
                    If sgx(id)\type & #sgxTypeScrollbar
                      sgx(id)\type=#sgxTypeScrollbarVer
                    ElseIf sgx(id)\type & #sgxTypeTrackbar
                      sgx(id)\type=#sgxTypeTrackbarVer
                    EndIf
                    sgx(id)\width=sgx(id)\h2
                    If sgx(id)\h < sgx(id)\width+1
                      sgx(id)\h = sgx(id)\width+1
                    EndIf
                    sgx(id)\x2 = (sgx(id)\w-sgx(id)\w2)/2
                    ;sgx(id)\y2 = (sgx(id)\h-sgx(id)\h2)/2
                  Else                     ; ### HOR --
                    If sgx(id)\type & #sgxTypeScrollbar
                      sgx(id)\type=#sgxTypeScrollbarHor
                    ElseIf sgx(id)\type & #sgxTypeTrackbar
                      sgx(id)\type=#sgxTypeTrackbarHor
                    EndIf
                    sgx(id)\width = sgx(id)\w2
                    If sgx(id)\w < sgx(id)\width+1
                      sgx(id)\w = sgx(id)\width+1
                    EndIf
                    ;sgx(id)\x2 = (sgx(id)\w-sgx(id)\w2)/2
                    sgx(id)\y2 = (sgx(id)\h-sgx(id)\h2)/2
                  EndIf
                  ;sgx(id)\scrollpos=0;sgx(id)\width/2
                  ;sgx_SetScrollPos(id,sgx(id)\scrollpos)
                  ;sgx(id)\state=0
                  sgx_DrawScrollbar(id, #sgx_GadgetSprNormal)  ; -- bar
                  sgx_DrawScrollbar(id, #sgx_GadgetSprHover)   ; /
                  sgx_DrawScrollbar(id, #sgx_GadgetSprNormal2) ; -- knob
                  sgx_DrawScrollbar(id, #sgx_GadgetSprHover2)  ; /
                  ;}
                  
                  ;- refresh edit
                Case #sgxTypeEdit ;{
                  ; get 'client area'
                  sgx(id)\x2 = sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w+2
                  sgx(id)\y2 = sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h+2
                  sgx(id)\w2 = sgx(id)\w - sgx(id)\x2 - sgx(id)\design\img[#sgx_DesignBorderNormalVer2]\w-2
                  sgx(id)\h2 = sgx(id)\h - sgx(id)\y2 - sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h-2
                  If sgx(id)\w2 < 1
                    sgx(id)\w2 = 1
                  EndIf
                  If sgx(id)\h2 < 1
                    sgx(id)\h2 = 1
                  EndIf
                  Static moveTextPos, scrollToCursor
                  
                  moveTextPos = #False   ; while normal typing / cursor keys
                  scrollToCursor = #True ; while normal typing / cursor keys
                  
                  If mode = #sgx_refreshEditMouseclickCursor
                    moveTextPos = #True ; while cursor positioning by mouseclick
                  ElseIf mode = #sgx_refreshEditMousewheelScroll
                    scrollToCursor = #False ; disable scrollToCursor while scrolling by mousewheel
                  EndIf
                  ; update metrics
                  sgx_DrawEdit(id, 0, #True, moveTextPos, scrollToCursor)
                  
                  ; redraw (while focused only redraw the hover sprite)
                  sgx_DrawEdit(id, #sgx_GadgetSprHover)
                  If sgx_editing <> id
                    sgx_DrawEdit(id, #sgx_GadgetSprNormal)
                    sgx_DrawEdit(id, #sgx_GadgetSprTextCursor)
                  EndIf
                  
                  ; clip textcursor sprite if necessary
                  
                  ; reset clipping to get proper sprite size
                  ClipSprite(sgx(id)\spr[#sgx_GadgetSprTextCursor], #PB_Default, #PB_Default, #PB_Default, #PB_Default)
                  sprW = SpriteWidth (sgx(id)\spr[#sgx_GadgetSprTextCursor])
                  sprH = SpriteHeight(sgx(id)\spr[#sgx_GadgetSprTextCursor])
                  
                  xClip = 0
                  yClip = 0
                  wClip = sprW
                  hClip = sprH
                  If sgx(id)\textcursor\x < 0
                    xClip = -sgx(id)\textcursor\x
                  EndIf
                  If sgx(id)\textcursor\x > (sgx(id)\w2 - sprW)
                    wClip = sprW - (sgx(id)\textcursor\x - (sgx(id)\w2 - sprw))
                  EndIf
                  If sgx(id)\textcursor\y < 0
                    yClip = -sgx(id)\textcursor\y
                  EndIf
                  If sgx(id)\textcursor\y > (sgx(id)\h2 - sprH)
                    hClip = sprH - (sgx(id)\textcursor\y - (sgx(id)\h2 - sprH))
                  EndIf
                  ClipSprite(sgx(id)\spr[#sgx_GadgetSprTextCursor], xClip, yClip, wClip, hClip)
                  
                  ; store offset for proper sprite positioning
                  sgx_textCursorClippingOffset\x = xClip
                  sgx_textCursorClippingOffset\y = yClip
                  ;}
                  
                  ;- refresh window
                Case #sgxTypeWin ;{
                  StartDrawing(ScreenOutput())
                    DrawingFont(sgx(id)\textfontID)
                    sgx(id)\textw = TextWidth(sgx(id)\text)
                    sgx(id)\texth = TextHeight(sgx(id)\text)
                  StopDrawing()
                  
                  ; get titlebar height
                  If sgx(id)\texth
                    sgx(id)\width = sgx(id)\texth + 2 + sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h + sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h
                  Else
                    sgx(id)\width = 0
                  EndIf
                  
                  ; close gadget (keep that order; second relies on first!)
                  sgx_DrawWin(id, #sgx_GadgetSprNormal2)
                  sgx_DrawWin(id, #sgx_GadgetSprHover2)
                  
                  ; get close button size and position
                  If sgx(id)\width
                    sgx(id)\sys0\x1 = sgx(id)\w - (sgx(id)\width-SpriteHeight(sgx(id)\spr[#sgx_GadgetSprNormal2]))/2 - SpriteWidth(sgx(id)\spr[#sgx_GadgetSprNormal2])
                    sgx(id)\sys0\y1 = (sgx(id)\width-SpriteHeight(sgx(id)\spr[#sgx_GadgetSprNormal2]))/2
                  Else
                    sgx(id)\sys0\x1 = sgx(id)\w - SpriteWidth(sgx(id)\spr[#sgx_GadgetSprNormal2]) - sgx(id)\design\img[#sgx_DesignBorderHoverVer2]\w - 1
                    sgx(id)\sys0\y1 = sgx(id)\design\img[#sgx_DesignBorderHoverHor2]\h + 1
                  EndIf
                  sgx(id)\sys0\x2 = sgx(id)\sys0\x1 + SpriteWidth(sgx(id)\spr[#sgx_GadgetSprNormal2])
                  sgx(id)\sys0\y2 = sgx(id)\sys0\y1 + SpriteHeight(sgx(id)\spr[#sgx_GadgetSprNormal2])
                  
                  ; these rely on sys0 to be initialized
                  sgx_DrawWin(id, #sgx_GadgetSprNormal)
                  sgx_DrawWin(id, #sgx_GadgetSprHover)
                  sgx_DrawWin(id, #sgx_GadgetSprPushed)
                  ;}
                  
                  ;- refresh table
                Case #sgxTypeTable ;{
                  StartDrawing(ScreenOutput())
                    DrawingFont(sgx(id)\textfontID)
                    sgx(id)\textw = TextWidth(sgx(id)\text)
                    sgx(id)\texth = TextHeight(sgx(id)\text)
                  StopDrawing()
                  ; in case the table has text, it gets a garbage titlebar (not realy usefull at the moment but inherited from window code)  
                  If sgx(id)\texth
                    sgx(id)\width = sgx(id)\texth + 2 + sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h + sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h
                  Else
                    sgx(id)\width = 0
                  EndIf
                  
                  ; store all elements metrics to check size change after the following call of sgx_ArrangeTable()
                  ; this is somewhat experimental and a bit oversized but with the avoidance of redundant table
                  ; child refreshes there is some perfomance gain.
                  ; could be reduced to only store table's childs recursivley and only if style fitW/H is set.
                  For sgxEach(_id_)
                    AddElement(sizes())
                    sizes()\id = _id_
                    sizes()\w = sgx(_id_)\w
                    sizes()\h = sgx(_id_)\h
                    sizes()\w2 = sgx(_id_)\w2
                    sizes()\h2 = sgx(_id_)\h2
                  Next
                  
                  ; collecting and adjusting metrics of table and childs without refreshing
                  sgx_ArrangeTable(id)
                  
                  ; table child refresh (flat, but sub tables will refresh their childs too, causing recursion)
                  If mode=#sgx_refreshNormalWithChildsOfTables And sgx(id)\treeElem\node
                    sgx_llIterator(Protected *it, sgx(id)\treeElem\node)
                    If sgx_BuildRefresh
                        ; get previous element size from before sgx_ArrangeTable()..
                        ForEach sizes()
                          If (sizes()\id = *it\id)
                            ; and refresh only if size changed
                            doRefresh = #False
                            If (sgx(*it\id)\type <> #sgxTypeCheckbox)  And  (sgx(*it\id)\w  <> sizes()\w  Or sgx(*it\id)\h  <> sizes()\h )
                              ; (non-checkboxes: check w/h change)
                              doRefresh = #True
                            ElseIf (sgx(*it\id)\type  = #sgxTypeCheckbox)  And  (sgx(*it\id)\w2 <> sizes()\w2 Or sgx(*it\id)\h2 <> sizes()\h2)
                              ; (checkboxes: check w2/h2 overall size change)
                              doRefresh = #True
                            EndIf
                            If doRefresh
                              sgxRefresh( *it\id, #sgx_refreshNormalWithChildsOfTables)
                            EndIf
                          EndIf
                        Next
                      Else
                        sgxRefresh( *it\id, #sgx_refreshNormalWithChildsOfTables)
                      EndIf
                    sgx_llNext(*it)
                  EndIf
                  
                  ; remove temporary metrics
                  ClearList(sizes())
                  
                  ; tables use the window visuals
                  sgx_DrawWin(id, #sgx_GadgetSprNormal)
                  sgx_DrawWin(id, #sgx_GadgetSprHover)
                  ;}
                  
                  ;- refresh combo
                Case #sgxTypeCombo ;{
                  StartDrawing(ScreenOutput())
                    DrawingFont(sgx(id)\textfontID)
                    sgx(id)\texth = TextHeight("X")
                  StopDrawing()
                  sgx_DrawCombo(id, #sgx_GadgetSprNormal)
                  sgx_DrawCombo(id, #sgx_GadgetSprHover)
                  sgx_DrawCombo(id, #sgx_GadgetSprPushed) ; the opened combo
                  sgx_DrawCombo(id, #sgx_GadgetSprHover2) ; open combo hover effect
                  ;}
                  
                  ;- refresh text
                Case #sgxTypeText ;{
                  StartDrawing(ScreenOutput())
                  DrawingFont(sgx(id)\textfontID)
                    ; get longest line widht and line heigth
                    sgx(id)\textw = sgx_textWidthLF(sgx(id)\text)
                    sgx(id)\texth = TextHeight("X")
                    
                    If sgx(id)\auto & #sgx_AutoW
                      ; (margin will be added when drawn)
                      sgx(id)\w = sgx(id)\textw +sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w +sgx(id)\design\img[#sgx_DesignBorderNormalVer2]\w
                    EndIf
                    
                    If sgx(id)\w > 0
                      
                      ; automatic word wrap (add newlines where necessary)
                      If sgx(id)\auto & #sgx_AutoNL
                        nbChars = Len(sgx(id)\text)
                        maxLineWidth = sgx(id)\w - 2*sgx(id)\margin
                        lastwhitespace = 0
                        For i=1 To nbChars
                          char = Mid(sgx(id)\text,i,1)
                          text + char
                          Select Asc(char)
                            Case 32,9 ; (space,tab)
                              lastwhitespace = i+inserted
                            Case #LF
                              lastwhitespace = i+inserted
                              ; next char will be begin of line
                              linestart = i+1
                          EndSelect
                          If TextWidth(Mid(text,linestart))  >  maxLineWidth  And  lastwhitespace<>linestart-1
                            ; dont break before first line/char
                            If lastwhitespace = 0
                              ; we're still in the first word of text, so we just go on and will break with next whitespace occurance
                              lastwhitespace+1
                              linestart = lastwhitespace+1
                            Else
                              ; insert newline (get text from beginning to last space, add newline, add the rest of the text)
                              text = Mid(text,1,lastwhitespace) + #LF$ + Mid(text,lastwhitespace+1)
                              ; count our additional added newline
                              inserted+1
                              lastwhitespace+1
                              linestart = lastwhitespace+1
                            EndIf
                          EndIf
                        Next
                        sgx(id)\text = text
                      EndIf
                      ; update longest line width
                      sgx(id)\textw = sgx_textWidthLF(sgx(id)\text)
                      
                      If sgx(id)\auto & #sgx_AutoW
                        sgx(id)\w = sgx(id)\margin + sgx(id)\textw + sgx(id)\margin
                      EndIf
                      If sgx(id)\auto & #sgx_AutoH
                        ; lines * line height
                        sgx(id)\h = sgx(id)\margin + (CountString(sgx(id)\text,#LF$)+1)*sgx(id)\texth + sgx(id)\margin
                      EndIf
                      
                    EndIf
                  StopDrawing()
                  sgx_DrawText(id, #sgx_GadgetSprNormal)
                  If sgx(id)\style & #sgxStyleHover
                    sgx_DrawText(id, #sgx_GadgetSprHover)
                  EndIf
                  ;}
                  
                  ;- refresh area
                Case #sgxTypeArea ;{
                  ; nothing to do
                  ;}
                  
              EndSelect
              
              
              ; disabled elements
              If sgx(id)\disabled
                ; we only modify those sprites that are used while disabled (i.e. not 'hover' etc.)
                ; user sprites are not modified but dimmed by alpha for disabled elements when displayed.
                sgx_DrawDisabled(id, #sgx_GadgetSprNormal)
                
                If sgx(id)\type & (#sgxTypeScrollbar|#sgxTypeTrackbar)
                  sgx_DrawDisabled(id, #sgx_GadgetSprNormal2) ; knob
                  
                ElseIf sgx(id)\type = #sgxTypeCheckbox
                  sgx_DrawDisabled(id, #sgx_GadgetSprChecked) ; checked
                  
                ElseIf sgx(id)\type = #sgxTypeButtonToggle
                  sgx_DrawDisabled(id, #sgx_GadgetSprPushed)  ; pushed
                  
                ElseIf sgx(id)\type = #sgxTypeWin
                  sgx_DrawDisabled(id, #sgx_GadgetSprNormal2)  ; close gadget
                EndIf
              EndIf
              
            EndIf
            
            ; check if size changed (for checkbox also check overall size change including label)
            If (sgx(id)\w <> wPrev Or sgx(id)\h <> hPrev)  Or  (sgx(id)\type = #sgxTypeCheckbox And (sgx(id)\w2 <> w2Prev Or sgx(id)\h2 <> h2Prev))
              ; prepare event triggering
              AddElement(sgx_pendingEvents())
              sgx_pendingEvents()\id = id
              sgx_pendingEvents()\resized = 1
            EndIf
            
          EndIf
        EndProcedure
        
        
        
        
        ; -------------------------------------- drawing auxiliary
        
        
        
        
        ;** sgx_Fill
        ;* draws a background to the current drawing output using the design's background image (will be tiled to fill specified area).
        ;** .*design.sgxS_Design : the design you want to use for the drawing.
        ;** .x                   : x-position to begin drawing
        ;** .y                   : y-position to begin drawing
        ;** .w                   : width of background to be drawn
        ;** .h                   : height of background to be drawn
        ;** .appearance          : a value like #sgx_AppearanceNormal, #sgx_AppearanceHover or #sgx_AppearancePushed etc.
        Procedure sgx_Fill(*design.sgxS_Design, x, y, w, h, appearance)
          Static i, k
          Static img2, img3, img4
          Static width2, height3
          Static imgType
          
          Select appearance
          Case #sgx_AppearanceNormal
            imgType = #sgx_DesignBackgroundNormal
          Case #sgx_AppearanceHover
            imgType = #sgx_DesignBackgroundHover
          Default
            ProcedureReturn
          EndSelect
          
          ; heeere (fill)
          If 0
            DrawingMode(#PB_2DDrawing_AllChannels)
            ;DrawingMode(#PB_2DDrawing_AlphaBlend)
            DrawImage(ImageID(*design\img[imgType]\nr), x,y, w,h)
            
            ;DrawingMode(#PB_2DDrawing_Default | #PB_2DDrawing_AlphaBlend)
        ;     DrawingMode(#PB_2DDrawing_Default | #PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Gradient)
            
            ;DrawingMode(#PB_2DDrawing_Gradient)      
        ;     BackColor($aa666666)
        ;     GradientColor(0.95, $aa666666)
        ;     FrontColor($aa222222)
        ;     BoxedGradient(0,0,w,h)
        ;     Box(x,y,w,h)
            
        ;     Box(x,y,w,h,$aa666666)
            
            ProcedureReturn
          EndIf
          
          width2 = w % *design\img[imgType]\w
          height3 = h % *design\img[imgType]\h
          If width2>0
            img2 = GrabImage(*design\img[imgType]\nr, #PB_Any, 0,0, width2, *design\img[imgType]\h)
          EndIf
          
          If height3>0
            img3 = GrabImage(*design\img[imgType]\nr, #PB_Any, 0,0, *design\img[imgType]\w, height3)
          EndIf
          
          If width2>0 And height3>0
            img4 = GrabImage(*design\img[imgType]\nr, #PB_Any, 0,0, width2, height3)
          EndIf
          
          ; full size
          If IsImage(*design\img[imgType]\nr)
            k=y
            While k <= y+h-*design\img[imgType]\h
              i=x
              While i <= x+w-*design\img[imgType]\w
                DrawImage(ImageID(*design\img[imgType]\nr),i,k)
                i+*design\img[imgType]\w
              Wend
              k+*design\img[imgType]\h
            Wend
          EndIf
        
          ; remaining right
          If IsImage(img2) 
            k=y
            While k <= y+h-*design\img[imgType]\h
              DrawImage(ImageID(img2),i,k)
              k+*design\img[imgType]\h
            Wend
            sgx_FreeImageChecked(img2)
          EndIf
        
          ; remaining bottom
          If IsImage(img3) 
            i=x
            While i <= x+w-*design\img[imgType]\w
              DrawImage(ImageID(img3),i,k)
              i+*design\img[imgType]\w
            Wend
            sgx_FreeImageChecked(img3)
          EndIf
        
          ; remaining bottom-right
          If IsImage(img4) 
            DrawImage(ImageID(img4),i,k)
            sgx_FreeImageChecked(img4)
          EndIf
        EndProcedure
        
        
        ;** sgx_BorderLine
        ;* draws a straight part (a line, as opposed to some corner part) of a border to the current drawing output using the design's border images. _
        ;* used by <a href=#f_sgx_borderbox>sgx_Borderbox</a>.
        ;** .*design.sgxS_Design : the design you want to use for the drawing.
        ;** .x                   : x-position to begin drawing
        ;** .y                   : y-position to begin drawing
        ;** .V                   : length of the line to be drawn. a positive value for horizontal lines and negative value for vertical lines.
        ;** .appearance          : a value like #sgx_AppearanceNormal, #sgx_AppearanceHover or #sgx_AppearancePushed etc.
        ;** .side                : the side of the border to be drawn: _
        ;*                         #sgx_SideTopOrLeft draws the top edge border (for horizontal) or left edge border (for vertical) respectively. _
        ;*                         #sgx_SideBottomOrRight draws the bottom edge border (for horizontal) or right edge border (for vertical) respectively.
        Procedure sgx_BorderLine(*design.sgxS_Design, x, y, V, appearance, side)
          Static i
          Static *designImg1.sgx_S_Image
          Static full
          Static img2
          Static width2
          Static height2
          
          
          ;horizontal line
          If V>0
            
            ; get the design image
            Select appearance
            Case #sgx_AppearanceNormal
              If side = #sgx_SideTopOrLeft
                *designImg1 = *design\img[#sgx_DesignBorderNormalHor1] ; use left border parts
              ElseIf side = #sgx_SideBottomOrRight
                *designImg1 = *design\img[#sgx_DesignBorderNormalHor2] ; use right border parts
              EndIf
            Case #sgx_AppearanceHover
              If side = #sgx_SideTopOrLeft
                *designImg1 = *design\img[#sgx_DesignBorderHoverHor1] ; left
              ElseIf side = #sgx_SideBottomOrRight
                *designImg1 = *design\img[#sgx_DesignBorderHoverHor2] ; right
              EndIf
            Case #sgx_AppearancePushed
              If side = #sgx_SideTopOrLeft
                *designImg1 = *design\img[#sgx_DesignBorderPushedHor1] ; left
              ElseIf side = #sgx_SideBottomOrRight
                *designImg1 = *design\img[#sgx_DesignBorderPushedHor2] ; right
              EndIf
            Default
              ProcedureReturn
            EndSelect
            
            ; get full amount of images to be drawn
            full  = V / *designImg1\w
            ; get the remaining image width to be drawn
            width2 = V % *designImg1\w
            
            ; draw full images
            For i=0 To full-1
              DrawImage(ImageID(*designImg1\nr), x+i**designImg1\w, y)
            Next
            ; draw remaining image part
            If width2>0
              img2 = GrabImage(*designImg1\nr, #PB_Any, 0,0, width2, *designImg1\h)
              DrawImage(ImageID(img2), x + i * *designImg1\w, y)
              FreeImage(img2)
            EndIf
          
          
          
          ;vertical lines
          ElseIf v<0
            
            ; get the design image
            Select appearance
            Case #sgx_AppearanceNormal
              If side = #sgx_SideTopOrLeft
                *designImg1 = *design\img[#sgx_DesignBorderNormalVer1]
              ElseIf side = #sgx_SideBottomOrRight
                *designImg1 = *design\img[#sgx_DesignBorderNormalVer2]
              EndIf
            Case #sgx_AppearanceHover
              If side = #sgx_SideTopOrLeft
                *designImg1 = *design\img[#sgx_DesignBorderHoverVer1]
              ElseIf side = #sgx_SideBottomOrRight
                *designImg1 = *design\img[#sgx_DesignBorderHoverVer2]
              EndIf
            Case #sgx_AppearancePushed
              If side = #sgx_SideTopOrLeft
                *designImg1 = *design\img[#sgx_DesignBorderPushedVer1]
              ElseIf side = #sgx_SideBottomOrRight
                *designImg1 = *design\img[#sgx_DesignBorderPushedVer2]
              EndIf
            Default
              ProcedureReturn
            EndSelect
            
            ; get positive length
            V = -V
            
            ; get full amount of images to be drawn
            full  = V / *designImg1\h
            ; get the remaining image height to be drawn
            height2 = V % *designImg1\h
            
            ; draw full images
            For i=0 To full-1
              DrawImage(ImageID(*designImg1\nr), x, y+i**designImg1\h)
            Next
            ; draw remaining image part
            If height2>0
              img2 = GrabImage(*designImg1\nr, #PB_Any, 0,0, *designImg1\w, height2)
              DrawImage(ImageID(img2), x, y+i* *designImg1\h)
              FreeImage(img2)
            EndIf
        
          EndIf
        EndProcedure
        
        
        ;** sgx_Borderbox
        ;* draws a box to the current drawing output using the design's border images. _
        ;** .*design.sgxS_Design : the design you want to use for the drawing.
        ;** .x                   : x-position to begin drawing
        ;** .y                   : y-position to begin drawing
        ;** .w                   : width of box to be drawn
        ;** .h                   : height of box to be drawn
        ;** .appearance          : a value like #sgx_AppearanceNormal, #sgx_AppearanceHover or #sgx_AppearancePushed etc.
        Procedure sgx_Borderbox(*design.sgxS_Design, x, y, w, h, appearance)
          Static *designImgCorner1.sgx_S_Image
          Static *designImgCorner2.sgx_S_Image
          Static *designImgCorner3.sgx_S_Image
          Static *designImgCorner4.sgx_S_Image
          Static i
          
          ; get design's corner images
          Select appearance
          ; ----------------
          Case #sgx_AppearanceNormal ;{
            *designImgCorner1 = *design\img[#sgx_DesignBorderNormalCorner1]
            *designImgCorner2 = *design\img[#sgx_DesignBorderNormalCorner2]
            *designImgCorner3 = *design\img[#sgx_DesignBorderNormalCorner3]
            *designImgCorner4 = *design\img[#sgx_DesignBorderNormalCorner4]
            ;}
          
          Case #sgx_AppearanceHover ;{
            *designImgCorner1 = *design\img[#sgx_DesignBorderHoverCorner1]
            *designImgCorner2 = *design\img[#sgx_DesignBorderHoverCorner2]
            *designImgCorner3 = *design\img[#sgx_DesignBorderHoverCorner3]
            *designImgCorner4 = *design\img[#sgx_DesignBorderHoverCorner4]
            ;}
          
          Case #sgx_AppearancePushed ;{
            *designImgCorner1 = *design\img[#sgx_DesignBorderPushedCorner1]
            *designImgCorner2 = *design\img[#sgx_DesignBorderPushedCorner2]
            *designImgCorner3 = *design\img[#sgx_DesignBorderPushedCorner3]
            *designImgCorner4 = *design\img[#sgx_DesignBorderPushedCorner4]
            ;}
          
          Default
            ProcedureReturn
          EndSelect
          
          ; heeere (borderbox)
          If 0
            DrawingMode(#PB_2DDrawing_Outlined | #PB_2DDrawing_AlphaBlend)
        ;     Box(x+0, y+0, w-0, h-0, $66336633)
            ;Box(x+1, y+1, w-2, h-2, $88336633)
            ;Box(x+2, y+2, w-4, h-4, $aa336633)
            ProcedureReturn
          EndIf
          
          ; draw borders:
          ; top 
          ; bottom
          ; left
          ; right
          sgx_BorderLine(*design, x+*designImgCorner1\w,                          y,                                              w-*designImgCorner1\w-*designImgCorner2\w,    appearance, #sgx_SideTopOrLeft)
          sgx_BorderLine(*design, x+*designImgCorner1\w,                          y+h-*design\img[#sgx_DesignBorderNormalHor2]\h, w-*designImgCorner1\w-*designImgCorner2\w,    appearance, #sgx_SideBottomOrRight)
          sgx_BorderLine(*design, x,                                              y+*designImgCorner1\h,                          -(h-*designImgCorner2\h-*designImgCorner4\h), appearance, #sgx_SideTopOrLeft)
          sgx_BorderLine(*design, x+w-*design\img[#sgx_DesignBorderNormalVer2]\w, y+*designImgCorner2\h,                          -(h-*designImgCorner2\h-*designImgCorner4\h), appearance, #sgx_SideBottomOrRight)
          
          ; draw corners:
          ; top left
          ; top right
          ; bottom left
          ; bottom right
          DrawImage(ImageID(*designImgCorner1\nr), x,                       y)
          DrawImage(ImageID(*designImgCorner2\nr), x+w-*designImgCorner2\w, y)
          DrawImage(ImageID(*designImgCorner3\nr), x,                       y+h-*designImgCorner3\h)
          DrawImage(ImageID(*designImgCorner4\nr), x+w-*designImgCorner4\w, y+h-*designImgCorner3\h)
          
        EndProcedure
        
        
        ;** sgx_CopyImageToSprite
        ;* creates and returns a sprite duplicated from an image (for example the mouse sprite from a loaded mouse design image, or the window's close buttons). _
        ;* the optional parameters will be specified by the element drawing routines to let this function also handle the custom drawing callbacks.
        ;** .imgNr            : a PB image number.
        ;** .onBeginDraw      : callback that should be called before drawing the image content to newly created sprite.
        ;** .onEndDraw        : callback that should be called after drawing the image content to newly created sprite.
        ;** .id               : id that should be passed to the callbacks.
        ;** .sprType          : sprType that should be passed to the callbacks.
        Procedure sgx_CopyImageToSprite(imgNr, onBeginDraw.sgx_onBeginDraw=0, onEndDraw.sgx_onEndDraw=0, id=0, sprType=0)
          Protected ret = 0
          Protected w = ImageWidth(imgNr)
          Protected h = ImageHeight(imgNr)
          Protected i
          Protected k
          Protected draw
          If IsImage(imgNr)
            ret = CreateSprite(#PB_Any, w, h, #PB_Sprite_AlphaBlending)
            If Not ret
              sgx_err("sprite creation failed with size "+w+", "+h) ; +": sgx("+id+")")
            Else
              TransparentSpriteColor(ret, RGB(0,0,0))
              StartDrawing(SpriteOutput(ret))
                draw = #sgxDrawDesign | #sgxDrawText
                If onBeginDraw
                  draw = onBeginDraw(id, sprType, w, h)
                EndIf
                If draw & #sgxDrawDesign
                  DrawingMode(#PB_2DDrawing_AllChannels)
                  DrawImage(ImageID(imgNr),0,0)
                EndIf
                If onEndDraw
                  onEndDraw(id, sprType, w, h)
                EndIf
              StopDrawing()
            EndIf
          EndIf
          ProcedureReturn ret
        EndProcedure
        
        
        
        ; -------------------------------------- draw sgx components ( refresh only; not display )
        
        
        ;** sgx_DrawCursor
        ;* redraws a particular state sprite of an sgxCursor element (called by <a href=#f_sgxrefresh>sgxRefresh</a>).
        ;** .id      : ID of the sgx element
        ;** .sprType : the state / sprite type that should be redrawn (not all elements use every state): #sgx_GadgetSprNormal or #sgx_GadgetSprHover etc.
        Procedure sgx_DrawCursor(id, sprType)
          sgx_FreeSpriteChecked(sgx(id)\spr[sprType])
          sgx(id)\spr[sprType] = sgx_CopyImageToSprite(sgx(id)\design\img[#sgx_DesignMouseCursor]\nr, sgx(id)\design\onBeginDraw, sgx(id)\design\onEndDraw, id, sprType)
        EndProcedure
        
        
        ;** sgx_DrawButton
        ;* redraws a particular state sprite of an sgxButton element (called by <a href=#f_sgxrefresh>sgxRefresh</a>).
        ;** .id      : ID of the sgx element
        ;** .sprType : the state / sprite type that should be redrawn (not all elements use every state): #sgx_GadgetSprNormal or #sgx_GadgetSprHover etc.
        Procedure sgx_DrawButton(id, sprType)
          Static color1
          Static appearance_fill
          Static appearance_border
          Static textOffset, x, y, lines
          Static draw
          
          sgx_prepareSprite(sgx(id)\spr[sprType], sgx(id)\w, sgx(id)\h, id)
          If sgx(id)\spr[sprType]
            StartDrawing(SpriteOutput(sgx(id)\spr[sprType]))
              DrawingMode(#PB_2DDrawing_AllChannels)
              Box(0,0,sgx(id)\w,sgx(id)\h, RGBA(0,0,0,0))
              draw = #sgxDrawDesign | #sgxDrawText
              If sgx(id)\design\onBeginDraw
                draw = sgx(id)\design\onBeginDraw(id, sprType, sgx(id)\w, sgx(id)\h)
                ; store text offset for 'pushed' for later display of user sprite
                If draw & #sgxDrawTextNoOffset
                  sgx(id)\margin = 0
                Else
                  sgx(id)\margin = 1
                EndIf
              EndIf
              
              DrawingMode(#PB_2DDrawing_AlphaBlend)
              Select sprType
              Case #sgx_GadgetSprNormal
                If sgx(id)\type=#sgxTypeButtonToggle : color1 = sgx(id)\design\c[03]
                Else                                 : color1 = sgx(id)\design\basecolor
                EndIf
                appearance_fill = #sgx_AppearanceNormal
                appearance_border = #sgx_AppearanceNormal
                textOffset = 0
              Case #sgx_GadgetSprHover
                If sgx(id)\type=#sgxTypeButtonToggle : color1 = sgx(id)\design\c[09]
                Else                                 : color1 = sgx(id)\design\c[09]
                EndIf
                appearance_fill = #sgx_AppearanceHover
                appearance_border = #sgx_AppearanceHover
                textOffset = 0
              Case #sgx_GadgetSprPushed
                If sgx(id)\type=#sgxTypeButtonToggle : color1 = sgx(id)\design\basecolor
                Else                                 : color1 = sgx(id)\design\c[03]
                EndIf
                appearance_fill = #sgx_AppearanceNormal
                appearance_border = #sgx_AppearancePushed
                textOffset = 1
                If draw &  #sgxDrawTextNoOffset
                  textOffset = 0
                EndIf
              Case #sgx_GadgetSprHoverPushed
                If sgx(id)\type=#sgxTypeButtonToggle : color1 = sgx(id)\design\c[09]
                Else                                 : color1 = sgx(id)\design\basecolor
                EndIf
                appearance_fill = #sgx_AppearanceHover
                appearance_border = #sgx_AppearancePushed
                textOffset = 1
                If draw &  #sgxDrawTextNoOffset
                  textOffset = 0
                EndIf
              Default
                ProcedureReturn
              EndSelect
              
              If draw & #sgxDrawDesign
                DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Outlined)
                sgx_Fill( sgx(id)\design, sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w, sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h, sgx(id)\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer2]\w, sgx(id)\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h, appearance_fill)
                
                DrawingMode(#PB_2DDrawing_AlphaBlend)
                sgx_BorderBox(sgx(id)\design, 0, 0, sgx(id)\w, sgx(id)\h, appearance_border)
              EndIf
              
              If draw & (#sgxDrawText | #sgxDrawTextNoOffset)
                DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Transparent)
                DrawingFont(sgx(id)\textfontID)
                
                ; get text start drawing position to have it centered on the button
                lines = CountString(sgx(id)\text, #LF$) + 1
                x = textOffset + (sgx(id)\w - sgx_textWidthLF(sgx(id)\text)) / 2
                y = textOffset + (sgx(id)\h - (lines * sgx(id)\texth) ) / 2
                ; draw text centered
                sgx_drawTextLF( x, y, sgx(id)\text, color1, -1, #LF$, #sgxAlignTextCenter)
              EndIf
              
              If sgx(id)\design\onEndDraw
                sgx(id)\design\onEndDraw(id, sprType, sgx(id)\w, sgx(id)\h)
              EndIf
            StopDrawing()
          EndIf
            
        EndProcedure
        
        
        ;** sgx_DrawCheckbox
        ;* redraws a particular state sprite of an sgxCheckbox element (called by <a href=#f_sgxrefresh>sgxRefresh</a>).
        ;** .id      : ID of the sgx element
        ;** .sprType : the state / sprite type that should be redrawn (not all elements use every state): #sgx_GadgetSprNormal or #sgx_GadgetSprHover etc.
        Procedure sgx_DrawCheckbox(id, sprType)
        ;   Static x1
        ;   Static y1
        ;   Static x2
        ;   Static y2
          Static color1
          Static imgType_fill
          Static appearance_fill
          Static appearance_border
          Static draw
          
          Select sprType
            Case #sgx_GadgetSprNormal
              color1 = sgx(id)\design\basecolor
              appearance_fill = #sgx_AppearanceNormal
              appearance_border = #sgx_AppearanceNormal
            Case #sgx_GadgetSprHover
              color1 = sgx(id)\design\c[09]
              appearance_fill = #sgx_AppearanceHover
              appearance_border = #sgx_AppearanceHover
            Case #sgx_GadgetSprChecked
              color1 = sgx(id)\design\basecolor
            Default
              ProcedureReturn
          EndSelect
          
          Select sprType
            Case #sgx_GadgetSprNormal, #sgx_GadgetSprHover
              sgx_prepareSprite(sgx(id)\spr[sprType], sgx(id)\w2, sgx(id)\h2, id)
              If sgx(id)\spr[sprType]
                StartDrawing(SpriteOutput(sgx(id)\spr[sprType]))
                  DrawingMode(#PB_2DDrawing_AllChannels)
                  Box(0,0,sgx(id)\w2,sgx(id)\h2, RGBA(0,0,0,0))
                  draw = #sgxDrawDesign | #sgxDrawText
                  If sgx(id)\design\onBeginDraw
                    draw = sgx(id)\design\onBeginDraw(id, sprType, sgx(id)\w2, sgx(id)\h2)
                  EndIf
                  If draw & (#sgxDrawDesign | #sgxDrawText | #sgxDrawTextNoOffset)
                    DrawingMode(#PB_2DDrawing_AlphaBlend)
                    Select sgx(id)\textalign
                      Case #sgxAlignCheckboxTextRight
                        
                        ; box left, text right
                        If draw & #sgxDrawDesign
                          sgx_Fill( sgx(id)\design, sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w, sgx(id)\h2/2-sgx(id)\h/2+sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h, sgx(id)\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer2]\w, sgx(id)\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h, appearance_fill)
                          sgx_Borderbox(sgx(id)\design, 0,sgx(id)\h2/2-sgx(id)\h/2, sgx(id)\w,sgx(id)\h, appearance_border)
                        EndIf
                        If draw & (#sgxDrawText | #sgxDrawTextNoOffset)
                          DrawingFont(sgx(id)\textfontID)
                          DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Transparent)
                          DrawText( sgx(id)\w+5, sgx(id)\h2/2-sgx(id)\texth/2, sgx(id)\text, color1 )
                        EndIf
                        
                      Case #sgxAlignCheckboxTextLeft
                        
                        ; text left, box right
                        If draw & #sgxDrawDesign
                          sgx_Fill( sgx(id)\design, sgx(id)\textw+5 + sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w, sgx(id)\h2/2-sgx(id)\h/2+sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h, sgx(id)\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer2]\w, sgx(id)\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h, appearance_fill)
                          sgx_Borderbox(sgx(id)\design, sgx(id)\textw+5,sgx(id)\h2/2-sgx(id)\h/2, sgx(id)\w,sgx(id)\h, appearance_border)
                        EndIf
                        If draw & (#sgxDrawText | #sgxDrawTextNoOffset)
                          DrawingFont(sgx(id)\textfontID)
                          DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Transparent)
                          DrawText( 0, sgx(id)\h2/2-sgx(id)\texth/2, sgx(id)\text, color1 )
                        EndIf
                      
                    EndSelect
                  EndIf
                  If sgx(id)\design\onEndDraw
                    sgx(id)\design\onEndDraw(id, sprType, sgx(id)\w2, sgx(id)\h2)
                  EndIf
                StopDrawing()
              EndIf
            Case #sgx_GadgetSprChecked
              ; this is only the 'checked' part (without the knob).
              ; we only need the knob width but we use the full height to not worry about y offset when displaying.
              sgx_prepareSprite(sgx(id)\spr[sprType], sgx(id)\w, sgx(id)\h, id)
              If sgx(id)\spr[sprType]
                StartDrawing(SpriteOutput(sgx(id)\spr[sprType]))
                  DrawingMode(#PB_2DDrawing_AllChannels)
                  Box(0,0,sgx(id)\w,sgx(id)\h, RGBA(0,0,0,0))
                  draw = #sgxDrawDesign | #sgxDrawText
                  If sgx(id)\design\onBeginDraw
                    draw = sgx(id)\design\onBeginDraw(id, sprType, sgx(id)\w, sgx(id)\h)
                  EndIf
                  If draw & #sgxDrawDesign
                    DrawingMode(#PB_2DDrawing_AlphaBlend)
                    ; old lowbudged 'checked' x
                    ;If sgx(id)\design\name = "lowbudget"
                    ;  x1 = sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w + 1
                    ;  x2 = sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w + sgx(id)\w2 - 2 - 2*sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w
                    ;  y1 = sgx(id)\h/2 - sgx(id)\h2/2 + sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h + 1
                    ;  y2 = sgx(id)\h/2 - sgx(id)\h2/2 + sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h + sgx(id)\h2 - 2 - 2*sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h
                    ;  LineXY(x1,y1,x2,y2,color1)
                    ;  LineXY(x1,y2,x2,y1,color1)
                    ;Else
                    DrawImage(ImageID(sgx(id)\design\img[#sgx_DesignCheckboxChecked]\nr),  sgx(id)\w/2 - ImageWidth(sgx(id)\design\img[#sgx_DesignCheckboxChecked]\nr)/2,  sgx(id)\h/2 - ImageHeight(sgx(id)\design\img[#sgx_DesignCheckboxChecked]\nr)/2  )
                    ;EndIf
                  EndIf
                  If sgx(id)\design\onEndDraw
                    sgx(id)\design\onEndDraw(id, sprType, sgx(id)\w, sgx(id)\h)
                  EndIf
                StopDrawing()
              EndIf
          EndSelect
        EndProcedure
        
        
        ;** sgx_DrawScrollbar
        ;* redraws a particular state sprite of an sgxScrollbar/sgxTrackbar element (called by <a href=#f_sgxrefresh>sgxRefresh</a>).
        ;** .id      : ID of the sgx element
        ;** .sprType : the state / sprite type that should be redrawn (not all elements use every state): #sgx_GadgetSprNormal or #sgx_GadgetSprHover etc.
        Procedure sgx_DrawScrollbar(id, sprType)
          Static appearance_fill
          Static appearance_border
          Static x, y
          Static sw, sh
          Static draw
          
          Select sprType
          Case #sgx_GadgetSprNormal, #sgx_GadgetSprNormal2
            appearance_fill = #sgx_AppearanceNormal
            appearance_border = #sgx_AppearanceNormal
          Case #sgx_GadgetSprHover, #sgx_GadgetSprHover2
            appearance_fill = #sgx_AppearanceHover
            appearance_border = #sgx_AppearanceHover
          Default
            ProcedureReturn
          EndSelect
          
          Select sprType
            Case #sgx_GadgetSprNormal, #sgx_GadgetSprHover
              ; bar
              sgx_prepareSprite(sgx(id)\spr[sprType], sgx(id)\w, sgx(id)\h, id)
              If sgx(id)\spr[sprType]
                sw = SpriteWidth(sgx(id)\spr[sprType])
                sh = SpriteHeight(sgx(id)\spr[sprType])
                StartDrawing(SpriteOutput(sgx(id)\spr[sprType]))
                  DrawingMode(#PB_2DDrawing_AllChannels)
                  Box(0,0,sgx(id)\w,sgx(id)\h, RGBA(0,0,0,0))
                  draw = #sgxDrawDesign | #sgxDrawText
                  If sgx(id)\design\onBeginDraw
                    draw = sgx(id)\design\onBeginDraw(id, sprType, sgx(id)\w, sgx(id)\h)
                  EndIf
                  If draw & #sgxDrawDesign
                    DrawingMode(#PB_2DDrawing_AlphaBlend)
                    sgx_Fill( sgx(id)\design, sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w, sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h, sgx(id)\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer2]\w, sgx(id)\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h, appearance_fill)
                    If sgx(id)\style & #sgxStyleGrid
                      Select sgx(id)\type
                        Case #sgxTypeScrollbarVer, #sgxTypeTrackbarVer
                          For y=0 To sgx(id)\h-2 Step 6
                            For x = 0 To sgx(id)\w-2
                              If x>=0 And x<sw And y>=0 And y<sh
                                Plot(x, y  , sgx_fade(Point(x, y  ),-200)|1 )
                              EndIf
                              If x>=0 And x<sw And (y+1)>=0 And (y+1)<sh
                                Plot(x, y+1, sgx_fade(Point(x, y+1),  50)|1 )
                              EndIf
                            Next
                          Next
                        Case #sgxTypeScrollbarHor, #sgxTypeTrackbarHor
                          For x=0 To sgx(id)\w-2 Step 6
                            For y = 0 To sgx(id)\h-2
                              If x>=0 And x<sw And y>=0 And y<sh
                                Plot(x,  y, sgx_fade(Point(x,  y),-200)|1 )
                              EndIf
                              If (x+1)>=0 And (x+1)<sw And y>=0 And y<sh
                                Plot(x+1,y, sgx_fade(Point(x+1,y),  50)|1 )
                              EndIf
                            Next
                          Next
                      EndSelect
                    EndIf
                    sgx_Borderbox(sgx(id)\design, 0,0, sgx(id)\w,sgx(id)\h, appearance_border)
                  EndIf
                  If sgx(id)\design\onEndDraw
                    sgx(id)\design\onEndDraw(id, sprType, sgx(id)\w, sgx(id)\h)
                  EndIf
                StopDrawing()
              EndIf
            Case #sgx_GadgetSprNormal2, #sgx_GadgetSprHover2
              ; knob
              sgx_prepareSprite(sgx(id)\spr[sprType], sgx(id)\w2, sgx(id)\h2, id)
              If sgx(id)\spr[sprType]
                StartDrawing(SpriteOutput(sgx(id)\spr[sprType]))
                  DrawingMode(#PB_2DDrawing_AllChannels)
                  Box(0,0,sgx(id)\w2,sgx(id)\h2, RGBA(0,0,0,0))
                  draw = #sgxDrawDesign | #sgxDrawText
                  If sgx(id)\design\onBeginDraw
                    draw = sgx(id)\design\onBeginDraw(id, sprType, sgx(id)\w2, sgx(id)\h2)
                  EndIf
                  If draw & #sgxDrawDesign
                    DrawingMode(#PB_2DDrawing_AlphaBlend)
                    sgx_Fill( sgx(id)\design, sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w, sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h, sgx(id)\w2-sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer2]\w, sgx(id)\h2-sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h, appearance_fill)
                    sgx_Borderbox(sgx(id)\design, 0,0, sgx(id)\w2,sgx(id)\h2, appearance_border)
                  EndIf
                  If sgx(id)\design\onEndDraw
                    sgx(id)\design\onEndDraw(id, sprType, sgx(id)\w2, sgx(id)\h2)
                  EndIf
                StopDrawing()
              EndIf
          EndSelect
        EndProcedure
        
        
        ;** sgx_DrawEdit
        ;* redraws a particular state sprite of an sgxEdit element (called by <a href=#f_sgxrefresh>sgxRefresh</a>).
        ;** .id      : ID of the sgx element
        ;** .sprType : the state / sprite type that should be redrawn (not all elements use every state): #sgx_GadgetSprNormal or #sgx_GadgetSprHover etc.
        ;** .updateMetricsOnly : #True to update only metrics (calculate text offset and cursor position) and skip the actual drawing. #False (default) to also redraw.
        ;** .moveTextpos       : #False (default) to use current char pos (while typing / cursor keys). #True to move the character position to the current textcursor pixel position (while cursor positioning by mouse click)
        ;** .scrollToCursor    : #True (default) to scroll to current character/cursor position. #False while scrolling via mouse wheel.
        Procedure sgx_DrawEdit(id, sprType, updateMetricsOnly=#False, moveTextpos=#False, scrollToCursor=#True)
          Static textCursorHeight
          Static color1, color2, color3, color4
          Static appearance_fill
          Static appearance_border
          Static tempImg
          Static linesAboveCursor, charsBeforeCursor, cursorX, cursorY
          Static i, textLen, lines, countChars, lineWidth, currentLine, line.s, skipTraversal
          Static *c.Character
          Static skipLines, useLines, startPos, endPos, check
          Static draw
          
          ; reuse tmp image if possible
          If  Not tempImg  Or  Not IsImage(tempImg)  Or  sgx(id)\w2 <> ImageWidth(tempImg)  Or  sgx(id)\h2 <> ImageHeight(tempImg)
            sgx_FreeImageChecked(tempImg)
            tempImg = CreateImage(#PB_Any, sgx(id)\w2, sgx(id)\h2, 32)
            If Not tempImg
              sgx_err("image creation failed with size "+Str(sgx(id)\w2)+", "+Str(sgx(id)\h2)+": sgx("+id+")")
            EndIf
          EndIf
          
          ; calculate text offset and cursor position
          If updateMetricsOnly
            
            If tempImg
              ; we only start drawing here for using TextHeight() and TextWidth()
              StartDrawing(ImageOutput(tempImg))
                DrawingFont(sgx(id)\textfontID)
                sgx(id)\texth = TextHeight("X")
                
                ; get cursor line and char pos
                
                linesAboveCursor = 0
                charsBeforeCursor = 0
                If moveTextpos=#False
                  ; calculate the cursor pixel position (textcursor\x \y) from current character position (textpos)
                  ; normal (i.e. typing and cursor keys)
                  
                  DisableDebugger
                  If sgx(id)\textpos > 0
                    ; go backwards in text from textpos and count the characters in it's line and count the lines above it
                    *c = @sgx(id)\text + (sgx(id)\textpos-1) * SizeOf(Character)
                    For i=sgx(id)\textpos-1 To 0 Step -1
                      ;c = PeekC(@sgx(id)\text + i * SizeOf(Character))
                      If *c\c = #LF
                        linesAboveCursor + 1
                      EndIf
                      If linesAboveCursor = 0
                        charsBeforeCursor + 1
                      EndIf
                      *c - SizeOf(Character)
                    Next
                  EndIf
                  EnableDebugger
                  
                Else
                  ; calculate the character position (textpos) from current cursor pixel position (textcursor\x \y) 
                  ; (i.e. cursor positioning by mouseclick)
                  
                  DisableDebugger
                  textLen = Len(sgx(id)\text)
                  countChars = #False
                  lineWidth = 0
                  linesAboveCursor = (sgx(id)\scrollpos + sgx(id)\textcursor\y) / sgx(id)\texth
                  lines = CountString(sgx(id)\text, #LF$) + 1
                  skipTraversal = #False
                  If linesAboveCursor > (lines-1)
                    ; if clicked in an empty space in the bottom part of an edit where no
                    ; text might exist then we use the last line / end of text
                    linesAboveCursor = lines - 1
                    line = StringField(sgx(id)\text, lines, #LF$)
                    charsBeforeCursor = Len(line)
                    sgx(id)\textpos = textLen
                    skipTraversal = #True
                  EndIf
                  
                  If Not skipTraversal
                    currentLine = 1
                    *c = @sgx(id)\text
                    For i=0 To textLen-1
                      ;c = PeekC(@sgx(id)\text + i * SizeOf(Character))
                      
                      ; count lines
                      If *c\c = #LF
                        If countChars
                          ; stop if eol while counting chars (i.e. if clicked beyound the end of line)
                          Break
                        Else
                          currentLine + 1
                        EndIf
                      EndIf
                      
                      ; count chars and sum up char width until intended x position is reached
                      If countChars
                        If *c\c <> #LF
                          lineWidth + TextWidth(Chr(*c\c))
                          If lineWidth < (sgx(id)\textalign + sgx(id)\textcursor\x)
                            charsBeforeCursor + 1
                          Else
                            ; reached intended x/y position
                            Break
                          EndIf
                        Else
                          Break
                        EndIf
                      ElseIf (currentLine > linesAboveCursor)
                        ; we reached the intended line. now we count characters from beginning to indented position or eol
                        countChars = #True
                        
                        ; if first line is intended..
                        If linesAboveCursor = 0
                          If *c\c = #LF
                            ; ..and is empty line then just stop
                            Break
                          Else
                            ; ..otherwise count the first char explicitly
                            lineWidth + TextWidth(Chr(*c\c))
                            If lineWidth < (sgx(id)\textalign + sgx(id)\textcursor\x)
                              charsBeforeCursor + 1
                            Else
                              ; reached intended x/y position (beginning of text)
                              Break
                            EndIf
                          EndIf
                        EndIf
                      EndIf
                      *c + SizeOf(Character)
                    Next
                    sgx(id)\textpos = i
                    EnableDebugger
                  EndIf
                EndIf
                
                ; cursor pixel position in complete text
                cursorX = TextWidth(Mid(sgx(id)\text, 1 + sgx(id)\textpos - charsBeforeCursor, charsBeforeCursor))
                cursorY = linesAboveCursor * sgx(id)\texth
                
                ; move view (scroll) if cursor would be outside of client area.
                ; scrollToCursor is false when scrolling by mousewheel.
                If scrollToCursor
                  ; left edge contact
                  If cursorX < sgx(id)\textalign
                    sgx(id)\textalign - (sgx(id)\textalign - cursorX)
                  EndIf
                  ; right edge contact
                  If cursorX > (sgx(id)\textalign + sgx(id)\w2)
                    sgx(id)\textalign + (cursorX - (sgx(id)\textalign + sgx(id)\w2))
                  EndIf
                  ; top edge contact
                  If cursorY < sgx(id)\scrollpos
                    sgx(id)\scrollpos - (sgx(id)\scrollpos - cursorY)
                  EndIf
                  ; bottom edge contact
                  If cursorY > (sgx(id)\scrollpos + sgx(id)\h2 - sgx(id)\texth)
                    sgx(id)\scrollpos + (cursorY - (sgx(id)\scrollpos + sgx(id)\h2 - sgx(id)\texth))
                  EndIf
                EndIf
                
                ; translate in-text pixel position to in-clientarea pixel position
                sgx(id)\textcursor\x = cursorX - sgx(id)\textalign
                sgx(id)\textcursor\y = cursorY - sgx(id)\scrollpos
                
              StopDrawing()
            EndIf
            
          Else
            ; actual drawing (not updateMetricsOnly)
            
            Select sprType
            Case #sgx_GadgetSprNormal, #sgx_GadgetSprHover
          
              If sprType = #sgx_GadgetSprNormal
                color1 = sgx(id)\design\basecolor
                appearance_fill = #sgx_AppearanceNormal
                appearance_border = #sgx_AppearanceNormal
              ElseIf sprType = #sgx_GadgetSprHover
                color1 = sgx(id)\design\c[09]
                appearance_fill = #sgx_AppearanceHover
                appearance_border = #sgx_AppearanceHover
              EndIf
              
              ; text will be drawn on temporary image first for clipping to client area
              If tempImg
                StartDrawing(ImageOutput(tempImg))
                  DrawingMode(#PB_2DDrawing_AllChannels)
                  Box(0,0,sgx(id)\w,sgx(id)\h, RGBA(0,0,0,0))
                  DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Transparent)
                  DrawingFont(sgx(id)\textfontID)
                  
                  ; get lines visible inside clientarea and get the start/end
                  ; character position in text to draw only non-clipped text
                  
                  DisableDebugger
                  skipLines = sgx(id)\scrollpos / sgx(id)\texth
                  useLines = sgx(id)\h2 / sgx(id)\texth + 3
                  textLen = Len(sgx(id)\text)
                  currentLine = 1
                  startPos = -1
                  endPos = -1
                  If skipLines = 0
                    startPos = 0
                  EndIf
                  
                  check = #True
                  
                  *c = @sgx(id)\text
                  For i=0 To textLen-1
                    ;c = PeekC(@sgx(id)\text + i * SizeOf(Character))
                    
                    ; check only once per line
                    If check
                      check = #False
                      ; get start character position if not done yet and unnecessary lines have been skipped
                      If startPos = -1 And currentLine > skipLines
                        startPos = i
                      EndIf
                      If startPos >= 0 And currentLine > (skipLines + useLines)
                        ; get end character position (outside the loop after 'Next' in case we never get here)
                        Break
                      EndIf
                    EndIf
                    If *c\c = #LF
                      currentLine + 1
                      check = #True
                    EndIf
                    *c + SizeOf(Character)
                  Next
                  endpos = i + 1
                  
                  ; draw only visible part of text but add y-offset for the skipped lines
                  sgx_DrawTextLF( -sgx(id)\textalign, -(sgx(id)\scrollpos - (skipLines) * sgx(id)\texth), Mid(sgx(id)\text, startPos+1, endPos-startPos), color1 )
                  EnableDebugger
                  
                StopDrawing()
              EndIf
              
              sgx_prepareSprite(sgx(id)\spr[sprType], sgx(id)\w, sgx(id)\h, id)
              If sgx(id)\spr[sprType]
                StartDrawing(SpriteOutput(sgx(id)\spr[sprType]))
                  DrawingMode(#PB_2DDrawing_AllChannels)
                  Box(0,0,sgx(id)\w,sgx(id)\h, RGBA(0,0,0,0))
                  draw = #sgxDrawDesign | #sgxDrawText
                  If sgx(id)\design\onBeginDraw
                    draw = sgx(id)\design\onBeginDraw(id, sprType, sgx(id)\w, sgx(id)\h)
                  EndIf
                  If draw & #sgxDrawDesign
                    DrawingMode(#PB_2DDrawing_AlphaBlend)
                    sgx_Fill( sgx(id)\design, sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w, sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h, sgx(id)\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer2]\w, sgx(id)\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h, appearance_fill)
                  EndIf
                  
                  If draw & (#sgxDrawText | #sgxDrawTextNoOffset)
                    ; draw our client area tempImage (the visible text) onto the sprite
                    If tempImg
                      DrawImage(ImageID(tempImg), sgx(id)\x2, sgx(id)\y2, sgx(id)\w2, sgx(id)\h2)
                    EndIf
                  EndIf
                  
                  If draw & #sgxDrawDesign
                    sgx_Borderbox(sgx(id)\design, 0,0, sgx(id)\w,sgx(id)\h, appearance_border)
                  EndIf
                  
                  If sgx(id)\design\onEndDraw
                    sgx(id)\design\onEndDraw(id, sprType, sgx(id)\w, sgx(id)\h)
                  EndIf
                StopDrawing()
              EndIf
              
            Case #sgx_GadgetSprTextCursor
              
              ; textcursors are hardcoded / not a design issue
              
              color1 = sgx(id)\design\c[09]
              color2 = sgx(id)\design\basecolor
              color3 = sgx(id)\design\c[05]
              color4 = sgx(id)\design\c[11]
              textCursorHeight = 1
              If sgx(id)\texth > 0
                textCursorHeight = sgx(id)\texth
              EndIf
              sgx_prepareSprite(sgx(id)\spr[sprType], 3, textCursorHeight, id)
              If sgx(id)\spr[sprType]
                StartDrawing(SpriteOutput(sgx(id)\spr[sprType]))
                  DrawingMode(#PB_2DDrawing_AllChannels)
                  Box(0,0,3,textCursorHeight, RGBA(0,0,0,0))
                  draw = #sgxDrawDesign | #sgxDrawText
                  If sgx(id)\design\onBeginDraw
                    draw = sgx(id)\design\onBeginDraw(id, sprType, 3, textCursorHeight)
                  EndIf
                  If draw & #sgxDrawDesign
                    DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Outlined)
                    Select sgx_textcursorType
                      Case #sgxTextcursorTypeIBeam
                        LineXY(0,0,0,0, color2)
                        LineXY(1,0,1,0, color3)
                        LineXY(2,0,2,0, color2)
                        LineXY(0,textCursorHeight-1,0,textCursorHeight-1, color2)
                        LineXY(1,textCursorHeight-1,1,textCursorHeight-1, color3)
                        LineXY(2,textCursorHeight-1,2,textCursorHeight-1, color2)
                        LineXY(1,1,1,textCursorHeight-2, color1)
                      Case #sgxTextcursorTypeTeeth
                        LineXY(0,0,2,0, color2)
                        LineXY(0,textCursorHeight-1,2,textCursorHeight-1, color2)
                        LineXY(1,0,1,2, color1)
                        LineXY(1,textCursorHeight-3,1,textCursorHeight-1, color1)
                        LineXY(0,1,0,1, color3)
                        LineXY(2,1,2,1, color3)
                        LineXY(0,textCursorHeight-2,0,textCursorHeight-2, color3)
                        LineXY(2,textCursorHeight-2,2,textCursorHeight-2, color3)
                      Case #sgxTextcursorTypeLine
                        LineXY(1,0,1,textCursorHeight-1, color1)
                      Case #sgxTextcursorTypeLine2
                        LineXY(0,0,0,textCursorHeight-1, color1)
                        LineXY(1,0,1,textCursorHeight-1, color1)
                      Case #sgxTextcursorTypeLine3
                        LineXY(0,0,0,textCursorHeight-1, RGBA(Red(color1),Green(color1),Blue(color1),Alpha(color1)*0.4))
                        LineXY(1,0,1,textCursorHeight-1, RGBA(Red(color4),Green(color4),Blue(color4),Alpha(color4)*0.5))
                        LineXY(2,0,2,textCursorHeight-1, RGBA(Red(color1),Green(color1),Blue(color1),Alpha(color1)*0.4))
                      Case #sgxTextcursorTypeCaret
                        LineXY(0,textCursorHeight-1,0,textCursorHeight-1, color1)
                        LineXY(1,textCursorHeight-2,1,textCursorHeight-2, color1)
                        LineXY(2,textCursorHeight-1,2,textCursorHeight-1, color1)
                        LineXY(1,textCursorHeight-1,1,textCursorHeight-1, color3)
                    EndSelect
                  EndIf
                  If sgx(id)\design\onEndDraw
                    sgx(id)\design\onEndDraw(id, sprType, 3, textCursorHeight)
                  EndIf
                StopDrawing()
              EndIf
            EndSelect
          EndIf
        EndProcedure
        
        
        ;** sgx_DrawWin
        ;* redraws a particular state sprite of an sgxWin/sgxTable element (called by <a href=#f_sgxrefresh>sgxRefresh</a>).
        ;** .id      : ID of the sgx element
        ;** .sprType : the state / sprite type that should be redrawn (not all elements use every state): #sgx_GadgetSprNormal or #sgx_GadgetSprHover etc.
        Procedure sgx_DrawWin(id, sprType)
          Static color1
          Static color2
          Static color3
          Static color4
          Static color5
          Static color6
          Static appearance_fill
          Static appearance_border
          Static margin = 4   ; (for minimized)
          Static i, k, x, y
          Static *c.Character, textLen, width, maxWidth, title.s
          Static text.s
          Static draw
          
          ;{ maximized
          If sgx(id)\state <> #sgxWinStateMinimized
            Select sprType
            Case #sgx_GadgetSprNormal
              color1 = sgx(id)\design\basecolor
              color2 = sgx(id)\design\c[01]
              color5 = sgx(id)\design\c[16]
              color6 = sgx(id)\design\c[17]
              appearance_fill = #sgx_AppearanceNormal
              appearance_border = #sgx_AppearanceNormal
            Case #sgx_GadgetSprHover
              color1 = sgx(id)\design\c[09]
              color2 = sgx(id)\design\c[01]
              color5 = sgx(id)\design\c[18]
              color6 = sgx(id)\design\c[19]
              appearance_fill = #sgx_AppearanceNormal
              If sgx(id)\type = #sgxTypeTable
                appearance_fill = #sgx_AppearanceHover
              EndIf
              appearance_border = #sgx_AppearanceHover
            Case #sgx_GadgetSprPushed
              color1 = sgx(id)\design\c[09]
              color2 = sgx(id)\design\c[01]
              color5 = sgx(id)\design\c[18]
              color6 = sgx(id)\design\c[19]
              appearance_fill = #sgx_AppearanceHover
              appearance_border = #sgx_AppearanceHover
            EndSelect
            
            Select sprType
            Case #sgx_GadgetSprNormal,  #sgx_GadgetSprHover,  #sgx_GadgetSprPushed
              
              If sgx(id)\type = #sgxTypeTable And sgx(id)\style & #sgxStyleNoBG
                ; no background sprites needed
              Else
                
                sgx_prepareSprite(sgx(id)\spr[sprType], sgx(id)\w, sgx(id)\h, id)
                If sgx(id)\spr[sprType]
                  StartDrawing(SpriteOutput(sgx(id)\spr[sprType]))
                    DrawingMode(#PB_2DDrawing_AllChannels)
                    Box(0,0,sgx(id)\w,sgx(id)\h, RGBA(0,0,0,0))
                    draw = #sgxDrawDesign | #sgxDrawText
                    If sgx(id)\design\onBeginDraw
                      draw = sgx(id)\design\onBeginDraw(id, sprType, sgx(id)\w, sgx(id)\h)
                    EndIf
                    If draw & #sgxDrawDesign
                      DrawingMode(#PB_2DDrawing_AlphaBlend)
                      
                      sgx_Fill( sgx(id)\design, sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w, sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h, sgx(id)\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer2]\w, sgx(id)\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h, appearance_fill)
                      
                      ;  ------------------------------
                      ;{ draw grid (tables only)
                      
                      If sgx(id)\type = #sgxTypeTable  And  Not sgx(id)\style & #sgxStyleNoBG
                        
                        ; ver lines |
                        
                        If sgx(id)\style & #sgxStyleGridV
                          For i=0 To sgx(id)\table\w-2
                            x = PeekL( sgx(id)\table\gridx + i*SizeOf(Long) )
                            ; don't draw useless grid lines for empty table columns
                            ;If x
                            ;  LineXY(x  ,0, x  ,sgx(id)\h-1, color5)
                            ;  LineXY(x+1,0, x+1,sgx(id)\h-1, color6)
                            ;EndIf
                            ; don't draw useless grid lines for empty table columns
                            If x And x < sgx(id)\w
                              For y=0 To sgx(id)\h-1
                                ;If ..check bounds
                                Plot(x  , y, sgx_fade(Point(x  , y),-200)|1 )
                                Plot(x+1, y, sgx_fade(Point(x+1, y),  50)|1 )
                              Next
                            EndIf
                          Next
                        EndIf
                        
                        ; hor lines --
                        
                        If sgx(id)\style & #sgxStyleGridH
                          For k=0 To sgx(id)\table\h-2
                            y = PeekL( sgx(id)\table\gridy + k*SizeOf(Long) )
                            ;; don't draw useless grid lines for empty table lines/rows
                            ;If y
                            ;  LineXY(0,y  , sgx(id)\w-1,y  , color5)
                            ;  LineXY(0,y+1, sgx(id)\w-1,y+1, color6)
                            ;EndIf
                            ; don't draw useless grid lines for empty table lines/rows
                            If y And y < sgx(id)\h
                              For x=0 To sgx(id)\w-1
                                ;If ..check bounds
                                Plot(x, y  , sgx_fade(Point(x, y  ),-200)|1 )
                                Plot(x, y+1, sgx_fade(Point(x, y+1),  50)|1 )
                              Next
                            EndIf
                          Next
                        EndIf
                      EndIf
                      ;}
                      ; ------------------------------
                      
                      ; separator below the titlebar
                      If sgx(id)\width
                        ; heeere (win line)
                        sgx_BorderLine(sgx(id)\design, 0,sgx(id)\width-sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h, sgx(id)\w, appearance_border, #sgx_SideBottomOrRight)
                      EndIf
                      
                      ; window border
                      sgx_Borderbox(sgx(id)\design, 0,0, sgx(id)\w, sgx(id)\h, appearance_border)
                    EndIf
                    
                    If draw & (#sgxDrawText | #sgxDrawTextNoOffset)
                      ; draw title if window has a titlebar
                      If sgx(id)\width
                        DrawingFont(sgx(id)\textfontID)
                        DrawingMode(#PB_2DDrawing_AlphaBlend |#PB_2DDrawing_Transparent)
                        ; this means: maxWidth = xBeginOfCloseGadget - xBeginningOfText
                        maxWidth = (sgx(id)\sys0\x1 - (sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w+1) - TextWidth("..") - 1)
                        width = 0
                        textLen = Len(sgx(id)\text)
                        *c = @sgx(id)\text
                        ; detect if title is to long
                        For i=0 To textLen-1
                          width + TextWidth(Chr(*c\c))
                          If width > maxWidth
                            Break
                          EndIf
                          *c + SizeOf(Character)
                        Next
                        ; shorten title if too long
                        title = Mid(sgx(id)\text,1,i)
                        If i < textLen
                          title + ".."
                        EndIf
                        sgx_DrawTextLF( sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w+1,sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h+1, title, color1 )
                      EndIf
                    EndIf
                    
                    If sgx(id)\design\onEndDraw
                      sgx(id)\design\onEndDraw(id, sprType, sgx(id)\w, sgx(id)\h)
                    EndIf
                  StopDrawing()
                EndIf
              EndIf
              
              ;close gadget
              Case #sgx_GadgetSprNormal2
                sgx_FreeSpriteChecked(sgx(id)\spr[sprType])
                sgx(id)\spr[sprType] = sgx_CopyImageToSprite(sgx(id)\design\img[#sgx_DesignCloseGadgetNormal]\nr, sgx(id)\design\onBeginDraw, sgx(id)\design\onEndDraw, id, sprType)
              Case #sgx_GadgetSprHover2
                sgx_FreeSpriteChecked(sgx(id)\spr[sprType])
                sgx(id)\spr[sprType] = sgx_CopyImageToSprite(sgx(id)\design\img[#sgx_DesignCloseGadgetHover]\nr, sgx(id)\design\onBeginDraw, sgx(id)\design\onEndDraw, id, sprType)
          
            EndSelect
          ;}
          
          
          ;{ minimized (draw as taskbar buttons)
          Else
            Select sprType
            Case #sgx_GadgetSprNormal
              color1 = sgx(id)\design\basecolor
              color2 = sgx(id)\design\c[05]
              color3 = sgx(id)\design\c[03]
              color4 = sgx(id)\design\c[02]
              appearance_fill = #sgx_AppearanceNormal
              appearance_border = #sgx_AppearanceNormal
            Case #sgx_GadgetSprHover
              color1 = sgx(id)\design\c[09]
              color2 = sgx(id)\design\c[01]
              color3 = sgx(id)\design\c[01]
              color4 = sgx(id)\design\c[03]
              appearance_fill = #sgx_AppearanceHover
              appearance_border = #sgx_AppearanceHover
            EndSelect
            
            sgx_prepareSprite(sgx(id)\spr[sprType], sgx(id)\w, sgx(id)\h, id)
            If sgx(id)\spr[sprType]
              StartDrawing(SpriteOutput(sgx(id)\spr[sprType]))
                DrawingMode(#PB_2DDrawing_AllChannels)
                Box(0,0,sgx(id)\w,sgx(id)\h, RGBA(0,0,0,0))
                draw = #sgxDrawDesign | #sgxDrawText
                If sgx(id)\design\onBeginDraw
                  draw = sgx(id)\design\onBeginDraw(id, sprType, sgx(id)\w, sgx(id)\h)
                EndIf
                If draw & #sgxDrawDesign
                  sgx_Fill( sgx(id)\design, sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w, sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h, sgx(id)\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer2]\w, sgx(id)\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h, appearance_fill)
                  sgx_Borderbox(sgx(id)\design, 0,0, sgx(id)\w, sgx(id)\h, appearance_border)
                EndIf
                If draw & (#sgxDrawText | #sgxDrawTextNoOffset)
                  DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Outlined | #PB_2DDrawing_Transparent)
                  ; use a uniform font for all taskbar buttons
                  DrawingFont(FontID(sgxStdFont))
                  ; shorten title if too long
                  If TextWidth(sgx(id)\text) < sgx(id)\w-margin
                    text = sgx(id)\text
                  Else
                    i=2
                    While TextWidth( Left(sgx(id)\text,i)+".." ) < sgx(id)\w-margin
                      i+1
                    Wend
                    text = Left(sgx(id)\text,i-1)+".."
                  EndIf
                  sgx_DrawTextLF( 3,3, text, color1 )
                EndIf
                If sgx(id)\design\onEndDraw
                  sgx(id)\design\onEndDraw(id, sprType, sgx(id)\w, sgx(id)\h)
                EndIf
              StopDrawing()
            EndIf
          EndIf ;}
        
        EndProcedure
        
        
        ;** sgx_DrawTip
        ;* redraws the tooltip sprite of an sgx element (called by <a href=#f_sgxrefresh>sgxRefresh</a>).
        ;** .id      : ID of the sgx element
        Procedure sgx_DrawTip(id)
          Protected i
          Protected texth
          Protected w
          Protected h
          Protected temp
          Protected max
          Protected margin = 3
          Protected draw
        
          StartDrawing(ScreenOutput())
            DrawingFont(sgx(id)\textfontID)
            texth = TextHeight(sgx(id)\tip\text)
            max = CountString(sgx(id)\tip\text,#LF$)+1
            For i=1 To max
              temp = TextWidth(StringField(sgx(id)\tip\text,i,#LF$))
              If temp > w
                w = temp
              EndIf
              h + texth
            Next
          StopDrawing()
          w + 2*margin
          h + 2*margin
          ; the tip has its own sprite, wich is not part of the sgx(id)\spr[sprType] array
          sgx_prepareSprite(sgx(id)\tip\spr, w, h, id)
          If sgx(id)\tip\spr
            StartDrawing(SpriteOutput(sgx(id)\tip\spr))
              DrawingMode(#PB_2DDrawing_AllChannels)
              Box(0,0,w,h, RGBA(0,0,0,0))
              draw = #sgxDrawDesign | #sgxDrawText
              If sgx(id)\design\onBeginDraw
                draw = sgx(id)\design\onBeginDraw(id, #sgx_GadgetSprTip, w, h)
              EndIf
              If draw & #sgxDrawDesign
                DrawingMode(#PB_2DDrawing_AlphaBlend)
                ;Box(0,0, w,h, sgx(id)\design\c[01])
                sgx_Fill( sgx(id)\design, 2,2, w-4,h-4, #sgx_AppearanceNormal)
                DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Outlined)
                Box(0,0, w,h, sgx(id)\design\c[03])
                Box(1,1, w-2,h-2, sgx(id)\design\c[02])
              EndIf
              If draw & (#sgxDrawText | #sgxDrawTextNoOffset)
                DrawingFont(sgx(id)\textfontID)
                DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Transparent)
                sgx_DrawTextLF(margin,margin,sgx(id)\tip\text,sgx(id)\design\basecolor)
              EndIf
              If sgx(id)\design\onEndDraw
                sgx(id)\design\onEndDraw(id, #sgx_GadgetSprTip, w, h)
              EndIf
            StopDrawing()
          EndIf
          sgx(id)\tip\w = w
          sgx(id)\tip\h = h
        EndProcedure
        
        
        ;** sgx_DrawResizeGadget
        ;* creates the resize gadget sprite of an sgxWin element if it does not exist yet. (called by <a href=#f_sgxrefresh>sgxRefresh</a>).
        ;** .id      : ID of the sgx element
        Procedure sgx_DrawResizeGadget(id)
          Static color1, color2, i, w, borderBottom, borderRight, border
          Static draw
          color1 = sgx(id)\design\basecolor
          color1 = RGBA($aa,$aa,$aa,$aa)
          color1 = $fffffffa
          color2 = $ff00000a
          color1 = sgx(id)\design\basecolor
          color2 = sgx(id)\design\c[01]
          w = sgx_resizeGadgetSize
          sgx_prepareSprite(sgx(id)\spr[#sgx_GadgetSprResize], w, w, id)
          If sgx(id)\spr[#sgx_GadgetSprResize]
            TransparentSpriteColor(sgx(id)\spr[#sgx_GadgetSprResize], RGBA(0,0,0,0))
            StartDrawing(SpriteOutput(sgx(id)\spr[#sgx_GadgetSprResize]))
              DrawingMode(#PB_2DDrawing_AllChannels)
              Box(0,0, w,w, RGBA(0,0,0,0))
              draw = #sgxDrawDesign | #sgxDrawText
              If sgx(id)\design\onBeginDraw
                draw = sgx(id)\design\onBeginDraw(id, #sgx_GadgetSprResize, w, w)
              EndIf
              If draw & #sgxDrawDesign
                DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Outlined | #PB_2DDrawing_Transparent)
                borderBottom = sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h
                borderRight  = sgx(id)\design\img[#sgx_DesignBorderNormalVer2]\w
                border = (borderBottom + borderRight) / 2
                For i=border To w-1 Step 4
                  LineXY(i+0, w-border,  w-border, i+0, color1)
                  LineXY(i+1, w-border,  w-border, i+1, color1)
                  LineXY(i+2, w-border,  w-border, i+2, color2)
                  LineXY(i+3, w-border,  w-border, i+3, color2)
                Next
                ; triangle
                ;color1 = $ffffffff
                ;LineXY(0,w, w,w, color1)
                ;LineXY(w,0, w,w, color1)
                ;LineXY(0,w, w,0, color1)
              EndIf
              If sgx(id)\design\onEndDraw
                sgx(id)\design\onEndDraw(id, #sgx_GadgetSprResize, w, w)
              EndIf
            StopDrawing()
          EndIf
        EndProcedure
        
        
        ;** sgx_DrawCombo
        ;* redraws a particular state sprite of an sgxCombo element (called by <a href=#f_sgxrefresh>sgxRefresh</a>).
        ;** .id      : ID of the sgx element
        ;** .sprType : the state / sprite type that should be redrawn (not all elements use every state): #sgx_GadgetSprNormal or #sgx_GadgetSprHover etc.
        Procedure sgx_DrawCombo(id, sprType)
          Static color1
          Static color2
          Static appearance_fill
          Static appearance_border
          Static arrowOffset
          Static x, y, i, k
          Static count
          Static h
          Static sw,sh,_x,_y,_c
          Static draw
          
          ; line height
          If sgx(id)\max=1
            h = sgx(id)\h2
          Else
            h = sgx(id)\h
          EndIf
          
          Select sprType
          Case #sgx_GadgetSprNormal
            color1 = sgx(id)\design\basecolor
            color2 = sgx(id)\design\c[01]
            appearance_fill = #sgx_AppearanceNormal
            appearance_border = #sgx_AppearanceNormal
            arrowOffset = 0
            count=1
          Case #sgx_GadgetSprHover
            color1 = sgx(id)\design\c[09]
            color2 = sgx(id)\design\c[01]
            appearance_fill = #sgx_AppearanceHover
            appearance_border = #sgx_AppearanceHover
            arrowOffset = 0
            count=1
          Case #sgx_GadgetSprPushed ; (the opened combo)
            color1 = sgx(id)\design\c[03]
            color2 = $0
            appearance_fill = #sgx_AppearanceNormal
            appearance_border = #sgx_AppearancePushed
            arrowOffset = 1
            count = CountString(sgx(id)\text,#LF$) + 2  ; (there is one more line/entry than number of linebreaks, plus 1 for displaying the currently selected entry)
          Case #sgx_GadgetSprHover2
            ; (the hover effect when combo is open)
            sgx_prepareSprite(sgx(id)\spr[sprType], sgx(id)\w, h+1, id)
            If sgx(id)\spr[sprType]
              StartDrawing(SpriteOutput(sgx(id)\spr[sprType]))
                DrawingMode(#PB_2DDrawing_AllChannels)
                Box(0,0,sgx(id)\w, h+1, RGBA(0,0,0,0))
                draw = #sgxDrawDesign | #sgxDrawText
                If sgx(id)\design\onBeginDraw
                  draw = sgx(id)\design\onBeginDraw(id, sprType, sgx(id)\w, h+1)
                EndIf
                If draw & #sgxDrawDesign
                  DrawingMode(#PB_2DDrawing_AlphaBlend)
                  sgx_Borderbox(sgx(id)\design, 0,0, sgx(id)\w-1, h+1, #sgx_AppearanceHover)
                  Box(0,0,sgx(id)\w-1, h+1, RGBA(0,0,0,0))
                EndIf
                If sgx(id)\design\onEndDraw
                  sgx(id)\design\onEndDraw(id, sprType, sgx(id)\w, h+1)
                EndIf
              StopDrawing()
            EndIf
            ProcedureReturn
            
          Default
            ProcedureReturn
          EndSelect
          
          ; (normal, hover and pushed/open)
          sgx_prepareSprite(sgx(id)\spr[sprType], sgx(id)\w, h*count, id)
          If sgx(id)\spr[sprType]
            sw = SpriteWidth(sgx(id)\spr[sprType])
            sh = SpriteHeight(sgx(id)\spr[sprType])
            StartDrawing(SpriteOutput(sgx(id)\spr[sprType]))
              DrawingMode(#PB_2DDrawing_AllChannels)
              Box(0,0,sgx(id)\w, h*count, RGBA(0,0,0,0))
              draw = #sgxDrawDesign | #sgxDrawText
              If sgx(id)\design\onBeginDraw
                draw = sgx(id)\design\onBeginDraw(id, sprType, sgx(id)\w, h*count)
              EndIf
              If draw & #sgxDrawDesign
                DrawingMode(#PB_2DDrawing_AlphaBlend)
                sgx_Fill( sgx(id)\design, sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w, sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h, sgx(id)\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer2]\w, h*count-sgx(id)\design\img[#sgx_DesignBorderNormalHor1]\h-sgx(id)\design\img[#sgx_DesignBorderNormalHor2]\h, appearance_fill)
              EndIf
              If draw & (#sgxDrawText | #sgxDrawTextNoOffset)
                DrawingFont(sgx(id)\textfontID)
                DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Transparent)
              EndIf
              
              ; draw the selected entry and all possible entries
              For i=0 To count-1
                If i=0
                  If draw & (#sgxDrawText | #sgxDrawTextNoOffset)
                    ; draw the selected entry
                    DrawText(  sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w+1,  h*i+(h-sgx(id)\texth)/2,  StringField(sgx(id)\text,sgx(id)\state,#LF$),  color1 )
                  EndIf
                Else
                  If draw & #sgxDrawDesign
                    ; draw horizontal separator lines
                    For x = 0 To sgx(id)\w-sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w-1
                      _y = h*i
                      If x>=0 And x<sw And _y>=0 And _y<sh
                        Plot(x, _y, sgx_fade(Point(x, _y),-200)|1 )
                      EndIf
                      _y = h*i+1
                      If x>=0 And x<sw And _y>=0 And _y<sh
                        Plot(x, _y, sgx_fade(Point(x, _y),  50)|1 )
                      EndIf
                    Next
                  EndIf
                  If draw & (#sgxDrawText | #sgxDrawTextNoOffset)
                    ; draw possible entries (i = 1..count-1)
                    DrawText(  sgx(id)\design\img[#sgx_DesignBorderNormalVer1]\w+1,  h*i+(h-sgx(id)\texth)/2,  StringField(sgx(id)\text,i,#LF$),  color1 )
                  EndIf
                EndIf
              Next
              If draw & #sgxDrawDesign
                sgx_Borderbox(sgx(id)\design, 0,0, sgx(id)\w, h*count, appearance_border)
                
                ; the 'open' arrow
                x = sgx(id)\w - h/2 - 8
                y = h/2 - 3
                For i=0 To 5
                  For k=0 To 12
                    If i And i>=k
                      _x = arrowOffset+x+i
                      _y = arrowOffset+y+k
                      If _x>=0 And _x<sw And _y>=0 And _y<sh
                        Plot(_x ,_y, sgx_fade(Point(_x ,_y),-250+arrowOffset*350)|1 )
                      EndIf
                      _x = arrowOffset+x+11-i
                      _y = arrowOffset+y+k
                      If _x>=0 And _x<sw And _y>=0 And _y<sh
                        Plot(_x ,_y, sgx_fade(Point(_x ,_y),-250+arrowOffset*350)|1 )
                      EndIf
                    EndIf
                  Next
                Next
              EndIf
              
              If sgx(id)\design\onEndDraw
                sgx(id)\design\onEndDraw(id, sprType, sgx(id)\w, h*count)
              EndIf
            StopDrawing()
          EndIf
        EndProcedure
        
        
        ;** sgx_DrawText
        ;* redraws a particular state sprite of an sgxText element (called by <a href=#f_sgxrefresh>sgxRefresh</a>).
        ;** .id      : ID of the sgx element
        ;** .sprType : the state / sprite type that should be redrawn (not all elements use every state): #sgx_GadgetSprNormal or #sgx_GadgetSprHover etc.
        Procedure sgx_DrawText(id, sprType)
          Static color1, color2, color3, color4
          Static appearance_fill, appearance_border
          Static w, h, margin
          Static draw
        
          Select sprType
          Case #sgx_GadgetSprNormal
            color1 = sgx(id)\design\basecolor
            color2 = sgx(id)\design\c[01]
            color3 = sgx(id)\design\c[03]
            color4 = sgx(id)\design\c[02]
            appearance_fill = #sgx_AppearanceNormal
            appearance_border = #sgx_AppearanceNormal
          Case #sgx_GadgetSprHover
            color1 = sgx(id)\design\c[09]
            color2 = sgx(id)\design\c[01]
            color3 = sgx(id)\design\c[05]
            color4 = sgx(id)\design\c[03]
            appearance_fill = #sgx_AppearanceHover
            appearance_border = #sgx_AppearanceHover
          Default
            ProcedureReturn
          EndSelect
          
          w = sgx(id)\w
          h = sgx(id)\h
          margin = sgx(id)\margin
          
          sgx_prepareSprite(sgx(id)\spr[sprType], w, h, id)
          If sgx(id)\spr[sprType]
            StartDrawing(SpriteOutput(sgx(id)\spr[sprType]))
              DrawingMode(#PB_2DDrawing_AllChannels)
              Box(0,0,w,h, RGBA(0,0,0,0))
              draw = #sgxDrawDesign | #sgxDrawText
              If sgx(id)\design\onBeginDraw
                draw = sgx(id)\design\onBeginDraw(id, sprType, w, h)
              EndIf
              If draw & #sgxDrawDesign
              DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Outlined)
                If sgx(id)\style & #sgxStyleThin 
                  sgx_Fill( sgx(id)\design, 2,2, w-4,h-4, appearance_fill)
                  ; heeere (text border)
                  DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Outlined)
                  Box(0,0, w,h, color3)
                  Box(1,1, w-2,h-2, color4)
                EndIf
              EndIf
              If draw & (#sgxDrawText | #sgxDrawTextNoOffset)
                DrawingMode(#PB_2DDrawing_AlphaBlend | #PB_2DDrawing_Transparent)
                DrawingFont(sgx(id)\textfontID)
                Select sgx(id)\textalign
                  Case #sgxAlignTextLeft
                    sgx_DrawTextLF(                 margin, margin, sgx(id)\text, color1 )
                  Case #sgxAlignTextRight
                    sgx_DrawTextLF( w-margin-sgx(id)\textw, margin, sgx(id)\text, color1, -1, #LF$, #sgxAlignTextRight )
                  Case #sgxAlignTextCenter
                    sgx_DrawTextLF( (w-sgx(id)\textw)/2,    margin, sgx(id)\text, color1, -1, #LF$, #sgxAlignTextCenter )
                  Case #sgxAlignTextJustifySpaces
                    sgx_DrawTextLF(                 margin, margin, sgx(id)\text, color1, -1, #LF$, #sgxAlignTextJustifySpaces )
                  Case #sgxAlignTextJustifyCharacters
                    sgx_DrawTextLF(                 margin, margin, sgx(id)\text, color1, -1, #LF$, #sgxAlignTextJustifyCharacters )
                EndSelect
              EndIf
              
              If sgx(id)\design\onEndDraw
                sgx(id)\design\onEndDraw(id, sprType, w, h)
              EndIf
            StopDrawing()
          EndIf
          
          ;sgx_Fill( sgx(id)\design, 0,0, sgx(id)\w, sgx(id)\h, appearance_fill)
          ;DrawingFont(sgx(id)\textfontID)
          ;DrawingMode(#PB_2DDrawing_Transparent|#PB_2DDrawing_Outlined)
          ;Box(0,0, sgx(id)\w, sgx(id)\h, $555555)
          ;DrawText(1,1, sgx(id)\text, color1)
        EndProcedure
        
        
        ;** sgx_DrawDisabled
        ;* modifies a particular state sprite of an sgx element to visually represent a 'disabled' state (called by <a href=#f_sgxrefresh>sgxRefresh</a>).
        ;** .id      : ID of the sgx element
        ;** .sprType : the state / sprite type that should be modified (not all states are modified to 'disabled'): #sgx_GadgetSprNormal or #sgx_GadgetSprPushed etc.
        Procedure sgx_DrawDisabled(id, sprType)
          Static w, h, x, y
          Static c, r,g,b,a, mean
          Static draw
          
          If sgx(id)\disabled  And  sgx(id)\spr[sprType]  And  IsSprite(sgx(id)\spr[sprType])
            w = SpriteWidth(sgx(id)\spr[sprType])
            h = SpriteHeight(sgx(id)\spr[sprType])
            ; reduce saturation and brightness of 'normal' sprite
            StartDrawing(SpriteOutput(sgx(id)\spr[sprType]))
              draw = #sgxDrawDesign | #sgxDrawText
              If sgx(id)\design\onBeginDraw
                draw = sgx(id)\design\onBeginDraw(id, sprType, w, h, #True)
              EndIf
              If draw & #sgxDrawDesign
                ; using alpha clip to keep transparent areas transparent (i.e. don't dim checkbox label background)
                DrawingMode(#PB_2DDrawing_Default | #PB_2DDrawing_AlphaClip)
                Box(0,0,w,h,sgx_disabledColor1)
                DrawingMode(#PB_2DDrawing_Default | #PB_2DDrawing_AlphaClip)
                Box(0,0,w,h,sgx_disabledColor2)
                
                ; (older method, slower)
                ; For x=0 To w-1
                ;   For y=0 To h-1
                ;     c = Point(x,y)
                ;     r = Red(c)
                ;     g = Green(c)
                ;     b = Blue(c)
                ;     a = Alpha(c)
                ;     ; reduce brightness
                ;     r * 0.8
                ;     g * 0.8
                ;     b * 0.8
                ;     ; get components mean brightness
                ;     mean = (r+g+b)/3
                ;     ; use mean between reduced component and components mean
                ;     r = (r + mean) / 2
                ;     g = (g + mean) / 2
                ;     b = (b + mean) / 2
                ;     Plot(x,y,RGBA(r,g,b,a))
                ;   Next
                ; Next
              EndIf
              If sgx(id)\design\onEndDraw
                sgx(id)\design\onEndDraw(id, sprType, w, h, #True)
              EndIf
            StopDrawing()
          EndIf
        EndProcedure

;}
;  <<<<<<<<<<<<<<<<<<<<<<<< merging end  sgx_draw.pbi

; XIncludeFile "sgx_examine.pbi"
;{ >>>>>>>>>>>>>>>>>>>>>>>> merging file sgx_examine.pbi

        
        ; Macro MouseWheel()
        ;   Bool(KeyboardPushed(#PB_Key_LeftControl)) * -sgx_m\dy
        ; EndMacro
        
        ; Procedure sgx_windowCallback(hWnd, uMsg, WParam, LParam)
        ;   Static wd.w
        ;   If uMsg = #WM_MOUSEWHEEL
        ;     wd.w = - ((WParam>>16) & $ffff)
        ;     wd.w / 120
        ;     sgx_mouseWheelDelta - wd
        ;   EndIf 
        ;   ProcedureReturn #PB_ProcessPureBasicEvents 
        ; EndProcedure 
        
        ;SetWindowCallback(@sgx_windowCallback())
        
        
        
        ;** sgxExamine
        ;* process actions / events and update the field data of all sgx element. _
        ;* it returns the ID of the sgx element that is hovered by the mouse, or -1 if the mouse isn't hovering an element. _
        ;* you can pass mouse coordinates if needed (for instance if you have self-calculated accelerated mouse movement). otherwise sgx will simply inquire the PB mouse library commands.
        ;** .mousex : (optional)
        ;** .mousey : (optional)
        Procedure sgxExamine( mousex=-1, mousey=-1)
          Static id, id2, parent, temp
          Static topID
          Static group
          Static emsPrev
          Static i, x, y, w, h, dy, lines
          Static textposPrev
          Static *it.sgx_llS_elem ; use Protected when using recursion!
          Static ignore
          Static otherFocused
          Static mousexPrev
          Static mouseyPrev
          Static *animation.sgxS_Animation
          Static changeLater.sgxS_ListWrapper
          Static destroyed
          Static editWasFocused
          Static sizeOk, dw, dh
          
          emsPrev=sgx_ems
          If *sgx_emsContainer
            sgx_ems = *sgx_emsContainer\i
          Else
            sgx_ems = ElapsedMilliseconds()
          EndIf
          
          ; ############################################################################
          ; ############################################################################
          
          ;{ -- GET MOUSE DATA
          
          mousexPrev = sgx_m\x
          mouseyPrev = sgx_m\y
          If mousex=-1
            sgx_m\x  = MouseX()
            sgx_m\dx = MouseDeltaX()
          Else
            sgx_m\x  = mousex
            sgx_m\dx = mousex - mousexPrev
          EndIf
          If mousey=-1
            sgx_m\y  = MouseY()
            sgx_m\dy = MouseDeltaY()
          Else
            sgx_m\y  = mousey
            sgx_m\dy = mousey - mouseyPrev
          EndIf
          
          ; (mouselib vs winapi)
          If 01
            ; use pb mouse lib for buttons and wheel
            ; problems:
            ; - mouse wheel may not work in windowed screen (http://www.purebasic.fr/english/viewtopic.php?f=4&t=65533)
            ; - mouse button may reset without actual button release (http://www.purebasic.fr/english/viewtopic.php?f=4&t=65799)
            
            If MouseWheel() >= 0
              sgx_m\wheelSign = 1
              sgx_m\wheel     = MouseWheel()
            Else
              sgx_m\wheelSign = -1
              sgx_m\wheel     = -MouseWheel()
            EndIf
            sgx_m\leftdown_prev = sgx_m\leftdown
            sgx_m\rightdown_prev = sgx_m\rightdown
            
            sgx_m\leftdown = MouseButton(1)
            sgx_m\rightdown = MouseButton(2)
            
            sgx_m\leftclick = 0
            If sgx_m\leftdown And Not sgx_m\leftdown_prev
              sgx_m\leftclick = 1
            EndIf
            sgx_m\leftup = 0
            If sgx_m\leftdown_prev And Not sgx_m\leftdown
              sgx_m\leftup = 1
            EndIf
            sgx_m\rightclick = 0
            If sgx_m\rightdown And Not sgx_m\rightdown_prev
              sgx_m\rightclick = 1
            EndIf
            sgx_m\rightup = 0
            If sgx_m\rightdown_prev And Not sgx_m\rightdown
              sgx_m\rightup = 1
            EndIf
          Else
            ; window callback (see above) not working in fullscreen?
            
            ; If sgx_mouseWheelDelta >= 0
            ;   ; scrolling 'up' (or not scrolling)
            ;   sgx_m\wheelSign = 1
            ;   sgx_m\wheel     = sgx_mouseWheelDelta
            ; Else
            ;   ; scrolling 'down'
            ;   sgx_m\wheelSign = -1
            ;   sgx_m\wheel     = -sgx_mouseWheelDelta
            ; EndIf
            ; sgx_mouseWheelDelta = 0
            
            ; If GetAsyncKeyState_(#VK_LBUTTON)&(1<<15)
            ;   sgx_m\leftdown = 1
            ; Else
            ;   sgx_m\leftdown = 0
            ; EndIf
            ; If GetAsyncKeyState_(#VK_RBUTTON)&(1<<15)
            ;   sgx_m\rightdown = 1
            ; Else
            ;   sgx_m\rightdown = 0
            ; EndIf
            
          EndIf
          ;}
          ;  --
          
          ; ############################################################################
          ; ############################################################################
          
          ; global topID
          sgxTopID = sgxGetElementAt(sgx_m\x, sgx_m\y)
          ; static local topID
          ; keep static topID until mouse release, for example during scrollbar/trackbar knob dragging even
          ; if mouse is outside the element
          If Not sgx_m\leftdown
            topID = sgxTopID
          EndIf
          
          ; reset window focus if clicked 'nowhere'
          If sgxActiveWin <> -1  And  topID = -1  And  (sgx_m\leftup Or sgx_m\rightup)
            sgxActiveWin = -1
          EndIf
          
          ; we check if an edit is currently focused before we loop through the elements because if the edit looses
          ; focus, elements processed after the edit would already receive events but we want to behave them as if
          ; the edit still had focus for this examine/frame.
          editWasFocused = 0
          If sgx_editing<>-1
            editWasFocused = sgx_editing
          EndIf
          
          ; ############################################################################
          ; ############################################################################
          
          For id=0 To sgx_CURRENT_MAX_DYNAMIC_GADGETS    ;   <<<<<<<      MAIN LOOP
            
            ; ############################################################################
            ; ############################################################################
            
            ;{ -- KICKOUTS / event reset / triggering the pending events
            
            ; skip gap from highest static ID to first dynamic one
            sgx_SkipUnused(id)
            
            ; skip invalid
            If Not sgx(id)
              Continue
            EndIf
            
            ; reset single fired events
            sgx(id)\leftclick = 0
            sgx(id)\leftup = 0
            sgx(id)\rightclick = 0
            sgx(id)\rightup = 0
            
            sgx(id)\lefthold = 0
            
            sgx(id)\changed = 0
            sgx(id)\resized = 0
            sgx(id)\refreshed = 0
            
            ; mouse button release -> reset element mousedown and focus
            If sgx_m\leftup
              sgx(id)\leftdown = 0
              sgx(id)\lefthold_timer = 0
              If id = sgx_focusCatchedBy
                sgx(id)\leftup = 1
                sgx_focusCatchedBy = -1
              EndIf
              If id = sgx_moving
                sgx(id)\leftup = 1
                sgx_moving = -1
              EndIf
            EndIf
            If sgx_m\rightup
              sgx(id)\rightdown = 0
              sgx(id)\lefthold_timer = 0
              If id = sgx_focusCatchedBy
                sgx(id)\rightup = 1
                sgx_focusCatchedBy = -1
              EndIf
            EndIf
            
            ; we trigger the pending events that sgxSetState() or sgxResize() etc. might have caused
            If ListSize(sgx_pendingEvents())
              ForEach sgx_pendingEvents()
                If sgx_pendingEvents()\id = id
                  If sgx_pendingEvents()\changed
                    sgx(id)\changed = sgx_pendingEvents()\changed
                  EndIf
                  If sgx_pendingEvents()\resized
                    sgx(id)\resized = sgx_pendingEvents()\resized
                  EndIf
                  If sgx_pendingEvents()\refreshed
                    sgx(id)\refreshed = sgx_pendingEvents()\refreshed
                  EndIf
                  DeleteElement(sgx_pendingEvents())
                EndIf
              Next
            EndIf
            
            ; ###### CONTINUE / IGNORE THE ELEMENT IN SOME CASES
            
            ; if an edit or tackbar for example have catched focus, the current element
            ; does not need hover/tooltips ect and can be handled briefly
            otherFocused = #False
            If (sgx_focusCatchedBy<>-1 And id<>sgx_focusCatchedBy)  Or  (sgx_editing<>-1 And id<>sgx_editing)
              otherFocused = #True
            EndIf
            
            ignore = #False
            ; in most situations we can handle the element in a brief way
            ; i.e. when no focus and element is not hovered and not being resized (exeption: animated windows must be processed anyway)
            If sgx_focusCatchedBy=-1 And id<>topID And id<>sgx_resizing And id<>sgx_editing And (Not (sgx(id)\type=#sgxTypeWin And sgx(id)\state<>#sgxWinStateNormal))  ; And (Not sgxIsParentage(id,topID)
              ignore = #True
            EndIf
            
            If sgx(id)\disabled
              ignore = #True
              If id = topID; Or id = sgxTopID
                ; make all parents up to the first window hover too.
                sgx_setParentWindowHovered(id)
              EndIf
            EndIf
            
            If Not sgx(id)\show
              Continue
            ElseIf (otherFocused Or ignore)
              ; avoid hover change (flickering) while manually resizing elements (mouse may temporarily move outside the element)
              If sgx_resizing = -1
                sgx(id)\hover=0
              EndIf
              If sgx(id)\tip
                sgx(id)\tip\show = 0
                sgx(id)\tip\timer = 0
              EndIf
              Continue
            EndIf
            ;}
            ;  --
            
            ; ############################################################################
            ; ############################################################################
            
            ; ######## HOVER (type independent)
            
            If sgx_isInElement(id, sgx_m\x, sgx_m\y)
              
              ;{ -- set element hovered
              If id = topID
                ; avoid hover change (flickering) while manually resizing elements (mouse may temporarily move outside the element)
                If sgx_resizing = -1
                  sgx(id)\hover=1
                EndIf
              EndIf
              
              ; make all parents up to the first window hover too.
              ; avoid hover change (flickering) while manually resizing elements (mouse may temporarily move outside the element)
              If sgx_resizing = -1
                sgx_setParentWindowHovered(id)
              EndIf
              ;}
              ;  --
              
              
              ;{ -- resize (manually via mouse)
              ; initialize resizing if not done yet and if resize gadget clicked
              If sgx(id)\resizable  And  sgx_resizing = -1  And  sgx_m\leftclick  And  sgx_MousehoverResize(id)  And  (Not editWasFocused Or id=sgx_editing)
                ; resize windows only in normal state, combos only when closed
                If  (sgx(id)\type <> #sgxTypeWin Or sgx(id)\state=#sgxWinStateNormal)  And  (sgx(id)\type <> #sgxTypeCombo Or sgx(id)\max<>1)
                  sgx_resizing = id
                  
                  ; get mouse cursor distance from bottom right element corner, [-2,-1] for example.
                  sgx_vec\x = sgx_m\x - (sgx(id)\x + sgx(id)\w)
                  sgx_vec\y = sgx_m\y - (sgx(id)\y + sgx(id)\h)
                  
                  ; exit edit focused on resizing
                  If sgx(id)\type = #sgxTypeEdit And sgx_editing=id
                    sgx_editing = -1
                  EndIf
                EndIf
              EndIf ;}
              ;  --
              
              
              ;{ -- mouse events (hover)
              
              If id <> sgx_resizing
                
                ; catch focus with first mousedown
                
                ; LEFT
                If sgx_m\leftclick  And  ((Not editWasFocused) Or (id=editWasFocused))
                  sgx(id)\leftclick=1
                  sgx_focusCatchedBy = id
                  sgx(id)\leftdown=1 ; this will stay set until mouse release
                  sgx(id)\lefthold_timer = sgx_ems + sgxHoldDelay ; init timer for first hold event
                EndIf
                
                ; RIGHT
                If sgx_m\rightclick  And  ((Not editWasFocused) Or (id=editWasFocused))
                  sgx(id)\rightclick=1
                  sgx_focusCatchedBy = id
                  sgx(id)\rightdown=1 ; this will stay set until mouse release
                  sgx(id)\lefthold_timer = sgx_ems + sgxHoldDelay ; init timer for first hold event
                EndIf
                
              EndIf
              ;}
              ;  --
              
              
              ;{ -- make direct parent window active if child is clicked
              If sgx(id)\leftclick Or sgx(id)\rightclick
                ; skip areas, they don't move in front (i'm not sure if a parent window should become active nonetheless for areas?)
                If sgx(id)\type <> #sgxTypeArea
                  sgxMoveOnTop(id)
                EndIf
              EndIf
              ;}
              ;  --
              
              
              ;{ -- TIPS (show / hide)
              If sgx(id)\tip
                ; hide tip if mouse moves fast enough
                If (sgx_m\dx > sgx_m\dlimit)  Or (sgx_m\dy > sgx_m\dlimit) Or (sgx_m\dx < -sgx_m\dlimit)  Or (sgx_m\dy < -sgx_m\dlimit)
                  sgx(id)\tip\timer=0
                  If sgx(id)\tip\show = 1
                    sgx(id)\tip\show = -1  ; (mark as 'has been shown')
                  EndIf
                ElseIf sgx(id)\tip\timer=0  And  sgx(id)\tip\show = 0   ; (show=-1 if has already been shown)
                  ; init timer
                  sgx(id)\tip\timer = sgx_ems + sgxTipDelay
                ElseIf (sgx_ems > sgx(id)\tip\timer)  And  sgx(id)\tip\show=0
                  sgx(id)\tip\show = 1
                EndIf
              EndIf ;}
              ;  --
              
              
              ; ############################################################################
              ; ############################################################################
              
              ; ###### HOVER (type dependent)
              
              Select sgx(id)\type
                
                
                ;  -- BUTTONTOGGLE (hover)
                ;{ -- CHECKBOX     (hover)
                Case #sgxTypeButtonToggle, #sgxTypeCheckbox
                  If (Not editWasFocused) And id <> sgx_resizing
                    If sgx(id)\leftclick
                      If sgxGetGroup(id) <> -1
                        ; radiogroup element (we don't untoggle those by leftclick)
                        sgxSetState(id, 1, #True, @changeLater)
                      Else
                        ; no radiogroup element (toggle state)
                        sgxSetState(id, sgx(id)\state!1, #True, @changeLater)
                      EndIf
                    ElseIf sgx(id)\rightclick And (sgxRightclickUnset=#sgxRightclickUnsetAll Or (sgxRightclickUnset=#sgxRightclickUnsetNoGroup And sgxGetGroup(id)=-1))
                      ; untoggle according to righclick settings
                      sgxSetState(id, 0, #False, @changeLater)
                    EndIf
                  EndIf ;}
                ;  --
                
                
                ;{ -- WIN (hover)
                Case #sgxTypeWin
                  
                  If (Not editWasFocused)
                    If sgx(id)\leftclick And id=topID
          
                      ; minimize window
                      If sgx(id)\state = #sgxWinStateNormal  And  sgx(id)\width  And  sgx_MousehoverClosebutton(id)
                        If sgxActiveWin <> id
                          sgxMoveOnTop(id)
                        EndIf
                        sgxMinimizeWin(id)
                        
                      ; restore window
                      ElseIf sgx(id)\state = #sgxWinStateMinimized
                        sgxRestoreWin(id)
                      EndIf
          
                    ElseIf sgx(id)\rightclick
                      
                      ; close window
                      If sgx(id)\state = #sgxWinStateNormal  And  sgx(id)\width  And  sgx_MousehoverClosebutton(id)  And  sgxWinCloseEnabled
                        If sgxActiveWin <> id
                          sgxMoveOnTop(id)
                        EndIf
                        destroyed = sgxCloseWin(id) ; returns 1 if the element has been destroyed by a finished closing animation
                        If destroyed
                          Continue
                        EndIf
                      EndIf
                      
                    ElseIf sgx(id)\state=#sgxWinStateNormal And sgx(id)\leftdown
                      
                      ; activate window (make foreground)
                      If sgxActiveWin <> id
                        sgxMoveOnTop(id)
                      EndIf
                      
                      ; move
                      If sgx_moving = -1  And  sgx_m\y < (sgx(id)\y+sgx(id)\width) ; if not already dragging and mouse is in titlebar
                        sgx_moving = id
                        ; store the in-titlebar mouse position for proper positioning while moving
                        sgx_vec\x = sgx_m\x - sgx(id)\x
                        sgx_vec\y = sgx_m\y - sgx(id)\y
                      EndIf
                      
                    EndIf
                    
                  EndIf ;}
                ;  --
                
                
                ;{ -- TABLE (hover) <<<<<<<<<<<<<<<<<<<<
                Case #sgxTypeTable
                  
                  ;If (Not editWasFocused)
                    ; [nothing to handle yet]
                  ;EndIf
        ;}
                ;  --
                
                
                ;{ -- EDIT (hover)
                Case #sgxTypeEdit
                  If (Not editWasFocused)
                    If sgx(id)\leftclick And id <> sgx_resizing
                      ; focus
                      sgx_editing = id
                      ; make tip appear immediately on edit focus (..?)
                      If sgx(id)\tip
                        sgx(id)\tip\timer = sgx_ems
                      EndIf
                    EndIf
                  ElseIf sgx_editing=id And sgx(id)\leftclick
                    ; position textcursor by mouseclick
                    x = sgx_m\x - (sgx(id)\x + sgx(id)\x2)
                    y = sgx_m\y - (sgx(id)\y + sgx(id)\y2)
                    ; clicked in client area?
                    If x >= 0  And  x < sgx(id)\w2  And  y >= 0  And  y < sgx(id)\h2
                      ; store mouse click position in client area
                      sgx(id)\textcursor\x = x
                      sgx(id)\textcursor\y = y
                      textposPrev = sgx(id)\textpos
                      ; reposition textpos from cursor position (as opposed to other way round normally)
                      sgxRefresh(id, #sgx_refreshEditMouseclickCursor)
                      ; trigger a 'changed' event if 'textpos' changed
                      If sgx(id)\textpos <> textposPrev
                        AddElement(sgx_pendingEvents())
                        sgx_pendingEvents()\id = id
                        sgx_pendingEvents()\changed = 1
                      EndIf
                      ; avoid confusion in sgx_keyboardEdit() and cursor key handling
                      sgx_keyboardEdit_lastTextPos = sgx(id)\textpos
                    EndIf
                  EndIf ;}
                ;  --
                
                
                ;{ -- COMBO (hover)
                Case #sgxTypeCombo
                  If (Not editWasFocused)  And id <> sgx_resizing
                    If sgx(id)\leftclick
                      sgx(id)\max!1
                      If sgx(id)\max=1
                        ; open
                        sgx(id)\h2 = sgx(id)\h
                        sgx(id)\h = sgx(id)\h*(2+CountString(sgx(id)\text,#LF$))
                        sgx_llMoveEnd(sgx(id)\treeElem)
                      Else
                        ; close (and select)
                        temp = (sgx_m\y - sgx(id)\y) / sgx(id)\h2
                        sgx(id)\h = sgx(id)\h2
                        If temp
                          If sgx(id)\state<>temp
                            sgx(id)\changed = 1
                            sgx(id)\state=temp
                            sgxRefresh(id)
                          EndIf
                        EndIf
                      EndIf
                    ElseIf sgx(id)\rightclick
                      ; close (cancel)
                      If sgx(id)\max=1
                        sgx(id)\h = sgx(id)\h2
                        sgx(id)\max=0
                      EndIf
                    EndIf
                  EndIf ;}
                ;  --
          
                
              EndSelect
              
              
              ;{ -- MOUSEWHEEL (hover) (scrollbars/trackbars and edit)
              If sgx_m\wheel And (Not editWasFocused Or sgx_editing=id) And id <> sgx_resizing
                
                ; scrollbars/trackbars handled only
                sgx_SetScrollPos(id, sgx(id)\scrollpos - sgx_m\wheelSign * 10 * sgx_m\wheel)
                
                If sgx(id)\type = #sgxTypeEdit
                  If sgx_m\wheel
                    dy = (sgx_m\wheel * sgx(id)\texth) ; (texth is lineheight)
                    
                    ; scrolling up (reduce scrollpos)
                    If sgx_m\wheelSign > 0
                      If (sgx(id)\scrollpos - dy) < 0
                        dy - -(sgx(id)\scrollpos - dy)
                      EndIf
                      sgx(id)\scrollpos - dy
                      sgx(id)\textcursor\y - dy
                      
                    ; scrolling down (increase scrollpos)
                    ElseIf sgx_m\wheelSign < 0
                      lines = CountString(sgx(id)\text, #LF$) + 1
                      If (lines * sgx(id)\texth) < sgx(id)\h2
                        ; less lines than clientarea height
                        sgx(id)\scrollpos = 0
                        sgx(id)\textcursor\y = 0
                        dy = 0
                      ElseIf (sgx(id)\scrollpos + dy) > (lines * sgx(id)\texth - sgx(id)\h2)
                        dy - ( (sgx(id)\scrollpos + dy) - (lines * sgx(id)\texth - sgx(id)\h2) )
                      EndIf
                      sgx(id)\scrollpos + dy
                      sgx(id)\textcursor\y + dy
                    EndIf
                    ; #sgx_refreshEditMousewheelScroll disables 'scrollToCursor'
                    sgxRefresh(id, #sgx_refreshEditMousewheelScroll)
                  EndIf
                EndIf
              EndIf ;}
              ;  --
            
            ; ############################################################################
            ; ############################################################################
            
            ; ######## NO HOVER
            Else ;{
              
              ; unset hover
              ; (we won't get here if other element has catched focus)
              ; avoid hover change (flickering) while manually resizing elements (mouse may temporarily move outside the element)
              If sgx_resizing = -1
                sgx(id)\hover=0
              EndIf
              If sgx(id)\tip
                sgx(id)\tip\show = 0
                sgx(id)\tip\timer = 0
              EndIf
              ;}
            EndIf
        
            
            ; ############################################################################
            ; ############################################################################
            
            ; ######## HOVER INDEPENDENT
            
            ; there are some 'Continue's up there
            ; so you might not get here!
            
            Select sgx(id)\type
              
              ;{ -- SCROLLBARS, TRACKBARS (click or drag)
              Case #sgxTypeScrollbarVer, #sgxTypeTrackbarVer ; |
                If  (Not editWasFocused) And id <> sgx_resizing
                  If sgx_focusCatchedBy=id
                    sgx_SetScrollPos(id, sgx_m\y - sgx(id)\y)
                  EndIf
                EndIf
              Case #sgxTypeScrollbarHor, #sgxTypeTrackbarHor ; --
                If (Not editWasFocused) And id <> sgx_resizing
                  If sgx_focusCatchedBy=id
                    sgx_SetScrollPos(id, sgx_m\x - sgx(id)\x)
                  EndIf
                EndIf ;}
              ;  --
              
              
              ;{ -- EDIT (focused)
              Case #sgxTypeEdit
                If sgx_editing=id
                  If KeyboardReleased(#PB_Key_Escape)  Or  ( (sgx_m\leftdown Or sgx_m\rightdown) And Not (sgx_isInElement(id, sgx_m\x, sgx_m\y) And id=topID) )
                    ; exit edit
                    sgx_editing = -1
                    ; disable 'scrollToCursor' with #sgx_refreshEditMousewheelScroll
                    sgxRefresh(id, #sgx_refreshEditMousewheelScroll) ; (this call might be redundant)
                  Else
                    ; typing/focused
                    ; (we need to do this continuously while edit is focused to have proper keystroke delay reset)
                    ;If KeyboardPushed(#PB_Key_All) Or KeyboardReleased(#PB_Key_All)
                      sgx(id)\text = sgx_keyboardEdit( sgx(id)\text , sgx(id)\textpos, sgx(id)\max)
                    ;EndIf
                    sgx(id)\textpos = sgx_keyboardEdit_lastTextPos
                    If sgx_keyboardEdit_TextModified
                      sgx(id)\changed = 1
                      sgxRefresh(id)
                    EndIf
                  EndIf
                EndIf ;}
              ;  --
              
              
              ;{ -- WIN (animation and moving)
              Case #sgxTypeWin
                
                ; ------------------------------------------------------
                ; window animations
                
                *animation = 0
                
                If sgx(id)\state <> #sgxWinStateNormal
                  *animation = FindMapElement(sgx_animations(), Str(id))
                EndIf
                
                If *animation
                  
                  ; adjust the element properties according to the current animation progress
                  destroyed = sgx_animate(id, *animation) ; returns 1 if the element has been destroyed by a finished closing animation
                  If destroyed
                    Continue
                  EndIf
                EndIf
                
                ; ------------------------------------------------------
                ; moving window
                
                If sgx_moving = id
                  sgxMove( id, sgx_m\x - sgx_vec\x, sgx_m\y - sgx_vec\y, #False )
                  If Not sgx_m\leftdown
                    sgx_moving = -1
                  EndIf
                EndIf ;}
              ;  --
                  
            EndSelect
            
            
            ;{ -- resizing manually via mouse
            If sgx_resizing = id
              If Not sgx_m\leftdown
                sgx_resizing = -1
              ElseIf sgx_m\dx Or sgx_m\dy
                ; new size = mouse-in-element + remaining-to-bottom-right-corner
                w = sgx_m\x - sgx(id)\x + (-sgx_vec\x)
                h = sgx_m\y - sgx(id)\y + (-sgx_vec\y)
                
                If w < sgx(id)\minWidth
                  w = sgx(id)\minWidth
                EndIf
                If h < sgx(id)\minHeight
                  h = sgx(id)\minHeight
                EndIf
                If w <> sgx(id)\w  Or  h <> sgx(id)\h
                  sgxResize(id, w, h)
                EndIf
                
              EndIf
            EndIf
            ;}
            ;  --
            
            
            ;{ -- trigger next hold event
            If sgx(id)\lefthold_timer And sgx(id)\lefthold_timer < sgx_ems
              ; fire a single hold event
              sgx(id)\lefthold = 1
              ; init timer for next hold event
              sgx(id)\lefthold_timer = sgx_ems + sgxHoldInterval
            EndIf
            ;}
            ;  --
            
            
          Next ; finished [For id=0 To ...]                                                                  <<<<<<< MAIN LOOP END
          
          ; ############################################################################
          ; ############################################################################
          
          
          ; elements in radiogroups might reset their 'changed' flag at the beginning of the loop though
          ; it was possibly set by another group element that has been processed in a previous turn
          ; so we stored the change and set it now after all element have been processed.
          ForEach changeLater\list_()
            sgx( changeLater\list_() )\changed = 1
          Next
          ClearList( changeLater\list_() )
          
          ProcedureReturn sgxTopID
        EndProcedure

;}
;  <<<<<<<<<<<<<<<<<<<<<<<< merging end  sgx_examine.pbi

; XIncludeFile "sgx_display.pbi"
;{ >>>>>>>>>>>>>>>>>>>>>>>> merging file sgx_display.pbi

        
        
        
        ;** sgxDisplay
        ;* display all sgx element on screen.
        ;* only DisplayTransparentSprite() is used.
        Procedure sgxDisplay()
          Static id, id2
          Static startpos
          Static endpos
          Static length
          Static temp
          Static x ,y
          Static display
          Static *it.sgx_llS_elem ; use Protected when using recursion!
          
          
          ; display as layered
          
          sgx_llIteratorRecursive(*it,*sgxLayeredView)
            If sgx(*it\id)\type = #sgxTypeWin And sgx(*it\id)\state = #sgxWinStateMinimized ; no minimized
            ;ElseIf sgx(sgxGetTopLevel(*it\id))\state = #sgxWinStateMinimized                ; no childs of minimized
            Else
              sgxDisplayID(*it\id)
            EndIf
          sgx_llNextRecursive(*it)
          
          
          ; display minimized
          
          sgx_llIteratorRecursive(*it,*sgxLayeredView)
            If sgx(*it\id)\type=#sgxTypeWin And sgx(*it\id)\state = #sgxWinStateMinimized
              sgxDisplayID(*it\id)
            EndIf
          sgx_llNextRecursive(*it)
          
          
          ;display tips
          
          sgx_llIteratorRecursive(*it,*sgxLayeredView)
            If sgx(*it\id)\tip And sgx(*it\id)\tip\show = 1
              ;{ calculate tip position from mouse
              x = sgx_m\x+20
              y = sgx_m\y+5
              ; display tip to the left if no space to the right
              If x > sgx_dimensions\w - sgx(*it\id)\tip\w
                x = sgx_m\x - sgx(*it\id)\tip\w - 10
              EndIf
              ; display tip above if no space beneath
              If y > sgx_dimensions\h - sgx(*it\id)\tip\h
                y = sgx_m\y - sgx(*it\id)\tip\h - 10
              EndIf
              ;}
              DisplayTransparentSprite(sgx(*it\id)\tip\spr, x,y)
            EndIf
          sgx_llNextRecursive(*it)
          
          
          ; debug: display hovered element info box and mouse position
          If sgx_debugInfos
            Static txt.s, txtw, txth, xpos, ypos
            xpos = sgx_m\x
            ypos = sgx_m\y
            id = sgxGetElementAt(xpos, ypos)
            StartDrawing(ScreenOutput())
              If id <> -1
                DrawingMode(#PB_2DDrawing_Outlined)
                Box(sgx(id)\x, sgx(id)\y, sgx(id)\w, sgx(id)\h, $ffffff)
                
                DrawingFont(FontID(sgx_fontMonoSmall))
                txt = ""
                txt + "mouse in element: "+Str(sgx_m\x - sgx(id)\x)+","+Str(sgx_m\y - sgx(id)\y) + #LF$ + #LF$
                txt + sgxDebugElem(id, #True)
                txtw = sgx_textWidthLF(txt)
                txth = TextHeight("X") * (CountString(txt, #LF$) + 1)
                If (xpos-40-txtw) > 0
                  xpos = xpos-40-txtw ; display left
                Else
                  xpos + 40 ; display right
                EndIf
                If ypos+(20+txth) > sgx_dimensions\h
                  ypos - txth
                Else
                  ypos + 20
                EndIf
                DrawingMode(#PB_2DDrawing_Default)
                Box(xpos, ypos, txtw, txth, $666666)
                DrawingMode(#PB_2DDrawing_Transparent)
                sgx_drawTextLF(xpos, ypos, txt, $dddddd)
              EndIf
              
              ; display mouse position
              DrawingFont(FontID(sgx_fontMonoSmall))
              DrawingMode(#PB_2DDrawing_Transparent)
              DrawText(sgx_m\x+20, sgx_m\y, ""+sgx_m\x+","+sgx_m\y)
              
            StopDrawing()
            
          EndIf
          
          ;display mouse cursor
          sgxDisplayCursor()
          
        EndProcedure
        
        
        ;** sgxDisplayCursor
        ;* display the sgx mouse cursor on screen. under normal circumstances you don't have to call this function. _
        ;* usefull if you want to overdraw/display something after you called sgxDisplay() but still want the mouse cursor to be visible, so you can redraw the cursor with this function after your drawing operations. _
        Procedure sgxDisplayCursor()
          If  sgx_m\cursor <> -1 And sgx(sgx_m\cursor)\show
            DisplayTransparentSprite( sgx(sgx_m\cursor)\spr[#sgx_GadgetSprNormal], sgx_m\x, sgx_m\y    )
          EndIf
        EndProcedure
        
        ;** sgxDisplayID
        ;* display an sgx element on screen.
        ;* only DisplayTransparentSprite() is used.
        ;** .id : ID of the sgx element to be displayed
        Procedure sgxDisplayID(id);, offsetx=0, offsety=0)
          Static offsetX, alpha, i, k, x, y
          
          If sgx(id)\show=0
            ProcedureReturn ;
          Else
            
            Select sgx(id)\type
                
                ;- Button
              Case #sgxTypeButton ;{
                If sgx(id)\leftdown And id <> sgx_resizing
                  DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprPushed], 0+sgx(id)\x, 0+sgx(id)\y, sgx(id)\alpha    )
                ElseIf sgx(id)\hover
                  DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprHover], sgx(id)\x, sgx(id)\y, sgx(id)\alpha    )
                Else
                  DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprNormal], sgx(id)\x, sgx(id)\y, sgx(id)\alpha    )
                EndIf
                ; user sprite
                If IsSprite( sgx(id)\spr[#sgx_GadgetSprNormal2] )
                  If sgx(id)\disabled
                    alpha = sgx(id)\alpha / 2
                  Else
                    alpha = sgx(id)\alpha
                  EndIf
                  If sgx(id)\leftdown And id <> sgx_resizing
                    ; margin is the text offset when 'pushed' and is stored during refresh, because offset can be
                    ; deactivated by a custom drawing callback via #sgxDrawTextNoOffset
                    DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprNormal2], sgx(id)\x+sgx(id)\x2+sgx(id)\margin, sgx(id)\y+sgx(id)\y2+sgx(id)\margin, alpha    )
                  Else
                    DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprNormal2], sgx(id)\x+sgx(id)\x2  , sgx(id)\y+sgx(id)\y2, alpha    )
                  EndIf
                EndIf
                ;}
        
              
              ;- ToggleButton
            Case #sgxTypeButtonToggle ;{
              If sgx(id)\state=1 ; toggled
                If sgx(id)\hover
                  DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprHoverPushed],   sgx(id)\x, sgx(id)\y, sgx(id)\alpha    )
                Else
                  DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprPushed], sgx(id)\x,  sgx(id)\y, sgx(id)\alpha    )
                EndIf
              Else
                If sgx(id)\hover
                  DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprHover], sgx(id)\x, sgx(id)\y, sgx(id)\alpha    )
                Else
                  DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprNormal], sgx(id)\x, sgx(id)\y, sgx(id)\alpha    )
                EndIf
              EndIf
              ; user sprite
              If IsSprite( sgx(id)\spr[#sgx_GadgetSprNormal2] )
                If sgx(id)\disabled
                  alpha = sgx(id)\alpha / 2
                Else
                  alpha = sgx(id)\alpha
                EndIf
                If sgx(id)\state=1 ; toggled
                  DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprNormal2], sgx(id)\x+sgx(id)\x2+1, sgx(id)\y+sgx(id)\y2+1, alpha    ) ; +1 = same as text offset for pushed
                Else
                  DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprNormal2], sgx(id)\x+sgx(id)\x2  , sgx(id)\y+sgx(id)\y2, alpha    )
                EndIf
              EndIf
              ;}
        
              
              ;- Checkbox
            Case #sgxTypeCheckbox ;{
              
              If sgx(id)\hover
                DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprHover],  sgx(id)\x + sgx(id)\x2,  sgx(id)\y + sgx(id)\y2, sgx(id)\alpha    )
              Else
                DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprNormal],  sgx(id)\x + sgx(id)\x2,  sgx(id)\y + sgx(id)\y2, sgx(id)\alpha    )
              EndIf
              ; 'checked' part
              If sgx(id)\state
                DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprChecked],  sgx(id)\x,  sgx(id)\y,  sgx(id)\alpha  )
              EndIf
              ;}
          
              
              ;- Scrollbar/Trackbar
            Case #sgxTypeScrollbarVer, #sgxTypeTrackbarVer, #sgxTypeScrollbarHor, #sgxTypeTrackbarHor ;{
              If sgx(id)\hover Or sgx_focusCatchedBy=id
                ; bar
                DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprHover],  sgx(id)\x , sgx(id)\y, sgx(id)\alpha   )
                ; knob
                DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprHover2], sgx(id)\x+sgx(id)\x2, sgx(id)\y+sgx(id)\y2, sgx(id)\alpha  )
              Else
                ; bar
                DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprNormal], sgx(id)\x , sgx(id)\y, sgx(id)\alpha   )
                ; knob
                DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprNormal2],sgx(id)\x+sgx(id)\x2, sgx(id)\y+sgx(id)\y2, sgx(id)\alpha  )
              EndIf
              ;}
          
              
              ;- Edit
            Case #sgxTypeEdit ;{
              If sgx(id)\hover Or id=sgx_editing
                DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprHover], sgx(id)\x, sgx(id)\y, sgx(id)\alpha    )
                If id=sgx_editing And ( (sgx_ems/200)%2 Or KeyboardPushed(#PB_Key_All) )
                  DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprTextCursor], sgx(id)\x + sgx(id)\x2 + sgx(id)\textcursor\x-1 + sgx_textCursorClippingOffset\x, sgx(id)\y + sgx(id)\y2 + sgx(id)\textcursor\y + sgx_textCursorClippingOffset\y, sgx(id)\alpha )
                EndIf
              Else
                DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprNormal], sgx(id)\x, sgx(id)\y, sgx(id)\alpha    )
              EndIf
              ;}
          
              
              ;- Window
            Case #sgxTypeWin ;{
              ; note: some window states get sorted out in sgxDisplay() already
              If id=sgxActiveWin Or id=sgx_focusCatchedBy ;Or sgxIsParentage(id, sgx_activeWin)
                DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprPushed], sgx(id)\x, sgx(id)\y, sgx(id)\alpha    )
              ElseIf sgx(id)\hover Or sgx_IsParentage(id, sgxActiveWin)
                DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprHover], sgx(id)\x, sgx(id)\y, sgx(id)\alpha    )
              Else
                DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprNormal], sgx(id)\x, sgx(id)\y, sgx(id)\alpha    )
              EndIf
              
              ;close gadget
              If sgx(id)\state <> #sgxWinStateMinimized  And  sgx(id)\width  And   sgx(id)\w > sgx(id)\width  And  sgx(id)\h > sgx(id)\width
                If sgx_MousehoverClosebutton(id) And (Not sgx(id)\disabled)
                  DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprHover2], sgx(id)\x+sgx(id)\sys0\x1, sgx(id)\y+sgx(id)\sys0\y1, sgx(id)\alpha )
                Else
                  DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprNormal2], sgx(id)\x+sgx(id)\sys0\x1, sgx(id)\y+sgx(id)\sys0\y1, sgx(id)\alpha )
                EndIf
              EndIf
              ;}
            
              
              ;- Table
            Case #sgxTypeTable ;{
              If sgx(id)\hover
                DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprHover], sgx(id)\x, sgx(id)\y, sgx(id)\alpha    )
              Else
                DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprNormal], sgx(id)\x, sgx(id)\y, sgx(id)\alpha    )
              EndIf
              ;}
            
              
              ;- Sprite
            Case #sgxTypeSpriteGadget ;{
              If IsSprite(sgx(id)\state)
                If sgx(id)\disabled
                  DisplayTransparentSprite( sgx(id)\state, sgx(id)\x,sgx(id)\y, sgx(id)\alpha / 2   )
                Else
                  DisplayTransparentSprite( sgx(id)\state, sgx(id)\x,sgx(id)\y, sgx(id)\alpha   )
                EndIf
              EndIf
              ;}
            
              
              ;- Combo
            Case #sgxTypeCombo ;{
              If sgx(id)\max=1
                ; the open combo
                DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprPushed], sgx(id)\x,  sgx(id)\y, sgx(id)\alpha    )
                If sgx(id)\hover
                  ; highlight the hovered entry
                  DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprHover2], sgx(id)\x, sgx(id)\y+(sgx_m\y-sgx(id)\y)/sgx(id)\h2*sgx(id)\h2, sgx(id)\alpha    )
                EndIf
              Else
                If sgx(id)\hover
                  DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprHover], sgx(id)\x, sgx(id)\y, sgx(id)\alpha    )
                Else
                  DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprNormal], sgx(id)\x, sgx(id)\y, sgx(id)\alpha    )
                EndIf
              EndIf
              ;}
            
              
              ;- Text
            Case #sgxTypeText ;{
              If sgx(id)\hover And sgx(id)\style & #sgxStyleHover
                DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprHover], sgx(id)\x, sgx(id)\y, sgx(id)\alpha    )
              Else
                DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprNormal], sgx(id)\x, sgx(id)\y, sgx(id)\alpha    )
              EndIf
              ;}
            
            
            EndSelect
            
            
            ; display resize gadget (for windows only if in normal state; combos only if not opened)
            If sgx(id)\resizable  And  (sgx(id)\type <> #sgxTypeWin Or sgx(id)\state=#sgxWinStateNormal)  And  (sgx(id)\type <> #sgxTypeCombo Or sgx(id)\max<>1)
              If sgx_MousehoverResize(id) And (Not sgx(id)\disabled)
                DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprResize],  sgx(id)\x + sgx(id)\w - sgx_resizeGadgetSize,  sgx(id)\y + sgx(id)\h - sgx_resizeGadgetSize,  sgx(id)\alpha/2    )
              Else
                DisplayTransparentSprite( sgx(id)\spr[#sgx_GadgetSprResize],  sgx(id)\x + sgx(id)\w - sgx_resizeGadgetSize,  sgx(id)\y + sgx(id)\h - sgx_resizeGadgetSize,  sgx(id)\alpha/4    )
              EndIf
            EndIf
            
            
            ; debug: display element frame
            If sgx_debugInfos
              If id = sgxTopID Or sgx_debugInfos = 2
                StartDrawing(ScreenOutput())
                  DrawingMode(#PB_2DDrawing_Outlined)
                  If sgx(id)\type = #sgxTypeCheckbox
                    Box( sgx(id)\x + sgx(id)\x2,  sgx(id)\y + sgx(id)\y2,  sgx(id)\w2, sgx(id)\h2, $66ff66)
                  ElseIf sgx(id)\type & (#sgxTypeScrollbar|#sgxTypeTrackbar)
                    Box( sgx(id)\x + sgx(id)\x2,  sgx(id)\y + sgx(id)\y2,  sgx(id)\w2, sgx(id)\h2, $66ff66)
                  ElseIf sgx(id)\type = #sgxTypeWin
                    Box( sgx(id)\x + sgx(id)\sys0\x1,  sgx(id)\y + sgx(id)\sys0\y1,  sgx(id)\sys0\x2 - sgx(id)\sys0\x1,  sgx(id)\sys0\y2 - sgx(id)\sys0\y1, $66ff66)
                  ElseIf sgx(id)\type = #sgxTypeCombo And sgx(id)\max=1
                    Box( sgx(id)\x,  sgx(id)\y+(sgx_m\y-sgx(id)\y)/sgx(id)\h2*sgx(id)\h2,  sgx(id)\w, sgx(id)\h2, $66ff66)
                  ElseIf sgx(id)\type = #sgxTypeTable  ;And  Not sgx(id)\style & #sgxStyleNoBG  ; (hidden anyway)
                    ; ver lines |
                    For i=0 To sgx(id)\table\w-2
                      x = PeekL( sgx(id)\table\gridx + i*SizeOf(Long) )
                      Line(sgx(id)\x + x, sgx(id)\y, 1, sgx(id)\h, $66ff66)
                    Next
                    ; hor lines --
                    For k=0 To sgx(id)\table\h-2
                      y = PeekL( sgx(id)\table\gridy + k*SizeOf(Long) )
                      Line(sgx(id)\x, sgx(id)\y + y, sgx(id)\w, 1, $66ff66)
                    Next
                  EndIf
                  If id = sgxTopID
                    Box(sgx(id)\x, sgx(id)\y, sgx(id)\w, sgx(id)\h, $ffffff)
                  ElseIf sgx_debugInfos = 2
                    Box(sgx(id)\x, sgx(id)\y, sgx(id)\w, sgx(id)\h, $aaaaaa)
                  EndIf
                StopDrawing()
              EndIf
            EndIf
            
            
          EndIf
        EndProcedure

;}
;  <<<<<<<<<<<<<<<<<<<<<<<< merging end  sgx_display.pbi


;DisableExplicit

;}
;  #################### end sgx



; IDE Options = PureBasic 5.50 beta 1 (Windows - x64)
; CursorPosition = 89
; FirstLine = 65
; Folding = -
; CompileSourceDirectory